/**
 *  ModelBus is the means to interact with models in a distributed
 * consistent way.
 * 
 *  Copyright (C) 2008  Tom Ritter, Christian Hein, Michael Wagner
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation, either version 3 of
 * the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.modelbus.team.eclipse.changemodelnotification.preferences;

import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.FieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

import org.modelbus.team.eclipse.changemodelnotification.Activator;

/**
 * This class represents a preference page that is contributed to the Preferences dialog. By subclassing <samp>FieldEditorPreferencePage</samp>, we
 * can use the field support built into JFace that allows us to create a page that is small and knows how to save, restore and apply itself.
 * <p>
 * This page is used to modify preferences only. They are stored in the preference store that belongs to the main plug-in class. That way, preferences
 * can be accessed directly via the preference store.
 */

public class InteractiveModePreferencePage extends FieldEditorPreferencePage implements IWorkbenchPreferencePage {

	private BooleanFieldEditor	commitOnSaveEditor;
	private BooleanFieldEditor	automaticLockUnlockEditor;
	private BooleanFieldEditor	interactivModelingEditor;

	public InteractiveModePreferencePage() {
		super(GRID);
		
		this.setPreferenceStore(Activator.getDefault().getPreferenceStore());
		this.setDescription("ModelBus Interactive Modeling Configuration");
	}

	/**
	 * Creates the field editors. Field editors are abstractions of the common GUI blocks needed to manipulate various types of preferences. Each
	 * field editor knows how to save and restore itself.
	 */
	public void createFieldEditors() {
		this.interactivModelingEditor = new BooleanFieldEditor(PreferenceConstants.AUTOMATIC_UPDATE, "Enable interactive modeling", this.getFieldEditorParent());

		this.addField(this.interactivModelingEditor);

		this.commitOnSaveEditor = new BooleanFieldEditor(PreferenceConstants.COMMIT_ON_SAVE,
				"Commit changes after save \n default: commit changes after each transaction", this.getFieldEditorParent());
		this.addField(this.commitOnSaveEditor);

		this.automaticLockUnlockEditor = new BooleanFieldEditor(PreferenceConstants.AUTOMATIC_LOCK_UNLOCK, "Automatic lock and unlock of elements",
				this.getFieldEditorParent());

		this.addField(this.automaticLockUnlockEditor);

		//
	}

	@Override
	protected void checkState() {
		super.checkState();

		if (!this.interactivModelingEditor.getBooleanValue()
				&& (this.commitOnSaveEditor.getBooleanValue() || this.automaticLockUnlockEditor.getBooleanValue())) {
			this.setValid(false);
		} else if (this.interactivModelingEditor.getBooleanValue() && !this.commitOnSaveEditor.getBooleanValue()
				&& this.automaticLockUnlockEditor.getBooleanValue()) {
			this.setValid(false);
		} else {
			this.setValid(true);
		}

	}

	@Override
	public void propertyChange(final PropertyChangeEvent event) {
		if (event.getProperty().equals(FieldEditor.VALUE)) {
			this.checkState();
		} else {
			super.propertyChange(event);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(final IWorkbench workbench) {
	}

}