/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.connector;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * LogEntry information container
 * 
 * The JavaHL API's is the only way to interact between ModelBus and Java-based tools. At the same time JavaHL connector library is not EPL compatible
 * and we won't to pin plug-in with concrete connector implementation. So, the only way to do this is providing our own connector interface which will
 * be covered by concrete connector implementation.
 * 
 * @author Alexander Gurov
 */
public class ModelBusLogEntry {
	/**
	 * The revision string
	 */
	public final String				revision;
	
	public final ModelBusRevision modelBusRevision;

	/**
	 * The date of the commit. Could be zero if the user who requested the log has no access rights to the specified resource.
	 */
	public final long				date;

	/**
	 * The commit author. Could be <code>null</code> if commit are performed with anonymous access.
	 */
	public final String				author;

	/**
	 * The log message for the revision. Could be <code>null</code> if revision has no message.
	 */
	public final String				message;

	/**
	 * The set of the items changed by this commit. Could be <code>null</code> when {@link IModelBusConnector#logMessages} is called with
	 * discoverPaths set to false or if the user who requested the log has no access rights to the specified resource.
	 */
	public final ModelBusLogPath[]	changedPaths;

	private List<ModelBusLogEntry>	children;

	/**
	 * The {@link ModelBusLogEntry} instance could be initialized only once because all fields are final
	 * 
	 * @param revision
	 *            the revision number associated with the commit
	 * @param date
	 *            the date of the commit
	 * @param author
	 *            the commit author
	 * @param message
	 *            the commit message text
	 * @param changedPaths
	 *            the set of the items changed by this commit
	 * @param hasChildren
	 *            if <code>true</code> then log entry will allocate memory for a child entries. Directly mapped to the hasChildren() method return
	 *            value.
	 */
	public ModelBusLogEntry(String revision, long date, String author, String message, ModelBusLogPath[] changedPaths, boolean hasChildren) {
		this.message = message;
		this.date = date;
		this.revision = revision;
		this.modelBusRevision = new ModelBusRevision(revision, date);
		this.author = author;
		this.changedPaths = changedPaths;
		this.children = hasChildren ? new ArrayList<ModelBusLogEntry>() : null;
	}

	public ModelBusLogEntry[] getChildren() {
		return this.children == null ? null : this.children.toArray(new ModelBusLogEntry[this.children.size()]);
	}

	public boolean hasChildren() {
		return this.children != null;
	}

	public void add(ModelBusLogEntry child) {
		this.children.add(child);
	}

	public void addAll(ModelBusLogEntry[] child) {
		this.children.addAll(Arrays.asList(child));
	}

	public int hashCode() {
		return this.revision.hashCode();
	}

	public boolean equals(Object obj) {
		if (obj instanceof ModelBusLogEntry) {
			return this.revision == ((ModelBusLogEntry) obj).revision;
		}
		return false;
	}

}
