/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.connector;

import java.text.DateFormat;
import java.util.Locale;

import org.modelbus.core.lib.IRepositoryHelper;
import org.modelbus.dosgi.repository.descriptor.ExtendedRepositoryLogEntry;
import org.modelbus.team.eclipse.repository.ModelBusRepositoryHelper;
import org.modelbus.team.eclipse.repository.UserSessionHelper;


/**
 * Revision information container
 * 
 * The JavaHL API's is the only way to interact between ModelBus and Java-based tools. At the same time JavaHL connector library is not EPL compatible
 * and we won't to pin plug-in with concrete connector implementation. So, the only way to do this is providing our own connector interface which will
 * be covered by concrete connector implementation.
 * 
 * @author Alexander Gurov
 */
public class ModelBusRevision {
	/**
	 * Invalid revision string
	 */
	public static final String					INVALID_REVISION_STRING	= "-1";

	//TODO check why -2
	public static final String					INVALID_REVISION_STRING_2	= "-2";
	
	
	/**
	 * Invalid revision string
	 */
	public static final String					REVISION_0				= "0";
	
	public final static ModelBusRevision REVISION_0_DESCRIPTOR = new ModelBusRevision(REVISION_0, null);
	public final static ModelBusRevision INVALID_REVISION_DESCRIPTOR = new ModelBusRevision(INVALID_REVISION_STRING, null);
	
	protected String 	revision;
	protected Long 		date;
	protected int		revKind;
	
	protected ModelBusRevision(String revision, Long date) {
		this.revision = revision;
		this.date = date;
		this.revKind = Kind.STRING;
	}
	
	protected ModelBusRevision(int kind) {
		this.revKind = kind;
	}

	/**
	 * Last committed revision
	 */
	public static final ModelBusRevision		HEAD					= new ModelBusRevision(Kind.HEAD);

	/**
	 * First existing revision
	 */
	public static final ModelBusRevision		START					= new ModelBusRevision(Kind.START);

	/**
	 * Last committed revision, needs working copy
	 */
	public static final ModelBusRevision		COMMITTED				= new ModelBusRevision(Kind.COMMITTED);

	/**
	 * Previous committed revision, needs working copy
	 */
	public static final ModelBusRevision		PREVIOUS				= new ModelBusRevision(Kind.PREVIOUS);

	/**
	 * Base revision of working copy
	 */
	public static final ModelBusRevision		BASE					= new ModelBusRevision(Kind.BASE);

	/**
	 * Working version in working copy
	 */
	public static final ModelBusRevision		WORKING					= new ModelBusRevision(Kind.WORKING);

	/**
	 * Invalid revision object
	 */
//	public static final ModelBusRevision.StringRevision	INVALID_REVISION		= new ModelBusRevision.Str(INVALID_REVISION_NUMBER);

	/**
	 * Creates revision object by revision number
	 * 
	 * @param revisionNumber
	 *            revision number
	 * @return revision object
	 * @throws IllegalArgumentException
	 *             if revisionNumber is negative
	 */
//	public static ModelBusRevision.Number fromNumber(long revisionNumber) {
//		if (revisionNumber < 0) {
//			throw new IllegalArgumentException("Negative revision numbers are not allowed");
//		}
//		return new ModelBusRevision.Number(revisionNumber);
//	}

	/**
	 * Creates revision object by revision string
	 * 
	 * @param revisionString
	 *            revision string
	 * @return revision object
	 */
//	public static ModelBusRevision.StringRevision fromString(String revisionString) {
//		return new ModelBusRevision.StringRevision(revisionString);
//	}
	
	/**
	 * Creates revision object by revision date
	 * 
	 * @param revisionDate
	 *            revision date in milliseconds
	 * @return revision object
	 * @throws IllegalArgumentException
	 *             if revisionDate is zero
	 */
	public static ModelBusRevision.DateRevision fromDate(long revisionDate) {
		if (revisionDate == -1) {
			throw new IllegalArgumentException("A date must be specified");
		}
		return new ModelBusRevision.DateRevision(revisionDate);
	}
	
	/**
	 * Creates revision object by revision string and date
	 * 
	 * @param revisionString
	 *            revision string
	 * @param revisionDate
	 *            revision date in milliseconds
	 * @return revision object
	 * @throws IllegalArgumentException
	 *             if revisionDate is zero
	 */
	public static ModelBusRevision fromStringAndDate(final String revisionString, long revisionDate) {
		if (revisionDate == -1) {
			throw new IllegalArgumentException("A date must be specified");
		}
		return new ModelBusRevision(revisionString, revisionDate);
	}

	/**
	 * Creates revision object by revision string
	 * 
	 * @param revisionString
	 *            string representation of one of valid revisions or revision kinds
	 * @return revision object
	 * @throws IllegalArgumentException
	 *             if invalid revision kind or revision value specified
	 */
	public static ModelBusRevision fromString(String revisionString) {
		revisionString = revisionString.toUpperCase();
		
		if ("BASE".equals(revisionString)) {
			return ModelBusRevision.BASE;
		}
		if ("WORKING".equals(revisionString)) {
			return ModelBusRevision.WORKING;
		}
		if ("COMMITTED".equals(revisionString)) {
			return ModelBusRevision.COMMITTED;
		}
		if ("HEAD".equals(revisionString)) {
			return ModelBusRevision.HEAD;
		}
		if ("PREVIOUS".equals(revisionString)) {
			return ModelBusRevision.PREVIOUS;
		}
		
//		/*try {
//			return ModelBusRevision.fromNumber(Long.parseLong(revisionString));
//		} catch (NumberFormatException ex) {*/
//			// check if revision specified as date (always locale-specific)
//			DateFormat dateTimeFormat = DateFormat.getDateTimeInstance(DateFormat.DEFAULT, DateFormat.DEFAULT, Locale.getDefault());
//			try {
//				return ModelBusRevision.fromDate(dateTimeFormat.parse(revisionString).getTime());
//			} catch (ParseException e) {
//				// do nothing
//			}
//		//}
		
		return new ModelBusRevision.StringRevision(revisionString);
		
//		throw new IllegalArgumentException("Invalid revision string: " + revisionString);
	}

	public String toString() {
		switch (this.revKind) {
			case Kind.BASE: {
				return "BASE";
			}
			case Kind.COMMITTED: {
				return "COMMITTED";
			}
			case Kind.HEAD: {
				return "HEAD";
			}
			case Kind.PREVIOUS: {
				return "PREV";
			}
			case Kind.WORKING: {
				return "WORKING";
			}
		}
		
		if(this.revision != null){
			return this.revision;
		}
		
		return "UNSPECIFIED";
	}

	/**
	 * Returns the kind of the revision object
	 * 
	 * @return the revision kind
	 */
	public int getKind() {
		return this.revKind;
	}
	
	/**
	 * Returns the revision string of the revision object
	 * 
	 * @return the revision string
	 */
	public String getRevision() {
		return revision;
	}

	/**
	 * Returns the revision date of the revision object
	 * 
	 * @return the revision date
	 */
	public Long getDate() {
		return date;
	}

	public boolean isGreaterThan(ModelBusRevision other){
		if(this.date == null ^ other.getDate() == null){
			return false;
		}
		
		return this.date > other.getDate();
	}
	
	public boolean isGreaterThanOrEqual(ModelBusRevision other){
		if(this.equals(other)){
			return true;
		}
		
		return this.isGreaterThan(other);
	}
	
	public boolean isLowerThan(ModelBusRevision other){
		if(this.date == null ^ other.getDate() == null){
			return false;
		}
		
		return this.date < other.getDate();
	}
	
	public boolean isLowerThanOrEqual(ModelBusRevision other){
		if(this.equals(other)){
			return true;
		}
		
		return this.isLowerThan(other);
	}
	
	public ModelBusRevision getPreviousRevision(){
		IRepositoryHelper repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();
		
		ExtendedRepositoryLogEntry logEntry;
		
		try {
			logEntry = repositoryHelper.getExtendedLogEntry(UserSessionHelper.getSession(), this.getRevision(), this.getDate());
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
		
		if(logEntry != null){
			final ModelBusRevision previousRevision = new ModelBusRevision(logEntry.getPreviousRevision(), logEntry.getPreviousRevisionDate().toGregorianCalendar().getTimeInMillis());
			
			return previousRevision;
		}
		
		return null;
	}
	
	public int compareTo(final ModelBusRevision other){
		if(this.equals(other)){
			return 0;
		}
		
		return this.isGreaterThan(other) ? 1 : -1;
	}
	
	public int hashCode() {
		return this.revKind;
	}

	public boolean equals(Object target) {
		if (this == target) {
			return true;
		}
		if (!this.getClass().equals(target.getClass())) {
			return false;
		}

		return ((ModelBusRevision) target).revKind == this.revKind;
	}

	/**
	 * The revision definition ways
	 */
	public static final class Kind {
		/**
		 * First existing revision
		 */
		public static final int	START		= 0;

		/**
		 * Number-based revision
		 */
		public static final int	STRING		= 1;

		/**
		 * Date-based revision
		 */
		public static final int	DATE		= 2;

		/**
		 * Last committed revision
		 */
		public static final int	COMMITTED	= 3;

		/**
		 * The revision before last committed
		 */
		public static final int	PREVIOUS	= 4;

		/**
		 * The working copy base revision
		 */
		public static final int	BASE		= 5;

		/**
		 * The working copy working revision
		 */
		public static final int	WORKING		= 6;

		/**
		 * The latest repository revision
		 */
		public static final int	HEAD		= 7;
	}
	
	/**
	 * String-based revision
	 */
	public static class StringRevision extends ModelBusRevision {

		public String toString() {
			return this.revision;
		}

		@Override
		public int hashCode() {
			int result = 31;
			result += this.revKind;
			result = 31 * result + (int) this.revision.hashCode();
			result = 31 * result + (int) (this.revision.hashCode() >> 32);
			
			return result;
		}

		public boolean equals(Object target) {
			return super.equals(target) && ((ModelBusRevision.StringRevision) target).revision.equals(this.revision);
		}

		protected StringRevision(String string) {
			super(Kind.STRING);
			this.revision = string;
		}

	}

//	/**
//	 * Date-based revision
//	 */
	public static class DateRevision extends ModelBusRevision {
		protected long	revDate;

		/**
		 * Returns the date of the revision
		 * 
		 * @return the date
		 */
		public Long getDate() {
			return this.revDate;
		}

		public String toString() {
			DateFormat dateTimeFormat = DateFormat.getDateTimeInstance(DateFormat.DEFAULT, DateFormat.DEFAULT, Locale.getDefault());
			return dateTimeFormat.format(new java.util.Date(this.revDate));
		}

		public int hashCode() {
			int result = 31;
			result += this.revKind;
			result = 31 * result + (int) this.revDate;
			result = 31 * result + (int) (this.revDate >> 32);
			return result;
		}

		public boolean equals(Object target) {
			return super.equals(target) && ((ModelBusRevision.DateRevision) target).revDate == this.revDate;
		}

		protected DateRevision(long date) {
			super(Kind.DATE);
			this.revDate = date;
		}

	}
	
	/**
	 * Creates revision object by revision kind
	 * 
	 * @param kind
	 *            revision kind
	 * @return revision object
	 * @throws IllegalArgumentException
	 *             if kind is {@link Kind#DATE}, {@link Kind#NUMBER} or exceeds kind limits
	 */
	public static ModelBusRevision fromKind(int kind) {
		switch (kind) {
			case Kind.BASE: {
				return ModelBusRevision.BASE;
			}
			case Kind.WORKING: {
				return ModelBusRevision.WORKING;
			}
			case Kind.HEAD: {
				return ModelBusRevision.HEAD;
			}
			case Kind.PREVIOUS: {
				return ModelBusRevision.PREVIOUS;
			}
			case Kind.START: {
				return ModelBusRevision.START;
			}
			case Kind.COMMITTED: {
				return ModelBusRevision.COMMITTED;
			}
			case Kind.STRING: {
				throw new IllegalArgumentException("Use fromString() method instead");
			}
		}
		
		throw new IllegalArgumentException("Invalid revision kind: " + kind);
	}
}
