/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *    Panagiotis Korros - [patch] bug fix: .modelbus folder is deleted by subversive
 *******************************************************************************/

package org.modelbus.team.eclipse.core.modelbusstorage;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.internal.preferences.Base64;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceVisitor;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.jobs.ISchedulingRule;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.team.core.RepositoryProvider;
import org.modelbus.team.eclipse.core.IConnectedProjectInformation;
import org.modelbus.team.eclipse.core.IStateFilter;
import org.modelbus.team.eclipse.core.ModelBusTeamPlugin;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Depth;
import org.modelbus.team.eclipse.core.connector.ModelBusChangeStatus;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusConnectorException;
import org.modelbus.team.eclipse.core.connector.ModelBusEntry;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryStatus;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.extension.CoreExtensionsManager;
import org.modelbus.team.eclipse.core.extension.options.IIgnoreRecommendations;
import org.modelbus.team.eclipse.core.operation.AbstractActionOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusNullProgressMonitor;
import org.modelbus.team.eclipse.core.operation.UnreportableException;
import org.modelbus.team.eclipse.core.resource.IChangeStateProvider;
import org.modelbus.team.eclipse.core.resource.ILocalFolder;
import org.modelbus.team.eclipse.core.resource.ILocalResource;
import org.modelbus.team.eclipse.core.resource.IRemoteStorage;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.resource.IResourceChange;
import org.modelbus.team.eclipse.core.resource.events.IResourceStatesListener;
import org.modelbus.team.eclipse.core.resource.events.ResourceStatesChangedEvent;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.core.utility.ProgressMonitorUtility;

/**
 * ModelBus based representation of IRemoteStorage
 * 
 * @author Alexander Gurov
 */
public class ModelBusRemoteStorage extends AbstractModelBusStorage implements IRemoteStorage {

	/**
	 * The name of the preferences node in the Subversive Core preferences that contains the known repositories as its children.
	 */
	public static final String							PREF_REPOSITORIES_NODE	= "repositories";

	/**
	 * The name of file containing the ModelBus repository locations information. Deprecated since Subversive 0.7.0 v20080404 - must not be used. The
	 * valid information is stored in preferences.
	 * 
	 * @see ModelBusRemoteStorage.PREF_REPOSITORIES_NODE
	 */
	public static final String							STATE_INFO_FILE_NAME	= ".modelbusRepositories";

	private static ModelBusRemoteStorage				instance				= new ModelBusRemoteStorage();

	protected Map										localResources;
	protected Map										switchedToUrls;
	protected Map										externalsLocations;
	protected Map										parent2Children;
	protected Map<Class, List<IResourceStatesListener>>	resourceStateListeners;
	protected LinkedList								fetchQueue;

	protected ISchedulingRule							notifyLock				= new ISchedulingRule() {
																					public boolean isConflicting(ISchedulingRule rule) {
																						return rule == this;
																					}

																					public boolean contains(ISchedulingRule rule) {
																						return rule == this;
																					}
																				};

	public static ModelBusRemoteStorage instance() {
		return ModelBusRemoteStorage.instance;
	}

	public void addResourceStatesListener(Class eventClass, IResourceStatesListener listener) {
		synchronized (this.resourceStateListeners) {
			List<IResourceStatesListener> listenersList = this.resourceStateListeners.get(eventClass);
			if (listenersList == null) {
				this.resourceStateListeners.put(eventClass, listenersList = new ArrayList<IResourceStatesListener>());
			}
			if (!listenersList.contains(listener)) {
				listenersList.add(listener);
			}
		}
	}

	public void removeResourceStatesListener(Class eventClass, IResourceStatesListener listener) {
		synchronized (this.resourceStateListeners) {
			List<IResourceStatesListener> listenersList = this.resourceStateListeners.get(eventClass);
			if (listenersList != null) {
				listenersList.remove(listener);
				if (listenersList.size() == 0) {
					this.resourceStateListeners.remove(eventClass);
				}
			}
		}
	}

	public void fireResourceStatesChangedEvent(final ResourceStatesChangedEvent event) {
		if (event.resources.length > 0) {
			// events should be serialized and called asynchronous to caller thread
			ProgressMonitorUtility.doTaskScheduled(new AbstractActionOperation("Operation.SendNotifications") {
				protected void runImpl(IProgressMonitor monitor) throws Exception {
					IResourceStatesListener[] listeners = null;
					synchronized (ModelBusRemoteStorage.this.resourceStateListeners) {
						List<IResourceStatesListener> listenersArray = ModelBusRemoteStorage.this.resourceStateListeners.get(event.getClass());
						if (listenersArray == null) {
							return;
						}
						listeners = listenersArray.toArray(new IResourceStatesListener[listenersArray.size()]);
					}
					for (int i = 0; i < listeners.length; i++) {
						listeners[i].resourcesStateChanged(event);
					}
				}

				public ISchedulingRule getSchedulingRule() {
					return ModelBusRemoteStorage.this.notifyLock;
				}
			}, true);
		}
	}

	public void initialize(IPath stateInfoLocation) throws Exception {
		this.setStateInfoFile(stateInfoLocation, ModelBusRemoteStorage.STATE_INFO_FILE_NAME);
		this.initializeImpl(ModelBusRemoteStorage.PREF_REPOSITORIES_NODE);
	}

	public IResourceChange asResourceChange(IChangeStateProvider changeState, boolean update) {
		IResource resource = null;
		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
		IPath location = new Path(changeState.getLocalPath());
		int nodeKind = changeState.getNodeKind();
		int propKind = changeState.getPropertiesChangeType();
		int textKind = changeState.getTextChangeType();
		boolean isCopied = changeState.isCopied();
		boolean isSwitched = changeState.isSwitched();
		ModelBusRevision remoteRevision = changeState.getChangeRevision();
		ModelBusRevision revision = remoteRevision != null ? remoteRevision : ModelBusRevision.INVALID_REVISION_DESCRIPTOR;
		// repositoryTextStatus can be StatusKind::none in two cases: resource not modified and non versioned
		// in the second case we should ignore repository status calculation
		String statusStr = /* revision == ModelBusRevision.INVALID_REVISION_NUMBER ? IStateFilter.ST_NOTEXISTS : */this.getStatusString(propKind,
				textKind, update);
		if (nodeKind == ModelBusEntry.Kind.DIR) {
			if ((resource = changeState.getExact(root.findContainersForLocation(location))) == null) {
				return null;
			}
			int changeMask = this.getChangeMask(textKind, propKind, isCopied, isSwitched);
			if (IStateFilter.SF_NOTEXISTS.accept(resource, statusStr, changeMask)) {
				revision = ModelBusRevision.INVALID_REVISION_DESCRIPTOR;
			}
			
//			IResource resource, ModelBusRevision revision, String status, int changeMask, String author, long lastCommitDate,
//			ModelBusRevision pegRevision, String comment
			
			return new ModelBusFolderChange(resource, revision, statusStr, changeMask, changeState.getChangeAuthor(), changeState.getChangeRevision().getDate().longValue(),
					null, changeState.getComment());
		}
		if ((resource = changeState.getExact(root.findFilesForLocation(location))) == null) {
			return null;
		}
		int changeMask = this.getChangeMask(textKind, propKind, isCopied, isSwitched);
		if (IStateFilter.SF_NOTEXISTS.accept(resource, statusStr, changeMask)) {
			revision = ModelBusRevision.INVALID_REVISION_DESCRIPTOR;
		}
		return new ModelBusFileChange(resource, revision, statusStr, changeMask, changeState.getChangeAuthor(), changeState.getChangeRevision().getDate(), null,
				changeState.getComment());
	}

	public byte[] resourceChangeAsBytes(IResourceChange resource) {
		if (resource == null) {
			return null;
		}
		int kind = resource.getPegRevision().getKind();
		String originatorData = null;
		if (resource.getOriginator() != null) {
			byte[] data = this.repositoryResourceAsBytes(resource.getOriginator());
			originatorData = new String(Base64.encode(data));
		}
		long lastCommitDate = resource.getLastCommitDate();
		String comment = resource.getComment();
		String retVal =
		/* 0 */String.valueOf(resource instanceof ILocalFolder)
				+ ";"
				+
				/* 1 */new String(Base64.encode(FileUtility.getWorkingCopyPath(resource.getResource()).getBytes()))
				+ ";"
				+
				/* 2 */resource.getRevision()
				+ ";"
				+
				/* 3 */resource.getStatus()
				+ ";"
				+
				/* 4 */resource.getAuthor()
				+ ";"
				+
				/* 5 */(lastCommitDate == 0 ? "null" : String.valueOf(lastCommitDate))
				+ ";"
				+
				/* 6 */String.valueOf(kind)
				+ ";"
				+
				/* 7 */(kind == ModelBusRevision.Kind.STRING ? ((ModelBusRevision.StringRevision) resource.getPegRevision()).getRevision()
						: String.valueOf(kind)) + ";" +
				/* 8 */(originatorData != null ? originatorData : "null") + ";" +
				/* 9 */(comment == null ? "null" : new String(Base64.encode(comment.getBytes()))) + ";" +
				/* 10 */resource.getChangeMask();
		return retVal.getBytes();
	}

	public IResourceChange resourceChangeFromBytes(byte[] bytes) {
		if (bytes == null) {
			return null;
		}
		String[] data = new String(bytes).split(";");
		boolean isFolder = "true".equals(data[0]);
		String name = new String(Base64.decode(data[1].getBytes()));
		String revision = data[2];
		String status = this.deserializeStatus(data[3]);
		String author = "null".equals(data[4]) ? null : data[4];
		long lastCommitDate = "null".equals(data[5]) ? 0 : Long.parseLong(data[5]);
		int revisionKind = Integer.parseInt(data[6]);
		ModelBusRevision pegRevision = null;
		if (revisionKind == ModelBusRevision.Kind.STRING) {
			String pegNum = data[7];
			pegRevision = pegNum == revision || revision == ModelBusRevision.INVALID_REVISION_STRING ? null : ModelBusRevision.fromString(pegNum);
		} else {
			pegRevision = ModelBusRevision.fromKind(revisionKind);
		}
		String comment = "null".equals(data[9]) ? null : new String(Base64.decode(data[9].getBytes()));
		int changeMask = "null".equals(data[10]) ? ILocalResource.NO_MODIFICATION : Integer.parseInt(data[10]);
		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
		
		ModelBusRevision revisionObj = ModelBusRevision.fromStringAndDate(revision, lastCommitDate);
		
		IResourceChange change = isFolder ? (IResourceChange) new ModelBusFolderChange(root.getContainerForLocation(new Path(name)), revisionObj,
				status, changeMask, author, lastCommitDate, pegRevision, comment) : new ModelBusFileChange(root.getFileForLocation(new Path(name)),
						revisionObj, status, changeMask, author, lastCommitDate, pegRevision, comment);

		if (!"null".equals(data[8])) {
			byte[] originatorData = Base64.decode(data[8].getBytes());
			change.setOriginator(this.repositoryResourceFromBytes(originatorData));
		}

		return change;
	}

	public synchronized IResource[] getRegisteredChildren(IContainer container) throws Exception {
		// for null, inaccessible resource and workspace root members shouldn't be provided
		if (container == null || !container.isAccessible() || container.getProject() == null) {
			return null;
		}
		IResource[] members = FileUtility.resourceMembers(container, false);

		Set retVal = (Set) this.parent2Children.get(container.getFullPath());
		if (retVal == null) {
			this.loadLocalResourcesSubTree(container, false);
			retVal = (Set) this.parent2Children.get(container.getFullPath());
		}
		if (retVal != null) {
			retVal = new HashSet(retVal);
			retVal.addAll(Arrays.asList(members));
		}

		return retVal == null ? members : (IResource[]) retVal.toArray(new IResource[retVal.size()]);
	}

	public ILocalResource asLocalResourceDirty(IResource resource) {
		if (!CoreExtensionsManager.instance().getOptionProvider().isModelBusCacheEnabled()) {
			return this.asLocalResource(resource);
		}
		// null resource and workspace root shouldn't be provided
		if (resource == null || resource.getProject() == null) {
			return this.wrapUnexistingResource(resource, IStateFilter.ST_INTERNAL_INVALID, 0);
		}
		ILocalResource retVal = (ILocalResource) this.localResources.get(resource.getFullPath());
		if (retVal == null) {
			ILocalResource parent = this.getFirstExistingParentLocal(resource);
			int mask = parent == null ? 0 : (parent.getChangeMask() & ILocalResource.IS_SWITCHED);
			retVal = this.wrapUnexistingResource(resource, IStateFilter.ST_NOTEXISTS, mask);
		}
		return retVal;
	}

	public ILocalResource asLocalResourceAccessible(IResource resource) {
		ILocalResource retVal = this.asLocalResource(resource);
		if (IStateFilter.SF_INTERNAL_INVALID.accept(retVal)) {
			throw new UnreportableException(ModelBusTeamPlugin.instance().getResource("Error.InaccessibleResource"));
		}
		return retVal;
	}

	public ILocalResource asLocalResource(IResource resource) {
		// null resource and workspace root shouldn't be provided
		if (resource == null || resource.getProject() == null || !resource.getProject().isAccessible()) {
			return this.wrapUnexistingResource(resource, IStateFilter.ST_INTERNAL_INVALID, 0);
		}
		ILocalResource local = (ILocalResource) this.localResources.get(resource.getFullPath());
		if (local == null) {
			synchronized (this) {
				local = (ILocalResource) this.localResources.get(resource.getFullPath());
				if (local == null) {
					try {
						local = this.loadLocalResourcesSubTree(resource, true);
					} catch (RuntimeException ex) {
						throw ex;
					} catch (ModelBusConnectorException ex) {
						return this.wrapUnexistingResource(resource, IStateFilter.ST_INTERNAL_INVALID, 0);
					} catch (Exception e) {
						throw new RuntimeException(e);
					}
				}
			}
		}
		return local;
	}

	public synchronized void refreshLocalResources(IResource[] resources, int depth) {
		if (depth == IResource.DEPTH_INFINITE) {
			resources = FileUtility.shrinkChildNodes(resources);
		}
		for (int i = 0; i < resources.length; i++) {
			this.refreshLocalResourceImpl(resources[i], depth);
		}
	}

	public ILocalResource asLocalResource(IProject project, String url, int kind) {
		synchronized (this.switchedToUrls) {
			for (Iterator it = this.switchedToUrls.entrySet().iterator(); it.hasNext();) {
				Map.Entry entry = (Map.Entry) it.next();
				String cachedUrl = (String) entry.getValue();
				if (new Path(cachedUrl).isPrefixOf(new Path(url))) {
					IPath target = ((IPath) entry.getKey()).append(url.substring(cachedUrl.length())).removeFirstSegments(1);
					return this.asLocalResource(kind == IResource.FOLDER ? (IResource) project.getFolder(target) : project.getFile(target));
				}
			}
		}
		String projectUrl = this.asRepositoryResource(project).getUrl();
		if (url.length() < projectUrl.length()) {
			return this.wrapUnexistingResource(null, IStateFilter.ST_INTERNAL_INVALID, 0);
		}
		String pathInProject = url.substring(projectUrl.length());
		if (pathInProject.length() == 0) {
			return this.asLocalResource(project);
		}
		return this.asLocalResource(kind == IResource.FOLDER ? (IResource) project.getFolder(pathInProject) : project.getFile(pathInProject));
	}

	public IRepositoryResource asRepositoryResource(IResource resource) {
		IProject project = resource.getProject();
		IRepositoryResource baseResource = this.getConnectedProjectInformation(project).getRepositoryResource();

		if (resource.equals(project)) {
			return ModelBusUtility.copyOf(baseResource);
		}

		String url = (String) this.switchedToUrls.get(resource.getFullPath());
		if (url == null) {
			ILocalResource parent = this.getFirstExistingParentLocal(resource);
			if (parent != null && (parent.getChangeMask() & ILocalResource.IS_SWITCHED) != 0) {
				IPath parentPath = parent.getResource().getFullPath();

				url = (String) this.switchedToUrls.get(parentPath) + "/"
						+ resource.getFullPath().removeFirstSegments(parentPath.segmentCount()).toString();
			}
		}

		if (url == null) {
			url = this.makeUrl(resource, baseResource);
		} else if (!new Path(ModelBusConnector.getRepositoryRootUrl()).isPrefixOf(new Path(url))) {
			// location = this.wrapLocationIfRequired(url, resource.getType() == IResource.FILE);
			// TODOMWA
			System.out.println("CHECKMWA19");
		}

		return resource instanceof IContainer ? (IRepositoryResource) ModelBusConnector.asRepositoryContainer(url, false) : ModelBusConnector
				.asRepositoryFile(url, false);
	}

	public IRepositoryResource asRepositoryResource(String url, boolean isFile) {
		// if (!new Path(location.getRepositoryRootUrl()).isPrefixOf(new Path(url))) {
		// location = this.wrapLocationIfRequired(location, url, isFile);
		// } CHECKMWA

		return !isFile ? (IRepositoryResource) ModelBusConnector.asRepositoryContainer(url, false) : ModelBusConnector.asRepositoryFile(url, false);
	}

	// public IRepositoryLocation getRepositoryLocation(IResource resource) {
	// return this.getConnectedProjectInformation(resource.getProject()).getRepositoryLocation();
	// }

	// protected IRepositoryLocation wrapLocationIfRequired(IRepositoryLocation location, String url, boolean isFile) {
	// if (!new Path(location.getRepositoryRootUrl()).isPrefixOf(new Path(url))) {
	// synchronized (this.externalsLocations) {
	// List locations = (List)this.externalsLocations.get(location);
	// if (locations == null) {
	// this.externalsLocations.put(location, locations = new ArrayList());
	// }
	// boolean found = false;
	// for (Iterator it = locations.iterator(); it.hasNext(); ) {
	// IRepositoryLocation tmp = (IRepositoryLocation)it.next();
	// if (url.startsWith(tmp.getUrl()) || tmp.getUrl().startsWith(url)) {
	// location = tmp;
	// found = true;
	// break;
	// }
	// }
	// if (!found) {
	// location = new ModelBusRepositoryLocationWrapper(location, isFile ? url.substring(0, url.lastIndexOf('/')) : url);
	// locations.add(location);
	// }
	// }
	// }
	// return location;
	// }

	protected ILocalResource wrapUnexistingResource(IResource resource, String state, int mask) {
		return resource == null || resource.getType() == IResource.FILE ? (ILocalResource) new ModelBusLocalFile(resource,
				ModelBusRevision.INVALID_REVISION_DESCRIPTOR, ModelBusRevision.INVALID_REVISION_DESCRIPTOR, state, mask, null, 0) : new ModelBusLocalFolder(
				resource, ModelBusRevision.INVALID_REVISION_DESCRIPTOR, ModelBusRevision.INVALID_REVISION_DESCRIPTOR, state, mask, null, 0);
	}

	protected String makeUrl(IResource resource, IRepositoryResource baseResource) {
		if (resource.getType() == IResource.PROJECT) {
			return baseResource.getUrl();
		}
		IProject project = resource.getProject();
		String url = resource.getFullPath().toString();
		// truncating of the project name allow us to remap a content of project with name 'A' to the remote folder called 'B'
		return baseResource.getUrl() + "/" + url.substring(project.getFullPath().toString().length() + 1);
	}

	protected IConnectedProjectInformation getConnectedProjectInformation(IProject project) {
		RepositoryProvider provider = RepositoryProvider.getProvider(project);
		if (provider == null) {
			String errMessage = ModelBusTeamPlugin.instance().getResource("Error.NotConnectedProject", new String[] { project.getName() });
			throw new UnreportableException(errMessage);
		}
		if (!(provider instanceof IConnectedProjectInformation)) {
			String errMessage = ModelBusTeamPlugin.instance().getResource("Error.AnotherProvider",
					new String[] { project.getName(), provider.getID() });
			throw new UnreportableException(errMessage);
		}

		return (IConnectedProjectInformation) provider;
	}

	protected void refreshLocalResourceImpl(IResource resource, int depth) {
		ArrayList<IResource> removed = new ArrayList<IResource>();
		removed.add(resource);
		if (resource.getType() == IResource.PROJECT) {
			IConnectedProjectInformation info = (IConnectedProjectInformation) RepositoryProvider.getProvider(resource.getProject(),
					ModelBusTeamPlugin.NATURE_ID);
			if (info != null) {
				try {
					info.relocateResource();
				} catch (CoreException ex) {
					throw new RuntimeException(ex);
				}
			}
		}
		for (Iterator it = this.localResources.values().iterator(); it.hasNext();) {
			ILocalResource local = (ILocalResource) it.next();
			IResource current = local.getResource();
			IPath currentPath = current.getFullPath();
			if (resource.getFullPath().isPrefixOf(currentPath) || IStateFilter.SF_NOTEXISTS.accept(local)) {
				int cachedSegmentsCount = currentPath.segmentCount();
				int matchingSegmentsCount = resource.getFullPath().matchingFirstSegments(currentPath);
				int difference = cachedSegmentsCount - matchingSegmentsCount;
				if (difference >= 0 && depth == IResource.DEPTH_INFINITE ? true : depth >= difference) {
					removed.add(current);
				}
			}
		}
		for (Iterator it = removed.iterator(); it.hasNext();) {
			IResource forRemove = (IResource) it.next();
			this.localResources.remove(forRemove.getFullPath());
			this.switchedToUrls.remove(forRemove.getFullPath());
			this.parent2Children.remove(forRemove.getFullPath());
			this.parent2Children.remove(forRemove.getParent().getFullPath());
		}
	}

	protected ILocalResource loadLocalResourcesSubTree(final IResource resource, boolean recurse) throws Exception {
		IConnectedProjectInformation provider = (IConnectedProjectInformation) RepositoryProvider.getProvider(resource.getProject(),
				ModelBusTeamPlugin.NATURE_ID);
		if (provider == null || FileUtility.isModelBusInternals(resource)) {
			return this.wrapUnexistingResource(resource, IStateFilter.ST_INTERNAL_INVALID, 0);
		}

		boolean isCacheEnabled = CoreExtensionsManager.instance().getOptionProvider().isModelBusCacheEnabled();
		if (!isCacheEnabled) {
			this.localResources.clear();
		}
		recurse &= isCacheEnabled;

		ILocalResource retVal = null;
		boolean isLinked = FileUtility.isLinked(resource);
		IResource parent = resource.getParent();
		boolean parentExists = parent != null && parent.isAccessible();
		if (parentExists && !isLinked) {
			ILocalResource parentLocal = this.getFirstExistingParentLocal(resource);
			if (parentLocal == null || !ModelBusRemoteStorage.SF_NONModelBus.accept(parentLocal)
					|| (parentLocal.getChangeMask() & ILocalResource.IS_EXTERNAL) != 0 && IStateFilter.SF_IGNORED.accept(parentLocal)) {
				retVal = this.loadLocalResourcesSubTreeModelBusImpl(provider, resource, recurse);
			}
		}

		return retVal == null || IStateFilter.SF_UNVERSIONED.accept(retVal) ? this.loadUnversionedSubtree(resource, isLinked, recurse) : retVal;
	}

	protected ILocalResource loadUnversionedSubtree(final IResource resource, boolean isLinked, boolean recurse) throws Exception {
		// if resource has unversioned parents it cannot be wrapped directly and it status should be calculated in other way
		String status = this.calculateUnversionedStatus(resource, isLinked);

		// delegate status
		final String fStatus = status;
		ILocalResource parent = this.getFirstExistingParentLocal(resource);
		final int parentCM = parent != null ? (parent.getChangeMask() & (ILocalResource.IS_EXTERNAL | ILocalResource.IS_SWITCHED)) : 0;
		final ILocalResource[] tmp = new ILocalResource[1];
		FileUtility.visitNodes(resource, new IResourceVisitor() {
			public boolean visit(IResource child) throws CoreException {
				if (FileUtility.isModelBusInternals(child)) {
					return false;
				}
				String state = child == resource ? fStatus : ModelBusRemoteStorage.this.getDelegatedStatus(child, fStatus, 0);
				int changeMask = (state == IStateFilter.ST_OBSTRUCTED || state == IStateFilter.ST_NEW) ? ILocalResource.TEXT_MODIFIED
						: ILocalResource.NO_MODIFICATION;
				changeMask |= parentCM;
				String path = FileUtility.getWorkingCopyPath(child);
				if (new File(path + "/" + ModelBusUtility.getModelBusFolderName()).exists()
						&& ModelBusUtility.getModelBusInfoForNotConnected(child) != null) {
					return false;
				}
				ILocalResource retVal = ModelBusRemoteStorage.this.registerResource(child, ModelBusRevision.INVALID_REVISION_DESCRIPTOR,
						ModelBusRevision.INVALID_REVISION_DESCRIPTOR, state, changeMask, null, -1);
				if (tmp[0] == null) {
					tmp[0] = retVal;
				}
				return true;
			}
		}, recurse ? IResource.DEPTH_INFINITE : IResource.DEPTH_ONE, false);

		return tmp[0] != null ? tmp[0] : this.wrapUnexistingResource(resource, IStateFilter.ST_INTERNAL_INVALID, 0);
	}

	protected String calculateUnversionedStatus(final IResource resource, boolean isLinked) {
		String status = IStateFilter.ST_NOTEXISTS;
		if (isLinked) {
			status = IStateFilter.ST_LINKED;
		} else {
			IPath location = resource.getLocation();
			if (location != null && this.isFileExists(location)) {
				// may be ignored ?
				status = IStateFilter.ST_IGNORED;
				if (!ModelBusUtility.isIgnored(resource)) {
					ILocalResource local = (ILocalResource) this.localResources.get(resource.getFullPath());
					if (local != null) {
						return local.getStatus();
					}
					status = this.getTopLevelStatus(resource, IStateFilter.ST_NEW, 0);
				}
			}
		}
		return status;
	}

	protected boolean isFileExists(IPath location) {
		if (new File(location.toString()).exists()) {
			if (FileUtility.isWindows()) {
				// should be case sensitive on Windows OS also
				String[] names = new File(location.removeLastSegments(1).toString()).list();
				if (names != null) {
					String name = location.lastSegment();
					for (int i = 0; i < names.length; i++) {
						if (names[i].equals(name)) {
							return true;
						}
					}
				}
			} else {
				return true;
			}
		}
		return false;
	}

	protected ILocalResource loadLocalResourcesSubTreeModelBusImpl(IConnectedProjectInformation provider, IResource resource, boolean recurse)
			throws Exception {
		IProject project = resource.getProject();
		IResource target = resource.getType() == IResource.FILE ? resource.getParent() : resource;

		IPath wcPath = project.getLocation();
		IPath resourcePath = target.getLocation();
		IPath requestedPath = resource.getLocation();
		if (wcPath == null || resourcePath == null || requestedPath == null) {
			return null;
		}
		String projectPath = wcPath.toString();
		int subPathStart = projectPath.length();

		// search for first parent which contains .modelbus folder
		// only folders due to condition above - check for presence of ModelBus meta directly at the path...
		boolean hasModelBusMeta = true;
		while (!this.canFetchStatuses(resourcePath)) {
			if (target == null || target.getType() == IResource.PROJECT) {
				return null;
			}
			hasModelBusMeta = false;
			// load statuses non-recursively for the found parent
			resourcePath = resourcePath.removeLastSegments(1);
			target = target.getParent();
		}
		IRepositoryResource baseResource = provider.getRepositoryResource();
		ModelBusChangeStatus[] statuses = this.getStatuses(resourcePath.toString());
		String desiredUrl = this.makeUrl(target, baseResource);
		ILocalResource retVal = this.fillCache(statuses, desiredUrl, resource, subPathStart, requestedPath);

		if (statuses.length == 1 || statuses.length > 1 && this.parent2Children.get(target.getFullPath()) == null) {
			// the caching is done for the folder if it is empty
			this.parent2Children.put(target.getFullPath(), new HashSet());
		}

		if (retVal != null && hasModelBusMeta && statuses.length > 1 && recurse) {
			this.scheduleStatusesFetch(statuses, target);
		}

		return retVal;
	}

	protected void scheduleStatusesFetch(ModelBusChangeStatus[] st, IResource target) {
		synchronized (this.fetchQueue) {
			this.fetchQueue.add(new Object[] { st, target });
			if (this.fetchQueue.size() == 1) {
				ProgressMonitorUtility.doTaskScheduledDefault(new AbstractActionOperation("Operation.UpdateModelBusCache") {
					public ISchedulingRule getSchedulingRule() {
						return null;
					}

					protected void runImpl(IProgressMonitor monitor) throws Exception {
						Thread.currentThread().setPriority(Thread.MIN_PRIORITY);
						while (CoreExtensionsManager.instance().getOptionProvider().isModelBusCacheEnabled() && !monitor.isCanceled()) {
							ModelBusChangeStatus[] st;
							IResource target;
							synchronized (ModelBusRemoteStorage.this.fetchQueue) {
								if (ModelBusRemoteStorage.this.fetchQueue.size() == 0) {
									break;
								}
								Object[] entry = (Object[]) ModelBusRemoteStorage.this.fetchQueue.get(0);
								st = (ModelBusChangeStatus[]) entry[0];
								target = (IResource) entry[1];
							}
							this.processEntry(monitor, st, target);
							synchronized (ModelBusRemoteStorage.this.fetchQueue) {
								ModelBusRemoteStorage.this.fetchQueue.remove(0);
							}
						}
						if (!CoreExtensionsManager.instance().getOptionProvider().isModelBusCacheEnabled()) {
							synchronized (ModelBusRemoteStorage.this.fetchQueue) {
								ModelBusRemoteStorage.this.fetchQueue.clear();
							}
						}
					}

					protected void processEntry(IProgressMonitor monitor, ModelBusChangeStatus[] st, IResource target) {
						IProject prj = target.getProject();
						IPath location = prj.getLocation();
						if (location != null) {
							ModelBusUtility.reorder(st, false);
							int projectEnd = location.toString().length();
							for (int i = 0; i < st.length && !monitor.isCanceled()
									&& CoreExtensionsManager.instance().getOptionProvider().isModelBusCacheEnabled(); i++) {
								ProgressMonitorUtility.progress(monitor, i, IProgressMonitor.UNKNOWN);

								if (st[i].nodeKind == ModelBusEntry.Kind.DIR && st[i].path.length() > projectEnd) {
									IResource folder = prj.getFolder(new Path(st[i].path.substring(projectEnd)));
									ProgressMonitorUtility.setTaskInfo(monitor, this, folder.getFullPath().toString());
									ILocalFolder local = (ILocalFolder) ModelBusRemoteStorage.this.asLocalResource(folder);
									if (!IStateFilter.SF_INTERNAL_INVALID.accept(local)) {
										local.getChildren();
									}
								}
							}
						}
					}
				}, false).setPriority(Job.DECORATE);
			}
		}
	}

	protected boolean canFetchStatuses(IPath path) {
		return path.append(ModelBusUtility.getModelBusFolderName()).toFile().exists();
	}

	protected ModelBusChangeStatus[] getStatuses(String path) throws Exception {
		IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		try {
			ModelBusChangeStatus[] statuses = ModelBusUtility.status(proxy, path, Depth.IMMEDIATES, IModelBusConnector.Options.INCLUDE_UNCHANGED
					| IModelBusConnector.Options.INCLUDE_IGNORED, new ModelBusNullProgressMonitor());
			ModelBusUtility.reorder(statuses, true);
			return statuses;
		}
		// FIXME The only discovered reason of SBV-3557 defect is that some
		// folders under the ignored path contains completely or partially
		// empty .modelbus folders. May be due to using external tools.
		// So, suppress throwing "Path is not a working copy directory" exception.
		// 155007 is error code for "Path is not a working copy directory".
		catch (ModelBusConnectorException cwe) {
			if (cwe.getErrorId() != 155007) {
				throw cwe;
			}
			return new ModelBusChangeStatus[0];
		} finally {
		}
	}

	protected ILocalResource fillCache(ModelBusChangeStatus[] statuses, String desiredUrl, IResource resource, int subPathStart, IPath requestedPath) {
		IProject project = resource.getProject();

		ILocalResource retVal = null;

		for (int i = 0; i < statuses.length; i++) {
			int nodeKind = ModelBusUtility.getNodeKind(statuses[i].path, statuses[i].nodeKind, true);
			// ignore files absent in the WC base and WC working. But what is the reason why it is reported ?
			if (nodeKind == ModelBusEntry.Kind.NONE) {
				continue;
			}

			String fsNodePath = statuses[i].path;
			String nodePath = statuses[i].path;

			nodePath = nodePath.length() >= subPathStart ? nodePath.substring(subPathStart) : "";
			if (nodePath.length() > 0 && nodePath.charAt(nodePath.length() - 1) == '/') {
				nodePath = nodePath.substring(0, nodePath.length() - 1);
			} else if (i > 0 && nodePath.trim().length() == 0) {
				// Debug code was removed. We already have all information about the JavaModelBus bug.
				continue;
			}

			IResource tRes = null;
			if (new Path(statuses[i].path).equals(requestedPath) && (resource.getType() > IResource.FOLDER || resource.getType() == nodeKind)) {// if
																																				// nodekind
																																				// not
																																				// equals
																																				// do
																																				// not
																																				// use
																																				// default
																																				// resource
				tRes = resource;
			} else {
				if (nodeKind == ModelBusEntry.Kind.DIR) {
					if (nodePath.length() == 0) {
						tRes = project;
					} else {
						tRes = project.getFolder(new Path(nodePath));
					}
				} else {
					tRes = project.getFile(new Path(nodePath));
				}
			}

			ILocalResource local = (ILocalResource) this.localResources.get(tRes.getFullPath());
			if (local == null) {
				ILocalResource parent = this.getFirstExistingParentLocal(tRes);
				int externalMask = statuses[i].textStatus == ModelBusEntryStatus.Kind.EXTERNAL ? ILocalResource.IS_EXTERNAL : 0;
				if (externalMask != 0) {
					statuses[i] = ModelBusUtility.getModelBusInfoForNotConnected(tRes);
					if (statuses[i] == null) {
						local = this.registerResource(tRes, ModelBusRevision.INVALID_REVISION_DESCRIPTOR, ModelBusRevision.INVALID_REVISION_DESCRIPTOR,
								IStateFilter.ST_IGNORED, externalMask, null, 0);
						if (tRes == resource) {
							retVal = local;
						}
						continue;
					}
				} else if (parent != null && (parent.getChangeMask() & ILocalResource.IS_EXTERNAL) != 0) {
					externalMask = ILocalResource.IS_EXTERNAL;
				}

				// get the IS_COPIED flag by parent node (it is not fetched for deletions)
				boolean forceCopied = parent != null && parent.isCopied();
				int changeMask = this.getChangeMask(statuses[i].textStatus, statuses[i].propStatus, forceCopied | statuses[i].isCopied,
						statuses[i].isSwitched);
				if (statuses[i].lockToken != null) {
					changeMask |= ILocalResource.IS_LOCKED;
				}
				changeMask |= externalMask;

				String status = this.getStatusString(statuses[i].propStatus, statuses[i].textStatus, false);
				if (!statuses[i].isSwitched && statuses[i].url != null && !ModelBusUtility.decodeURL(statuses[i].url).startsWith(desiredUrl)) {
					changeMask |= ILocalResource.IS_SWITCHED;
				}

				if ((changeMask & ILocalResource.IS_SWITCHED) != 0) {
					this.switchedToUrls.put(tRes.getFullPath(), ModelBusUtility.decodeURL(statuses[i].url));
				}

				if (status == IStateFilter.ST_DELETED && nodeKind == ModelBusEntry.Kind.FILE && new File(statuses[i].path).exists()) {
					status = IStateFilter.ST_PREREPLACED;
				}

				if (FileUtility.isLinked(tRes)) {
					status = IStateFilter.ST_LINKED;
				} else if (status != IStateFilter.ST_OBSTRUCTED
						&& statuses[i].textStatus == org.modelbus.team.eclipse.core.connector.ModelBusEntryStatus.Kind.UNVERSIONED) {
					status = this.getDelegatedStatus(tRes, IStateFilter.ST_NEW, changeMask);
				}

				if (status == IStateFilter.ST_NEW && this.canFetchStatuses(new Path(fsNodePath))) {
					status = IStateFilter.ST_OBSTRUCTED;
				}

				// fetch revision for "copied from"
				ModelBusRevision revision = statuses[i].lastChangedRevision == ModelBusRevision.INVALID_REVISION_DESCRIPTOR
						&& (changeMask & ILocalResource.IS_COPIED) != 0 ? statuses[i].revision : statuses[i].lastChangedRevision;
				local = this.registerResource(tRes, revision, statuses[i].revision, status, changeMask, statuses[i].lastCommitAuthor,
						statuses[i].lastChangedDate);
			} else {
				this.writeChild(local.getResource(), local.getStatus(), local.getChangeMask());
			}

			if (tRes == resource) {
				retVal = local;
			}
		}

		return retVal;
	}

	protected String getDelegatedStatus(IResource resource, String status, int mask) {
		// calculate applicability of delegated status
		if (IStateFilter.SF_LINKED.accept(resource, status, mask) || FileUtility.isLinked(resource)) {
			return IStateFilter.ST_LINKED;
		}
		if (IStateFilter.SF_OBSTRUCTED.accept(resource, status, mask)) {
			return IStateFilter.ST_OBSTRUCTED;
		}
		if (IStateFilter.SF_IGNORED.accept(resource, status, mask) || ModelBusUtility.isIgnored(resource)) {
			return IStateFilter.ST_IGNORED;
		}
		return this.getTopLevelStatus(resource, status, mask);
	}

	protected String getTopLevelStatus(IResource resource, String status, int mask) {
		ILocalResource topLevel = this.getFirstExistingParentLocal(resource);
		if (topLevel != null) {
			String topLevelStatus = topLevel.getStatus();
			IResource topLevelResource = topLevel.getResource();
			if (IStateFilter.SF_OBSTRUCTED.accept(topLevelResource, topLevelStatus, mask)
					|| IStateFilter.SF_LINKED.accept(topLevelResource, topLevelStatus, mask)
					|| IStateFilter.SF_IGNORED.accept(topLevelResource, topLevelStatus, mask)) {
				return topLevelStatus;
			}
		}
		return status;
	}

	protected ILocalResource registerResource(IResource current, ModelBusRevision revision, ModelBusRevision baseRevision, String status, int changeMask, String author,
			long date) {
		ModelBusLocalResource local = null;

		if (IStateFilter.SF_OBSTRUCTED.accept(current, status, changeMask)) {
			try {
				IIgnoreRecommendations[] ignores = CoreExtensionsManager.instance().getIgnoreRecommendations();
				for (int i = 0; i < ignores.length; i++) {
					if (ignores[i].isAcceptableNature(current) && ignores[i].isOutput(current)) {
						IPath location = current.getProject().getLocation();
						if (location != null) {
							String projectPath = location.removeLastSegments(1).toString();
							File checkedResource = new File(projectPath + current.getFullPath().toString());
							status = !checkedResource.exists() ? IStateFilter.ST_NOTEXISTS : IStateFilter.ST_IGNORED;
							// Bug fix: incorrect path calculation; thanks to Panagiotis Korros
							IPath invalidMetaPath = new Path(projectPath + current.getFullPath().toString()).append(ModelBusUtility
									.getModelBusFolderName());
							FileUtility.deleteRecursive(invalidMetaPath.toFile());
						}
						break;
					}
				}
			} catch (CoreException e1) {
				// Cannot detect output folder due to some reasons. So, do not process the resource.
			}
		}

		if (IStateFilter.SF_UNVERSIONED.accept(current, status, changeMask)
				&& !(IStateFilter.SF_PREREPLACEDREPLACED.accept(current, status, changeMask) || IStateFilter.SF_DELETED.accept(current, status,
						changeMask)) || IStateFilter.SF_LINKED.accept(current, status, changeMask)) {
			revision = ModelBusRevision.INVALID_REVISION_DESCRIPTOR;
			author = null;
			date = -1;
		}

		local = current instanceof IContainer ? (ModelBusLocalResource) new ModelBusLocalFolder(current, revision, baseRevision, status, changeMask,
				author, date) : new ModelBusLocalFile(current, revision, baseRevision, status, changeMask, author, date);

		// handle parent-to-child relations
		this.writeChild(current, status, changeMask);

		this.localResources.put(current.getFullPath(), local);

		return local;
	}

	protected void writeChild(IResource current, String status, int mask) {
		if (!ModelBusRemoteStorage.SF_NONModelBus.accept(current, status, mask)) {
			IResource parent = current.getParent();
			Set children = (Set) this.parent2Children.get(parent.getFullPath());
			if (children == null) {
				this.parent2Children.put(parent.getFullPath(), children = new HashSet());
			}
			children.add(current);
		}
	}

	protected ILocalResource getFirstExistingParentLocal(IResource node) {
		IResource parent = node.getParent();
		if (parent == null) {
			return null;
		}
		ILocalResource local = (ILocalResource) this.localResources.get(parent.getFullPath());
		if (local != null) {
			return local;
		}
		return this.getFirstExistingParentLocal(parent);
	}

	protected String deserializeStatus(String status) {
		if ("null".equals(status)) {
			return IStateFilter.ST_NOTEXISTS;
		} else if (IStateFilter.ST_IGNORED.equals(status)) {
			return IStateFilter.ST_IGNORED;
		} else if (IStateFilter.ST_NEW.equals(status)) {
			return IStateFilter.ST_NEW;
		} else if (IStateFilter.ST_ADDED.equals(status)) {
			return IStateFilter.ST_ADDED;
		} else if (IStateFilter.ST_NORMAL.equals(status)) {
			return IStateFilter.ST_NORMAL;
		} else if (IStateFilter.ST_MODIFIED.equals(status)) {
			return IStateFilter.ST_MODIFIED;
		} else if (IStateFilter.ST_CONFLICTING.equals(status)) {
			return IStateFilter.ST_CONFLICTING;
		} else if (IStateFilter.ST_DELETED.equals(status)) {
			return IStateFilter.ST_DELETED;
		} else if (IStateFilter.ST_MISSING.equals(status)) {
			return IStateFilter.ST_MISSING;
		} else if (IStateFilter.ST_OBSTRUCTED.equals(status)) {
			return IStateFilter.ST_OBSTRUCTED;
		} else if (IStateFilter.ST_PREREPLACED.equals(status)) {
			return IStateFilter.ST_PREREPLACED;
		} else if (IStateFilter.ST_REPLACED.equals(status)) {
			return IStateFilter.ST_REPLACED;
		}
		throw new RuntimeException(ModelBusTeamPlugin.instance().getResource("Error.UnknownStatus"));
	}

	public String getStatusString(int propKind, int textKind, boolean isRemoteStatus) {
		String status = IStateFilter.ST_NORMAL;

		switch (textKind) {
			case org.modelbus.team.eclipse.core.connector.ModelBusEntryStatus.Kind.IGNORED: {
				status = IStateFilter.ST_IGNORED;
				break;
			}
			case org.modelbus.team.eclipse.core.connector.ModelBusEntryStatus.Kind.UNVERSIONED: {
				status = isRemoteStatus ? IStateFilter.ST_NOTEXISTS : IStateFilter.ST_NEW;
				break;
			}
			case org.modelbus.team.eclipse.core.connector.ModelBusEntryStatus.Kind.ADDED: {
				status = IStateFilter.ST_ADDED;
				break;
			}
			case org.modelbus.team.eclipse.core.connector.ModelBusEntryStatus.Kind.DELETED: {
				status = IStateFilter.ST_DELETED;
				break;
			}
			case org.modelbus.team.eclipse.core.connector.ModelBusEntryStatus.Kind.MISSING: {
				status = IStateFilter.ST_MISSING;
				break;
			}
			case org.modelbus.team.eclipse.core.connector.ModelBusEntryStatus.Kind.CONFLICTED: {
				status = isRemoteStatus ? IStateFilter.ST_MODIFIED : IStateFilter.ST_CONFLICTING;
				break;
			}
			case org.modelbus.team.eclipse.core.connector.ModelBusEntryStatus.Kind.MODIFIED: {
				status = IStateFilter.ST_MODIFIED;
				break;
			}
			case org.modelbus.team.eclipse.core.connector.ModelBusEntryStatus.Kind.OBSTRUCTED: {
				status = IStateFilter.ST_OBSTRUCTED;
				break;
			}
			case org.modelbus.team.eclipse.core.connector.ModelBusEntryStatus.Kind.REPLACED: {
				status = IStateFilter.ST_REPLACED;
				break;
			}
			case org.modelbus.team.eclipse.core.connector.ModelBusEntryStatus.Kind.NONE: {
				if (!isRemoteStatus && propKind == org.modelbus.team.eclipse.core.connector.ModelBusEntryStatus.Kind.NONE) {
					status = IStateFilter.ST_NOTEXISTS;
				}
				break;
			}
		}
		if (status == IStateFilter.ST_NORMAL) {
			if (propKind == org.modelbus.team.eclipse.core.connector.ModelBusEntryStatus.Kind.CONFLICTED) {
				status = IStateFilter.ST_CONFLICTING;
			} else if (propKind == org.modelbus.team.eclipse.core.connector.ModelBusEntryStatus.Kind.MODIFIED) {
				status = IStateFilter.ST_MODIFIED;
			}
		}

		return status;
	}

	protected int getChangeMask(int textStatus, int propKind, boolean isCopied, boolean isSwitched) {
		int changeMask = ILocalResource.NO_MODIFICATION;
		if (propKind == org.modelbus.team.eclipse.core.connector.ModelBusEntryStatus.Kind.MODIFIED
				|| propKind == org.modelbus.team.eclipse.core.connector.ModelBusEntryStatus.Kind.CONFLICTED) {
			changeMask |= ILocalResource.PROP_MODIFIED;
		}
		if (textStatus != org.modelbus.team.eclipse.core.connector.ModelBusEntryStatus.Kind.NORMAL) {
			changeMask |= ILocalResource.TEXT_MODIFIED;
		}
		if (isCopied) {
			changeMask |= ILocalResource.IS_COPIED;
		}
		if (isSwitched) {
			changeMask |= ILocalResource.IS_SWITCHED;
		}
		return changeMask;
	}

	private ModelBusRemoteStorage() {
		super();
		this.localResources = Collections.synchronizedMap(new HashMap());
		this.switchedToUrls = Collections.synchronizedMap(new LinkedHashMap());
		this.parent2Children = new HashMap();
		this.externalsLocations = new HashMap();
		this.resourceStateListeners = new HashMap<Class, List<IResourceStatesListener>>();
		this.fetchQueue = new LinkedList();
	}

	private static final IStateFilter	SF_NONModelBus	= new IStateFilter.AbstractStateFilter() {
															protected boolean acceptImpl(ILocalResource local, IResource resource, String state,
																	int mask) {
																return state == IStateFilter.ST_PREREPLACED || state == IStateFilter.ST_NEW
																		|| state == IStateFilter.ST_IGNORED || state == IStateFilter.ST_NOTEXISTS
																		|| state == IStateFilter.ST_LINKED || state == IStateFilter.ST_OBSTRUCTED;
															}

															protected boolean allowsRecursionImpl(ILocalResource local, IResource resource,
																	String state, int mask) {
																return true;
															}
														};

}
