/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.file;

import java.io.File;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Depth;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusNotification.NodeStatus;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.operation.IActionOperation;
import org.modelbus.team.eclipse.core.operation.IConsoleStream;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;

/**
 * Update resources operation
 * 
 * @author Alexander Gurov
 */
public class UpdateOperation extends AbstractFileConflictDetectionOperation implements IFileProvider {
	protected boolean			updateUnresolved;
	protected ModelBusRevision	selectedRevision;

	public UpdateOperation(File[] files, ModelBusRevision selectedRevision, boolean updateUnresolved) {
		super("Operation.UpdateFile", files);
		this.updateUnresolved = updateUnresolved;
		this.selectedRevision = selectedRevision;
	}

	public UpdateOperation(IFileProvider provider, ModelBusRevision selectedRevision, boolean updateUnresolved) {
		super("Operation.UpdateFile", provider);
		this.updateUnresolved = updateUnresolved;
		this.selectedRevision = selectedRevision;
	}

	public File[] getFiles() {
		return this.getProcessed();
	}

	protected void runImpl(final IProgressMonitor monitor) throws Exception {
		File[] files = this.operableData();

		// container with resources that is really updated
		this.defineInitialResourceSet(files);

		Map wc2Resources = ModelBusUtility.splitWorkingCopies(files);
		for (Iterator it = wc2Resources.entrySet().iterator(); it.hasNext() && !monitor.isCanceled();) {
			Map.Entry entry = (Map.Entry) it.next();

			IRepositoryResource wcRoot = ModelBusFileStorage.instance().asRepositoryResource((File) entry.getKey(), false);

			final String[] paths = FileUtility.asPathArray((File[]) ((List) entry.getValue()).toArray(new File[0]));

			this.complexWriteToConsole(new Runnable() {
				public void run() {
					UpdateOperation.this.writeToConsole(IConsoleStream.LEVEL_CMD, "modelbus update");
					for (int i = 0; i < paths.length && !monitor.isCanceled(); i++) {
						UpdateOperation.this.writeToConsole(IConsoleStream.LEVEL_CMD, " \"" + paths[i] + "\"");
					}
					UpdateOperation.this.writeToConsole(IConsoleStream.LEVEL_CMD,
							" -r " + UpdateOperation.this.selectedRevision + FileUtility.getUsernameParam(ModelBusConnector.getUserName()) + "\n");
				}
			});

			final IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
			proxy.setTouchUnresolved(this.updateUnresolved);
			this.protectStep(new IUnprotectedOperation() {
				public void run(IProgressMonitor monitor) throws Exception {
					proxy.update(paths, UpdateOperation.this.selectedRevision, Depth.infinityOrFiles(true), IModelBusConnector.Options.NONE,
							new ConflictDetectionProgressMonitor(UpdateOperation.this, monitor, null));
				}
			}, monitor, wc2Resources.size());
			proxy.setTouchUnresolved(false);

		}
	}

	protected class ConflictDetectionProgressMonitor extends ModelBusProgressMonitor {
		public ConflictDetectionProgressMonitor(IActionOperation parent, IProgressMonitor monitor, IPath root) {
			super(parent, monitor, root);
		}

		public void progress(int current, int total, ItemState state) {
			super.progress(current, total, state);
			if (state.contentState == NodeStatus.CONFLICTED || state.propState == NodeStatus.CONFLICTED) {
				UpdateOperation.this.hasUnresolvedConflict = true;
				UpdateOperation.this.unprocessed.add(new File(state.path));
				IPath conflictPath = new Path(state.path);
				for (Iterator it = UpdateOperation.this.processed.iterator(); it.hasNext();) {
					File res = (File) it.next();
					if (new Path(res.getAbsolutePath()).equals(conflictPath)) {
						it.remove();
						break;
					}
				}
			}
		}
	}

}
