/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.local;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryReference;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryRevisionReference;
import org.modelbus.team.eclipse.core.connector.ModelBusRevisionRange;
import org.modelbus.team.eclipse.core.operation.IActionOperation;
import org.modelbus.team.eclipse.core.operation.IConsoleStream;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryResourceProvider;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;

/**
 * JavaHL-mode merge implementation
 * 
 * @author Alexander Gurov
 */
public class JavaHLMergeOperation extends AbstractWorkingCopyOperation {
	protected IRepositoryResourceProvider	from;
	protected IRepositoryResourceProvider	fromEnd;
	protected ModelBusRevisionRange[]		revisions;
	protected boolean						dryRun;
	protected boolean						ignoreAncestry;
	protected int							depth;

	protected IModelBusProgressMonitor		externalMonitor;

	public JavaHLMergeOperation(IResource[] localTo, IRepositoryResource[] from, ModelBusRevisionRange[] revisions, boolean dryRun,
			boolean ignoreAncestry, int depth) {
		this(localTo, new IRepositoryResourceProvider.DefaultRepositoryResourceProvider(from), revisions, dryRun, ignoreAncestry, depth);
	}

	public JavaHLMergeOperation(IResource[] localTo, IRepositoryResource[] fromStart, IRepositoryResource[] fromEnd, boolean dryRun,
			boolean ignoreAncestry, int depth) {
		this(localTo, new IRepositoryResourceProvider.DefaultRepositoryResourceProvider(fromStart),
				new IRepositoryResourceProvider.DefaultRepositoryResourceProvider(fromEnd), dryRun, ignoreAncestry, depth);
	}

	public JavaHLMergeOperation(IResource[] localTo, IRepositoryResource[] from, boolean dryRun) {
		this(localTo, new IRepositoryResourceProvider.DefaultRepositoryResourceProvider(from), dryRun);
	}

	public JavaHLMergeOperation(IResource[] localTo, IRepositoryResourceProvider from, ModelBusRevisionRange[] revisions, boolean dryRun,
			boolean ignoreAncestry, int depth) {
		super("Operation.JavaHLMerge", localTo);
		this.from = from;
		this.revisions = revisions;
		this.dryRun = dryRun;
		this.ignoreAncestry = ignoreAncestry;
		this.depth = depth;
	}

	public JavaHLMergeOperation(IResource[] localTo, IRepositoryResourceProvider fromStart, IRepositoryResourceProvider fromEnd, boolean dryRun,
			boolean ignoreAncestry, int depth) {
		super("Operation.JavaHLMerge", localTo);
		this.from = fromStart;
		this.fromEnd = fromEnd;
		this.dryRun = dryRun;
		this.ignoreAncestry = ignoreAncestry;
		this.depth = depth;
	}

	public JavaHLMergeOperation(IResource[] localTo, IRepositoryResourceProvider from, boolean dryRun) {
		super("Operation.JavaHLMerge", localTo);
		this.from = from;
		this.dryRun = dryRun;
	}

	public int getOperationWeight() {
		return 19;
	}

	public void setExternalMonitor(IModelBusProgressMonitor externalMonitor) {
		this.externalMonitor = externalMonitor;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		IResource[] resources = this.operableData();
		IRepositoryResource[] fromStart = this.from.getRepositoryResources();
		IRepositoryResource[] fromEnd = this.fromEnd != null ? this.fromEnd.getRepositoryResources() : null;

		for (int i = 0; i < resources.length && !monitor.isCanceled(); i++) {
			final IResource resource = resources[i];
			final IRepositoryResource from1 = fromStart[i];
			final IRepositoryResource from2 = fromEnd == null ? null : fromEnd[i];
			this.protectStep(new IUnprotectedOperation() {
				public void run(IProgressMonitor monitor) throws Exception {
					if (from2 != null) {
						JavaHLMergeOperation.this.doMerge2URL(resource, from1, from2, monitor);
					} else if (JavaHLMergeOperation.this.revisions != null) {
						JavaHLMergeOperation.this.doMerge1URL(resource, from1, monitor);
					} else {
						JavaHLMergeOperation.this.doMergeReintegrate(resource, from1, monitor);
					}
				}
			}, monitor, resources.length);
		}
	}

	protected void doMerge1URL(IResource resource, IRepositoryResource from, IProgressMonitor monitor) throws Exception {
		IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();

		proxy.setTouchUnresolved(true);
		try {
			String wcPath = FileUtility.getWorkingCopyPath(resource);
			ModelBusEntryReference ref1 = ModelBusUtility.getEntryReference(from);
			long options = this.ignoreAncestry ? IModelBusConnector.Options.IGNORE_ANCESTRY : IModelBusConnector.Options.NONE;
			options |= this.dryRun ? IModelBusConnector.Options.SIMULATE : IModelBusConnector.Options.NONE;
			String changes = "";
			String ranges = "";
			for (ModelBusRevisionRange range : this.revisions) {
				if (range.from.equals(range.to)) {
					changes += changes.length() > 0 ? ("," + range.from.toString()) : range.from.toString();
				} else {
					String rangeStr = range.from.toString() + ":" + range.to.toString();
					ranges += ranges.length() > 0 ? (" " + rangeStr) : rangeStr;
				}
			}
			if (changes.length() > 0) {
				changes = " -c " + changes;
			}
			if (ranges.length() > 0) {
				ranges = " -r " + ranges;
			}
			this.writeToConsole(
					IConsoleStream.LEVEL_CMD,
					"modelbus merge" + changes + ranges + " \"" + from.getUrl() + "@" + from.getPegRevision() + "\" \""
							+ FileUtility.normalizePath(wcPath) + "\"" + ModelBusUtility.getDepthArg(this.depth) + (this.dryRun ? " --dry-run" : "")
							+ (this.ignoreAncestry ? " --ignore-ancestry" : "") + FileUtility.getUsernameParam(ModelBusConnector.getUserName())
							+ "\n");
			proxy.merge(ref1, this.revisions, wcPath, this.depth, options, new MergeProgressMonitor(this, monitor, null));
		} finally {
			proxy.setTouchUnresolved(false);
		}
	}

	protected void doMerge2URL(IResource resource, IRepositoryResource from1, IRepositoryResource from2, IProgressMonitor monitor) throws Exception {
		IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();

		proxy.setTouchUnresolved(true);
		try {
			String wcPath = FileUtility.getWorkingCopyPath(resource);
			ModelBusEntryRevisionReference ref1 = ModelBusUtility.getEntryRevisionReference(from1);
			ModelBusEntryRevisionReference ref2 = ModelBusUtility.getEntryRevisionReference(from2);
			long options = this.ignoreAncestry ? IModelBusConnector.Options.IGNORE_ANCESTRY : IModelBusConnector.Options.NONE;
			options |= this.dryRun ? IModelBusConnector.Options.SIMULATE : IModelBusConnector.Options.NONE;
			this.writeToConsole(
					IConsoleStream.LEVEL_CMD,
					"modelbus merge \"" + from1.getUrl() + "@" + from1.getSelectedRevision() + "\" \"" + from2.getUrl() + "@"
							+ from2.getSelectedRevision() + "\" \"" + FileUtility.normalizePath(wcPath) + "\""
							+ ModelBusUtility.getDepthArg(this.depth) + (this.dryRun ? " --dry-run" : "")
							+ (this.ignoreAncestry ? " --ignore-ancestry" : "") + FileUtility.getUsernameParam(ModelBusConnector.getUserName())
							+ "\n");
			proxy.merge(ref1, ref2, wcPath, this.depth, options, new MergeProgressMonitor(this, monitor, null));
		} finally {
			proxy.setTouchUnresolved(false);
		}
	}

	protected void doMergeReintegrate(IResource resource, IRepositoryResource from1, IProgressMonitor monitor) throws Exception {
		IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();

		proxy.setTouchUnresolved(true);
		try {
			String wcPath = FileUtility.getWorkingCopyPath(resource);
			ModelBusEntryReference ref1 = ModelBusUtility.getEntryReference(from1);
			long options = this.ignoreAncestry ? IModelBusConnector.Options.IGNORE_ANCESTRY : IModelBusConnector.Options.NONE;
			options |= this.dryRun ? IModelBusConnector.Options.SIMULATE : IModelBusConnector.Options.NONE;
			this.writeToConsole(IConsoleStream.LEVEL_CMD,
					"modelbus merge --reintegrate \"" + from1.getUrl() + "@" + from1.getPegRevision() + "\" \"" + FileUtility.normalizePath(wcPath)
							+ "\"" + (this.dryRun ? " --dry-run" : "") + FileUtility.getUsernameParam(ModelBusConnector.getUserName()) + "\n");
			proxy.mergeReintegrate(ref1, wcPath, options, new MergeProgressMonitor(this, monitor, null));
		} finally {
			proxy.setTouchUnresolved(false);
		}
	}

	protected class MergeProgressMonitor extends ModelBusProgressMonitor {
		public MergeProgressMonitor(IActionOperation parent, IProgressMonitor monitor, IPath root) {
			super(parent, monitor, root, !JavaHLMergeOperation.this.dryRun);
		}

		public void progress(int current, int total, ItemState state) {
			super.progress(current, total, state);
			if (JavaHLMergeOperation.this.externalMonitor != null) {
				JavaHLMergeOperation.this.externalMonitor.progress(current, total, state);
			}
		}
	}

}
