/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.local;

import java.io.File;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Depth;
import org.modelbus.team.eclipse.core.connector.IModelBusNotificationCallback;
import org.modelbus.team.eclipse.core.connector.IModelbusEntryStatusCallback;
import org.modelbus.team.eclipse.core.connector.ModelBusChangeStatus;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusEntry;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryStatus;
import org.modelbus.team.eclipse.core.connector.ModelBusNotification;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRemoteStorage;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.resource.IResourceChange;
import org.modelbus.team.eclipse.core.resource.IResourceProvider;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.core.utility.ProgressMonitorUtility;

/**
 * This operation fetch remote resource statuses
 * 
 * @author Alexander Gurov
 */
public class RemoteStatusOperation extends AbstractWorkingCopyOperation implements IRemoteStatusOperation, IModelBusNotificationCallback {
	protected ModelBusChangeStatus[]	statuses;
	protected Map<String, ModelBusRevision>		pegRevisions;

	public RemoteStatusOperation(IResource[] resources) {
		super("Operation.UpdateStatus", resources);
		this.pegRevisions = new HashMap<String, ModelBusRevision>();
	}

	public RemoteStatusOperation(IResourceProvider provider) {
		super("Operation.UpdateStatus", provider);
		this.pegRevisions = new HashMap<String, ModelBusRevision>();
	}

	public IResource[] getScope() {
		return this.operableData();
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		IResource[] resources = FileUtility.shrinkChildNodes(this.operableData());

		final HashSet<Path> projectPaths = new HashSet<Path>();
		for (int i = 0; i < resources.length && !monitor.isCanceled(); i++) {
			projectPaths.add(new Path(FileUtility.getWorkingCopyPath(resources[i].getProject())));
		}
		final HashMap<Path, ModelBusChangeStatus> result = new HashMap<Path, ModelBusChangeStatus>();
		final IModelbusEntryStatusCallback cb = new IModelbusEntryStatusCallback() {
			public void next(ModelBusChangeStatus status) {
				result.put(new Path(status.path), status);
				String parent = new File(status.path).getParent();
				if (parent != null) {// can be null for drive roots
					Path projectPath = this.getProjectPath(parent);
					if (projectPath != null) {
						if (status.reposKind != ModelBusEntry.Kind.DIR) {
							this.postStatus(parent, status);
						}
						this.postStatus(projectPath.toString(), status);
					}
				}
			}

			private void postStatus(String path, ModelBusChangeStatus baseStatus) {
				Path tPath = new Path(path);
				ModelBusChangeStatus st = result.get(tPath);
				if (st == null || st.reposLastCmtRevision.isLowerThan(baseStatus.reposLastCmtRevision)) {
					ModelBusChangeStatus status = this.makeStatus(path, baseStatus);
					result.put(tPath, status);
				}
			}

			private ModelBusChangeStatus makeStatus(String path, ModelBusChangeStatus status) {
				int deltaSegments = new Path(status.path).segmentCount() - new Path(path).segmentCount();
				return new ModelBusChangeStatus(path, status.url != null ? new Path(status.url).removeLastSegments(deltaSegments).toString() : null,
						ModelBusEntry.Kind.DIR, ModelBusRevision.INVALID_REVISION_DESCRIPTOR, ModelBusRevision.INVALID_REVISION_DESCRIPTOR, 0, null,
						ModelBusEntryStatus.Kind.NORMAL, ModelBusEntryStatus.Kind.NONE, ModelBusEntryStatus.Kind.NORMAL,
						ModelBusEntryStatus.Kind.MODIFIED, false, false, null, null, null, null, ModelBusRevision.INVALID_REVISION_DESCRIPTOR, false,
						null, null, null, 0, null, status.reposLastCmtRevision, status.reposLastCmtDate, ModelBusEntry.Kind.DIR,
						status.reposLastCmtAuthor);
			}

			private Path getProjectPath(String path) {
				Path tPath = new Path(path);
				for (Path projectPath : projectPaths) {
					if (projectPath.isPrefixOf(tPath)) {
						return projectPath;
					}
				}
				return null;
			}
		};
		for (int i = 0; i < resources.length && !monitor.isCanceled(); i++) {
			final IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();

			ModelBusUtility.addModelBusNotifyListener(proxy, this);
			final IResource current = resources[i];
			ProgressMonitorUtility.setTaskInfo(monitor, this, current.getFullPath().toString());
			// this.writeToConsole(IConsoleStream.LEVEL_CMD, "modelbus status -u \"" + FileUtility.normalizePath(current.getLocation().toString()) +
			// "\\"" + FileUtility.getUsernameParam(location.getUsername()) + "\\n"
			this.protectStep(new IUnprotectedOperation() {
				public void run(IProgressMonitor monitor) throws Exception {
					proxy.status(FileUtility.getWorkingCopyPath(current), Depth.UNKNOWN, IModelBusConnector.Options.SERVER_SIDE, null, cb,
							new ModelBusProgressMonitor(RemoteStatusOperation.this, monitor, null, false));
				}
			}, monitor, resources.length);
			ModelBusUtility.removeModelBusNotifyListener(proxy, this);
		}
		this.statuses = result.values().toArray(new ModelBusChangeStatus[result.size()]);
	}

	public ModelBusEntryStatus[] getStatuses() {
		return this.statuses;
	}

	public void setPegRevision(IResourceChange change) {
		IPath resourcePath = FileUtility.getResourcePath(change.getResource());
		int prefixLength = 0;
		ModelBusRevision revision = ModelBusRevision.INVALID_REVISION_DESCRIPTOR;
		for (Iterator<?> it = this.pegRevisions.entrySet().iterator(); it.hasNext();) {
			Map.Entry entry = (Map.Entry) it.next();
			IPath rootPath = new Path((String) entry.getKey());
			int segments = rootPath.segmentCount();
			if (rootPath.isPrefixOf(resourcePath) && segments > prefixLength) {
				prefixLength = segments;
				revision = (ModelBusRevision) entry.getValue();
			}
		}
		if (revision != ModelBusRevision.INVALID_REVISION_DESCRIPTOR) {
			change.setPegRevision(revision);
		} else if (change.getResource().getType() == IResource.PROJECT) {
			IRepositoryResource remote = ModelBusRemoteStorage.instance().asRepositoryResource(change.getResource());
			change.setPegRevision(remote.getPegRevision());
		}
	}

	public void notify(ModelBusNotification info) {
		if (info.revision != ModelBusRevision.INVALID_REVISION_DESCRIPTOR) {
			this.pegRevisions.put(info.path, info.revision);
		}
	}

}
