/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.local.management;

import java.io.File;
import java.io.IOException;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.ModelBusTeamPlugin;
import org.modelbus.team.eclipse.core.ModelBusTeamProjectMapper;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Depth;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusConnectorCancelException;
import org.modelbus.team.eclipse.core.connector.ModelBusConnectorException;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.operation.UnreportableException;
import org.modelbus.team.eclipse.core.operation.local.AbstractWorkingCopyOperation;
import org.modelbus.team.eclipse.core.resource.IRepositoryContainer;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.core.utility.ProgressMonitorUtility;

/**
 * Share project operation implementation
 * 
 * @author Alexander Gurov
 */
public class ShareProjectOperation extends AbstractWorkingCopyOperation {
	public static final int	LAYOUT_DEFAULT	= 0;
	public static final int	LAYOUT_SINGLE	= 1;
	public static final int	LAYOUT_MULTIPLE	= 2;

	public interface IFolderNameMapper {
		public String getRepositoryFolderName(IProject project);
	}

	protected IFolderNameMapper		mapper;
	protected int					shareLayout;
	protected String				rootName;
	protected boolean				managementFoldersEnabled;
	protected String				commitComment;
	protected IShareProjectPrompt	shareProjectPrompt;

	public ShareProjectOperation(IProject[] projects, IFolderNameMapper mapper) {
		this(projects, mapper, null);
	}

	public ShareProjectOperation(IProject[] projects, IFolderNameMapper mapper, String commitComment) {
		this(projects, mapper, null, ShareProjectOperation.LAYOUT_DEFAULT, true, commitComment);
	}

	public ShareProjectOperation(IProject[] projects, IFolderNameMapper mapper, String rootName, int shareLayout, boolean managementFoldersEnabled) {
		this(projects, mapper, rootName, shareLayout, managementFoldersEnabled, null);
	}

	public ShareProjectOperation(IProject[] projects, IFolderNameMapper mapper, String rootName, int shareLayout, boolean managementFoldersEnabled,
			String commitComment) {
		super("Operation.ShareProject", projects);
		this.mapper = mapper;
		this.shareLayout = shareLayout;
		this.rootName = rootName;
		this.managementFoldersEnabled = managementFoldersEnabled;
		this.commitComment = commitComment;
	}

	public void setSharePrompt(IShareProjectPrompt prompt) {
		this.shareProjectPrompt = prompt;
	}

	public static String getTargetUrl(int shareLayout, String projectName, String rootName, boolean managementFoldersEnabled) {
		return ModelBusUtility.normalizeURL(ShareProjectOperation.getTargetUrlImpl(shareLayout, projectName, rootName, managementFoldersEnabled));
	}

	public static String getDefaultComment(IProject project, IRepositoryResource remote) {
		return ModelBusTeamPlugin.instance().getResource("Operation.ShareProject.DefaultComment",
				new String[] { project.getName(), ModelBusUtility.encodeURL(remote.getUrl()) });
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		IResource[] resources = this.operableData();

		HashMap local2remote = new HashMap();
		for (int i = 0; i < resources.length; i++) {
			String url = ShareProjectOperation.getTargetUrl(this.shareLayout,
					this.mapper == null ? resources[i].getName() : this.mapper.getRepositoryFolderName((IProject) resources[i]), this.rootName,
					this.managementFoldersEnabled);
			IRepositoryContainer remote = ModelBusConnector.asRepositoryContainer(url, false);
			local2remote.put(resources[i], remote);
		}

		if (this.commitComment == null) {
			this.commitComment = "";
			for (Iterator it = local2remote.entrySet().iterator(); it.hasNext();) {
				Map.Entry entry = (Map.Entry) it.next();
				String commentPart = ShareProjectOperation.getDefaultComment((IProject) entry.getKey(), (IRepositoryContainer) entry.getValue());
				this.commitComment += this.commitComment.length() == 0 ? commentPart : ("\n" + commentPart);
			}
		}

		Set fullSet = null;
		switch (this.shareLayout) {
			case ShareProjectOperation.LAYOUT_DEFAULT: {
				fullSet = this.doDefaultLayout(local2remote);
				break;
			}
			case ShareProjectOperation.LAYOUT_SINGLE: {
				fullSet = this.doSingleLayout(local2remote);
				break;
			}
			case ShareProjectOperation.LAYOUT_MULTIPLE: {
				fullSet = this.doMultipleLayout(local2remote);
				break;
			}
			default: {
				String message = this.getNationalizedString("Error.UnknownProjectLayoutType");
				throw new Exception(MessageFormat.format(message, new Object[] { String.valueOf(this.shareLayout) }));
			}
		}

		final HashSet existingProjects = new HashSet();
		for (Iterator it = local2remote.entrySet().iterator(); it.hasNext() && !monitor.isCanceled();) {
			final Map.Entry entry = (Map.Entry) it.next();
			this.protectStep(new IUnprotectedOperation() {
				public void run(IProgressMonitor monitor) throws Exception {
					try {
						IRepositoryContainer remote = (IRepositoryContainer) entry.getValue();
						if (remote.getChildren().length > 0) {
							existingProjects.add(entry.getKey());
						}
					} catch (ModelBusConnectorException ex) {
						// do nothing
					}
				}
			}, monitor, local2remote.size() * 2);
		}

		if (existingProjects.size() > 0 && this.shareProjectPrompt != null
				&& !this.shareProjectPrompt.prompt((IProject[]) existingProjects.toArray(new IProject[existingProjects.size()]))) {
			throw new ModelBusConnectorCancelException(this.getNationalizedString("Error.ShareCanceled"));
		}

		final IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		try {
			IRepositoryResource[] resourceSet = this.getOrderedSet(fullSet);
			this.mkdir(proxy, resourceSet, monitor);

			for (Iterator it = local2remote.entrySet().iterator(); it.hasNext() && !monitor.isCanceled();) {
				final Map.Entry entry = (Map.Entry) it.next();
				this.protectStep(new IUnprotectedOperation() {
					public void run(IProgressMonitor monitor) throws Exception {
						IRepositoryContainer remote = (IRepositoryContainer) entry.getValue();
						IProject project = (IProject) entry.getKey();

						File tempDir = existingProjects.contains(project) ? ShareProjectOperation.this.createTempDirectory(project) : null;
						String checkoutTo = tempDir != null ? tempDir.toString() : FileUtility.getWorkingCopyPath(project);

						proxy.checkout(ModelBusUtility.getEntryRevisionReference(remote), checkoutTo, Depth.INFINITY,
								IModelBusConnector.Options.NONE, new ModelBusProgressMonitor(ShareProjectOperation.this, monitor, null));

						if (tempDir != null) {
							ShareProjectOperation.this.copyModelBusMeta(tempDir, FileUtility.getResourcePath(project).toFile());
							FileUtility.deleteRecursive(tempDir, monitor);
						}

						ModelBusTeamProjectMapper.map(project, remote);
					}
				}, monitor, local2remote.size() * 2);
			}
		} finally {
		}
	}

	protected Set doDefaultLayout(Map local2remote) throws Exception {
		HashSet fullSet = new HashSet();
		// for (Iterator it = local2remote.values().iterator(); it.hasNext(); ) {
		// IRepositoryContainer remote = (IRepositoryContainer)it.next();
		// IRepositoryResource []resources = ModelBusUtility.makeResourceSet(remote.getRepositoryLocation().getRoot(), remote);
		// fullSet.addAll(Arrays.asList(resources));
		// }
		throw new RuntimeException("TODOMWA");
		// return fullSet;
	}

	protected Set doSingleLayout(Map local2remote) throws Exception {
		if (this.managementFoldersEnabled) {
			// HashSet fullSet = new HashSet();
			// for (Iterator it = local2remote.values().iterator(); it.hasNext(); ) {
			// IRepositoryContainer remote = (IRepositoryContainer)it.next();
			// IRepositoryResource []resources = ModelBusUtility.makeResourceSet(remote.getRepositoryLocation().getRoot(), remote);
			// fullSet.addAll(Arrays.asList(resources));
			// IRepositoryContainer parent = (IRepositoryContainer)remote.getParent();
			// fullSet.add(this.makeChild(parent, ShareProjectOperation.getTagsName(this.location)));
			// fullSet.add(this.makeChild(parent, ShareProjectOperation.getBranchesName(this.location)));
			// }
			// return fullSet;
			throw new RuntimeException("TODOMWA");
		}
		return this.doDefaultLayout(local2remote);
	}

	protected Set doMultipleLayout(Map local2remote) throws Exception {
		if (this.managementFoldersEnabled) {
			// HashSet fullSet = new HashSet();
			// for (Iterator it = local2remote.values().iterator(); it.hasNext(); ) {
			// IRepositoryContainer remote = (IRepositoryContainer)it.next();
			// IRepositoryResource []resources = ModelBusUtility.makeResourceSet(remote.getRepositoryLocation().getRoot(), remote);
			// fullSet.addAll(Arrays.asList(resources));
			//
			// String targetUrl = ShareProjectOperation.getTargetUrl(this.location, this.shareLayout, "", this.rootName, false);
			// IRepositoryContainer parent = this.location.asRepositoryContainer(targetUrl, false);
			// fullSet.add(this.makeChild(parent, ShareProjectOperation.getTagsName(this.location)));
			// fullSet.add(this.makeChild(parent, ShareProjectOperation.getBranchesName(this.location)));
			// }
			// return fullSet;
			throw new RuntimeException("TODOMWA");
		}
		return this.doDefaultLayout(local2remote);
	}

	protected IRepositoryResource[] getOrderedSet(Set fullSet) {
		IRepositoryResource[] resources = (IRepositoryResource[]) fullSet.toArray(new IRepositoryResource[fullSet.size()]);
		Arrays.sort(resources, new Comparator() {
			public int compare(Object arg0, Object arg1) {
				IRepositoryResource first = (IRepositoryResource) arg0;
				IRepositoryResource second = (IRepositoryResource) arg1;
				return first.getUrl().compareTo(second.getUrl());
			}
		});
		return resources;
	}

	protected void mkdir(IModelBusConnector proxy, IRepositoryResource[] resourceSet, IProgressMonitor monitor) throws Exception {
		ArrayList urlsList = new ArrayList();
		for (int i = 0; i < resourceSet.length && !monitor.isCanceled(); i++) {
			ProgressMonitorUtility.setTaskInfo(monitor, this, resourceSet[i].getUrl());
			if (!resourceSet[i].exists()) {
				urlsList.add(ModelBusUtility.encodeURL(resourceSet[i].getUrl()));
			}
			ProgressMonitorUtility.progress(monitor, IProgressMonitor.UNKNOWN, resourceSet.length);
		}
		String[] urls = (String[]) urlsList.toArray(new String[urlsList.size()]);
		proxy.mkdir(urls, this.commitComment, IModelBusConnector.Options.INCLUDE_PARENTS, null, new ModelBusProgressMonitor(this, monitor, null));
	}

	protected IRepositoryContainer makeChild(IRepositoryContainer parent, String name) {
		return ModelBusConnector.asRepositoryContainer(parent.getUrl() + "/" + name, false);
	}

	protected File createTempDirectory(IProject project) {
		try {
			File tempDirectory = File.createTempFile("save_" + project.getName(), ".tmp", FileUtility.getResourcePath(project).toFile()
					.getParentFile());
			tempDirectory.deleteOnExit();
			tempDirectory.delete();
			return tempDirectory;
		} catch (IOException ex) {
			String message = ModelBusTeamPlugin.instance()
					.getResource("Error.CannotCheckOutMeta", new String[] { String.valueOf(project.getName()) });
			throw new UnreportableException(message, ex);
		}
	}

	protected void copyModelBusMeta(File fromFolder, File toFolder) {
		if (!toFolder.isDirectory()) {
			return;
		}
		File[] tempFiles = fromFolder.listFiles();
		if (tempFiles != null) {
			for (int i = 0; i < tempFiles.length; i++) {
				File renameToFile = new File(toFolder + "/" + tempFiles[i].getName());
				if (!renameToFile.exists()) {
					tempFiles[i].renameTo(renameToFile);
				} else if (tempFiles[i].isDirectory()) {
					this.copyModelBusMeta(tempFiles[i], renameToFile);
				}
			}
		}
	}

	protected static String getTargetUrlImpl(int shareLayout, String projectName, String rootName, boolean managementFoldersEnabled) {
		throw new RuntimeException("TODOMWA18");
		// String trunkName = managementFoldersEnabled ? ("/" + ShareProjectOperation.getTrunkName(location)) : "";
		// switch (shareLayout) {
		// case ShareProjectOperation.LAYOUT_DEFAULT: {
		// return location.getUrl() + trunkName + "/" + projectName;
		// }
		// case ShareProjectOperation.LAYOUT_SINGLE: {
		// return location.getUrl() + "/" + projectName + trunkName;
		// }
		// case ShareProjectOperation.LAYOUT_MULTIPLE: {
		// return location.getUrl() + "/" + rootName + trunkName + "/" + projectName;
		// }
		// default: {
		// String message = ModelBusTeamPlugin.instance().getResource("Error.UnknownProjectLayoutType", new String[] {String.valueOf(shareLayout)});
		// throw new RuntimeException(message);
		// }
		// }
	}

	// public static String getTrunkName(IRepositoryLocation location) {
	// if (location.isStructureEnabled()) {
	// return location.getTrunkLocation();
	// }
	// return ModelBusTeamPlugin.instance().getOptionProvider().getDefaultTrunkName();
	// }
	//
	// public static String getBranchesName(IRepositoryLocation location) {
	// if (location.isStructureEnabled()) {
	// return location.getBranchesLocation();
	// }
	// return ModelBusTeamPlugin.instance().getOptionProvider().getDefaultBranchesName();
	// }
	//
	// public static String getTagsName(IRepositoryLocation location) {
	// if (location.isStructureEnabled()) {
	// return location.getTagsLocation();
	// }
	// return ModelBusTeamPlugin.instance().getOptionProvider().getDefaultTagsName();
	// }

}
