/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.remote;

import java.text.MessageFormat;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.ModelBusTeamPlugin;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusNotificationCallback;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusNotification;
import org.modelbus.team.eclipse.core.operation.IConsoleStream;
import org.modelbus.team.eclipse.core.operation.IRevisionProvider;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.core.utility.ProgressMonitorUtility;

/**
 * Create remote folder operation
 * 
 * @author Alexander Gurov
 */
public class CreateFolderOperation extends AbstractRepositoryOperation implements IRevisionProvider {
	protected String[]			names;
	protected String			comment;
	protected RevisionPair[]	revisionPair;

	public CreateFolderOperation(IRepositoryResource parent, String name, String comment) {
		this(parent, new String[] { name }, comment);
	}

	public CreateFolderOperation(IRepositoryResource parent, String[] names, String comment) {
		super("Operation.CreateFolder", new IRepositoryResource[] { parent });
		this.names = names;
		this.comment = comment;
	}

	protected void runImpl(final IProgressMonitor monitor) throws Exception {
		this.revisionPair = new RevisionPair[1];
		IRepositoryResource parent = this.operableData()[0];
		ProgressMonitorUtility.setTaskInfo(monitor, this, parent.getUrl() + "/" + this.names[0]);

		Set<IRepositoryResource> fullSet = new HashSet<IRepositoryResource>();
		for (int i = 0; i < this.names.length; i++) {
			fullSet.addAll(Arrays.asList(ModelBusUtility.makeResourceSet(parent, this.names[i], false)));
		}
		IRepositoryResource[] toBeCreated = fullSet.toArray(new IRepositoryResource[fullSet.size()]);

		final String[] childUrls = ModelBusUtility.asURLArray(toBeCreated, true);
		Arrays.sort(childUrls);

		IModelBusNotificationCallback notify = new IModelBusNotificationCallback() {
			public void notify(ModelBusNotification info) {
				String[] path = childUrls;
				CreateFolderOperation.this.revisionPair[0] = new RevisionPair(info.revision, path);
				String message = ModelBusTeamPlugin.instance().getResource("Console.CommittedRevision",
						new String[] { String.valueOf(info.revision) });
				CreateFolderOperation.this.writeToConsole(IConsoleStream.LEVEL_OK, message);
			}
		};

		this.complexWriteToConsole(new Runnable() {
			public void run() {
				CreateFolderOperation.this.writeToConsole(IConsoleStream.LEVEL_CMD, "modelbus mkdir");
				for (int i = 0; i < childUrls.length && !monitor.isCanceled(); i++) {
					CreateFolderOperation.this.writeToConsole(IConsoleStream.LEVEL_CMD, " \"" + ModelBusUtility.decodeURL(childUrls[i]) + "\"");
				}
				CreateFolderOperation.this.writeToConsole(IConsoleStream.LEVEL_CMD, " -m \"" + CreateFolderOperation.this.comment + "\""
						+ FileUtility.getUsernameParam(ModelBusConnector.getUserName()) + "\n");
			}
		});

		IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		try {
			ModelBusUtility.addModelBusNotifyListener(proxy, notify);
			proxy.mkdir(childUrls, this.comment, IModelBusConnector.Options.NONE, null, new ModelBusProgressMonitor(this, monitor, null));
		} finally {
			ModelBusUtility.removeModelBusNotifyListener(proxy, notify);
		}

		ProgressMonitorUtility.progress(monitor, 1, 1);
	}

	public RevisionPair[] getRevisions() {
		return this.revisionPair;
	}

	protected String getShortErrorMessage(Throwable t) {
		return MessageFormat.format(super.getShortErrorMessage(t), new Object[] { this.name });
	}

}
