/**
 *  ModelBus is the means to interact with models in a distributed
 * consistent way.
 * 
 *  Copyright (C) 2008  Tom Ritter, Christian Hein, Michael Wagner
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation, either version 3 of
 * the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.modelbus.team.eclipse.notification.views;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;
import org.eclipse.ui.progress.UIJob;
import org.modelbus.core.lib.notification.INotificationListener;
import org.modelbus.core.lib.notification.NotificationListenerManager;
import org.modelbus.team.eclipse.notification.Activator;
import org.modelbus.team.eclipse.notification.preferences.PreferenceConstants;

/**
 * This sample class demonstrates how to plug-in a new workbench view. The view shows data obtained from the model. The sample creates a dummy model
 * on the fly, but a real implementation would connect to the model available either in this or another plug-in (e.g. the workspace). The view is
 * connected to the model using a content provider.
 * <p>
 * The view uses a label provider to define how model objects should be presented in the view. Each view can present the same model objects using
 * different labels and icons, if needed. Alternatively, a single label provider can be shared between views in order to ensure that objects of the
 * same type are presented in the same way everywhere.
 * <p>
 */

public class NotificationView extends ViewPart implements INotificationListener {
	public final static String			VIEW_ID					= "org.modelbus.team.eclipse.notification.views.NotificationView";

	private TableViewer					viewer;
	private Action						togglePopupAction;
	private Action						synchronizeModelAction;
	private Action						doubleClickAction;

	private static NotificationView		currentNotificationView	= null;

	/*
	 * The content provider class is responsible for providing objects to the view. It can wrap existing objects in adapters or simply return objects
	 * as-is. These objects may be sensitive to the current input of the view, or ignore it and always show the same content (like Task List, for
	 * example).
	 */

	public static ArrayList<String[]>	messages				= new ArrayList<String[]>();

	public static void addMessage(String url, String type) {
		if (FilterHelper.filter(url)) {
			messages.add(new String[] { "", url, type, new SimpleDateFormat("dd.MM.yyyy HH:mm:ss.SSS").format(new Date()) });
			if (currentNotificationView != null) {

				UIJob uiJob = new UIJob("Model " + url + " has been updated.") {

					@Override
					public IStatus runInUIThread(IProgressMonitor monitor) {
						currentNotificationView.viewer.refresh();

						if (checked && currentNotificationView != null) {
							MessageDialog.openInformation(currentNotificationView.getViewSite().getShell(), "Notification View", getName());
						}

						return Status.OK_STATUS;
					}
				};

				uiJob.schedule();
			}
		}
	}

	class ViewContentProvider implements IStructuredContentProvider {
		public void inputChanged(Viewer v, Object oldInput, Object newInput) {
		}

		public void dispose() {
		}

		public Object[] getElements(Object parent) {
			return NotificationView.messages.toArray();
		}
	}

	class ViewLabelProvider extends LabelProvider implements ITableLabelProvider {
		public String getColumnText(Object obj, int index) {
			return getText(((String[]) obj)[index]);
		}

		public Image getColumnImage(Object obj, int index) {
			return null;// getImage(obj);
		}

		public Image getImage(Object obj) {
			return null;// PlatformUI.getWorkbench().
			// getSharedImages().getImage(ISharedImages.IMG_OBJ_ELEMENT);
		}
	}

	class NameSorter extends ViewerSorter {
	}

	/**
	 * The constructor.
	 */
	public NotificationView() {
		currentNotificationView = this;

		final NotificationListenerManager nManager = Activator.getDefault().getNotificationListenerManager();

		if (nManager != null) {
			nManager.addNotificationListener(this);
		}
	}

	/**
	 * This is a callback that will allow us to create the viewer and initialize it.
	 */
	public void createPartControl(Composite parent) {
		Table table = new Table(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);

		// int style = SWT.SINGLE | SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL |
		// SWT.FULL_SELECTION | SWT.HIDE_SELECTION;

		// final int NUMBER_COLUMNS = 4;

		// table = new Table(parent, style);

		GridData gridData = new GridData(GridData.FILL_BOTH);
		gridData.grabExcessVerticalSpace = true;
		gridData.horizontalSpan = 3;
		table.setLayoutData(gridData);

		table.setLinesVisible(true);
		table.setHeaderVisible(true);

		// 1st column with image/checkboxes - NOTE: The SWT.CENTER has no effect!!
		TableColumn column = new TableColumn(table, SWT.CENTER, 0);
		column.setText("!");
		column.setWidth(20);

		// 2nd column with task Description
		column = new TableColumn(table, SWT.LEFT, 1);
		column.setText("Model");
		column.setWidth(400);
		// Add listener to column so tasks are sorted by description when clicked
		// column.addSelectionListener(new SelectionAdapter() {
		//
		// public void widgetSelected(SelectionEvent e) {
		// tableViewer.setSorter(new ExampleTaskSorter(ExampleTaskSorter.DESCRIPTION));
		// }
		// });

		// 3rd column with task Owner
		column = new TableColumn(table, SWT.LEFT, 2);
		column.setText("Type");
		column.setWidth(100);
		// Add listener to column so tasks are sorted by owner when clicked
		// column.addSelectionListener(new SelectionAdapter() {
		//
		// public void widgetSelected(SelectionEvent e) {
		// tableViewer.setSorter(new ExampleTaskSorter(ExampleTaskSorter.OWNER));
		// }
		// });

		// 4th column with task PercentComplete
		column = new TableColumn(table, SWT.CENTER, 3);
		column.setText("Time");
		column.setWidth(120);
		// Add listener to column so tasks are sorted by percent when clicked
		// column.addSelectionListener(new SelectionAdapter() {
		//
		// public void widgetSelected(SelectionEvent e) {
		// tableViewer.setSorter(new ExampleTaskSorter(ExampleTaskSorter.PERCENT_COMPLETE));
		// }
		// });

		viewer = new TableViewer(table);
		// viewer.setColumnProperties(new String[] { "asdd", "adasd" , "sadsdaf" });
		viewer.setUseHashlookup(true);
		viewer.setContentProvider(new ViewContentProvider());
		viewer.setLabelProvider(new ViewLabelProvider());
		// viewer.setSorter(new NameSorter());
		viewer.setInput(getViewSite());

		// Create the help context id for the viewer's control
		PlatformUI.getWorkbench().getHelpSystem().setHelp(viewer.getControl(), "org.modelbus.team.eclipse.notification.viewer");
		makeActions();
		hookContextMenu();
		hookDoubleClickAction();
		contributeToActionBars();
		new TableSortSelectionListener(viewer, column, new TextSorter(3), SWT.DOWN, true).chooseColumnForSorting();

	}

	private void hookContextMenu() {
		MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				NotificationView.this.fillContextMenu(manager);
			}
		});
		Menu menu = menuMgr.createContextMenu(viewer.getControl());
		viewer.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, viewer);
	}

	private void contributeToActionBars() {
		IActionBars bars = getViewSite().getActionBars();
		fillLocalPullDown(bars.getMenuManager());
		fillLocalToolBar(bars.getToolBarManager());
		bars.getToolBarManager().update(true);
	}

	private void fillLocalPullDown(IMenuManager manager) {
		manager.add(togglePopupAction);
		manager.add(new Separator());
		manager.add(synchronizeModelAction);
	}

	private void fillContextMenu(IMenuManager manager) {
		// manager.add(togglePopupAction);
		manager.add(synchronizeModelAction);
		// Other plug-ins can contribute there actions here
		manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}

	private void fillLocalToolBar(IToolBarManager manager) {
		manager.add(togglePopupAction);
		// manager.add(action2);
	}

	public static boolean	checked	= Activator.getDefault().getPreferenceStore().getBoolean(PreferenceConstants.POPUP_NOTIFICATION);

	private void makeActions() {
		togglePopupAction = new Action("Enable PopUp", IAction.AS_CHECK_BOX) {
			public void run() {
				// showMessage("Action 1 executed");
				checked = !checked;
				setChecked(checked);
			}
		};
		togglePopupAction.setChecked(true);
		togglePopupAction.setText("Enable PopUp");
		togglePopupAction.setToolTipText("Action 1 tooltip");
		togglePopupAction.setImageDescriptor(PlatformUI.getWorkbench().getSharedImages().getImageDescriptor(ISharedImages.IMG_OBJS_INFO_TSK));

		synchronizeModelAction = new Action() {
			public void run() {
				showMessage("Synchronize Model");
			}
		};
		synchronizeModelAction.setText("Synchronize Model");
		synchronizeModelAction.setToolTipText("Synchronizes the local model information with the repository");
		synchronizeModelAction.setImageDescriptor(PlatformUI.getWorkbench().getSharedImages().getImageDescriptor(ISharedImages.IMG_OBJS_INFO_TSK));
		doubleClickAction = new Action() {
			public void run() {
				ISelection selection = viewer.getSelection();
				Object obj = ((IStructuredSelection) selection).getFirstElement();
				showMessage("Double-click detected on " + obj.toString());
			}
		};
	}

	private void hookDoubleClickAction() {
		viewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				doubleClickAction.run();
			}
		});
	}

	private void showMessage(String message) {
		MessageDialog.openInformation(viewer.getControl().getShell(), "Notification View", message);
	}

	/**
	 * Passing the focus request to the viewer's control.
	 */
	public void setFocus() {
		viewer.getControl().setFocus();
	}

	@Override
	public void notification(String url, String mode, String user, String sessionID) {
		// TODO Auto-generated method stub
		addMessage(url, mode);
	}

	@Override
	public void commitChangeModelNotification(String modelUri, String messageID, String changeModelContent) {
		// TODO Auto-generated method stub

	}
}