/*******************************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.modelbus.team.eclipse.repository.ui;

import java.io.IOException;

import org.eclipse.core.resources.mapping.ResourceTraversal;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.team.core.diff.IDiff;
import org.eclipse.team.core.diff.IDiffVisitor;
import org.eclipse.team.core.diff.IThreeWayDiff;
import org.eclipse.team.core.subscribers.SubscriberScopeManager;
import org.eclipse.ui.IWorkbenchPart;
import org.modelbus.dosgi.repository.descriptor.InvalidRevisionException;
import org.modelbus.dosgi.repository.descriptor.NonExistingResourceException;
import org.modelbus.dosgi.repository.descriptor.RepositoryAuthentificationException;
import org.modelbus.team.eclipse.repository.ModelBusRepositoryProvider;
import org.modelbus.team.eclipse.repository.Policy;
import org.modelbus.team.eclipse.repository.subscriber.ModelBusRepositorySubscriber;

/**
 * Operation for getting the contents of the selected resources
 */
public class GetOperation extends ModelBusRepositoryOperation {

	private boolean			overwriteOutgoing;
	private IWorkbenchPart	workbenchPart;

	public GetOperation(IWorkbenchPart part, SubscriberScopeManager manager) {
		super(part, manager);
		this.workbenchPart = part;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.modelbus.team.eclipse.repository.ui.ModelBusRepositoryOperation#execute(org.modelbus.team.eclipse.repository
	 * .ModelBusRepositoryProvider, org.eclipse.core.resources.mapping.ResourceTraversal[], org.eclipse.core.runtime.SubProgressMonitor)
	 */
	protected void execute(ModelBusRepositoryProvider provider, ResourceTraversal[] traversals, IProgressMonitor monitor) throws CoreException {
		if (!isOverwriteOutgoing() && hasOutgoingChanges(traversals)) {

			// GDU
			if (workbenchPart != null) {

				final Shell shell = workbenchPart.getSite().getShell();
				final Display display = shell.getDisplay();
				display.asyncExec(new Runnable() {

					@Override
					public void run() {
						MessageDialog
								.openInformation(getShell(), getTaskName(),
										"Could not get all changes due to conflicts or outgoing changes. You have to synchronize your project before updating.");
					}

				});
			}
			// throw new TeamException("Could not get all changes due to conflicts or outgoing changes.");
			/*
			 * Don't do anything. The exception will just result in an ugly accumulation of faulty jobs in the progress view. Can't use shouldRun()
			 * for this. Scope not initialized yet, when this is called.
			 */
			return;
		}
		try {
			provider.getOperations().get(traversals, isOverwriteOutgoing(), monitor);
		} catch (RepositoryAuthentificationException e) {
			// TODO Auto-generated catch block
			throw new RuntimeException(e);
		} catch (NonExistingResourceException e) {
			// TODO Auto-generated catch block
			throw new RuntimeException(e);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			throw new RuntimeException(e);
		} catch (InvalidRevisionException e) {
			// TODO Auto-generated catch block
			throw new RuntimeException(e);
		}

	}

	private boolean hasOutgoingChanges(ResourceTraversal[] traversals) throws CoreException {
		final RuntimeException found = new RuntimeException();
		try {
			ModelBusRepositorySubscriber.getInstance().accept(traversals, new IDiffVisitor() {
				public boolean visit(IDiff diff) {
					if (diff instanceof IThreeWayDiff) {
						IThreeWayDiff twd = (IThreeWayDiff) diff;
						if (twd.getDirection() == IThreeWayDiff.OUTGOING || twd.getDirection() == IThreeWayDiff.CONFLICTING) {
							throw found;
						}
					}
					return false;
				}
			});
		} catch (RuntimeException e) {
			if (e == found)
				return true;
			throw e;
		}
		return false;
	}

	/**
	 * Indicate whether the operation should overwrite outgoing changes. By default, the get operation does not override local modifications.
	 * 
	 * @return whether the operation should overwrite outgoing changes.
	 */
	protected boolean isOverwriteOutgoing() {
		return overwriteOutgoing;
	}

	/**
	 * Set whether the operation should overwrite outgoing changes.
	 * 
	 * @param overwriteOutgoing
	 *            whether the operation should overwrite outgoing changes
	 */
	public void setOverwriteOutgoing(boolean overwriteOutgoing) {
		this.overwriteOutgoing = overwriteOutgoing;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.modelbus.team.eclipse.repository.ui.ModelBusRepositoryOperation#getTaskName()
	 */
	protected String getTaskName() {
		return Policy.bind("GetAction.working"); //$NON-NLS-1$
	}
}
