/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.action;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.team.internal.ui.actions.TeamAction;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.ide.IDE;
import org.modelbus.team.eclipse.core.operation.AbstractActionOperation;
import org.modelbus.team.eclipse.core.operation.IActionOperation;
import org.modelbus.team.eclipse.core.operation.LoggedOperation;
import org.modelbus.team.eclipse.core.utility.ProgressMonitorUtility;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.utility.DefaultOperationWrapperFactory;
import org.modelbus.team.eclipse.ui.utility.ICancellableOperationWrapper;
import org.modelbus.team.eclipse.ui.utility.IOperationWrapperFactory;
import org.modelbus.team.eclipse.ui.utility.UIMonitorUtility;

/**
 * This class implements operation running policies
 * 
 * @author Alexander Gurov
 */
public abstract class AbstractModelBusTeamAction extends TeamAction {
	// copy paste in order to fix problems with Eclipse 3.0.x->3.1.x->3.2 API changes
	private IWorkbenchWindow	window;
	private Shell				shell;
	private ISelectionListener	selectionListener	= new ISelectionListener() {
														public void selectionChanged(IWorkbenchPart part, ISelection selection) {
															if (selection instanceof IStructuredSelection) {
																AbstractModelBusTeamAction.this.checkSelection((IStructuredSelection) selection);
															}
														}
													};

	public AbstractModelBusTeamAction() {
		super();
	}

	/**
	 * Save all dirty editors in the workbench that are open on files that may be affected by this operation. Opens a dialog to prompt the user if
	 * <code>confirm</code> is true. Return true if successful. Return false if the user has canceled the command. Must be called from the UI thread.
	 * 
	 * @param confirm
	 *            prompt the user if true
	 * @return boolean false if the operation was canceled.
	 */
	public final boolean saveAllEditors(boolean confirm) {
		return IDE.saveAllEditors(this.getSelectedResources(), confirm);
	}

	public abstract boolean isEnabled();

	public abstract void runImpl(IAction action);

	protected final void execute(final IAction action) throws InvocationTargetException, InterruptedException {
		ProgressMonitorUtility.doTaskExternal(new AbstractActionOperation("Operation.CallMenuAction") {
			protected void runImpl(IProgressMonitor monitor) throws Exception {
				if (AbstractModelBusTeamAction.this.isEnabled()) {
					if (AbstractModelBusTeamAction.this.needsToSaveDirtyEditors()
							&& !AbstractModelBusTeamAction.this.saveAllEditors(AbstractModelBusTeamAction.this.confirmSaveOfDirtyEditor())) {
						return;
					}
					AbstractModelBusTeamAction.this.runImpl(action);
				}
			}
		}, new NullProgressMonitor());
	}

	/**
	 * Return whether dirty editor should be saved before this action is run. Default is <code>false</code>.
	 * 
	 * @return whether dirty editor should be saved before this action is run
	 */
	protected boolean needsToSaveDirtyEditors() {
		return false;
	}

	/**
	 * Returns whether the user should be prompted to save dirty editors. The default is <code>true</code>.
	 * 
	 * @return whether the user should be prompted to save dirty editors
	 */
	protected boolean confirmSaveOfDirtyEditor() {
		return true;
	}

	protected ICancellableOperationWrapper runBusy(IActionOperation operation) {
		return UIMonitorUtility.doTaskBusy(operation, this.getOperationWrapperFactory());
	}

	protected ICancellableOperationWrapper runScheduled(IActionOperation operation) {
		return UIMonitorUtility.doTaskScheduled(this.getTargetPart(), operation, this.getOperationWrapperFactory());
	}

	protected IOperationWrapperFactory getOperationWrapperFactory() {
		return new DefaultOperationWrapperFactory();
	}

	protected void handleException(Exception ex) {
		this.handle(ex, ModelBusTeamUIPlugin.instance().getResource("Error.ActionFailed"),
				ModelBusTeamUIPlugin.instance().getResource("Error.ActionFailed.Message"));
	}

	public void selectionChanged(IAction action, ISelection selection) {
		try {
			if (selection == null || selection.isEmpty() || !(selection instanceof IStructuredSelection)) {
				if (action != null) {
					action.setEnabled(false);
				}
				return;
			}

			this.checkSelection((IStructuredSelection) selection);

			super.selectionChanged(action, selection);
		} catch (Throwable ex) {
			LoggedOperation.reportError(ModelBusTeamUIPlugin.instance().getResource("Error.MenuEnablement"), ex);
		}
	}

	protected Shell getShell() {
		return this.shell != null ? this.shell : super.getShell();
	}

	public IWorkbenchWindow getWindow() {
		return this.window;
	}

	public void init(IWorkbenchWindow window) {
		this.window = window;
		this.shell = this.window.getShell();
		this.window.getSelectionService().addPostSelectionListener(this.selectionListener);
	}

	public void dispose() {
		if (this.window != null) {
			this.window.getSelectionService().removePostSelectionListener(this.selectionListener);
		}
		super.dispose();
	}

	protected abstract void checkSelection(IStructuredSelection selection);

	protected abstract IStructuredSelection getSelection();

}
