/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Sergiy Logvin (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.action.remote;

import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import org.eclipse.jface.action.IAction;
import org.eclipse.swt.widgets.Shell;
import org.modelbus.team.eclipse.core.connector.ModelBusConnectorException;
import org.modelbus.team.eclipse.core.operation.CompositeOperation;
import org.modelbus.team.eclipse.core.operation.IActionOperation;
import org.modelbus.team.eclipse.core.operation.remote.PreparedBranchTagOperation;
import org.modelbus.team.eclipse.core.resource.IRepositoryContainer;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryRoot;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.action.AbstractRepositoryTeamAction;
import org.modelbus.team.eclipse.ui.dialog.DefaultDialog;
import org.modelbus.team.eclipse.ui.dialog.OperationErrorDialog;
import org.modelbus.team.eclipse.ui.operation.GetRemoteFolderChildrenOperation;
import org.modelbus.team.eclipse.ui.operation.RefreshRemoteResourcesOperation;
import org.modelbus.team.eclipse.ui.panel.common.AbstractBranchTagPanel;
import org.modelbus.team.eclipse.ui.panel.common.BranchPanel;
import org.modelbus.team.eclipse.ui.panel.common.TagPanel;
import org.modelbus.team.eclipse.ui.preferences.ModelBusTeamPreferences;
import org.modelbus.team.eclipse.ui.utility.DefaultOperationWrapperFactory;
import org.modelbus.team.eclipse.ui.utility.UIMonitorUtility;

/**
 * Branch or tag action on repository
 * 
 * @author Sergiy Logvin
 */
public class BranchTagAction extends AbstractRepositoryTeamAction {
	public static final int	BRANCH_ACTION	= 0;
	public static final int	TAG_ACTION		= 1;

	protected int			nodeType;

	public BranchTagAction(int nodeType) {
		super();
		this.nodeType = nodeType;
	}

	public void runImpl(IAction action) {
		IRepositoryResource[] resources = this.getSelectedRepositoryResources();
		PreparedBranchTagOperation op = BranchTagAction.getBranchTagOperation(resources, this.getShell(), this.nodeType);

		if (op != null) {
			CompositeOperation composite = new CompositeOperation(op.getId());
			composite.add(op);
			composite.add(new RefreshRemoteResourcesOperation(new IRepositoryResource[] { op.getDestination().getParent() }),
					new IActionOperation[] { op });
			this.runScheduled(composite);
		}
	}

	public static PreparedBranchTagOperation getBranchTagOperation(IRepositoryResource[] resources, Shell shell, int nodeType) {
		if (!OperationErrorDialog.isAcceptableAtOnce(
				resources,
				ModelBusTeamUIPlugin.instance().getResource(
						nodeType == BranchTagAction.BRANCH_ACTION ? "BranchTagAction.Error.Branch" : "BranchTagAction.Error.Tag"), shell)) {
			return null;
		}

		resources = ModelBusUtility.shrinkChildNodes(resources);
		boolean isStructureEnabled =
		// CHECKMWA resources[0].getRepositoryLocation().isStructureEnabled()&&
		ModelBusTeamPreferences.getRepositoryBoolean(ModelBusTeamUIPlugin.instance().getPreferenceStore(),
				ModelBusTeamPreferences.BRANCH_TAG_CONSIDER_STRUCTURE_NAME);
		// no structure -> copy content to destination
		// structure detection disabled -> copy selection content to destination
		// consider structure disabled -> copy selection content to destination

		// single-project layout:trunk selected -> copy selection content to destination
		// single-project layout:child of trunk selected -> copy selection content to destination

		// multiple-project layout:trunk selected -> copy selection content to destination
		// multiple-project layout:trunk children selected -> copy selection to destination
		// multiple-project layout:one child of trunk selected -> copy selection to destination (forceCreate)

		// tagging from branch and branching from tag uses the same rules, i.e. respects a project layout.
		Set<String> nodeNames = Collections.emptySet();
		boolean forceCreate = false;
		if (isStructureEnabled) {
			// allows to use project root for branching and tagging of the whole project
			int kind = ((IRepositoryRoot) resources[0].getRoot()).getKind();
			if (kind == IRepositoryRoot.KIND_LOCATION_ROOT || kind == IRepositoryRoot.KIND_ROOT) {

				// throw new RuntimeException("TODOMWA");
				try {

					IRepositoryContainer supposedTrunk = resources[0].asRepositoryContainer(
					/* resources[0].getRepositoryLocation().getTrunkLocation() */"trunk", false);
					if (supposedTrunk.exists()) {
						resources[0] = supposedTrunk;
					}
				} catch (ModelBusConnectorException ex) {
					// do nothing
				}
			}
			nodeNames = BranchTagAction.getExistingNodeNames(nodeType == BranchTagAction.BRANCH_ACTION ? ModelBusUtility
					.getBranchesLocation(resources[0]) : ModelBusUtility.getTagsLocation(resources[0]));
			forceCreate = resources.length == 1
					&& !(resources[0] instanceof IRepositoryRoot && ((IRepositoryRoot) resources[0]).getKind() == IRepositoryRoot.KIND_TRUNK)
					&& !BranchTagAction.isSingleProjectLayout(resources[0]) && BranchTagAction.isProjectFileExists(resources[0]);
		}

		AbstractBranchTagPanel panel = nodeType == BranchTagAction.BRANCH_ACTION ? (AbstractBranchTagPanel) new BranchPanel(
				ModelBusUtility.getBranchesLocation(resources[0]), false, nodeNames, resources) : new TagPanel(
				ModelBusUtility.getTagsLocation(resources[0]), false, nodeNames, resources);
		DefaultDialog dialog = new DefaultDialog(shell, panel);
		if (dialog.open() == 0) {
			IRepositoryResource[] resourcesWithSpecifiedRevision = new IRepositoryResource[resources.length];
			for (int i = 0; i < resources.length; i++) {
				resourcesWithSpecifiedRevision[i] = ModelBusUtility.copyOf(resources[i]);
				resourcesWithSpecifiedRevision[i].setSelectedRevision(panel.getRevisionForRemoteResources());
				resourcesWithSpecifiedRevision[i].setPegRevision(resources[i].getPegRevision());
			}
			return new PreparedBranchTagOperation(nodeType == BranchTagAction.BRANCH_ACTION ? "Branch" : "Tag", resourcesWithSpecifiedRevision,
					panel.getDestination(), panel.getMessage(), forceCreate);
		} else {
			return null;
		}
		// return dialog.open() != 0 ? null : new PreparedBranchTagOperation(nodeType == BranchTagAction.BRANCH_ACTION ? "Branch" : "Tag", resources,
		// panel.getDestination(), panel.getMessage(), forceCreate);
		// throw new RuntimeException("TODOMWA");
	}

	public static Set<String> getExistingNodeNames(IRepositoryContainer parent) {
		HashSet<String> nodeNames = new HashSet<String>();
		IRepositoryResource[] existentNodes = BranchTagAction.getRemoteChildren(parent);
		if (existentNodes != null) {
			for (int i = 0; i < existentNodes.length; i++) {
				nodeNames.add(existentNodes[i].getName());
			}
		}
		return nodeNames;
	}

	// MES: Disabled by default.
	public boolean isEnabled() {
		IRepositoryResource[] resources = this.getSelectedRepositoryResources();
		if (resources.length == 0) {
			return false;
		}
		// disable branching/tagging resources from different repositories
		return true;
		// return false;
	}

	protected static IRepositoryResource[] getRemoteChildren(final IRepositoryContainer parent) {
		GetRemoteFolderChildrenOperation op = new GetRemoteFolderChildrenOperation(parent, false);
		UIMonitorUtility.doTaskBusy(op, new DefaultOperationWrapperFactory() {
			public IActionOperation getLogged(IActionOperation operation) {
				return operation;
			}
		});
		return op.getChildren();
	}

	public static boolean isSingleProjectLayout(IRepositoryResource resource) {
		System.out.println("TODOMWA28");
		return true;
		// return BranchTagAction.isProjectFileExists(ModelBusUtility.getTrunkLocation(resource));
	}

	protected static boolean isProjectFileExists(IRepositoryResource resource) {
		try {
			return resource.asRepositoryFile(".project", false).exists();
		} catch (ModelBusConnectorException ex) {
			return false;
		}
	}

}
