/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.annotate;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jface.internal.text.revisions.RevisionSelectionProvider;
import org.eclipse.jface.text.revisions.RevisionInformation;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.team.ui.history.IHistoryView;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.editors.text.EditorsUI;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.ide.ResourceUtil;
import org.eclipse.ui.texteditor.AbstractDecoratedTextEditor;
import org.modelbus.team.eclipse.core.ModelBusTeamPlugin;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusAnnotationData;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusLogEntry;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.operation.AbstractActionOperation;
import org.modelbus.team.eclipse.core.operation.CompositeOperation;
import org.modelbus.team.eclipse.core.operation.IActionOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.operation.remote.GetResourceAnnotationOperation;
import org.modelbus.team.eclipse.core.resource.IRepositoryFile;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.core.utility.ProgressMonitorUtility;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.history.ModelBusHistoryPage;
import org.modelbus.team.eclipse.ui.operation.OpenRemoteFileOperation;
import org.modelbus.team.eclipse.ui.preferences.ModelBusTeamPreferences;
import org.modelbus.team.eclipse.ui.utility.UIMonitorUtility;

/**
 * Annotation based on Eclipse Platform support.
 * 
 * @author Alexander Gurov
 */
public class BuiltInAnnotate {
	protected AbstractDecoratedTextEditor	textEditor;
	protected ModelBusHistoryPage			historyPage;

	public void open(IWorkbenchPage page, IRepositoryResource remote, IFile resource) {
		GetResourceAnnotationOperation annotateOp = new GetResourceAnnotationOperation(remote);
		annotateOp.setIncludeMerged(ModelBusTeamPreferences.getMergeBoolean(ModelBusTeamUIPlugin.instance().getPreferenceStore(),
				ModelBusTeamPreferences.MERGE_INCLUDE_MERGED_NAME));
		IActionOperation showOp = this.prepareBuiltInAnnotate(annotateOp, page, remote, resource);
		CompositeOperation op = new CompositeOperation(showOp.getId());
		op.add(annotateOp);
		op.add(showOp, new IActionOperation[] { annotateOp });
		UIMonitorUtility.doTaskScheduledDefault(page.getActivePart(), op);
	}

	protected IActionOperation prepareBuiltInAnnotate(final GetResourceAnnotationOperation annotateOp, final IWorkbenchPage page,
			final IRepositoryResource remote, final IFile resource) {
		CompositeOperation op = new CompositeOperation("Operation.BuiltInShowAnnotation");
		final RevisionInformation info = new RevisionInformation();
		IActionOperation prepareRevisions = new AbstractActionOperation("Operation.PrepareRevisions") {
			protected void runImpl(IProgressMonitor monitor) throws Exception {
				Map<String, BuiltInAnnotateRevision> revisions = new HashMap<String, BuiltInAnnotateRevision>();
				ModelBusAnnotationData[] data = annotateOp.getAnnotatedLines();
				if (data == null || data.length == 0) {
					return;
				}
				String noAuthor = ModelBusTeamPlugin.instance().getResource("ModelBusInfo.NoAuthor");
				for (int i = 0; i < data.length; i++) {
					String revisionId = data[i].revision;
					BuiltInAnnotateRevision revision = revisions.get(revisionId);
					if (revision == null) {
						//TODO date for revision?
						ModelBusRevision modelBusRevision = ModelBusRevision.fromString(revisionId);
						
						revisions.put(revisionId, revision = new BuiltInAnnotateRevision(modelBusRevision, data[i].author, CommitterColors.getDefault()
								.getCommitterRGB(data[i].author == null ? noAuthor : data[i].author)));
						info.addRevision(revision);
					}
					revision.addLine(i + 1);
					if (data[i].mergedRevision != ModelBusRevision.INVALID_REVISION_STRING && data[i].mergedRevision != data[i].revision) {
						revision.addMergeInfo(i + 1, data[i].mergedRevision, data[i].mergedDate, data[i].mergedAuthor == null ? noAuthor
								: data[i].mergedAuthor, data[i].mergedPath);
					}
				}
				ModelBusRevision from = ModelBusRevision.INVALID_REVISION_DESCRIPTOR, to = ModelBusRevision.INVALID_REVISION_DESCRIPTOR;
				for (BuiltInAnnotateRevision revision : revisions.values()) {
					revision.addLine(BuiltInAnnotateRevision.END_LINE);
//					String revisionNum = revision.getRevision();
					if (from.isGreaterThan(revision.getRevision()) || from == ModelBusRevision.INVALID_REVISION_DESCRIPTOR) {
						from = revision.getRevision();
					}
					if (to.isLowerThan(revision.getRevision())) {
						to = revision.getRevision();
					}
				}
				IRepositoryResource resource = annotateOp.getRepositoryResource();
				IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
				// resource.getRepositoryLocation().acquireModelBusProxy();
				try {
					ModelBusLogEntry[] msgs = ModelBusUtility.logEntries(proxy, ModelBusUtility.getEntryReference(resource),
							to, from, IModelBusConnector.Options.NONE,
							IModelBusConnector.DEFAULT_LOG_ENTRY_PROPS, 0, new ModelBusProgressMonitor(this, monitor, null));
					for (int i = 0; i < msgs.length; i++) {
						BuiltInAnnotateRevision revision = revisions.get(msgs[i].revision);
						if (revision != null) {
							revision.setLogMessage(msgs[i]);
						}
					}
				} finally {
					// resource.getRepositoryLocation().releaseModelBusProxy(proxy);
				}
			}
		};
		op.add(prepareRevisions, new IActionOperation[] { annotateOp });
		IActionOperation attachMessages = new AbstractActionOperation("Operation.BuiltInShowView") {
			protected void runImpl(IProgressMonitor monitor) throws Exception {
				page.getActivePart().getSite().getShell().getDisplay().syncExec(new Runnable() {
					public void run() {
						try {
							BuiltInAnnotate.this.initializeEditor(page, remote, resource, info);
						} catch (PartInitException ex) {
							throw new RuntimeException(ex);
						}
					}
				});
			}
		};
		op.add(attachMessages, new IActionOperation[] { annotateOp, prepareRevisions });
		return op;
	}

	protected void initializeEditor(final IWorkbenchPage page, final IRepositoryResource remote, final IFile resource, RevisionInformation info)
			throws PartInitException {
		IEditorPart editor = resource != null ? this.openEditor(page, resource) : this.openEditor(page, remote);
		if (editor instanceof AbstractDecoratedTextEditor) {
			this.textEditor = (AbstractDecoratedTextEditor) editor;
			final ISelectionProvider provider = (ISelectionProvider) this.textEditor.getAdapter(RevisionSelectionProvider.class);
			if (provider != null) {
				final ISelectionChangedListener selectionListener = new ISelectionChangedListener() {
					public void selectionChanged(SelectionChangedEvent event) {
						if (event.getSelection() instanceof IStructuredSelection) {
							BuiltInAnnotateRevision selected = (BuiltInAnnotateRevision) ((IStructuredSelection) event.getSelection())
									.getFirstElement();
							if (selected != null) {
								if (BuiltInAnnotate.this.historyPage != null) {
									BuiltInAnnotate.this.historyPage.selectRevision(selected.getId());
								}
							}
						}
					}
				};
				provider.addSelectionChangedListener(selectionListener);
				page.addPartListener(new IPartListener() {
					public void partClosed(IWorkbenchPart part) {
						if (part instanceof IHistoryView || part == BuiltInAnnotate.this.textEditor) {
							page.removePartListener(this);
							provider.removeSelectionChangedListener(selectionListener);
						}
					}

					public void partActivated(IWorkbenchPart part) {
					}

					public void partBroughtToTop(IWorkbenchPart part) {
					}

					public void partDeactivated(IWorkbenchPart part) {
					}

					public void partOpened(IWorkbenchPart part) {
					}
				});
			}
			this.textEditor.showRevisionInformation(info, ModelBusTeamQuickDiffProvider.class.getName());
		}
	}

	protected IEditorPart openEditor(IWorkbenchPage page, IRepositoryResource remote) throws PartInitException {
		OpenRemoteFileOperation op = new OpenRemoteFileOperation(new IRepositoryFile[] { (IRepositoryFile) remote },
				OpenRemoteFileOperation.OPEN_DEFAULT);
		op.setRequiredDefaultEditorKind(AbstractDecoratedTextEditor.class);
		ProgressMonitorUtility.doTaskExternal(op, new NullProgressMonitor());
		if (op.getExecutionState() == IActionOperation.OK) {
			IEditorPart part = op.getEditors()[0];
			if (part instanceof AbstractDecoratedTextEditor) {
				return part;
			}
		}
		return null;
	}

	protected IEditorPart openEditor(IWorkbenchPage page, IFile resource) throws PartInitException {
		IEditorPart part = ResourceUtil.findEditor(page, resource);
		if (part != null && part instanceof AbstractDecoratedTextEditor) {
			page.activate(part);
			return part;
		}

		return IDE.openEditor(page, resource, EditorsUI.DEFAULT_TEXT_EDITOR_ID);
	}

}
