/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *    Thomas Champagne - Bug 217561 : additional date formats for label decorations
 *    Igor Burilo - Bug 211415: Export History log
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.history;

import java.io.FileOutputStream;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.LinkedHashSet;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFileState;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.team.ui.history.HistoryPage;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchPartSite;
import org.eclipse.ui.PlatformUI;
import org.modelbus.team.eclipse.core.IStateFilter;
import org.modelbus.team.eclipse.core.connector.ModelBusLogEntry;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRemoteStorage;
import org.modelbus.team.eclipse.core.modelbusstorage.events.IRevisionPropertyChangeListener;
import org.modelbus.team.eclipse.core.modelbusstorage.events.RevisonPropertyChangeEvent;
import org.modelbus.team.eclipse.core.operation.AbstractActionOperation;
import org.modelbus.team.eclipse.core.operation.CompositeOperation;
import org.modelbus.team.eclipse.core.operation.IActionOperation;
import org.modelbus.team.eclipse.core.operation.LoggedOperation;
import org.modelbus.team.eclipse.core.operation.remote.GetLogMessagesOperation;
import org.modelbus.team.eclipse.core.resource.ILocalResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.resource.events.IResourceStatesListener;
import org.modelbus.team.eclipse.core.resource.events.ResourceStatesChangedEvent;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.core.utility.ProgressMonitorUtility;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.dialog.DefaultDialog;
import org.modelbus.team.eclipse.ui.history.HistoryActionManager.HistoryAction;
import org.modelbus.team.eclipse.ui.history.data.ModelBusLocalFileRevision;
import org.modelbus.team.eclipse.ui.history.filter.AuthorNameLogEntryFilter;
import org.modelbus.team.eclipse.ui.history.filter.ChangeNameLogEntryFilter;
import org.modelbus.team.eclipse.ui.history.filter.CommentLogEntryFilter;
import org.modelbus.team.eclipse.ui.history.filter.CompositeLogEntryFilter;
import org.modelbus.team.eclipse.ui.history.filter.ILogEntryFilter;
import org.modelbus.team.eclipse.ui.history.model.ILogNode;
import org.modelbus.team.eclipse.ui.operation.CorrectRevisionOperation;
import org.modelbus.team.eclipse.ui.operation.UILoggedOperation;
import org.modelbus.team.eclipse.ui.panel.view.HistoryFilterPanel;
import org.modelbus.team.eclipse.ui.panel.view.HistoryRangePanel;
import org.modelbus.team.eclipse.ui.preferences.ModelBusTeamPreferences;
import org.modelbus.team.eclipse.ui.repository.model.RepositoryResource;
import org.modelbus.team.eclipse.ui.utility.UIMonitorUtility;

/**
 * Generic HistoryView page
 * 
 * @author Alexander Gurov
 */
public class ModelBusHistoryPage extends HistoryPage implements IModelBusHistoryView, IResourceStatesListener, IPropertyChangeListener,
		IRevisionPropertyChangeListener {
	protected IResource						wcResource;
	protected IRepositoryResource			repositoryResource;

	protected ModelBusRevision				endRevision;
	protected ModelBusRevision				startRevision;

	protected LogMessagesComposite			history;

	protected CommentLogEntryFilter			commentFilter;
	protected AuthorNameLogEntryFilter		authorFilter;
	protected ChangeNameLogEntryFilter		changeFilter;
	protected CompositeLogEntryFilter		logEntriesFilter;

	protected Action						showCommentViewerAction;
	protected Action						showAffectedPathsViewerAction;
	protected Action						hideUnrelatedAction;
	protected Action						hideUnrelatedDropDownAction;
	protected Action						stopOnCopyAction;
	protected Action						stopOnCopyDropDownAction;
	protected Action						groupByDateDropDownAction;
	protected Action						getNextPageAction;
	protected Action						getAllPagesAction;
	protected Action						clearFilterDropDownAction;
	protected Action						filterDropDownAction;
	protected Action						revisionsRangeDropDownAction;
	protected Action						hierarchicalAction;
	protected Action						flatAction;
	protected Action						compareModeAction;
	protected Action						compareModeDropDownAction;
	protected Action						groupByDateAction;
	protected Action						showLocalAction;
	protected Action						showRemoteAction;
	protected Action						showBothAction;
	protected Action						collapseAllAction;
	protected Action						exportLogAction;
	protected Action						showLocalActionDropDown;
	protected Action						showRemoteActionDropDown;
	protected Action						showBothActionDropDown;

	protected long							limit			= 25;
	protected boolean						pagingEnabled	= false;
	protected boolean						pending;
	protected int							options			= 0;

	protected IResource						compareWith;

	protected ModelBusRevision			currentRevision	= ModelBusRevision.fromString("0");
	protected ModelBusLogEntry[]			logMessages;
	protected ModelBusLocalFileRevision[]	localHistory;

	protected HistoryActionManager			actionManager;

	public ModelBusHistoryPage(Object input) {
		ModelBusRemoteStorage.instance().addResourceStatesListener(ResourceStatesChangedEvent.class, this);
		ModelBusRemoteStorage.instance().addRevisionPropertyChangeListener(this);
		this.actionManager = new HistoryActionManager(this);
		ModelBusTeamUIPlugin.instance().getPreferenceStore().addPropertyChangeListener(this);
		this.authorFilter = new AuthorNameLogEntryFilter();
		this.commentFilter = new CommentLogEntryFilter();
		this.changeFilter = new ChangeNameLogEntryFilter();
		this.logEntriesFilter = new CompositeLogEntryFilter(new ILogEntryFilter[] { this.authorFilter, this.commentFilter, this.changeFilter });
	}

	public void propertyChange(PropertyChangeEvent event) {
		if (event.getProperty().equals(ModelBusTeamPreferences.fullHistoryName(ModelBusTeamPreferences.HISTORY_PAGE_SIZE_NAME))
				|| event.getProperty().equals(ModelBusTeamPreferences.fullHistoryName(ModelBusTeamPreferences.HISTORY_PAGING_ENABLE_NAME))) {
			this.refreshLimitOption();
		}
		if (event.getProperty().startsWith(ModelBusTeamPreferences.DATE_FORMAT_BASE)) {
			this.refresh(IModelBusHistoryView.REFRESH_VIEW);
		}
		if (event.getProperty().equals(ModelBusTeamPreferences.fullMergeName(ModelBusTeamPreferences.MERGE_INCLUDE_MERGED_NAME))) {
			this.refresh(IModelBusHistoryView.REFRESH_REMOTE);
		}
	}

	public void revisionPropertyChanged(RevisonPropertyChangeEvent event) {
		if (event.getProperty().name.equals("modelbus:author") || event.getProperty().name.equals("modelbus:log")
				|| event.getProperty().name.equals("modelbus:date")) {
			for (int i = 0; i < this.logMessages.length; i++) {
				ModelBusLogEntry current = this.logMessages[i];
				if (ModelBusRevision.fromString(current.revision).equals(event.getRevision())) {
					if (event.getProperty().name.equals("modelbus:author")) {
						this.logMessages[i] = new ModelBusLogEntry(current.revision, current.date, event.getProperty().value, current.message,
								current.changedPaths, current.hasChildren());
						if (current.hasChildren()) {
							this.logMessages[i].addAll(current.getChildren());
						}
					}
					if (event.getProperty().name.equals("modelbus:log")) {
						this.logMessages[i] = new ModelBusLogEntry(current.revision, current.date, current.author, event.getProperty().value,
								current.changedPaths, current.hasChildren());
						if (current.hasChildren()) {
							this.logMessages[i].addAll(current.getChildren());
						}
					}
					if (event.getProperty().name.equals("modelbus:date")) {
						try {
							this.logMessages[i] = new ModelBusLogEntry(current.revision, new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'").parse(
									event.getProperty().value).getTime(), current.author, current.message, current.changedPaths,
									current.hasChildren());
						} catch (ParseException ex) {
							// uninteresting in this context
						}
						if (current.hasChildren()) {
							this.logMessages[i].addAll(current.getChildren());
						}
					}
				}
			}
			UIMonitorUtility.getDisplay().syncExec(new Runnable() {
				public void run() {
					ModelBusHistoryPage.this.refresh(IModelBusHistoryView.REFRESH_VIEW);
				}
			});
		}
	}

	public void resourcesStateChanged(ResourceStatesChangedEvent event) {
		if (this.wcResource == null) {
			return;
		}
		ILocalResource local = ModelBusRemoteStorage.instance().asLocalResource(this.wcResource);
		if (!IStateFilter.SF_INTERNAL_INVALID.accept(local)) {
			if (IStateFilter.SF_ONREPOSITORY.accept(local) && this.logMessages == null) {
				this.refreshChanges(IModelBusHistoryView.REFRESH_ALL);
			} else if (this.wcResource instanceof IFile) {
				this.refreshChanges(IModelBusHistoryView.REFRESH_LOCAL);
			}
			if ((event.contains(this.wcResource) || event.contains(this.wcResource.getProject()))
					&& (!this.wcResource.exists() || !FileUtility.isConnected(this.wcResource))) {
				this.disconnectView();
			}
		} else {
			this.disconnectView();
		}
	}

	public void addFilter(ILogEntryFilter filter) {
		this.logEntriesFilter.addFilter(filter);
	}

	public void removeFilter(ILogEntryFilter filter) {
		this.logEntriesFilter.removeFilter(filter);
	}

	public void showHistory(IResource resource) {
		if (!resource.equals(this.wcResource)) {
			this.clear();

			this.wcResource = resource;
			this.refresh(IModelBusHistoryView.REFRESH_ALL);
		}
	}

	public void showHistory(IRepositoryResource remoteResource) {
		if (!remoteResource.equals(this.repositoryResource)) {
			this.clear();

			this.repositoryResource = remoteResource;

			this.refresh(IModelBusHistoryView.REFRESH_ALL);
		}
	}

	public synchronized void clear() {
		this.currentRevision = ModelBusRevision.INVALID_REVISION_DESCRIPTOR;
		this.repositoryResource = null;
		this.wcResource = null;
		this.logMessages = null;
		this.localHistory = null;
		this.authorFilter.setAuthorNameToAccept(null);
		this.commentFilter.setCommentToAccept(null);
		this.pending = false;

		this.setButtonsEnablement();
		this.history.refresh(LogMessagesComposite.REFRESH_ALL);
	}

	public void selectRevision(String revision) {
		this.history.setSelectedRevision(revision);
	}

	public void setCompareWith(IResource compareWith) {
		this.compareWith = compareWith;
	}

	public void setOptions(int mask, int values) {
		this.options = (this.options & ~mask) | (mask & values);
		this.refreshOptionButtons();
	}

	public IResource getResource() {
		return this.wcResource;
	}

	public IRepositoryResource getRepositoryResource() {
		return this.repositoryResource;
	}

	public IResource getCompareWith() {
		return this.compareWith == null ? this.wcResource : this.compareWith;
	}

	public HistoryPage getHistoryPage() {
		return this;
	}

	public ModelBusLogEntry[] getRemoteHistory() {
		return ModelBusHistoryPage.filterMessages(this.logMessages, this.logEntriesFilter);
	}

	public ModelBusLogEntry[] getFullRemoteHistory() {
		return this.logMessages;
	}

	public ModelBusLocalFileRevision[] getLocalHistory() {
		return this.localHistory;
	}

	public boolean isAllRemoteHistoryFetched() {
		return !this.getNextPageAction.isEnabled();
	}

	public boolean isFilterEnabled() {
		return this.authorFilter.getAuthorNameToAccept() != null || this.commentFilter.getCommentToAccept() != null
				|| this.changeFilter.getGangedPathToAccept() != null;
	}

	public int getOptions() {
		return this.options;
	}

	public void clearFilter() {
		this.authorFilter.setAuthorNameToAccept(null);
		this.commentFilter.setCommentToAccept(null);
		this.changeFilter.setGangedPathToAccept(null);
		this.clearFilterDropDownAction.setEnabled(false);
		this.history.refresh(LogMessagesComposite.REFRESH_ALL);
	}

	public void setFilter() {
		HistoryFilterPanel panel = new HistoryFilterPanel(this.authorFilter.getAuthorNameToAccept(), this.commentFilter.getCommentToAccept(),
				this.changeFilter.getGangedPathToAccept(), ModelBusHistoryPage.getSelectedAuthors(this.logMessages));
		DefaultDialog dialog = new DefaultDialog(this.getPartSite().getShell(), panel);
		if (dialog.open() == 0) {
			this.authorFilter.setAuthorNameToAccept(panel.getAuthor());
			this.commentFilter.setCommentToAccept(panel.getComment());
			this.changeFilter.setGangedPathToAccept(panel.getChangedPath());
			this.clearFilterDropDownAction.setEnabled(this.isFilterEnabled());
			this.history.refresh(LogMessagesComposite.REFRESH_ALL);
		}
	}

	public void refresh(int refreshType) {
		if (this.wcResource != null) {
			ILocalResource local = ModelBusRemoteStorage.instance().asLocalResource(this.wcResource);
			if (IStateFilter.SF_ONREPOSITORY.accept(local)) {
				this.currentRevision = local.getRevision();
				this.repositoryResource = local.isCopied() ? ModelBusUtility.getCopiedFrom(this.wcResource) : ModelBusRemoteStorage.instance()
						.asRepositoryResource(this.wcResource);
			} else {
				this.repositoryResource = null;
			}

			if (this.wcResource instanceof IFile && refreshType != IModelBusHistoryView.REFRESH_REMOTE
					&& refreshType != IModelBusHistoryView.REFRESH_VIEW) {
				try {
					this.fetchLocalHistory(local, new NullProgressMonitor());
				} catch (CoreException ex) {
					UILoggedOperation.reportError(ModelBusTeamUIPlugin.instance().getResource("HistoryView.Name"), ex);
				}
			}
		}

		if (this.repositoryResource != null
				&& (refreshType == IModelBusHistoryView.REFRESH_ALL || refreshType == IModelBusHistoryView.REFRESH_REMOTE)) {
			this.logMessages = null;
			this.pending = true;
			this.setButtonsEnablement();
			this.history.refresh(LogMessagesComposite.REFRESH_ALL);
			GetLogMessagesOperation msgOp = this.createRemoteHistoryFetcher();
			if (this.startRevision != null) {
				msgOp.setStartRevision(this.startRevision);
			}
			msgOp.setLimit(this.limit);
			this.fetchRemoteHistory(msgOp);
		} else {
			this.setButtonsEnablement();
			this.history.refresh(LogMessagesComposite.REFRESH_ALL);
		}
	}

	public ModelBusRevision getCurrentRevision() {
		return this.currentRevision;
	}

	public boolean isGrouped() {
		return this.groupByDateAction.isChecked();
	}

	public boolean isPending() {
		return this.pending;
	}

	public int getMode() {
		return this.options & IModelBusHistoryViewInfo.MODE_MASK;
	}

	public boolean isRelatedPathsOnly() {
		return this.hideUnrelatedAction.isChecked();
	}

	public void dispose() {
		ModelBusRemoteStorage.instance().removeResourceStatesListener(ResourceStatesChangedEvent.class, this);
		ModelBusRemoteStorage.instance().removeRevisionPropertyChangeListener(this);
		ModelBusTeamUIPlugin.instance().getPreferenceStore().removePropertyChangeListener(this);
		// log messages composite is disposed by HistoryPage.dispose()
		super.dispose();
	}

	public boolean inputSet() {
		if (this.getInput() instanceof IResource) {
			this.showHistory((IResource) this.getInput());
			return true;
		} else if (this.getInput() instanceof IRepositoryResource) {
			this.showHistory((IRepositoryResource) this.getInput());
			return true;
		} else if (this.getInput() instanceof RepositoryResource) {
			this.showHistory(((RepositoryResource) this.getInput()).getRepositoryResource());
			return true;
		}
		// else if (this.getInput() instanceof RepositoryLocation) {
		// this.showHistory(((RepositoryLocation)this.getInput()).getRepositoryResource());
		// return true;
		// }
		return false;
	}

	public Control getControl() {
		return this.history;
	}

	public void setFocus() {

	}

	public String getDescription() {
		return this.getName();
	}

	public String getName() {
		if (this.getResource() != null) {
			return this.getResource().getFullPath().toString().substring(1);
		}
		if (this.getRepositoryResource() != null) {
			return this.getRepositoryResource().getUrl();
		}
		return ModelBusTeamUIPlugin.instance().getResource("ModelBusView.ResourceNotSelected");
	}

	public boolean isValidInput(Object object) {
		return ModelBusHistoryPage.isValidData(object);
	}

	public void refresh() {
		this.refresh(IModelBusHistoryView.REFRESH_ALL);
	}

	@SuppressWarnings("unchecked")
	public Object getAdapter(Class adapter) {
		return null;
	}

	public void createControl(Composite parent) {
		IActionBars actionBars = this.getHistoryPageSite().getWorkbenchPageSite().getActionBars();

		this.groupByDateAction = new HistoryAction("HistoryView.GroupByDate", "icons/views/history/group_by_date.gif", IAction.AS_CHECK_BOX) {
			public void run() {
				ModelBusHistoryPage.this.options ^= IModelBusHistoryView.GROUP_BY_DATE;
				ModelBusHistoryPage.this.groupByDateDropDownAction
						.setChecked((ModelBusHistoryPage.this.options & IModelBusHistoryView.GROUP_BY_DATE) != 0);
				ModelBusHistoryPage
						.saveInt(
								ModelBusTeamPreferences.HISTORY_GROUPING_TYPE_NAME,
								(ModelBusHistoryPage.this.options & IModelBusHistoryView.GROUP_BY_DATE) == 0 ? ModelBusTeamPreferences.HISTORY_GROUPING_TYPE_NONE
										: ModelBusTeamPreferences.HISTORY_GROUPING_TYPE_DATE);
				ModelBusHistoryPage.this.history.refresh(LogMessagesComposite.REFRESH_UI_ALL);
			}
		};

		this.showBothAction = new HistoryAction("HistoryView.ShowBoth", "icons/views/history/both_history_mode.gif", IAction.AS_RADIO_BUTTON) {
			public void run() {
				ModelBusHistoryPage.this.options = ModelBusHistoryPage.this.options
						& ~(IModelBusHistoryViewInfo.MODE_LOCAL | IModelBusHistoryViewInfo.MODE_REMOTE) | IModelBusHistoryViewInfo.MODE_BOTH;
				ModelBusHistoryPage.this.showBothActionDropDown.setChecked(true);
				ModelBusHistoryPage.this.showLocalActionDropDown.setChecked(false);
				ModelBusHistoryPage.this.showRemoteActionDropDown.setChecked(false);
				ModelBusHistoryPage.this.saveShowMode();
				ModelBusHistoryPage.this.history.refresh(LogMessagesComposite.REFRESH_UI_ALL);
			}
		};
		this.showLocalAction = new HistoryAction("HistoryView.ShowLocal", "icons/views/history/local_history_mode.gif", IAction.AS_RADIO_BUTTON) {
			public void run() {
				ModelBusHistoryPage.this.options = ModelBusHistoryPage.this.options
						& ~(IModelBusHistoryViewInfo.MODE_REMOTE | IModelBusHistoryViewInfo.MODE_BOTH) | IModelBusHistoryViewInfo.MODE_LOCAL;
				ModelBusHistoryPage.this.showLocalActionDropDown.setChecked(true);
				ModelBusHistoryPage.this.showRemoteActionDropDown.setChecked(false);
				ModelBusHistoryPage.this.showBothActionDropDown.setChecked(false);
				ModelBusHistoryPage.this.saveShowMode();
				ModelBusHistoryPage.this.history.refresh(LogMessagesComposite.REFRESH_UI_ALL);
			}
		};
		this.showRemoteAction = new HistoryAction("HistoryView.ShowRemote", "icons/views/history/remote_history_mode.gif", IAction.AS_RADIO_BUTTON) {
			public void run() {
				ModelBusHistoryPage.this.options = ModelBusHistoryPage.this.options
						& ~(IModelBusHistoryViewInfo.MODE_LOCAL | IModelBusHistoryViewInfo.MODE_BOTH) | IModelBusHistoryViewInfo.MODE_REMOTE;
				ModelBusHistoryPage.this.showRemoteActionDropDown.setChecked(true);
				ModelBusHistoryPage.this.showLocalActionDropDown.setChecked(false);
				ModelBusHistoryPage.this.showBothActionDropDown.setChecked(false);
				ModelBusHistoryPage.this.saveShowMode();
				ModelBusHistoryPage.this.history.refresh(LogMessagesComposite.REFRESH_UI_ALL);
			}
		};

		this.hideUnrelatedAction = new HistoryAction("HistoryView.HideUnrelatedPaths", "icons/views/history/hide_unrelated.gif", IAction.AS_CHECK_BOX) {
			public void run() {
				ModelBusHistoryPage.this.options ^= IModelBusHistoryView.HIDE_UNRELATED;
				ModelBusHistoryPage.this.hideUnrelatedDropDownAction
						.setChecked((ModelBusHistoryPage.this.options & IModelBusHistoryView.HIDE_UNRELATED) != 0);
				ModelBusHistoryPage.this.history.refresh(LogMessagesComposite.REFRESH_UI_AFFECTED);
			}
		};
		this.stopOnCopyAction = new HistoryAction("HistoryView.StopOnCopy", "icons/views/history/stop_on_copy.gif", IAction.AS_CHECK_BOX) {
			public void run() {
				ModelBusHistoryPage.this.options ^= IModelBusHistoryView.STOP_ON_COPY;
				ModelBusHistoryPage.this.stopOnCopyDropDownAction
						.setChecked((ModelBusHistoryPage.this.options & IModelBusHistoryView.STOP_ON_COPY) != 0);
				ModelBusHistoryPage.this.refresh(IModelBusHistoryView.REFRESH_REMOTE);
			}
		};

		this.getNextPageAction = new HistoryAction("HistoryView.GetNextPage", "icons/views/history/paging.gif") {
			public void run() {
				GetLogMessagesOperation msgOp = ModelBusHistoryPage.this.createRemoteHistoryFetcher();
				msgOp.setLimit(ModelBusHistoryPage.this.limit + 1);
				if (ModelBusHistoryPage.this.logMessages != null) {
					ModelBusLogEntry lm = ModelBusHistoryPage.this.logMessages[ModelBusHistoryPage.this.logMessages.length - 1];
					msgOp.setStartRevision(ModelBusRevision.fromString(lm.revision));
				}
				ModelBusHistoryPage.this.fetchRemoteHistory(msgOp);
			}
		};
		String msg = this.limit > 0 ? ModelBusTeamUIPlugin.instance().getResource("HistoryView.ShowNextX",
				new String[] { String.valueOf(this.limit) }) : ModelBusTeamUIPlugin.instance().getResource("HistoryView.ShowNextPage");
		this.getNextPageAction.setToolTipText(msg);
		this.getAllPagesAction = new HistoryAction("HistoryView.ShowAll", "icons/views/history/paging_all.gif") {
			public void run() {
				GetLogMessagesOperation msgOp = ModelBusHistoryPage.this.createRemoteHistoryFetcher();
				msgOp.setLimit(0);
				if (ModelBusHistoryPage.this.logMessages != null) {
					ModelBusLogEntry lm = ModelBusHistoryPage.this.logMessages[ModelBusHistoryPage.this.logMessages.length - 1];
					msgOp.setStartRevision(ModelBusRevision.fromString(lm.revision));
				}
				ModelBusHistoryPage.this.fetchRemoteHistory(msgOp);
			}
		};

		this.collapseAllAction = new HistoryAction("RepositoriesView.CollapseAll.Label", "icons/common/collapseall.gif") {
			public void run() {
				ModelBusHistoryPage.this.history.collapseAll();
			}
		};
		this.compareModeAction = new HistoryAction("HistoryView.CompareMode", "icons/views/history/compare_mode.gif", IAction.AS_CHECK_BOX) {
			public void run() {
				ModelBusHistoryPage.this.options ^= IModelBusHistoryView.COMPARE_MODE;
				ModelBusHistoryPage.this.compareModeDropDownAction
						.setChecked((ModelBusHistoryPage.this.options & IModelBusHistoryView.COMPARE_MODE) != 0);
				ModelBusHistoryPage.saveBoolean(ModelBusTeamPreferences.HISTORY_COMPARE_MODE,
						(ModelBusHistoryPage.this.options & IModelBusHistoryView.COMPARE_MODE) != 0);
			}
		};

		IToolBarManager tbm = actionBars.getToolBarManager();
		tbm.add(new Separator());
		tbm.add(this.groupByDateAction);
		tbm.add(new Separator());
		tbm.add(this.showBothAction);
		tbm.add(this.showLocalAction);
		tbm.add(this.showRemoteAction);
		tbm.add(new Separator());
		tbm.add(this.hideUnrelatedAction);
		tbm.add(this.stopOnCopyAction);
		tbm.add(new Separator());
		tbm.add(this.getNextPageAction);
		tbm.add(this.getAllPagesAction);
		tbm.add(new Separator());
		tbm.add(this.collapseAllAction);
		tbm.add(this.compareModeAction);

		// drop-down menu
		this.showCommentViewerAction = new HistoryAction("HistoryView.ShowCommentViewer") {
			public void run() {
				ModelBusHistoryPage.this.history.setCommentViewerVisible(ModelBusHistoryPage.this.showCommentViewerAction.isChecked());
				ModelBusHistoryPage.saveBoolean(ModelBusTeamPreferences.HISTORY_SHOW_MULTILINE_COMMENT_NAME, this.isChecked());
			}
		};
		this.showAffectedPathsViewerAction = new HistoryAction("HistoryView.ShowAffectedPathsViewer") {
			public void run() {
				boolean showAffected = this.isChecked();
				ModelBusHistoryPage.this.history.setAffectedPathsViewerVisible(showAffected);
				ModelBusHistoryPage.this.flatAction.setEnabled(showAffected);
				ModelBusHistoryPage.this.hierarchicalAction.setEnabled(showAffected);
				ModelBusHistoryPage.saveBoolean(ModelBusTeamPreferences.HISTORY_SHOW_AFFECTED_PATHS_NAME, showAffected);
			}
		};

		this.hideUnrelatedDropDownAction = new HistoryAction("HistoryView.HideUnrelatedPaths", "icons/views/history/hide_unrelated.gif") {
			public void run() {
				ModelBusHistoryPage.this.options ^= IModelBusHistoryView.HIDE_UNRELATED;
				ModelBusHistoryPage.this.hideUnrelatedAction
						.setChecked((ModelBusHistoryPage.this.options & IModelBusHistoryView.HIDE_UNRELATED) != 0);
				ModelBusHistoryPage.this.history.refresh(LogMessagesComposite.REFRESH_UI_AFFECTED);
			}
		};
		this.stopOnCopyDropDownAction = new HistoryAction("HistoryView.StopOnCopy", "icons/views/history/stop_on_copy.gif") {
			public void run() {
				ModelBusHistoryPage.this.options ^= IModelBusHistoryView.STOP_ON_COPY;
				ModelBusHistoryPage.this.stopOnCopyAction.setChecked((ModelBusHistoryPage.this.options & IModelBusHistoryView.STOP_ON_COPY) != 0);
				ModelBusHistoryPage.this.refresh(IModelBusHistoryView.REFRESH_REMOTE);
			}
		};

		this.groupByDateDropDownAction = new HistoryAction("HistoryView.GroupByDate", "icons/views/history/group_by_date.gif") {
			public void run() {
				ModelBusHistoryPage.this.options ^= IModelBusHistoryView.GROUP_BY_DATE;
				ModelBusHistoryPage.this.groupByDateAction.setChecked((ModelBusHistoryPage.this.options & IModelBusHistoryView.GROUP_BY_DATE) != 0);
				ModelBusHistoryPage
						.saveInt(
								ModelBusTeamPreferences.HISTORY_GROUPING_TYPE_NAME,
								(ModelBusHistoryPage.this.options & IModelBusHistoryView.GROUP_BY_DATE) == 0 ? ModelBusTeamPreferences.HISTORY_GROUPING_TYPE_NONE
										: ModelBusTeamPreferences.HISTORY_GROUPING_TYPE_DATE);
				ModelBusHistoryPage.this.history.refresh(LogMessagesComposite.REFRESH_UI_ALL);
			}
		};

		this.exportLogAction = new HistoryAction("HistoryView.ExportLog") {
			public void run() {
				FileDialog dlg = new FileDialog(UIMonitorUtility.getShell(), SWT.PRIMARY_MODAL | SWT.SAVE);
				dlg.setText(ModelBusTeamUIPlugin.instance().getResource("ExportLogDialog.Title"));
				String caption = ModelBusHistoryPage.this.getName();
				dlg.setFileName(caption.substring(caption.lastIndexOf('/') + 1) + "_history.log");
				dlg.setFilterExtensions(new String[] { "log", "*.*" });
				String file = dlg.open();
				if (file != null) {
					ILogNode input = (ILogNode) ModelBusHistoryPage.this.history.getTreeViewer().getInput();
					try {
						FileOutputStream stream = new FileOutputStream(file);
						stream.write(ModelBusHistoryPage.this.actionManager.getSelectedMessagesAsString(new ILogNode[] { input }).getBytes());
						stream.flush();
						stream.close();
					} catch (Exception ex) {
						LoggedOperation.reportError(ModelBusTeamUIPlugin.instance().getResource("Operation.ExportLog"), ex);
					}
				}
			}
		};

		this.showBothActionDropDown = new HistoryAction("HistoryView.ShowBoth", "icons/views/history/both_history_mode.gif", IAction.AS_RADIO_BUTTON) {
			public void run() {
				ModelBusHistoryPage.this.options = ModelBusHistoryPage.this.options
						& ~(IModelBusHistoryViewInfo.MODE_LOCAL | IModelBusHistoryViewInfo.MODE_REMOTE) | IModelBusHistoryViewInfo.MODE_BOTH;
				ModelBusHistoryPage.this.showBothAction.setChecked(true);
				ModelBusHistoryPage.this.showLocalAction.setChecked(false);
				ModelBusHistoryPage.this.showRemoteAction.setChecked(false);
				ModelBusHistoryPage.this.saveShowMode();
				ModelBusHistoryPage.this.history.refresh(LogMessagesComposite.REFRESH_UI_ALL);
			}
		};
		this.showLocalActionDropDown = new HistoryAction("HistoryView.ShowLocal", "icons/views/history/local_history_mode.gif",
				IAction.AS_RADIO_BUTTON) {
			public void run() {
				ModelBusHistoryPage.this.options = ModelBusHistoryPage.this.options
						& ~(IModelBusHistoryViewInfo.MODE_REMOTE | IModelBusHistoryViewInfo.MODE_BOTH) | IModelBusHistoryViewInfo.MODE_LOCAL;
				ModelBusHistoryPage.this.showBothAction.setChecked(false);
				ModelBusHistoryPage.this.showLocalAction.setChecked(true);
				ModelBusHistoryPage.this.showRemoteAction.setChecked(false);
				ModelBusHistoryPage.this.saveShowMode();
				ModelBusHistoryPage.this.history.refresh(LogMessagesComposite.REFRESH_UI_ALL);
			}
		};
		this.showRemoteActionDropDown = new HistoryAction("HistoryView.ShowRemote", "icons/views/history/remote_history_mode.gif",
				IAction.AS_RADIO_BUTTON) {
			public void run() {
				ModelBusHistoryPage.this.options = ModelBusHistoryPage.this.options
						& ~(IModelBusHistoryViewInfo.MODE_LOCAL | IModelBusHistoryViewInfo.MODE_BOTH) | IModelBusHistoryViewInfo.MODE_REMOTE;
				ModelBusHistoryPage.this.showBothAction.setChecked(false);
				ModelBusHistoryPage.this.showLocalAction.setChecked(false);
				ModelBusHistoryPage.this.showRemoteAction.setChecked(true);
				ModelBusHistoryPage.this.saveShowMode();
				ModelBusHistoryPage.this.history.refresh(LogMessagesComposite.REFRESH_UI_ALL);
			}
		};

		this.revisionsRangeDropDownAction = new HistoryAction("HistoryView.RevisionsRange") {
			public void run() {
				ModelBusHistoryPage.this.defineRevisionsRange();
			}
		};

		this.filterDropDownAction = new HistoryAction("HistoryView.QuickFilter", "icons/views/history/filter.gif") {
			public void run() {
				ModelBusHistoryPage.this.setFilter();
			}
		};
		this.clearFilterDropDownAction = new HistoryAction("HistoryView.ClearFilter", "icons/views/history/clear_filter.gif") {
			public void run() {
				ModelBusHistoryPage.this.clearFilter();
			}
		};

		this.compareModeDropDownAction = new HistoryAction("HistoryView.CompareMode", "icons/views/history/compare_mode.gif") {
			public void run() {
				ModelBusHistoryPage.this.options ^= IModelBusHistoryView.COMPARE_MODE;
				ModelBusHistoryPage.this.compareModeAction.setChecked((ModelBusHistoryPage.this.options & IModelBusHistoryView.COMPARE_MODE) != 0);
				ModelBusHistoryPage.saveBoolean(ModelBusTeamPreferences.HISTORY_COMPARE_MODE,
						(ModelBusHistoryPage.this.options & IModelBusHistoryView.COMPARE_MODE) != 0);
			}
		};

		this.flatAction = new HistoryAction("HistoryView.Flat", "icons/views/history/flat_layout.gif", IAction.AS_RADIO_BUTTON) {
			public void run() {
				ModelBusHistoryPage.this.history.setResourceTreeVisible(false);
				ModelBusHistoryPage.saveBoolean(ModelBusTeamPreferences.HISTORY_HIERARCHICAL_LAYOUT, false);
			}
		};
		this.hierarchicalAction = new HistoryAction("HistoryView.Hierarchical", "icons/views/history/tree_layout.gif", IAction.AS_RADIO_BUTTON) {
			public void run() {
				ModelBusHistoryPage.this.history.setResourceTreeVisible(true);
				ModelBusHistoryPage.saveBoolean(ModelBusTeamPreferences.HISTORY_HIERARCHICAL_LAYOUT, true);
			}
		};

		IMenuManager actionBarsMenu = actionBars.getMenuManager();
		actionBarsMenu.add(this.showCommentViewerAction);
		actionBarsMenu.add(this.showAffectedPathsViewerAction);
		MenuManager sub = new MenuManager(ModelBusTeamUIPlugin.instance().getResource("HistoryView.AffectedPathLayout"),
				IWorkbenchActionConstants.GROUP_MANAGING);
		sub.add(this.flatAction);
		sub.add(this.hierarchicalAction);
		actionBarsMenu.add(sub);
		actionBarsMenu.add(new Separator());
		actionBarsMenu.add(this.groupByDateDropDownAction);
		actionBarsMenu.add(new Separator());
		actionBarsMenu.add(this.showBothActionDropDown);
		actionBarsMenu.add(this.showLocalActionDropDown);
		actionBarsMenu.add(this.showRemoteActionDropDown);
		actionBarsMenu.add(new Separator());
		actionBarsMenu.add(this.hideUnrelatedDropDownAction);
		actionBarsMenu.add(this.stopOnCopyDropDownAction);
		actionBarsMenu.add(new Separator());
		actionBarsMenu.add(this.revisionsRangeDropDownAction);
		actionBarsMenu.add(this.filterDropDownAction);
		actionBarsMenu.add(this.clearFilterDropDownAction);
		actionBarsMenu.add(this.exportLogAction);
		actionBarsMenu.add(new Separator());
		actionBarsMenu.add(this.compareModeDropDownAction);

		this.history = new LogMessagesComposite(parent, true, false, this);

		GridData data = new GridData(GridData.FILL_BOTH);
		this.history.setLayoutData(data);

		this.history.registerActionManager(this.actionManager, this.getPartSite());

		// Setting context help
		PlatformUI.getWorkbench().getHelpSystem().setHelp(parent, "org.modelbus.team.eclipse.help.historyViewContext");

		this.refreshOptionButtons();
	}

	public static String[] getSelectedAuthors(ModelBusLogEntry[] logMessages) {
		HashSet<String> authors = new HashSet<String>();
		if (logMessages != null) {
			for (ModelBusLogEntry entry : logMessages) {
				if (entry.author != null) {
					authors.add(entry.author);
				}
			}
		}
		return authors.toArray(new String[authors.size()]);
	}

	public static ModelBusLogEntry[] filterMessages(ModelBusLogEntry[] msgs, ILogEntryFilter filter) {
		if (msgs == null) {
			return null;
		}
		ArrayList<ModelBusLogEntry> filteredMessages = new ArrayList<ModelBusLogEntry>();
		for (int i = 0; i < msgs.length; i++) {
			if (filter.accept(msgs[i])) {
				filteredMessages.add(msgs[i]);
			}
		}
		if (filteredMessages.size() == 0) {
			return null;
		}
		return filteredMessages.toArray(new ModelBusLogEntry[filteredMessages.size()]);
	}

	public static boolean isValidData(Object object) {
		return object instanceof IRepositoryResource || object instanceof RepositoryResource ||
		// object instanceof RepositoryLocation ||
				object instanceof IResource && FileUtility.isConnected((IResource) object);
	}

	protected IWorkbenchPartSite getPartSite() {
		IWorkbenchPart part = this.getHistoryPageSite().getPart();
		if (part == null) {
			return null;
		}
		IWorkbenchPartSite site = part.getSite();
		while (site == null) {
			try {
				// await while site is initialized, see IWorkbenchPart.getSite() documentation
				Thread.sleep(100);
			} catch (InterruptedException ex) {
				break;
			}
			site = part.getSite();
		}
		return site;
	}

	protected GetLogMessagesOperation createRemoteHistoryFetcher() {
		GetLogMessagesOperation msgOp = new GetLogMessagesOperation(this.repositoryResource, this.stopOnCopyAction.isChecked());
		if (this.endRevision != null) {
			msgOp.setEndRevision(this.endRevision);
		}
		// set revision range here
		return msgOp;
	}

	protected void fetchRemoteHistory(final GetLogMessagesOperation msgsOp) {
		msgsOp.setIncludeMerged(ModelBusTeamPreferences.getMergeBoolean(ModelBusTeamUIPlugin.instance().getPreferenceStore(),
				ModelBusTeamPreferences.MERGE_INCLUDE_MERGED_NAME));

		final IStructuredSelection selected = (IStructuredSelection) this.history.getTreeViewer().getSelection();
		IActionOperation showOp = new AbstractActionOperation("Operation.HShowHistory") {
			private ModelBusRevision	revision = ModelBusHistoryPage.this.currentRevision;

			protected void runImpl(IProgressMonitor monitor) throws Exception {
				IRepositoryResource resource = ModelBusHistoryPage.this.repositoryResource;
				if (msgsOp.getExecutionState() != IActionOperation.OK || resource == null) {
					ModelBusHistoryPage.this.pending = false;
					UIMonitorUtility.getDisplay().syncExec(new Runnable() {
						public void run() {
							ModelBusHistoryPage.this.history.refresh(LogMessagesComposite.REFRESH_ALL);
						}
					});
					return;
				}
				if (ModelBusHistoryPage.this.wcResource == null) {
					this.revision = resource.getRevision();
				}

				if (!resource.equals(msgsOp.getResource())) {
					return;
				}

				ModelBusHistoryPage.this.currentRevision = this.revision;
				ModelBusHistoryPage.this.addPage(msgsOp.getMessages());

				UIMonitorUtility.getDisplay().syncExec(new Runnable() {
					public void run() {
						ModelBusHistoryPage.this.history.refresh(LogMessagesComposite.REFRESH_ALL);
						ModelBusHistoryPage.this.setButtonsEnablement();

						TreeViewer treeTable = ModelBusHistoryPage.this.history.getTreeViewer();
						if (!treeTable.getTree().isDisposed() && treeTable.getTree().getItems().length > 0) {
							if (selected.size() != 0) {
								treeTable.setSelection(selected, true);
							} else {
								TreeItem firstItem = treeTable.getTree().getItem(0);
								if (((ILogNode) firstItem.getData()).getType() == ILogNode.TYPE_CATEGORY) {
									firstItem = firstItem.getItem(0);
								}
								treeTable.getTree().setSelection(firstItem);
							}
							ModelBusHistoryPage.this.history.refresh(LogMessagesComposite.REFRESH_UI_AFFECTED);
						}
					}
				});
			}
		};
		CompositeOperation op = new CompositeOperation(showOp.getId(), true);
		op.add(new CorrectRevisionOperation(msgsOp, this.repositoryResource, this.currentRevision, this.wcResource));
		op.add(msgsOp);
		op.add(showOp);

		ProgressMonitorUtility.doTaskScheduled(op, false);
	}

	protected void fetchLocalHistory(ILocalResource local, IProgressMonitor monitor) throws CoreException {
		IFile file = (IFile) this.wcResource;
		ArrayList<ModelBusLocalFileRevision> history = new ArrayList<ModelBusLocalFileRevision>();
		IFileState[] states = file.getHistory(monitor);
		if (states.length > 0 || IStateFilter.SF_NOTONREPOSITORY.accept(local)) {
			history.add(new ModelBusLocalFileRevision(file));
		}
		for (IFileState state : states) {
			history.add(new ModelBusLocalFileRevision(state));
		}
		this.localHistory = history.size() == 0 ? null : history.toArray(new ModelBusLocalFileRevision[history.size()]);
	}

	protected void addPage(ModelBusLogEntry[] newMessages) {
		if (this.logMessages == null) {
			if (newMessages.length > 0) {
				this.pending = false;
				this.logMessages = newMessages;
				this.pagingEnabled = this.limit > 0 && newMessages.length == this.limit;
			}
		} else if (newMessages.length > 1) {
			LinkedHashSet<ModelBusLogEntry> entries = new LinkedHashSet<ModelBusLogEntry>(Arrays.asList(this.logMessages));
			int oldSize = entries.size();
			entries.addAll(Arrays.asList(newMessages));
			this.logMessages = entries.toArray(new ModelBusLogEntry[entries.size()]);
			this.pagingEnabled = this.limit > 0 && (newMessages.length == this.limit + 1 || entries.size() - oldSize < newMessages.length - 1);
		}
	}

	protected void refreshOptionButtons() {
		IPreferenceStore store = ModelBusTeamUIPlugin.instance().getPreferenceStore();

		boolean showMultiline = ModelBusTeamPreferences.getHistoryBoolean(store, ModelBusTeamPreferences.HISTORY_SHOW_MULTILINE_COMMENT_NAME);
		this.showCommentViewerAction.setChecked(showMultiline);
		this.showCommentViewerAction.run();

		boolean showAffected = ModelBusTeamPreferences.getHistoryBoolean(store, ModelBusTeamPreferences.HISTORY_SHOW_AFFECTED_PATHS_NAME);
		this.showAffectedPathsViewerAction.setChecked(showAffected);
		this.showAffectedPathsViewerAction.run();

		boolean hierarchicalAffectedView = ModelBusTeamPreferences.getHistoryBoolean(store, ModelBusTeamPreferences.HISTORY_HIERARCHICAL_LAYOUT);
		int groupingType = ModelBusTeamPreferences.getHistoryInt(store, ModelBusTeamPreferences.HISTORY_GROUPING_TYPE_NAME);
		int revisionMode = ModelBusTeamPreferences.getHistoryInt(store, ModelBusTeamPreferences.HISTORY_REVISION_MODE_NAME);
		if (revisionMode == 0) {
			revisionMode = IModelBusHistoryViewInfo.MODE_BOTH;
		} else if (revisionMode == 1) {
			revisionMode = IModelBusHistoryViewInfo.MODE_REMOTE;
		} else {
			revisionMode = IModelBusHistoryViewInfo.MODE_LOCAL;
		}

		boolean compareMode = ModelBusTeamPreferences.getHistoryBoolean(store, ModelBusTeamPreferences.HISTORY_COMPARE_MODE);
		if (compareMode) {
			this.options |= IModelBusHistoryView.COMPARE_MODE;
		}
		this.options |= groupingType == ModelBusTeamPreferences.HISTORY_GROUPING_TYPE_DATE ? IModelBusHistoryView.GROUP_BY_DATE : 0;
		this.options = this.options
				& ~(IModelBusHistoryViewInfo.MODE_BOTH | IModelBusHistoryViewInfo.MODE_LOCAL | IModelBusHistoryViewInfo.MODE_REMOTE) | revisionMode;

		this.hideUnrelatedDropDownAction.setChecked((this.options & IModelBusHistoryView.HIDE_UNRELATED) != 0);
		this.hideUnrelatedAction.setChecked((this.options & IModelBusHistoryView.HIDE_UNRELATED) != 0);
		this.stopOnCopyDropDownAction.setChecked((this.options & IModelBusHistoryView.STOP_ON_COPY) != 0);
		this.stopOnCopyAction.setChecked((this.options & IModelBusHistoryView.STOP_ON_COPY) != 0);
		this.groupByDateAction.setChecked((this.options & IModelBusHistoryView.GROUP_BY_DATE) != 0);
		this.showBothAction.setChecked((this.options & IModelBusHistoryViewInfo.MODE_BOTH) != 0);
		this.showBothActionDropDown.setChecked((this.options & IModelBusHistoryViewInfo.MODE_BOTH) != 0);
		this.showLocalAction.setChecked((this.options & IModelBusHistoryViewInfo.MODE_LOCAL) != 0);
		this.showLocalActionDropDown.setChecked((this.options & IModelBusHistoryViewInfo.MODE_LOCAL) != 0);
		this.showRemoteAction.setChecked((this.options & IModelBusHistoryViewInfo.MODE_REMOTE) != 0);
		this.showRemoteActionDropDown.setChecked((this.options & IModelBusHistoryViewInfo.MODE_REMOTE) != 0);
		this.groupByDateDropDownAction.setChecked((this.options & IModelBusHistoryView.GROUP_BY_DATE) != 0);
		this.compareModeDropDownAction.setChecked((this.options & IModelBusHistoryView.COMPARE_MODE) != 0);
		this.compareModeAction.setChecked((this.options & IModelBusHistoryView.COMPARE_MODE) != 0);

		this.flatAction.setChecked(!hierarchicalAffectedView);
		this.hierarchicalAction.setChecked(hierarchicalAffectedView);
		this.history.setResourceTreeVisible(hierarchicalAffectedView);

		this.refreshLimitOption();
		this.setButtonsEnablement();
	}

	protected void defineRevisionsRange() {
		HistoryRangePanel panel = new HistoryRangePanel(this.repositoryResource, this.startRevision, this.endRevision);
		DefaultDialog dialog = new DefaultDialog(this.getPartSite().getShell(), panel);
		if (dialog.open() == 0) {
			this.startRevision = panel.getStartRevision();
			this.endRevision = panel.getStopRevision();
			this.refresh(IModelBusHistoryView.REFRESH_REMOTE);
		}
	}

	protected void refreshLimitOption() {
		IPreferenceStore store = ModelBusTeamUIPlugin.instance().getPreferenceStore();

		if (ModelBusTeamPreferences.getHistoryBoolean(store, ModelBusTeamPreferences.HISTORY_PAGING_ENABLE_NAME)) {
			this.limit = ModelBusTeamPreferences.getHistoryInt(store, ModelBusTeamPreferences.HISTORY_PAGE_SIZE_NAME);
			this.getNextPageAction.setToolTipText("Show Next " + this.limit);
			this.options |= IModelBusHistoryView.PAGING_ENABLED;
		} else {
			this.limit = 0;
			this.getNextPageAction.setToolTipText("Show Next Page");
			this.options &= ~IModelBusHistoryView.PAGING_ENABLED;
		}
	}

	protected void setButtonsEnablement() {
		ILocalResource local = ModelBusRemoteStorage.instance().asLocalResource(this.wcResource);
		boolean enableRepo = (IStateFilter.SF_ONREPOSITORY.accept(local) || this.repositoryResource != null) && !this.pending;

		this.revisionsRangeDropDownAction.setEnabled(enableRepo && this.repositoryResource != null);
		this.filterDropDownAction.setEnabled(enableRepo && this.repositoryResource != null && this.logMessages != null);
		this.clearFilterDropDownAction.setEnabled(this.isFilterEnabled());
		this.getNextPageAction.setEnabled(enableRepo && this.pagingEnabled & ((this.options & IModelBusHistoryView.PAGING_ENABLED) != 0));
		this.getAllPagesAction.setEnabled(enableRepo && this.pagingEnabled & ((this.options & IModelBusHistoryView.PAGING_ENABLED) != 0));

		this.stopOnCopyAction.setEnabled(enableRepo);
		this.stopOnCopyDropDownAction.setEnabled(enableRepo);
		this.hideUnrelatedAction.setEnabled(enableRepo);
		this.hideUnrelatedDropDownAction.setEnabled(enableRepo);
	}

	protected void saveShowMode() {
		int prefToSet = ModelBusTeamPreferences.HISTORY_REVISION_MODE_REMOTE;
		if ((this.options & IModelBusHistoryViewInfo.MODE_BOTH) != 0) {
			prefToSet = ModelBusTeamPreferences.HISTORY_REVISION_MODE_BOTH;
		} else if ((this.options & IModelBusHistoryViewInfo.MODE_LOCAL) != 0) {
			prefToSet = ModelBusTeamPreferences.HISTORY_REVISION_MODE_LOCAL;
		}
		ModelBusHistoryPage.saveInt(ModelBusTeamPreferences.HISTORY_REVISION_MODE_NAME, prefToSet);
	}

	protected static void saveBoolean(String name, boolean value) {
		ModelBusTeamPreferences.setHistoryBoolean(ModelBusTeamUIPlugin.instance().getPreferenceStore(), name, value);
		ModelBusTeamUIPlugin.instance().savePluginPreferences();
	}

	protected static void saveInt(String name, int value) {
		ModelBusTeamPreferences.setHistoryInt(ModelBusTeamUIPlugin.instance().getPreferenceStore(), name, value);
		ModelBusTeamUIPlugin.instance().savePluginPreferences();
	}

	protected void disconnectView() {
		UIMonitorUtility.getDisplay().syncExec(new Runnable() {
			public void run() {
				ModelBusHistoryPage.this.clear();
			}
		});
	}

	protected void refreshChanges(final int refreshType) {
		UIMonitorUtility.getDisplay().syncExec(new Runnable() {
			public void run() {
				ModelBusHistoryPage.this.refresh(refreshType);
			}
		});
	}

}
