/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.operation;

import java.text.MessageFormat;
import java.util.ArrayList;

import org.eclipse.compare.CompareConfiguration;
import org.eclipse.compare.internal.CompareEditor;
import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.ModelBusTeamPlugin;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Depth;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusDiffStatus;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryRevisionReference;
import org.modelbus.team.eclipse.core.operation.AbstractActionOperation;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.operation.remote.LocateResourceURLInHistoryOperation;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryResourceProvider;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.core.utility.ProgressMonitorUtility;
import org.modelbus.team.eclipse.ui.compare.ResourceCompareInput;
import org.modelbus.team.eclipse.ui.compare.TwoWayResourceCompareInput;
import org.modelbus.team.eclipse.ui.utility.UIMonitorUtility;

/**
 * Two way compare for repository resources operation implementation
 * 
 * @author Alexander Gurov
 */
public class CompareRepositoryResourcesOperation extends AbstractActionOperation {
	protected IRepositoryResource			next;
	protected IRepositoryResource			prev;
	protected IRepositoryResourceProvider	provider;
	protected boolean						forceReuse;
	protected String						forceId;

	public CompareRepositoryResourcesOperation(IRepositoryResource prev, IRepositoryResource next) {
		this(prev, next, false);
	}

	public CompareRepositoryResourcesOperation(IRepositoryResource prev, IRepositoryResource next, boolean forceReuse) {
		super("Operation.CompareRepository");
		this.prev = prev;
		this.next = next;
		this.forceReuse = forceReuse;
	}

	public CompareRepositoryResourcesOperation(IRepositoryResourceProvider provider) {
		this(null, null, false);
		this.provider = provider;
	}

	public CompareRepositoryResourcesOperation(IRepositoryResourceProvider provider, boolean forceReuse) {
		this(null, null, forceReuse);
		this.provider = provider;
	}

	public void setForceId(String forceId) {
		this.forceId = forceId;
	}

	public String getForceId() {
		return this.forceId;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		if (this.provider != null) {
			IRepositoryResource[] toCompare = this.provider.getRepositoryResources();
			this.prev = toCompare[0];
			this.next = toCompare[1];
		}

		final LocateResourceURLInHistoryOperation op = new LocateResourceURLInHistoryOperation(new IRepositoryResource[] { this.next, this.prev });
		this.protectStep(new IUnprotectedOperation() {
			public void run(IProgressMonitor monitor) throws Exception {
				ProgressMonitorUtility.doTaskExternal(op, monitor);
			}
		}, monitor, 100, 60);
		this.next = op.getRepositoryResources()[0];
		this.prev = op.getRepositoryResources()[1];

		final IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		final ArrayList<ModelBusDiffStatus> statuses = new ArrayList<ModelBusDiffStatus>();

		ProgressMonitorUtility.setTaskInfo(monitor, this, ModelBusTeamPlugin.instance().getResource("Progress.Running"));
		this.protectStep(new IUnprotectedOperation() {
			public void run(IProgressMonitor monitor) throws Exception {
				ModelBusEntryRevisionReference refPrev = ModelBusUtility.getEntryRevisionReference(CompareRepositoryResourcesOperation.this.prev);
				ModelBusEntryRevisionReference refNext = ModelBusUtility.getEntryRevisionReference(CompareRepositoryResourcesOperation.this.next);
				if (ModelBusUtility.useSingleReferenceSignature(refPrev, refNext)) {
					ModelBusUtility.diffStatus(proxy, statuses, refPrev, refPrev.revision, refNext.revision, Depth.INFINITY,
							IModelBusConnector.Options.NONE, new ModelBusProgressMonitor(CompareRepositoryResourcesOperation.this, monitor, null,
									false));
				} else {
					ModelBusUtility.diffStatus(proxy, statuses, refPrev, refNext, Depth.INFINITY, IModelBusConnector.Options.NONE,
							new ModelBusProgressMonitor(CompareRepositoryResourcesOperation.this, monitor, null, false));
				}
			}
		}, monitor, 100, 20);

		if (!monitor.isCanceled()) {
			this.protectStep(new IUnprotectedOperation() {
				public void run(IProgressMonitor monitor) throws Exception {
					CompareConfiguration cc = new CompareConfiguration();
					cc.setProperty(CompareEditor.CONFIRM_SAVE_PROPERTY, Boolean.FALSE);
					final TwoWayResourceCompareInput compare = new TwoWayResourceCompareInput(cc, CompareRepositoryResourcesOperation.this.next,
							CompareRepositoryResourcesOperation.this.prev, statuses);
					compare.setForceId(CompareRepositoryResourcesOperation.this.forceId);
					compare.initialize(monitor);
					UIMonitorUtility.getDisplay().syncExec(new Runnable() {
						public void run() {
							ResourceCompareInput.openCompareEditor(compare, CompareRepositoryResourcesOperation.this.forceReuse);
						}
					});
				}
			}, monitor, 100, 20);
		}
	}

	protected String getShortErrorMessage(Throwable t) {
		return MessageFormat.format(super.getShortErrorMessage(t), new Object[] { this.next.getName(), this.prev.getName() });
	}

}
