/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.operation;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;

import org.eclipse.compare.CompareConfiguration;
import org.eclipse.compare.CompareEditorInput;
import org.eclipse.compare.internal.CompareEditor;
import org.eclipse.compare.internal.Utilities;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Shell;
import org.modelbus.team.eclipse.core.ModelBusTeamPlugin;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Depth;
import org.modelbus.team.eclipse.core.connector.IModelbusEntryStatusCallback;
import org.modelbus.team.eclipse.core.connector.ModelBusChangeStatus;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusDiffStatus;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryRevisionReference;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision.Kind;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRemoteStorage;
import org.modelbus.team.eclipse.core.operation.AbstractActionOperation;
import org.modelbus.team.eclipse.core.operation.IActionOperation;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.operation.remote.LocateResourceURLInHistoryOperation;
import org.modelbus.team.eclipse.core.resource.ILocalResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.core.utility.ProgressMonitorUtility;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.compare.ComparePanel;
import org.modelbus.team.eclipse.ui.compare.ResourceCompareInput;
import org.modelbus.team.eclipse.ui.compare.ThreeWayResourceCompareInput;
import org.modelbus.team.eclipse.ui.dialog.DefaultDialog;
import org.modelbus.team.eclipse.ui.utility.UIMonitorUtility;

/**
 * This operation calculate and show differences between WORKING and BASE revisions of a local resources
 * 
 * @author Alexander Gurov
 */
public class CompareResourcesOperation extends AbstractActionOperation {
	protected ILocalResource		local;
	protected IRepositoryResource	ancestor;
	protected IRepositoryResource	remote;
	protected boolean				showInDialog;
	protected boolean				forceReuse;
	protected String				forceId;

	public CompareResourcesOperation(ILocalResource local, IRepositoryResource remote) {
		this(local, remote, false, false);
	}

	public CompareResourcesOperation(ILocalResource local, IRepositoryResource remote, boolean forceReuse) {
		this(local, remote, forceReuse, false);
	}

	public CompareResourcesOperation(ILocalResource local, IRepositoryResource remote, boolean forceReuse, boolean showInDialog) {
		super("Operation.CompareLocal");
		this.local = local;
		this.ancestor = local.isCopied() ? ModelBusUtility.getCopiedFrom(local.getResource()) : ModelBusRemoteStorage.instance()
				.asRepositoryResource(local.getResource());
		this.ancestor.setSelectedRevision(local.getBaseRevision());
		this.remote = remote;
		this.showInDialog = showInDialog;
		this.forceReuse = forceReuse;
	}

	public void setForceId(String forceId) {
		this.forceId = forceId;
	}

	public String getForceId() {
		return this.forceId;
	}

	protected void runImpl(final IProgressMonitor monitor) throws Exception {
		final ArrayList<ModelBusDiffStatus> localChanges = new ArrayList<ModelBusDiffStatus>();
		final ArrayList<ModelBusDiffStatus> remoteChanges = new ArrayList<ModelBusDiffStatus>();

		final IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();

		final IRepositoryResource[] diffPair = new IRepositoryResource[] { this.ancestor, this.remote };
		ModelBusRevision revision = this.remote.getSelectedRevision();
		boolean fetchRemote = revision.getKind() == Kind.HEAD || revision.getKind() == Kind.STRING;

		this.protectStep(new IUnprotectedOperation() {
			public void run(IProgressMonitor monitor) throws Exception {
				proxy.status(FileUtility.getWorkingCopyPath(CompareResourcesOperation.this.local.getResource()), Depth.INFINITY,
						IModelBusConnector.Options.IGNORE_EXTERNALS, null, new IModelbusEntryStatusCallback() {
							public void next(ModelBusChangeStatus status) {
								localChanges.add(new ModelBusDiffStatus(status.path, status.path, status.nodeKind, status.textStatus,
										status.propStatus));
							}
						}, new ModelBusProgressMonitor(CompareResourcesOperation.this, monitor, null, false));
			}
		}, monitor, 100, fetchRemote ? 5 : 60);

		if (!monitor.isCanceled() && fetchRemote) {
			this.protectStep(new IUnprotectedOperation() {
				public void run(IProgressMonitor monitor) throws Exception {
					LocateResourceURLInHistoryOperation op = new LocateResourceURLInHistoryOperation(diffPair);
					ProgressMonitorUtility.doTaskExternal(op, monitor);
					if (op.getExecutionState() != IActionOperation.OK) {
						CompareResourcesOperation.this.reportStatus(op.getStatus());
						return;
					}
					diffPair[0] = op.getRepositoryResources()[0];
					diffPair[1] = op.getRepositoryResources()[1];
				}
			}, monitor, 100, 55);
			if (this.getExecutionState() == IActionOperation.ERROR) {
				return;
			}
			this.protectStep(new IUnprotectedOperation() {
				public void run(IProgressMonitor monitor) throws Exception {
					ProgressMonitorUtility.setTaskInfo(monitor, CompareResourcesOperation.this,
							ModelBusTeamPlugin.instance().getResource("Progress.Running"));

					ModelBusEntryRevisionReference refPrev = ModelBusUtility.getEntryRevisionReference(diffPair[0]);
					ModelBusEntryRevisionReference refNext = ModelBusUtility.getEntryRevisionReference(diffPair[1]);
					if (ModelBusUtility.useSingleReferenceSignature(refPrev, refNext)) {
						ModelBusUtility.diffStatus(proxy, remoteChanges, refPrev, refPrev.revision, refNext.revision, Depth.INFINITY,
								IModelBusConnector.Options.NONE, new ModelBusProgressMonitor(CompareResourcesOperation.this, monitor, null, false));
					} else {
						ModelBusUtility.diffStatus(proxy, remoteChanges, refPrev, refNext, Depth.INFINITY, IModelBusConnector.Options.NONE,
								new ModelBusProgressMonitor(CompareResourcesOperation.this, monitor, null, false));
					}
				}
			}, monitor, 100, 5);
		}

		if (!monitor.isCanceled()) {
			this.protectStep(new IUnprotectedOperation() {
				public void run(IProgressMonitor monitor) throws Exception {
					CompareConfiguration cc = new CompareConfiguration();
					cc.setProperty(CompareEditor.CONFIRM_SAVE_PROPERTY, Boolean.TRUE);
					diffPair[0].setSelectedRevision(ModelBusRevision.BASE);
					final ThreeWayResourceCompareInput compare = new ThreeWayResourceCompareInput(cc, CompareResourcesOperation.this.local,
							diffPair[0], diffPair[1], localChanges, remoteChanges);
					compare.setForceId(CompareResourcesOperation.this.forceId);
					compare.initialize(monitor);
					UIMonitorUtility.getDisplay().syncExec(new Runnable() {
						public void run() {
							if (CompareResourcesOperation.this.showInDialog) {
								if (CompareResourcesOperation.this.compareResultOK(compare)) {
									ComparePanel panel = new ComparePanel(compare, CompareResourcesOperation.this.local.getResource());
									DefaultDialog dialog = new DefaultDialog(UIMonitorUtility.getShell(), panel);
									dialog.open();
								}
							} else {
								ResourceCompareInput.openCompareEditor(compare, CompareResourcesOperation.this.forceReuse);
							}
						}
					});
				}
			}, monitor, 100, 40);
		}
	}

	protected boolean compareResultOK(CompareEditorInput input) {
		final Shell shell = UIMonitorUtility.getShell();

		try {
			ModelBusTeamUIPlugin.instance().getWorkbench().getProgressService().run(true, true, input);

			String message = input.getMessage();
			if (message != null) {
				MessageDialog.openError(shell, Utilities.getString("CompareUIPlugin.compareFailed"), message);
			} else if (input.getCompareResult() == null) {
				MessageDialog.openInformation(shell, Utilities.getString("CompareUIPlugin.dialogTitle"),
						Utilities.getString("CompareUIPlugin.noDifferences"));
			} else {
				return true;
			}
		} catch (InterruptedException x) {
			// cancelled by user
		} catch (InvocationTargetException x) {
			MessageDialog.openError(shell, Utilities.getString("CompareUIPlugin.compareFailed"), x.getTargetException().getMessage());
		}
		return false;
	}

}
