/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.operation;

import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.modelbus.team.eclipse.core.ModelBusTeamPlugin;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryRevisionReference;
import org.modelbus.team.eclipse.core.connector.ModelBusProperty;
import org.modelbus.team.eclipse.core.connector.ModelBusProperty.BuiltIn;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRemoteStorage;
import org.modelbus.team.eclipse.core.operation.AbstractActionOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.operation.UnreportableException;
import org.modelbus.team.eclipse.core.resource.IRepositoryContainer;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource.Information;
import org.modelbus.team.eclipse.core.resource.IRepositoryRoot;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.preferences.ModelBusTeamPreferences;

/**
 * Load folder children. Used in asynchronous repository view refresh.
 * 
 * @author Alexander Gurov
 */
public class GetRemoteFolderChildrenOperation extends AbstractActionOperation {
	protected IRepositoryContainer				parent;
	protected IRepositoryResource[]				children;
	protected boolean							sortChildren;
	protected Map<IRepositoryResource, String>	externalsNames;

	public GetRemoteFolderChildrenOperation(IRepositoryContainer parent) {
		this(parent, true);
	}

	public GetRemoteFolderChildrenOperation(IRepositoryContainer parent, boolean sortChildren) {
		super("Operation.GetRemoteChildren");
		this.parent = parent;
		this.sortChildren = sortChildren;
		this.externalsNames = new HashMap<IRepositoryResource, String>();
	}

	public IRepositoryResource[] getChildren() {
		return this.children;
	}

	public String getExternalsName(IRepositoryResource resource) {
		return this.externalsNames.get(resource);
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		IRepositoryResource[] tmp = this.parent.getChildren();

		// handle modelbus:externals, if present:
		Information info = this.parent.getInfo();
		if (info != null
				&& info.hasProperties
				&& ModelBusTeamPreferences.getRepositoryBoolean(ModelBusTeamUIPlugin.instance().getPreferenceStore(),
						ModelBusTeamPreferences.REPOSITORY_SHOW_EXTERNALS_NAME)) {
			IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
			try {
				ModelBusProperty data = proxy.getProperty(ModelBusUtility.getEntryRevisionReference(this.parent), BuiltIn.EXTERNALS,
						new ModelBusProgressMonitor(this, monitor, null));
				if (data != null) {
					// Map externals;
					try {
						Map<String, ModelBusEntryRevisionReference> externals = ModelBusUtility.parseModelBusExternalsProperty(data.value,
								this.parent);
						IRepositoryResource[] newTmp = new IRepositoryResource[tmp.length + externals.size()];
						System.arraycopy(tmp, 0, newTmp, 0, tmp.length);
						int i = 0;
						for (Iterator<Map.Entry<String, ModelBusEntryRevisionReference>> it = externals.entrySet().iterator(); it.hasNext(); i++) {
							Map.Entry<String, ModelBusEntryRevisionReference> entry = it.next();
							String name = entry.getKey();
							ModelBusEntryRevisionReference ref = entry.getValue();
							newTmp[tmp.length + i] = ModelBusRemoteStorage.instance().asRepositoryResource(ref.path, false);
							newTmp[tmp.length + i].setSelectedRevision(ref.revision);
							newTmp[tmp.length + i].setPegRevision(ref.pegRevision);
							this.externalsNames.put(newTmp[tmp.length + i], name);
						}
						tmp = newTmp;
					} catch (UnreportableException ex) {
						this.reportStatus(new Status(IStatus.WARNING, ModelBusTeamPlugin.NATURE_ID, IStatus.OK, this.getShortErrorMessage(ex), ex));
					}
				}
			} finally {
			}
		}

		if (this.sortChildren) {
			Arrays.sort(tmp, new Comparator<IRepositoryResource>() {
				public int compare(IRepositoryResource first, IRepositoryResource second) {
					boolean firstContainer = first instanceof IRepositoryContainer;
					boolean secondContainer = second instanceof IRepositoryContainer;
					if (firstContainer && secondContainer) {
						boolean firstRoot = first instanceof IRepositoryRoot;
						boolean secondRoot = second instanceof IRepositoryRoot;
						return firstRoot == secondRoot ? (firstRoot ? this.compareRoots(((IRepositoryRoot) first).getKind(),
								((IRepositoryRoot) second).getKind()) : first.getUrl().compareTo(second.getUrl())) : (firstRoot ? -1 : 1);
					}
					return firstContainer == secondContainer ? first.getUrl().compareTo(second.getUrl()) : (firstContainer ? -1 : 1);
				}

				public int compareRoots(int firstKind, int secondKind) {
					return firstKind < secondKind ? -1 : 1;
				}
			});
		}
		this.children = tmp;
	}

}
