/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexei Goncharov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.properties;

import java.io.File;
import java.io.FileInputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.connector.ModelBusProperty;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRemoteStorage;
import org.modelbus.team.eclipse.core.modelbusstorage.events.RevisonPropertyChangeEvent;
import org.modelbus.team.eclipse.core.operation.AbstractActionOperation;
import org.modelbus.team.eclipse.core.operation.CompositeOperation;
import org.modelbus.team.eclipse.core.operation.IActionOperation;
import org.modelbus.team.eclipse.core.operation.local.property.IRevisionPropertiesProvider;
import org.modelbus.team.eclipse.core.operation.remote.SetRevisionPropertyOperation;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.extension.factory.PredefinedProperty;
import org.modelbus.team.eclipse.ui.preferences.ModelBusTeamPropsPreferencePage.CustomProperty;
import org.modelbus.team.eclipse.ui.utility.UIMonitorUtility;
import org.modelbus.team.eclipse.ui.verifier.PropertyVerifier;

/**
 * Panel for editing revision properties.
 * 
 * @author Alexei Goncharov
 */
public class RevPropertiesEditPanel extends AbstractPropertyEditPanel {

	protected ModelBusRevision	revision;

	/**
	 * Creates a panel.
	 * 
	 * @param revProperties
	 *            - existent revision properties
	 * @param revision
	 *            - the revision to edit properties for
	 */
	public RevPropertiesEditPanel(ModelBusProperty[] revProperties, ModelBusRevision revision) {
		super(revProperties, ModelBusTeamUIPlugin.instance().getResource("RevisionPropertyEditPanel.Title"), ModelBusTeamUIPlugin.instance()
				.getResource("RevisionPropertyEditPanel.Description", new String[] { String.valueOf(revision) }));
		this.revision = revision;
		ArrayList<CustomProperty> customPropList = new ArrayList<CustomProperty>(Arrays.asList(this.customProps));
		ArrayList<String> givenNames = new ArrayList<String>();
		for (CustomProperty current : this.customProps) {
			givenNames.add(current.propName);
		}
		for (PredefinedProperty current : this.predefinedProperties) {
			givenNames.add(current.name);
		}
		for (ModelBusProperty current : revProperties) {
			if (!givenNames.contains(current.name)) {
				customPropList.add(new CustomProperty(current.name, "", ""));
			}
		}
		this.customProps = customPropList.toArray(new CustomProperty[customPropList.size()]);
		this.fillVerifiersMap();
	}

	protected void saveChangesImpl() {
		super.saveChangesImpl();
	}

	protected void cancelChangesImpl() {
	}

	protected void fillVerifiersMap() {
		for (PredefinedProperty current : this.predefinedProperties) {
			this.verifiers.put(current.name, new PropertyVerifier("EditPropertiesInputField", current.name.equals("modelbus:autoversioned") ? null
					: "", current.name, null));
		}
	}

	protected List<PredefinedProperty> getPredefinedProperties() {
		ArrayList<PredefinedProperty> properties = new ArrayList<PredefinedProperty>();
		properties.add(new PredefinedProperty(ModelBusTeamUIPlugin.instance().getResource("AbstractPropertyEditPanel.modelbus_description"), "", ""));
		properties.add(new PredefinedProperty("modelbus:log", this.getDescription("ModelBus.Log"), ""));
		properties.add(new PredefinedProperty("modelbus:author", this.getDescription("ModelBus.Author"), ""));
		properties.add(new PredefinedProperty("modelbus:date", this.getDescription("ModelBus.Date"), ""));
		properties.add(new PredefinedProperty("modelbus:autoversioned", this.getDescription("ModelBus.Autoversioned"), ""));
		return properties;
	}

	protected String getDescription(String id) {
		return ModelBusTeamUIPlugin.instance().getResource("Property." + id);
	}

	protected Map<String, String> getPredefinedPropertiesRegexps() {
		return Collections.emptyMap();
	}

	public static void doSetRevisionProperty(RevPropertiesEditPanel panel, final ModelBusRevision revision) {
		final ModelBusProperty[] data = new ModelBusProperty[] { new ModelBusProperty(panel.getPropertyName(), panel.getPropertyValue()) };
		SetRevisionPropertyOperation setPropOp = null;
		CompositeOperation op = new CompositeOperation("");
		if (panel.isFileSelected()) {
			final File f = new File(panel.getPropertyFile());
			AbstractActionOperation loadOp = new AbstractActionOperation("Operation.SLoadFileContent") {
				protected void runImpl(IProgressMonitor monitor) throws Exception {
					FileInputStream input = null;
					try {
						input = new FileInputStream(f);
						byte[] binary = new byte[(int) f.length()];
						input.read(binary);
						data[0] = new ModelBusProperty(data[0].name, new String(binary));
					} finally {
						if (input != null) {
							input.close();
						}
					}
				}
			};
			op.add(loadOp);
			IRevisionPropertiesProvider provider = new IRevisionPropertiesProvider() {
				public ModelBusProperty[] getRevisionProperties() {
					return data;
				}
			};
			setPropOp = new SetRevisionPropertyOperation(revision, provider);
		} else {
			setPropOp = new SetRevisionPropertyOperation(revision, data[0]);
		}
		op.setOperationName(setPropOp.getOperationName());
		op.add(setPropOp);
		op.add(new AbstractActionOperation(setPropOp.getOperationName()) {
			protected void runImpl(IProgressMonitor monitor) throws Exception {
				ModelBusRemoteStorage.instance().fireRevisionPropertyChangeEvent(
						new RevisonPropertyChangeEvent(RevisonPropertyChangeEvent.SET, revision, data[0]));
			}
		}, new IActionOperation[] { setPropOp });
		UIMonitorUtility.doTaskNowDefault(op, true);
	}

}