/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.repository.model;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.provider.EcoreItemProviderAdapterFactory;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.XMIResource;
import org.eclipse.emf.ecore.xmi.impl.BasicResourceHandler;
import org.eclipse.emf.edit.provider.ComposedAdapterFactory;
import org.eclipse.emf.edit.provider.DelegatingWrapperItemProvider;
import org.eclipse.emf.edit.provider.ReflectiveItemProviderAdapterFactory;
import org.eclipse.emf.edit.provider.resource.ResourceItemProviderAdapterFactory;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryContentProvider;
import org.eclipse.ui.model.IWorkbenchAdapter;
import org.eclipse.ui.model.WorkbenchContentProvider;
import org.eclipse.uml2.uml.edit.providers.UMLItemProviderAdapterFactory;
import org.modelbus.core.lib.IRepositoryHelper;
import org.modelbus.dosgi.repository.descriptor.NonExistingResourceException;
import org.modelbus.dosgi.repository.descriptor.RepositoryAuthentificationException;
import org.modelbus.dosgi.repository.descriptor.UnresolvedReferencesException;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.repository.ModelBusRepositoryHelper;
import org.modelbus.team.eclipse.repository.UserSessionHelper;
import org.modelbus.team.eclipse.ui.repository.RepositoryTreeViewer;
import org.modelbus.team.eclipse.ui.repository.preferences.PreferenceInitializer;

/**
 * Repository content provider
 * 
 * @author Alexander Gurov
 */
public class RepositoryContentProvider extends WorkbenchContentProvider {
	protected RepositoryTreeViewer			repositoryTree;
	protected IRepositoryContentFilter		filter;
	protected ComposedAdapterFactory		composedAdapterFactory;
	protected AdapterFactoryContentProvider	adapterFactoryContentProvider;

	public RepositoryContentProvider(RepositoryTreeViewer repositoryTree) {
		this.repositoryTree = repositoryTree;
		List<AdapterFactory> factories = new ArrayList<AdapterFactory>();
		factories.add(new ResourceItemProviderAdapterFactory());
		factories.add(new ReflectiveItemProviderAdapterFactory());
		factories.add(new UMLItemProviderAdapterFactory());
		factories.add(new EcoreItemProviderAdapterFactory());
		composedAdapterFactory = new ComposedAdapterFactory(factories);
		adapterFactoryContentProvider = new AdapterFactoryContentProvider(composedAdapterFactory);
	}

	public IRepositoryContentFilter getFilter() {
		return this.filter;
	}

	public void setFilter(IRepositoryContentFilter filter) {
		this.filter = filter;
	}

	private String[] getModelExtensions() {
		String[] modelExtensions = new String[0];
		modelExtensions = PreferenceInitializer.getExtensionPreference();
		return modelExtensions;
	}

	private boolean isModelToBrowse(String path) {
		List<String> modelExtensions = Arrays.asList(getModelExtensions());
		URI uri = URI.createURI(path);
		String fileExtension = uri.fileExtension();
		for (String modelExtension : modelExtensions) {
			if (modelExtension.startsWith("*.")) {
				modelExtension = modelExtension.substring(1);
			}
			if (modelExtension.startsWith(".") && path.endsWith(modelExtension)) {
				return true;
			} else if (modelExtension.equals(fileExtension)) {
				return true;
			}
		}
		return false;
	}

	public boolean hasChildren(Object element) {
		IWorkbenchAdapter adapter = this.getAdapter(element);
		if (adapter instanceof IParentTreeNode) {
			return ((IParentTreeNode) adapter).hasChildren();
		} else if (adapter instanceof RepositoryFile) {
			String path = ((RepositoryFile) adapter).getRepositoryResource().getUrl();

			if (isModelToBrowse(path)/*
									 * path.endsWith(".uml")|| path.endsWith(".ecore") || path.endsWith(".emx") || path.endsWith(".epx") ||
									 * path.endsWith(".simulink")
									 */) {
				return true;
			}

		} else if (adapter == null/* && element instanceof EObject */) {
			if (element instanceof EObject) {
				EObject eObject = (EObject) element;
				if (eObject.eIsProxy()) {
					return true;
				}
			}
			boolean hasChildren = adapterFactoryContentProvider.hasChildren(element);
			return adapterFactoryContentProvider.hasChildren(element);
		}
		return false;
	}

	public Object[] getChildren(Object parentElement) {
		IWorkbenchAdapter adapter = this.getAdapter(parentElement);
		if (adapter instanceof IParentTreeNode) {
			if (adapter instanceof IResourceTreeNode) {
				((IResourceTreeNode) adapter).setViewer(this.repositoryTree);
			}
			ArrayList<Object> filtered = new ArrayList<Object>();
			Object[] children = adapter.getChildren(parentElement);
			if (children != null) {
				for (int i = 0; i < children.length; i++) {
					if (this.filter == null || this.filter.accept(children[i])) {
						if (children[i] instanceof IResourceTreeNode) {
							((IResourceTreeNode) children[i]).setViewer(this.repositoryTree);
						}
						filtered.add(children[i]);
					}
				}
			}
			return filtered.toArray();
		} else if (adapter instanceof RepositoryFile) {
			RepositoryFile modelFile = (RepositoryFile) adapter;
			String resourcePath = modelFile.getRepositoryResource().getUrl();
			ResourceSet resourceSet = new ResourceSetImpl();
			Map options = new HashMap();
			if (resourcePath.endsWith(".emx") || resourcePath.endsWith(".epx")) {
				// resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put("*", new XMIResourceFactoryImpl());
				options.put(XMIResource.OPTION_RESOURCE_HANDLER, new BasicResourceHandler());
			}

			IRepositoryHelper repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();
			try {
				Resource modelResource = resourceSet.createResource(URI.createURI(resourcePath));
				try {
					repositoryHelper.checkOutModel(UserSessionHelper.getSession(), modelResource,/* Collections.EMPTY_MAP */options);
				} catch (RepositoryAuthentificationException e) {
					throw new RuntimeException(e);
				} catch (UnresolvedReferencesException e) {
					throw new RuntimeException(e);
				} catch (NonExistingResourceException e) {
					throw new RuntimeException(e);
				}
				EList<EObject> contents = modelResource.getContents();
				if (contents != null && !contents.isEmpty()) {
					EObject content = contents.get(0);
					Resource tmp = resourceSet.createResource(URI.createURI(resourcePath));
					tmp.getContents().add(content);
					return adapterFactoryContentProvider.getChildren(tmp);
				}
			} catch (IOException e) {
				e.printStackTrace();
				throw new RuntimeException(e.getMessage(), e);
			}
			return new Object[0];
		} else if (adapter == null && parentElement instanceof EObject) {
			EObject eObject = (EObject) parentElement;
			if (eObject.eIsProxy()) {
				ResourceSet resourceSet = new ResourceSetImpl();
				URI proxyURI = ((InternalEObject) eObject).eProxyURI();
				String resourceUri = proxyURI.trimFragment().toString();
				String fragment = proxyURI.fragment();
				Map options = new HashMap();
				if (resourceUri.endsWith(".emx") || resourceUri.endsWith(".epx")) {
					// resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put("*", new XMIResourceFactoryImpl());
					options.put(XMIResource.OPTION_RESOURCE_HANDLER, new BasicResourceHandler());
				}

				Resource resource = resourceSet.createResource(URI.createURI(resourceUri));
				IRepositoryHelper repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();
				try {
					try {
						repositoryHelper.checkOutModel(UserSessionHelper.getSession(), resource, /* Collections.EMPTY_MAP */options);
					} catch (RepositoryAuthentificationException e) {
						throw new RuntimeException(e);
					} catch (UnresolvedReferencesException e) {
						throw new RuntimeException(e);
					} catch (NonExistingResourceException e) {
						throw new RuntimeException(e);
					}
					parentElement = resource.getEObject(fragment);
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
					throw new RuntimeException(e.getMessage(), e);
				}
			}
			return adapterFactoryContentProvider.getChildren((EObject) parentElement);
		} else if (adapter == null && parentElement instanceof DelegatingWrapperItemProvider) {

			return adapterFactoryContentProvider.getChildren(parentElement);
		}
		return new Object[0];
	}

	@Override
	public Object getParent(Object element) {
		// TODO Auto-generated method stub
		Object parent = super.getParent(element);
		if (parent != null) {
			return parent;
		}
		if (element instanceof EObject) {
			EObject eObject = (EObject) element;
			if (eObject.eContainer() == null) {
				// RepositoryHelper repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();
				// String path = eObject.eResource().getURI().toString();
				// ResourceSet resourceSet = new ResourceSetImpl();
				// Resource resource = resourceSet.createResource(URI.createURI(path));
				// try {
				// repositoryHelper.checkOutModel(UserSessionHelper.getSession(), resource);
				// return resource;
				// } catch (IOException e) {
				// // TODO Auto-generated catch block
				// e.printStackTrace();
				// }

				return ModelBusUtility.asRepositoryResource(eObject.eResource().getURI().toString(), false);

			}
			return adapterFactoryContentProvider.getParent(eObject);
		}
		return null;

	}
}
