package org.modelbus.team.eclipse.ui.repository.preferences;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;

/**
 * This class represents a preference page that is contributed to the Preferences dialog. By subclassing <samp>FieldEditorPreferencePage</samp>, we
 * can use the field support built into JFace that allows us to create a page that is small and knows how to save, restore and apply itself.
 * <p>
 * This page is used to modify preferences only. They are stored in the preference store that belongs to the main plug-in class. That way, preferences
 * can be accessed directly via the preference store.
 */

public class ModelBusRepositoryPreferencePage extends PreferencePage implements IWorkbenchPreferencePage {

	private static final int	VERTICAL_DIALOG_UNITS_PER_CHAR	= 8;
	private static final int	LIST_HEIGHT_IN_CHARS			= 10;
	private static final int	LIST_HEIGHT_IN_DLUS				= LIST_HEIGHT_IN_CHARS * VERTICAL_DIALOG_UNITS_PER_CHAR;

	private List				modelExtensionList;
	private Text				textField;
	private Button				removeModelExtension;

	public ModelBusRepositoryPreferencePage() {
		super();

		// Set the preference store for the preference page.
		IPreferenceStore store = ModelBusTeamUIPlugin.instance().getPreferenceStore();
		setPreferenceStore(store);
		setDescription("ModelBus Repository Browser Configuration");
	}

	/**
	 * @see org.eclipse.jface.preference. PreferencePage#createContents(Composite)
	 */
	protected Control createContents(Composite parent) {
		Composite top = new Composite(parent, SWT.LEFT);

		// Sets the layout data for the top composite's
		// place in its parent's layout.
		top.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		// Sets the layout for the top composite's
		// children to populate.
		top.setLayout(new GridLayout());

		Label listLabel = new Label(top, SWT.NONE);
		listLabel.setText("Set model types to browse:");

		modelExtensionList = new List(top, SWT.BORDER);
		modelExtensionList.setItems(PreferenceInitializer.getExtensionPreference());

		// Create a data that takes up the extra space
		// in the dialog and spans both columns.
		GridData listData = new GridData(GridData.FILL_HORIZONTAL);
		listData.heightHint = convertVerticalDLUsToPixels(LIST_HEIGHT_IN_DLUS);
		modelExtensionList.setLayoutData(listData);

		modelExtensionList.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				selectionChanged();
			}
		});

		Composite addRemoveGroup = new Composite(top, SWT.NONE);
		addRemoveGroup.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		GridLayout addRemoveLayout = new GridLayout();
		addRemoveLayout.numColumns = 2;
		addRemoveLayout.marginHeight = 0;
		addRemoveLayout.marginWidth = 0;
		addRemoveGroup.setLayout(addRemoveLayout);

		// Create a composite for the add and remove buttons.
		Composite buttonGroup = new Composite(addRemoveGroup, SWT.NONE);
		buttonGroup.setLayoutData(new GridData());
		GridLayout buttonLayout = new GridLayout();
		buttonLayout.marginHeight = 0;
		buttonLayout.marginWidth = 0;
		buttonGroup.setLayout(buttonLayout);

		Button addModelExtension = new Button(buttonGroup, SWT.NONE);
		addModelExtension.setText("Add Extension");
		addModelExtension.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				addExtension();
			}
		});
		GridData addExtensionData = new GridData(GridData.FILL_HORIZONTAL);
		// addExtensionData.heightHint = convertVerticalDLUsToPixels(IDialogConstants.BUTTON_HEIGHT);
		// addExtensionData.widthHint = convertHorizontalDLUsToPixels(IDialogConstants.BUTTON_WIDTH);
		addModelExtension.setLayoutData(addExtensionData);

		removeModelExtension = new Button(buttonGroup, SWT.NONE);
		removeModelExtension.setText("Remove Extension");
		removeModelExtension.setEnabled(false);
		removeModelExtension.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				modelExtensionList.remove(modelExtensionList.getSelectionIndex());
				selectionChanged();
			}
		});
		GridData removeExtensionData = new GridData(GridData.FILL_HORIZONTAL);
		// removeExtensionData.heightHint = convertVerticalDLUsToPixels(IDialogConstants.BUTTON_HEIGHT);
		// removeExtensionData.widthHint = convertHorizontalDLUsToPixels(IDialogConstants.BUTTON_WIDTH);
		removeModelExtension.setLayoutData(removeExtensionData);

		textField = new Text(addRemoveGroup, SWT.BORDER);

		GridData textData = new GridData(GridData.FILL_HORIZONTAL);
		textData.verticalAlignment = GridData.BEGINNING;
		textField.setLayoutData(textData);

		return top;
	}

	/**
	 * @see IWorkbenchPreferencePage#init
	 */
	public void init(IWorkbench wb) {
	}

	/*
	 * The user has pressed "Restore defaults". Restore all default preferences.
	 */
	protected void performDefaults() {
		modelExtensionList.setItems(PreferenceInitializer.getDefaultExtensionPreference());
		// getDefaultExemptTagsPreference() is a convenience
		// method which retrieves the default preference from
		// the preference store.
		super.performDefaults();
	}

	/*
	 * The user has pressed Ok or Apply. Store/apply this page's values appropriately.
	 */
	public boolean performOk() {

		PreferenceInitializer.setExtensionPreference(modelExtensionList.getItems());

		return super.performOk();
	}

	private void addExtension() {
		String extension = textField.getText();
		if (extension != null && extension.length() > 0) {
			modelExtensionList.add(extension);
		}
		textField.setText("");
	}

	/*
	 * Sets the enablement of the remove button depending on the selection in the list.
	 */
	private void selectionChanged() {
		int index = modelExtensionList.getSelectionIndex();
		removeModelExtension.setEnabled(index >= 0);
	}
}