/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.synchronize.merge;

import java.util.HashSet;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.team.core.synchronize.SyncInfo;
import org.modelbus.team.eclipse.core.IStateFilter;
import org.modelbus.team.eclipse.core.connector.ModelBusEntry;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryStatus;
import org.modelbus.team.eclipse.core.connector.ModelBusMergeStatus;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRemoteStorage;
import org.modelbus.team.eclipse.core.operation.CompositeOperation;
import org.modelbus.team.eclipse.core.operation.local.AbstractMergeSet;
import org.modelbus.team.eclipse.core.operation.local.IRemoteStatusOperation;
import org.modelbus.team.eclipse.core.operation.local.MergeSet1URL;
import org.modelbus.team.eclipse.core.operation.local.MergeSet2URL;
import org.modelbus.team.eclipse.core.operation.local.MergeSetReintegrate;
import org.modelbus.team.eclipse.core.operation.local.MergeStatusOperation;
import org.modelbus.team.eclipse.core.operation.local.RefreshResourcesOperation;
import org.modelbus.team.eclipse.core.operation.local.RestoreProjectMetaOperation;
import org.modelbus.team.eclipse.core.operation.local.SaveProjectMetaOperation;
import org.modelbus.team.eclipse.core.resource.IChangeStateProvider;
import org.modelbus.team.eclipse.core.resource.ILocalResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.resource.IResourceChange;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.ui.synchronize.AbstractModelbusSubscriber;
import org.modelbus.team.eclipse.ui.synchronize.RemoteStatusCache;
import org.modelbus.team.eclipse.ui.synchronize.update.UpdateSyncInfo;

/**
 * Merge view data provider
 * 
 * @author Alexander Gurov
 */
public class MergeSubscriber extends AbstractModelbusSubscriber {
	private static MergeSubscriber	instance	= null;

	protected MergeScope			scope;
	protected MergeStatusOperation	mergeStatusOp;
	protected RemoteStatusCache		baseStatusCache;

	public static synchronized MergeSubscriber instance() {
		if (MergeSubscriber.instance == null) {
			MergeSubscriber.instance = new MergeSubscriber();
		}
		return MergeSubscriber.instance;
	}

	public MergeScope getMergeScope() {
		return this.scope;
	}

	public void setMergeScope(MergeScope scope) {
		this.scope = scope;
	}

	protected SyncInfo getModelBusSyncInfo(ILocalResource localStatus, IResourceChange remoteStatus) {
		IResourceChange baseStatus = ModelBusRemoteStorage.instance().resourceChangeFromBytes(
				this.baseStatusCache.getBytes(localStatus.getResource()));
		// provide correct base resource: same as right but with the start revision specified
		return remoteStatus != null && IStateFilter.SF_NOTMODIFIED.accept(remoteStatus) ? new UpdateSyncInfo(localStatus, null,
				this.getResourceComparator()) : new MergeSyncInfo(localStatus, baseStatus, remoteStatus, this.getResourceComparator());
	}

	protected IRemoteStatusOperation addStatusOperation(CompositeOperation op, IResource[] resources, int depth) {
		MergeStatusOperation mergeOp = this.mergeStatusOp = (this.scope == null ? null
				: new MergeStatusOperation(this.scope.getMergeSet(), resources));
		if (mergeOp == null) {
			return null;
		}
		SaveProjectMetaOperation saveOp = new SaveProjectMetaOperation(resources);
		op.add(saveOp);
		op.add(mergeOp);
		op.add(new RestoreProjectMetaOperation(saveOp));
		op.add(new RefreshResourcesOperation(resources, depth, RefreshResourcesOperation.REFRESH_CHANGES));
		return mergeOp;
	}

	protected HashSet<IResource> clearRemoteStatusesImpl(IResource[] resources) {
		this.clearRemoteStatusesImpl(this.baseStatusCache, resources);
		return super.clearRemoteStatusesImpl(resources);
	}

	public void refresh(final IResource[] resources, final int depth, IProgressMonitor monitor) {
		if (this.scope != null) {
			this.baseStatusCache.clearAll();
			this.scope.getMergeSet().setStatuses(new ModelBusMergeStatus[0]);
		}
		super.refresh(resources, depth, monitor);
	}

	protected IResourceChange handleResourceChange(IRemoteStatusOperation rStatusOp, ModelBusEntryStatus status) {
		final ModelBusMergeStatus current = (ModelBusMergeStatus) status;
		IChangeStateProvider endProvider = new IChangeStateProvider() {
			public long getChangeDate() {
				return current.date;
			}

			public String getChangeAuthor() {
				return current.author;
			}

			public ModelBusRevision getChangeRevision() {
				return current.endRevision == ModelBusRevision.INVALID_REVISION_DESCRIPTOR ? null : current.endRevision;
			}

			public int getTextChangeType() {
				return current.skipped ? current.textStatus : ModelBusEntryStatus.Kind.NORMAL;
			}

			public int getPropertiesChangeType() {
				return current.skipped ? current.propStatus : ModelBusEntryStatus.Kind.NONE;
			}

			public int getNodeKind() {
				int kind = ModelBusUtility.getNodeKind(current.path, current.nodeKind, true);
				// if not exists on repository try to check it with WC kind...
				return kind == ModelBusEntry.Kind.NONE ? ModelBusUtility.getNodeKind(current.path, current.nodeKind, false) : kind;
			}

			public String getLocalPath() {
				return current.path;
			}

			public String getComment() {
				return current.comment;
			}

			public boolean isCopied() {
				return false;
			}

			public boolean isSwitched() {
				return false;
			}

			public IResource getExact(IResource[] set) {
				return FileUtility.selectOneOf(MergeSubscriber.this.scope.getRoots(), set);
			}
		};
		if (endProvider.getNodeKind() == ModelBusEntry.Kind.NONE) {
			return null;
		}
		IResourceChange endResourceChange = ModelBusRemoteStorage.instance().asResourceChange(endProvider, false);
		if (endResourceChange == null) {
			return null;
		}
		if (endResourceChange.getRevision() != ModelBusRevision.INVALID_REVISION_DESCRIPTOR) {
			IRepositoryResource originator = this.getEndOriginator();
			String decodedUrl = ModelBusUtility.decodeURL(current.endUrl);
			originator = endProvider.getNodeKind() == ModelBusEntry.Kind.DIR ? (IRepositoryResource) originator.asRepositoryContainer(decodedUrl,
					false) : originator.asRepositoryFile(decodedUrl, false);
			originator.setSelectedRevision(current.textStatus == ModelBusEntryStatus.Kind.DELETED ? current.endRevision.getPreviousRevision() : current.endRevision);
			endResourceChange.setOriginator(originator);
		}

		IChangeStateProvider startProvider = new IChangeStateProvider() {
			public long getChangeDate() {
				return current.date;
			}

			public String getChangeAuthor() {
				return null;
			}

			public ModelBusRevision getChangeRevision() {
				return current.startRevision == ModelBusRevision.INVALID_REVISION_DESCRIPTOR ? null : current.startRevision;
			}

			public int getTextChangeType() {
				return current.startRevision == ModelBusRevision.INVALID_REVISION_DESCRIPTOR ? ModelBusEntryStatus.Kind.NONE
						: ModelBusEntryStatus.Kind.NORMAL;
			}

			public int getPropertiesChangeType() {
				return ModelBusEntryStatus.Kind.NONE;
			}

			public int getNodeKind() {
				int kind = ModelBusUtility.getNodeKind(current.path, current.nodeKind, true);
				// if not exists on repository try to check it with WC kind...
				return kind == ModelBusEntry.Kind.NONE ? ModelBusUtility.getNodeKind(current.path, current.nodeKind, false) : kind;
			}

			public String getLocalPath() {
				return current.path;
			}

			public String getComment() {
				return null;
			}

			public boolean isCopied() {
				return false;
			}

			public boolean isSwitched() {
				return false;
			}

			public IResource getExact(IResource[] set) {
				return FileUtility.selectOneOf(MergeSubscriber.this.scope.getRoots(), set);
			}
		};
		IResourceChange startResourceChange = ModelBusRemoteStorage.instance().asResourceChange(startProvider, false);
		if (startResourceChange.getRevision() != ModelBusRevision.INVALID_REVISION_DESCRIPTOR) {
			String decodedUrl = ModelBusUtility.decodeURL(current.startUrl);
			IRepositoryResource originator = this.getStartOriginator();
			originator = startProvider.getNodeKind() == ModelBusEntry.Kind.DIR ? (IRepositoryResource) originator.asRepositoryContainer(decodedUrl,
					false) : originator.asRepositoryFile(decodedUrl, false);
			originator.setSelectedRevision(current.startRevision);
			startResourceChange.setOriginator(originator);
		}
		this.baseStatusCache.setBytes(startResourceChange.getResource(), ModelBusRemoteStorage.instance().resourceChangeAsBytes(startResourceChange));

		return endResourceChange;
	}

	protected IRepositoryResource getEndOriginator() {
		AbstractMergeSet mergeSet = this.scope.getMergeSet();
		if (mergeSet instanceof MergeSet1URL) {
			return ((MergeSet1URL) mergeSet).from[0];
		} else if (mergeSet instanceof MergeSet2URL) {
			return ((MergeSet2URL) mergeSet).fromEnd[0];
		} else {
			return ((MergeSetReintegrate) mergeSet).from[0];
		}
	}

	protected IRepositoryResource getStartOriginator() {
		AbstractMergeSet mergeSet = this.scope.getMergeSet();
		if (mergeSet instanceof MergeSet1URL) {
			return ((MergeSet1URL) mergeSet).from[0];
		} else if (mergeSet instanceof MergeSet2URL) {
			return ((MergeSet2URL) mergeSet).fromStart[0];
		} else {
			return ((MergeSetReintegrate) mergeSet).from[0];
		}
	}

	protected boolean isIncoming(ModelBusEntryStatus status) {
		return true;
	}

	private MergeSubscriber() {
		super();
		this.baseStatusCache = new RemoteStatusCache();
	}

}
