/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *    Alessandro Nistico - [patch] Change Set's implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.synchronize.update.action;

import java.util.Arrays;
import java.util.HashSet;

import org.eclipse.compare.structuremergeviewer.IDiffElement;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.team.core.synchronize.FastSyncInfoFilter;
import org.eclipse.team.core.synchronize.SyncInfo;
import org.eclipse.team.ui.synchronize.ISynchronizePageConfiguration;
import org.modelbus.team.eclipse.core.IStateFilter;
import org.modelbus.team.eclipse.core.operation.AbstractActionOperation;
import org.modelbus.team.eclipse.core.operation.CompositeOperation;
import org.modelbus.team.eclipse.core.operation.IActionOperation;
import org.modelbus.team.eclipse.core.operation.local.AddToModelBusWithPropertiesOperation;
import org.modelbus.team.eclipse.core.operation.local.ClearLocalStatusesOperation;
import org.modelbus.team.eclipse.core.operation.local.CommitOperation;
import org.modelbus.team.eclipse.core.operation.local.MarkAsMergedOperation;
import org.modelbus.team.eclipse.core.operation.local.RefreshResourcesOperation;
import org.modelbus.team.eclipse.core.resource.ILocalResource;
import org.modelbus.team.eclipse.core.resource.IResourceProvider;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.ui.dialog.NotifyNodeKindChangedDialog;
import org.modelbus.team.eclipse.ui.extension.ExtensionsManager;
import org.modelbus.team.eclipse.ui.extension.factory.ICommitDialog;
import org.modelbus.team.eclipse.ui.operation.ClearUpdateStatusesOperation;
import org.modelbus.team.eclipse.ui.panel.local.CommitPanel;
import org.modelbus.team.eclipse.ui.synchronize.ModelBusChangeSetCapability;
import org.modelbus.team.eclipse.ui.synchronize.action.AbstractSynchronizeModelAction;
import org.modelbus.team.eclipse.ui.synchronize.action.ISyncStateFilter;
import org.modelbus.team.eclipse.ui.synchronize.update.UpdateSyncInfo;
import org.modelbus.team.eclipse.ui.utility.UIMonitorUtility;
import org.modelbus.team.eclipse.ui.utility.UnacceptableOperationNotificator;

/**
 * Override and commit conflicting files action
 * 
 * @author Alexander Gurov
 */
public class OverrideAndCommitAction extends AbstractSynchronizeModelAction {

	public OverrideAndCommitAction(String text, ISynchronizePageConfiguration configuration) {
		super(text, configuration);
	}

	protected FastSyncInfoFilter getSyncInfoFilter() {
		return new FastSyncInfoFilter.SyncInfoDirectionFilter(new int[] { SyncInfo.CONFLICTING, SyncInfo.OUTGOING }) {
			public boolean select(SyncInfo info) {
				UpdateSyncInfo sync = (UpdateSyncInfo) info;
				return super.select(info) && !(IStateFilter.SF_OBSTRUCTED.accept(sync.getLocalResource()));
			}
		};
	}

	protected IActionOperation getOperation(ISynchronizePageConfiguration configuration, IDiffElement[] elements) {
		String msg = null;
		boolean keepLocks = false;
		final IResource[][] resources = new IResource[1][];

		IResource[] changedResources = OverrideAndCommitAction.this.syncInfoSelector.getSelectedResourcesRecursive(ISyncStateFilter.SF_OVERRIDE);
		IResource[] overrideResources = UnacceptableOperationNotificator.shrinkResourcesWithNotOnRespositoryParents(configuration.getSite()
				.getShell(), changedResources);
		if (overrideResources != null && overrideResources.length > 0) {
			overrideResources = FileUtility.addOperableParents(overrideResources, IStateFilter.SF_NOTONREPOSITORY);
			HashSet<IResource> allResourcesSet = new HashSet<IResource>(Arrays.asList(overrideResources));
			String proposedComment = ModelBusChangeSetCapability.getProposedComment(overrideResources);
			CommitPanel commitPanel = new CommitPanel(overrideResources, overrideResources, CommitPanel.MSG_OVER_AND_COMMIT, proposedComment);
			ICommitDialog commitDialog = ExtensionsManager.getInstance().getCurrentCommitFactory()
					.getCommitDialog(configuration.getSite().getShell(), allResourcesSet, commitPanel);
			if (commitDialog.open() != 0) {
				return null;
			}
			resources[0] = commitPanel.getSelectedResources().length == 0 ? null : commitPanel.getSelectedResources();
			msg = commitDialog.getMessage();
			keepLocks = commitPanel.getKeepLocks();
		}

		CompositeOperation op = new CompositeOperation("Operation.UOverrideAndCommit");

		final MarkAsMergedOperation mergeOp = new MarkAsMergedOperation(resources[0], true, msg, keepLocks);
		op.add(mergeOp);
		final IResource[] addition = FileUtility.getResourcesRecursive(resources[0], OverrideAndCommitAction.SF_NEW);
		if (addition.length != 0) {
			IResourceProvider additionProvider = new IResourceProvider() {
				protected IResource[]	result;

				public IResource[] getResources() {
					if (this.result == null) {
						HashSet<IResource> tAdd = new HashSet<IResource>(Arrays.asList(addition));
						IResource[] restricted = mergeOp.getHavingDifferentNodeKind();
						for (int i = 0; i < restricted.length; i++) {
							if (restricted[i] instanceof IContainer) {// delete from add to ModelBus list, resources, with nodekind changed, and all
																		// their children
								IResource[] restrictedChildren = FileUtility.getResourcesRecursive(resources[0], IStateFilter.SF_ALL);
								tAdd.removeAll(Arrays.asList(restrictedChildren));
							} else {
								tAdd.remove(restricted[i]);
							}

						}

						this.result = tAdd.toArray(new IResource[tAdd.size()]);
					}
					return this.result;
				}
			};
			op.add(new AddToModelBusWithPropertiesOperation(additionProvider, false), new IActionOperation[] { mergeOp });
			op.add(new ClearLocalStatusesOperation(additionProvider));
		}
		CommitOperation mainOp = new CommitOperation(mergeOp, msg, true, keepLocks);
		IActionOperation[] dependsOn = new IActionOperation[] { mergeOp };
		op.add(mainOp, dependsOn);
		op.add(new AbstractActionOperation("Operation.UNodeKindChanged") {
			protected void runImpl(IProgressMonitor monitor) throws Exception {
				final IResource[] diffNodeKind = mergeOp.getHavingDifferentNodeKind();
				if (diffNodeKind.length > 0) {
					UIMonitorUtility.getDisplay().syncExec(new Runnable() {
						public void run() {
							new NotifyNodeKindChangedDialog(UIMonitorUtility.getShell(), diffNodeKind).open();
						}
					});
				}
			}
		});
		op.add(new ClearUpdateStatusesOperation(resources[0]));
		op.add(new RefreshResourcesOperation(resources[0]));
		ExtensionsManager.getInstance().getCurrentCommitFactory().performAfterCommitTasks(op, mainOp, dependsOn, configuration.getSite().getPart());
		return op;
	}

	public static final IStateFilter	SF_NEW	= new IStateFilter.AbstractStateFilter() {
													protected boolean acceptImpl(ILocalResource local, IResource resource, String state, int mask) {
														return state == IStateFilter.ST_NEW;
													}

													protected boolean allowsRecursionImpl(ILocalResource local, IResource resource, String state,
															int mask) {
														return true;
													}
												};

}
