/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *    Zend Technologies - [patch] Workaround for 'thread "Worker-3" timed out waiting (5000ms) for thread...' problem 
 *******************************************************************************/

package org.modelbus.team.eclipse.core;

import java.io.File;
import java.io.FileFilter;
import java.text.MessageFormat;

import org.eclipse.core.net.proxy.IProxyService;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.resources.WorkspaceJob;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Plugin;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.preferences.IEclipsePreferences;
import org.eclipse.core.runtime.preferences.InstanceScope;
import org.eclipse.team.core.RepositoryProvider;
import org.modelbus.team.eclipse.core.extension.CoreExtensionsManager;
import org.modelbus.team.eclipse.core.extension.crashrecovery.DefaultErrorHandlingFacility;
import org.modelbus.team.eclipse.core.extension.crashrecovery.IErrorHandlingFacility;
import org.modelbus.team.eclipse.core.extension.options.IOptionProvider;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRemoteStorage;
import org.modelbus.team.eclipse.core.operation.AbstractActionOperation;
import org.modelbus.team.eclipse.core.operation.LoggedOperation;
import org.modelbus.team.eclipse.core.operation.file.ModelBusFileStorage;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ProgressMonitorUtility;
import org.osgi.framework.BundleContext;
import org.osgi.util.tracker.ServiceTracker;

/**
 * Team provider plug-in implementation
 * 
 * @author Alexander Gurov
 */
public class ModelBusTeamPlugin extends Plugin {
	// all projects shared with subversion will have this nature
	public static final String					NATURE_ID					= "org.modelbus.team.eclipse.core.modelbusnature";

	public static final String					CORE_ModelBusCLIENT_NAME	= "modelbusconnector";

	private volatile static ModelBusTeamPlugin	instance					= null;

	private ProjectCloseListener				pcListener;
	private ResourceChangeListener				rcListener;
	private ModelBusFolderListener				modelbusListener;
	private ServiceTracker						tracker;

	private IErrorHandlingFacility				errorHandlingFacility;

	/**
	 * Allows to save changes in repository root URL and UUID
	 */
	private boolean								isLocationsDirty;

	public ModelBusTeamPlugin() {
		super();

		ModelBusTeamPlugin.instance = this;

		this.pcListener = new ProjectCloseListener();
		this.rcListener = new ResourceChangeListener();
		this.modelbusListener = new ModelBusFolderListener();

		this.errorHandlingFacility = new DefaultErrorHandlingFacility();
	}

	public void setLocationsDirty(boolean isLocationsDirty) {
		this.isLocationsDirty = isLocationsDirty;
	}

	public boolean isLocationsDirty() {
		return this.isLocationsDirty;
	}

	public static ModelBusTeamPlugin instance() {
		return ModelBusTeamPlugin.instance;
	}

	public String getResource(String key) {
		return FileUtility.getResource(Platform.getResourceBundle(this.getBundle()), key);
	}

	public String getResource(String key, Object[] args) {
		String message = this.getResource(key);
		return MessageFormat.format(message, args);
	}

	public IOptionProvider getOptionProvider() {
		return CoreExtensionsManager.instance().getOptionProvider();
	}

	public IErrorHandlingFacility getErrorHandlingFacility() {
		return this.errorHandlingFacility;
	}

	// FIXME remove later after Integration API is changed
	public void setOptionProvider(IOptionProvider optionProvider) {
		CoreExtensionsManager.instance().setOptionProvider(optionProvider == null ? IOptionProvider.DEFAULT : optionProvider);

		ModelBusRemoteStorage.instance().reconfigureLocations();

		// remove temporary files if IDE is crached time ago...
		ProgressMonitorUtility.doTaskScheduledDefault(new AbstractActionOperation("Remove Temporary Files") {
			protected void runImpl(IProgressMonitor monitor) throws Exception {
				ModelBusTeamPlugin.instance().getStateLocation().toFile().listFiles(new FileFilter() {
					public boolean accept(File pathname) {
						String name = pathname.getName();
						if (!name.equals(ModelBusRemoteStorage.STATE_INFO_FILE_NAME) && !name.equals(ModelBusFileStorage.STATE_INFO_FILE_NAME)) {
							FileUtility.deleteRecursive(pathname);
						}
						return false;
					}
				});
			}
		});
	}

	/**
	 * Return the Subversive Core preferences node in the instance scope
	 */
	public IEclipsePreferences getModelBusCorePreferences() {
		return new InstanceScope().getNode(this.getBundle().getSymbolicName());
	}

	public void start(BundleContext context) throws Exception {
		super.start(context);

		this.tracker = new ServiceTracker(context, IProxyService.class.getName(), null);
		this.tracker.open();

		IPath stateLocation = this.getStateLocation();
		ModelBusFileStorage.instance().initialize(stateLocation);

		ModelBusRemoteStorage storage = ModelBusRemoteStorage.instance();
		storage.initialize(stateLocation);

		WorkspaceJob job = new WorkspaceJob("") {
			public IStatus runInWorkspace(IProgressMonitor monitor) throws CoreException {
				IWorkspace workspace = ResourcesPlugin.getWorkspace();

				workspace.addResourceChangeListener(ModelBusTeamPlugin.this.rcListener, IResourceChangeEvent.POST_CHANGE);
				workspace.addResourceChangeListener(ModelBusTeamPlugin.this.modelbusListener, IResourceChangeEvent.PRE_BUILD);
				workspace.addResourceChangeListener(ModelBusTeamPlugin.this.pcListener, IResourceChangeEvent.PRE_CLOSE
						| IResourceChangeEvent.PRE_DELETE);

				// shouldn't prevent plugin start
				try {
					ModelBusTeamPlugin.this.rcListener.handleInitialWorkspaceDelta();
				} catch (Throwable ex) {
					LoggedOperation.reportError("Handle Initial Workspace Delta", ex);
				}

				// if some team provider is missing the code below "enables" team menu "Share Project" action...
				IProject[] projects = workspace.getRoot().getProjects();
				for (int i = 0; i < projects.length; i++) {
					RepositoryProvider.getProvider(projects[i]);
				}
				return Status.OK_STATUS;
			}
		};
		job.setRule(ResourcesPlugin.getWorkspace().getRuleFactory().buildRule());
		job.setUser(false);
		job.schedule();
	}

	public IProxyService getProxyService() {
		return (IProxyService) this.tracker.getService();
	}

	public void stop(BundleContext context) throws Exception {
		IWorkspace workspace = ResourcesPlugin.getWorkspace();

		workspace.removeResourceChangeListener(this.modelbusListener);
		workspace.removeResourceChangeListener(this.rcListener);
		workspace.removeResourceChangeListener(this.pcListener);

		if (this.isLocationsDirty) {
			ModelBusRemoteStorage.instance().saveConfiguration();
			ModelBusFileStorage.instance().saveConfiguration();
		}
		ModelBusRemoteStorage.instance().dispose();
		ModelBusFileStorage.instance().dispose();

		this.tracker.close();

		// cleanup temporary files if any
		File temporaryFilesStorage = ModelBusTeamPlugin.instance().getStateLocation().toFile();
		File[] files = temporaryFilesStorage.listFiles(new FileFilter() {
			public boolean accept(File pathname) {
				return pathname.getName().indexOf(".tmp") != -1;
			}
		});
		if (files != null) {
			for (int i = 0; i < files.length; i++) {
				FileUtility.deleteRecursive(files[i]);
			}
		}

		super.stop(context);
	}

}
