/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceRuleFactory;
import org.eclipse.core.resources.team.FileModificationValidator;
import org.eclipse.core.resources.team.IMoveDeleteHook;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.team.core.RepositoryProvider;
import org.eclipse.team.core.history.IFileHistoryProvider;
import org.modelbus.team.eclipse.core.extension.crashrecovery.ErrorDescription;
import org.modelbus.team.eclipse.core.history.ModelBusFileHistoryProvider;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRemoteStorage;
import org.modelbus.team.eclipse.core.operation.AbstractActionOperation;
import org.modelbus.team.eclipse.core.operation.CompositeOperation;
import org.modelbus.team.eclipse.core.operation.HiddenException;
import org.modelbus.team.eclipse.core.operation.ModelBusResourceRuleFactory;
import org.modelbus.team.eclipse.core.operation.UnreportableException;
import org.modelbus.team.eclipse.core.operation.local.management.DisconnectOperation;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.resource.events.ResourceStatesChangedEvent;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.core.utility.ProgressMonitorUtility;

/**
 * IProject attachment, that allows us to associate a project with a repository
 * 
 * @author Alexander Gurov
 */
public class ModelBusTeamProvider extends RepositoryProvider implements IConnectedProjectInformation {
	public final static QualifiedName	RESOURCE_PROPERTY	= new QualifiedName("org.modelbus.team.eclipse", "resource");
	public final static QualifiedName	LOCATION_PROPERTY	= new QualifiedName("org.modelbus.team.eclipse", "location");

	protected String					loaction;
	protected IRepositoryResource		resource;
	protected String					relocatedTo;
	protected String					locationId;
	protected int						errorCode;
	protected int						state;

	public ModelBusTeamProvider() {
		super();
		this.state = 0;
	}

	public synchronized IRepositoryResource getRepositoryResource() throws HiddenException {
		if (this.state != 1) {
			this.connectToProject();
		}
		return this.resource;
	}

	public synchronized void switchResource(IRepositoryResource resource) throws CoreException {
		// does not affect finite automate state
		this.resource = ModelBusUtility.copyOf(resource);
		ModelBusTeamProvider.setRepositoryLocation(this.getProject(), resource.getUrl());
	}

	public synchronized void relocateResource() throws CoreException {
		// does not affect finite automate state
		// throw new RuntimeException("NYI");
		ModelBusTeamProvider.setRepositoryLocation(this.getProject(), this.loaction);
	}

	public static void map(IProject project, IRepositoryResource resource) throws CoreException {
		ModelBusTeamProvider.setRepositoryLocation(project, resource.getUrl());
		RepositoryProvider.map(project, ModelBusTeamPlugin.NATURE_ID);
	}

	public String getID() {
		return ModelBusTeamPlugin.NATURE_ID;
	}

	public void configureProject() {
		this.connectToProject();
		ModelBusRemoteStorage.instance()
				.fireResourceStatesChangedEvent(
						new ResourceStatesChangedEvent(new IResource[] { this.getProject() }, IResource.DEPTH_ZERO,
								ResourceStatesChangedEvent.CHANGED_NODES));
	}

	public void deconfigure() throws CoreException {

	}

	public IMoveDeleteHook getMoveDeleteHook() {
		return new ModelBusTeamMoveDeleteHook();
	}

	public FileModificationValidator getFileModificationValidator2() {
		return ModelBusTeamPlugin.instance().getOptionProvider().getFileModificationValidator();
	}

	public boolean canHandleLinkedResources() {
		// deprecated in Eclipse 3.2
		return this.canHandleLinkedResourceURI();
	}

	public boolean canHandleLinkedResourceURI() {
		// since Eclipse 3.2
		return true;
	}

	public IResourceRuleFactory getRuleFactory() {
		return ModelBusResourceRuleFactory.INSTANCE;
	}

	public IFileHistoryProvider getFileHistoryProvider() {
		return new ModelBusFileHistoryProvider();
	}

	protected void deconfigured() {
		IProject project = this.getProject();
		if (project != null) {
			try {
				project.setPersistentProperty(ModelBusTeamProvider.LOCATION_PROPERTY, null);
			} catch (Exception ex) {
			}
			// compatibility with previous versions
			try {
				project.setPersistentProperty(ModelBusTeamProvider.RESOURCE_PROPERTY, null);
			} catch (Exception ex) {
			}
		}
		ModelBusRemoteStorage.instance()
				.fireResourceStatesChangedEvent(
						new ResourceStatesChangedEvent(new IResource[] { this.getProject() }, IResource.DEPTH_ZERO,
								ResourceStatesChangedEvent.CHANGED_NODES));
		super.deconfigured();
	}

	protected static void setRepositoryLocation(IProject project, String path) throws CoreException {

		// CHECKMWA!!!!
		project.setPersistentProperty(ModelBusTeamProvider.LOCATION_PROPERTY, path);
	}

	protected void restoreLocation() throws HiddenException {
		if (this.state == 0) {
			this.loaction = null;
			this.locationId = null;
			if ((this.errorCode = this.uploadRepositoryLocation()) == ErrorDescription.SUCCESS
					|| (this.errorCode = this.acquireResolution(false)) == ErrorDescription.SUCCESS) {
				return;
			}

			this.performDisconnect();
		}
		this.breakThreadExecution();
	}

	protected void connectToProject() throws HiddenException {
		// serialize initialization
		synchronized (ModelBusTeamProvider.class) {
			if (this.state == 0) {
				this.loaction = null;
				this.locationId = null;
				this.resource = null;
				this.relocatedTo = null;
				if ((this.errorCode = this.uploadRepositoryResource()) == ErrorDescription.SUCCESS
						|| (this.errorCode = this.acquireResolution(true)) == ErrorDescription.SUCCESS) {
					this.state = 1;
					return;
				}

				this.performDisconnect();
			}
			this.breakThreadExecution();
		}
	}

	protected int acquireResolution(boolean full) {
		while (true) {
			Object context = null;
			if (this.errorCode == ErrorDescription.REPOSITORY_LOCATION_IS_DISCARDED) {
				context = new Object[] { this.getProject(), loaction };
			} else if (this.errorCode == ErrorDescription.PROJECT_IS_RELOCATED_OUTSIDE_PLUGIN) {
				context = new Object[] { this.getProject(), this.relocatedTo, this.loaction };
			} else if (this.errorCode == ErrorDescription.CANNOT_READ_LOCATION_DATA) {
				context = new Object[] { this.getProject(), this.relocatedTo, this.locationId };
			} else if (this.errorCode == ErrorDescription.CANNOT_READ_PROJECT_METAINFORMATION) {
				context = this.getProject();
			}

			if (ModelBusTeamPlugin.instance().getErrorHandlingFacility().acquireResolution(new ErrorDescription(this.errorCode, context))) {
				int newError = full ? this.uploadRepositoryResource() : this.uploadRepositoryLocation();
				if (newError != ErrorDescription.SUCCESS) {
					if (newError != this.errorCode) {
						this.errorCode = newError;
						continue;
					}
					return newError;
				}
				return ErrorDescription.SUCCESS;
			}
			return this.errorCode;
		}
	}

	protected void performDisconnect() {
		this.state = -1;
		CompositeOperation op = new CompositeOperation("Operation.OpenProject");
		op.add(new DisconnectOperation(new IProject[] { this.getProject() }, false));
		// notify user about the problem is happened
		op.add(new AbstractActionOperation(op.getId()) {
			protected void runImpl(IProgressMonitor monitor) throws Exception {
				throw new UnreportableException(ModelBusTeamProvider.this.getAutoDisconnectMessage());
			}
		});
		ProgressMonitorUtility.doTaskScheduled(op);
	}

	protected void breakThreadExecution() {
		throw new HiddenException(this.getAutoDisconnectMessage());
	}

	protected String getAutoDisconnectMessage() {
		return ModelBusTeamPlugin.instance().getResource("Error.AutoDisconnect", new String[] { this.getProject().getName() });
	}

	protected int uploadRepositoryResource() {
		int errorCode = this.uploadRepositoryLocation();

		// IProject project = this.getProject();
		//
		// ModelBusChangeStatus st = ModelBusUtility.getModelBusInfoForNotConnected(project);
		// if (st != null) {
		// this.relocatedTo = ModelBusUtility.decodeURL(st.url);
		// this.resource = ModelBusConnector.asRepositoryContainer(this.relocatedTo, true);
		// if (this.resource == null) {
		// return ErrorDescription.PROJECT_IS_RELOCATED_OUTSIDE_PLUGIN;
		// }
		// }
		// else {
		// return ErrorDescription.CANNOT_READ_PROJECT_METAINFORMATION;
		// }

		return errorCode;
	}

	protected int uploadRepositoryLocation() {
		try {
			IProject project = this.getProject();

			String data = project.getPersistentProperty(ModelBusTeamProvider.LOCATION_PROPERTY);
			System.out.println("TODOMWACHECKLOCATION");
			if (data != null) {
				this.loaction = data;
				// this.locationId = this.location.getId();
				// if (ModelBusRemoteStorage.instance().getRepositoryLocation(this.location.getId()) == null) {
				// return ErrorDescription.REPOSITORY_LOCATION_IS_DISCARDED;
				// }
			}
			// else {
			// // compatibility with previous versions
			// data = project.getPersistentProperty(ModelBusTeamProvider.RESOURCE_PROPERTY);
			// if (data != null) {
			// this.locationId = ModelBusTeamProvider.extractLocationId(data);
			// if (this.locationId != null) {
			// ModelBusTeamProvider.setRepositoryLocation(project, path);
			// }
			// }
			// }
			// throw new RuntimeException("NYI");

			return this.loaction == null ? ErrorDescription.CANNOT_READ_LOCATION_DATA : ErrorDescription.SUCCESS;
		} catch (CoreException ex) {
			return ErrorDescription.CANNOT_READ_LOCATION_DATA;
		}
	}

	// compatibility with old plugin versions
	protected static String extractLocationId(String resourceData) {
		if (resourceData != null) {
			String[] data = resourceData.split(";");
			if (data.length >= 2) {
				return data[1];
			}
		}
		return null;
	}

}
