/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.modelbusstorage;

import java.io.File;
import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.internal.preferences.Base64;
import org.eclipse.core.net.proxy.IProxyChangeEvent;
import org.eclipse.core.net.proxy.IProxyChangeListener;
import org.eclipse.core.net.proxy.IProxyData;
import org.eclipse.core.net.proxy.IProxyService;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.preferences.IEclipsePreferences;
import org.eclipse.core.runtime.preferences.IEclipsePreferences.IPreferenceChangeListener;
import org.eclipse.core.runtime.preferences.IEclipsePreferences.PreferenceChangeEvent;
import org.modelbus.team.eclipse.core.ModelBusTeamPlugin;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision.Kind;
import org.modelbus.team.eclipse.core.modelbusstorage.events.IRepositoriesStateChangedListener;
import org.modelbus.team.eclipse.core.modelbusstorage.events.IRevisionPropertyChangeListener;
import org.modelbus.team.eclipse.core.modelbusstorage.events.RepositoriesStateChangedEvent;
import org.modelbus.team.eclipse.core.modelbusstorage.events.RevisonPropertyChangeEvent;
import org.modelbus.team.eclipse.core.operation.LoggedOperation;
import org.modelbus.team.eclipse.core.resource.IModelBusStorage;
import org.modelbus.team.eclipse.core.resource.IRepositoryContainer;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryRoot;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.osgi.service.prefs.BackingStoreException;

/**
 * Basic IRemoteStorage implementation
 * 
 * @author Alexander Gurov
 */
public abstract class AbstractModelBusStorage implements IModelBusStorage {

	private static final String								URL_TO_STORE	= "http://eclipse.org/subversive/";

	protected File											stateInfoFile;
	protected String										preferencesNode;
	protected IPreferenceChangeListener						repoPrefChangeListener;
	protected ModelBusCachedProxyCredentialsManager			proxyCredentialsManager;

	protected List<IRepositoriesStateChangedListener>		repositoriesStateChangedListeners;
	protected ArrayList<IRevisionPropertyChangeListener>	revPropChangeListeners;

	protected class RepositoryPreferenceChangeListener implements IPreferenceChangeListener {
		public void preferenceChange(PreferenceChangeEvent event) {
			try {
				((IEclipsePreferences) event.getSource()).flush();
			} catch (BackingStoreException e) {
				LoggedOperation.reportError("preferenceChange", e);
			}
			AbstractModelBusStorage.this.fireRepositoriesStateChanged(new RepositoriesStateChangedEvent(RepositoriesStateChangedEvent.ADDED));
		}
	}

	public AbstractModelBusStorage() {
		this.repositoriesStateChangedListeners = new ArrayList<IRepositoriesStateChangedListener>();
		this.revPropChangeListeners = new ArrayList<IRevisionPropertyChangeListener>();
	}

	public void dispose() {
		// synchronization is innecessary
	}

	public void reconfigureLocations() {
		// synchronization is innecessary
	}

	public void addRepositoriesStateChangedListener(IRepositoriesStateChangedListener listener) {
		synchronized (this.repositoriesStateChangedListeners) {
			this.repositoriesStateChangedListeners.add(listener);
		}
	}

	public void removeRepositoriesStateChangedListener(IRepositoriesStateChangedListener listener) {
		synchronized (this.repositoriesStateChangedListeners) {
			this.repositoriesStateChangedListeners.remove(listener);
		}
	}

	public void fireRepositoriesStateChanged(RepositoriesStateChangedEvent event) {
		synchronized (this.repositoriesStateChangedListeners) {
			for (IRepositoriesStateChangedListener listener : this.repositoriesStateChangedListeners) {
				listener.repositoriesStateChanged(event);
			}
		}
	}

	public void addRevisionPropertyChangeListener(IRevisionPropertyChangeListener listener) {
		synchronized (this.revPropChangeListeners) {
			this.revPropChangeListeners.add(listener);
		}
	}

	public void fireRevisionPropertyChangeEvent(RevisonPropertyChangeEvent event) {
		synchronized (this.revPropChangeListeners) {
			for (IRevisionPropertyChangeListener current : this.revPropChangeListeners) {
				current.revisionPropertyChanged(event);
			}
		}
	}

	public void removeRevisionPropertyChangeListener(IRevisionPropertyChangeListener listener) {
		synchronized (this.revPropChangeListeners) {
			this.revPropChangeListeners.remove(listener);
		}
	}

	/**
	 * Return the preferences node whose child nodes are the repositories, stored in preferences.
	 * 
	 * @return preferences node
	 */
	public static IEclipsePreferences getRepositoriesPreferences(String prefNode) {
		return (IEclipsePreferences) ModelBusTeamPlugin.instance().getModelBusCorePreferences().node(prefNode);
	}

	public ModelBusCachedProxyCredentialsManager getProxyCredentialsManager() {
		return this.proxyCredentialsManager;
	}

	public synchronized void saveConfiguration() throws Exception {
		this.saveLocations();
	}

	public byte[] repositoryResourceAsBytes(IRepositoryResource resource) {
		if (resource == null) {
			return null;
		}
		int selectedKind = resource.getSelectedRevision().getKind();
		int pegKind = resource.getPegRevision().getKind();
		String retVal = new String(Base64.encode(String.valueOf(resource instanceof IRepositoryContainer).getBytes()))
				+ ";"
				+
				// CHECKMWA resource.getRepositoryLocation().getId() + ";" +
				new String(Base64.encode(resource.getUrl().getBytes())) + ";" + String.valueOf(selectedKind) + ";"
				+ (selectedKind == Kind.STRING ? ((ModelBusRevision.StringRevision) resource.getSelectedRevision()).getRevision() : "0") + ";"
				+ String.valueOf(IRepositoryRoot.KIND_ROOT) + ";" + String.valueOf(pegKind) + ";"
				+ (pegKind == Kind.STRING ? ((ModelBusRevision.StringRevision) resource.getPegRevision()).getRevision() : "0");
		return retVal.getBytes();
	}

	public IRepositoryResource repositoryResourceFromBytes(byte[] bytes) {
		if (bytes == null) {
			return null;
		}
		String[] data = new String(bytes).split(";");
		boolean isFolder = false;
		boolean base64Label = false;
		if ("true".equals(data[0])) {
			isFolder = true;
		} else if (!"false".equals(data[0])) {
			isFolder = "true".equals(new String(Base64.decode(data[0].getBytes())));
			base64Label = true;
		}

		// MES: Skipped location-ID above. Shift following data items to the left accordingly.
		// int revisionKind = Integer.parseInt(data[3]);
		int revisionKind = Integer.parseInt(data[2]);

		// long revNum = Long.parseLong(data[4]);
		String revNum = data[3];

		ModelBusRevision selectedRevision = (revisionKind == Kind.STRING ? (revNum == ModelBusRevision.INVALID_REVISION_STRING ? ModelBusRevision.INVALID_REVISION_DESCRIPTOR
				: (ModelBusRevision) ModelBusRevision.fromString(revNum))
				: ModelBusRevision.fromKind(revisionKind));
		ModelBusRevision pegRevision = null;
		// if (data.length > 6) {
		if (data.length > 5) {
			// int pegKind = Integer.parseInt(data[6]);
			int pegKind = Integer.parseInt(data[5]);
			// int pegNum = Integer.parseLong(data[7]);
			String pegNum = data[6];
			pegRevision = pegKind == Kind.STRING ? (pegNum == ModelBusRevision.INVALID_REVISION_STRING ? null : (ModelBusRevision) ModelBusRevision
					.fromString(pegNum)) : ModelBusRevision.fromKind(pegKind);
		}

		// String urlPart = base64Label ? new String(Base64.decode(data[2].getBytes())) : data[2];
		String urlPart = base64Label ? new String(Base64.decode(data[1].getBytes())) : data[1];

		try {
			ModelBusUtility.getModelBusUrl(urlPart);
		} catch (MalformedURLException e) {
			// old-style partial url
			// String prefix = AbstractModelBusStorage.getRootPrefix(Integer.parseInt(data[5]));
			String prefix = AbstractModelBusStorage.getRootPrefix(Integer.parseInt(data[4]));
			urlPart = prefix + urlPart;
		}

		// IRepositoryResource retVal = isFolder ? (IRepositoryResource)location.asRepositoryContainer(urlPart, false) :
		// location.asRepositoryFile(urlPart, false);
		IRepositoryResource retVal = isFolder ? ModelBusUtility.asRepositoryResource(urlPart, true) : ModelBusUtility.asRepositoryResource(urlPart,
				false);
		retVal.setSelectedRevision(selectedRevision);
		retVal.setPegRevision(pegRevision);
		return retVal;
		// throw new RuntimeException("TODOMWA31");
	}

	protected static String getRootPrefix(int rootKind) {
		// MES: Only one kind of repository root (IRepositoryRoot.KIND_ROOT) at the moment.
		// switch (rootKind) {
		// case IRepositoryRoot.KIND_ROOT: {
		// return location.getRepositoryRootUrl();
		// }
		// case IRepositoryRoot.KIND_LOCATION_ROOT: {
		// return location.getUrl();
		// }
		// case IRepositoryRoot.KIND_TRUNK: {
		// return location.isStructureEnabled() ? (location.getUrl() + "/" + location.getTrunkLocation()) : location.getUrl();
		// }
		// case IRepositoryRoot.KIND_BRANCHES: {
		// return location.isStructureEnabled() ? (location.getUrl() + "/" + location.getBranchesLocation()) : location.getUrl();
		// }
		// case IRepositoryRoot.KIND_TAGS: {
		// return location.isStructureEnabled() ? (location.getUrl() + "/" + location.getTagsLocation()) : location.getUrl();
		// }
		// }
		// return null;
		return ModelBusConnector.getRepositoryRootUrl();
	}

	protected void initializeImpl(String preferencesNode) throws Exception {
		IProxyService proxyService = ModelBusTeamPlugin.instance().getProxyService();
		this.proxyCredentialsManager = new ModelBusCachedProxyCredentialsManager(proxyService);

		proxyService.addProxyChangeListener(new IProxyChangeListener() {
			public void proxyInfoChanged(IProxyChangeEvent event) {
				IProxyData[] newDatas = event.getChangedProxyData();
				for (IProxyData current : newDatas) {
					if (current.isRequiresAuthentication()) {
						AbstractModelBusStorage.this.proxyCredentialsManager.setPassword(current.getPassword());
						AbstractModelBusStorage.this.proxyCredentialsManager.setUsername(current.getUserId());
						break;
					}
				}
				AbstractModelBusStorage.this.dispose();
			}
		});

		this.preferencesNode = preferencesNode;
		this.repoPrefChangeListener = new RepositoryPreferenceChangeListener();
		IEclipsePreferences repositoryPreferences = AbstractModelBusStorage.getRepositoriesPreferences(this.preferencesNode);
		repositoryPreferences.addPreferenceChangeListener(this.repoPrefChangeListener);
		// if the file exists, we should convert the data and delete the file.
		if (this.stateInfoFile.exists()) {
			this.loadLocationsFromFile();
			this.stateInfoFile.delete();
			this.saveLocations();
			return;
		}
		try {
			// this.loadLocations();
		} catch (Exception ex) {
			LoggedOperation.reportError(ModelBusTeamPlugin.instance().getResource("Error.LoadLocations"), ex);
		}
	}

	protected void saveLocations() throws Exception {
		IEclipsePreferences repositoryPreferences = AbstractModelBusStorage.getRepositoriesPreferences(this.preferencesNode);
		repositoryPreferences.removePreferenceChangeListener(this.repoPrefChangeListener);
		repositoryPreferences.clear();
		// for (IRepositoryLocation current : this.repositories) {
		// repositoryPreferences.put(current.getId(), this.repositoryLocationAsReference(current));
		// this.saveAuthInfo(current, "");
		// String [] realms = current.getRealms().toArray(new String[0]);
		// for (String realm : realms) {
		// if (current.isPasswordSavedForRealm(realm)) {
		// this.saveAuthInfo(current, realm);
		// }
		// }
		// }
		repositoryPreferences.flush();
		ModelBusTeamPlugin.instance().savePluginPreferences();
		repositoryPreferences.addPreferenceChangeListener(this.repoPrefChangeListener);
	}

	protected void saveAuthInfo(String realm) throws Exception {
		// Platform.flushAuthorizationInfo(new URL(AbstractModelBusStorage.URL_TO_STORE), location.getId(), realm);
		// IRepositoryLocation tmp = realm.equals("") ? location : location.getLocationForRealm(realm);
		// boolean toStorePass = tmp.isPasswordSaved();
		// HashMap<String, String> authInfo = new HashMap<String, String>();
		// CHECKMWA auth info
		// //store normal password settings
		// authInfo.put("username", tmp.getUsername());
		// authInfo.put("password", toStorePass ? tmp.getPassword() : "");
		// authInfo.put("password_saved", String.valueOf(toStorePass));
		//
		// //store SSH settings
		// SSHSettings sshSettings = tmp.getSSHSettings();
		// boolean useKeyFile = sshSettings.isUseKeyFile();
		// authInfo.put("ssh_use_key", String.valueOf(useKeyFile));
		// boolean savePassphrase = sshSettings.isPassPhraseSaved();
		// authInfo.put("ssh_passphrase_saved", useKeyFile ? String.valueOf(savePassphrase) : "false");
		// authInfo.put("ssh_key", useKeyFile ? sshSettings.getPrivateKeyPath() : "");
		// authInfo.put("ssh_passprase", (useKeyFile && savePassphrase) ? sshSettings.getPassPhrase() : "");
		//
		// //store SSL settings
		// SSLSettings sslSettings = tmp.getSSLSettings();
		// boolean clientAuthEnabled = sslSettings.isAuthenticationEnabled();
		// savePassphrase = sslSettings.isPassPhraseSaved();
		// authInfo.put("ssl_enabled", String.valueOf(clientAuthEnabled));
		// authInfo.put("ssl_certificate", clientAuthEnabled ? sslSettings.getCertificatePath() : "");
		// authInfo.put("ssl_passphrase_saved", clientAuthEnabled ? String.valueOf(savePassphrase) : "false");
		// authInfo.put("ssl_passphrase", (clientAuthEnabled && savePassphrase) ? sslSettings.getPassPhrase() : "");

		// store the map in platform
		// Platform.addAuthorizationInfo(new URL(AbstractModelBusStorage.URL_TO_STORE), location.getId(), realm, authInfo);
	}

	// protected void loadAuthInfo(IRepositoryLocation location, String realm) throws Exception {
	// Map<String, String> authInfo = Platform.getAuthorizationInfo(new URL(AbstractModelBusStorage.URL_TO_STORE), location.getId(), realm);
	// if (authInfo != null) {
	// IRepositoryLocation tmp;
	// boolean toAddRealm = !realm.equals("");
	// if (toAddRealm) {
	// tmp = this.newRepositoryLocation();
	// }
	// else {
	// tmp = location;
	// }
	//
	// //recover normal password settings
	// tmp.setUsername(authInfo.get("username"));
	// tmp.setPasswordSaved(authInfo.get("password_saved").equals("true"));
	// tmp.setPassword(authInfo.get("password"));
	//
	// //recover SSH settings
	// SSHSettings sshSettings = tmp.getSSHSettings();
	// sshSettings.setUseKeyFile(authInfo.get("ssh_use_key").equals("true"));
	// sshSettings.setPrivateKeyPath(authInfo.get("ssh_key"));
	// sshSettings.setPassPhraseSaved(authInfo.get("ssh_passphrase_saved").equals("true"));
	// sshSettings.setPassPhrase(authInfo.get("ssh_passprase"));
	//
	// //recover SSL settings
	// SSLSettings sslSettings = tmp.getSSLSettings();
	// sslSettings.setAuthenticationEnabled(authInfo.get("ssl_enabled").equals("true"));
	// sslSettings.setCertificatePath(authInfo.get("ssl_certificate"));
	// sslSettings.setPassPhraseSaved(authInfo.get("ssl_passphrase_saved").equals("true"));
	// sslSettings.setPassPhrase(authInfo.get("ssl_passphrase"));
	//
	// //if realm, add it to realms
	// if (toAddRealm) {
	// location.addRealm(realm, tmp);
	// }
	// }
	// }

	// public void removeAuthInfoForLocation(IRepositoryLocation location, String realm) {
	// try {
	// Platform.flushAuthorizationInfo(new URL(AbstractModelBusStorage.URL_TO_STORE), location.getId(), realm);
	// }
	// catch (Exception ex) {
	// LoggedOperation.reportError("Remove Authorization Info Operation", ex);
	// }
	// }

	// protected void loadLocations() throws Exception {
	// IEclipsePreferences repositoryPreferences = AbstractModelBusStorage.getRepositoriesPreferences(this.preferencesNode);
	// String [] keys = repositoryPreferences.keys();
	// ArrayList<IRepositoryLocation> readLocations = new ArrayList<IRepositoryLocation>();
	// for (String current : keys) {
	// IRepositoryLocation location = this.newRepositoryLocation(repositoryPreferences.get(current, null));
	// readLocations.add(location);
	// }
	// this.repositories = readLocations.toArray(new IRepositoryLocation[readLocations.size()]);
	// }

	/**
	 * Sets the file, containing locations info preferences. Used for compatibility with the versions of Subversive, where the locations info was
	 * stored in files.
	 */
	protected void setStateInfoFile(IPath stateInfoLocation, String fileName) {
		this.stateInfoFile = stateInfoLocation.append(fileName).toFile();
	}

	/**
	 * Loads locations info from a file. Left for compatibility with earlier versions - to have a probability to read locations and convert to
	 * preferences. Used for compatibility with the versions of Subversive, where the locations info was stored in files.
	 * 
	 * @throws Exception
	 */
	protected void loadLocationsFromFile() throws Exception {
		// List<IRepositoryLocation> tmp = new ArrayList<IRepositoryLocation>(Arrays.asList(this.repositories));
		// ObjectInputStream stream = null;
		// try {
		// stream = new ObjectInputStream(new FileInputStream(this.stateInfoFile));
		//
		// // stream.available() does not provide any EOF information
		// while (true) {
		// ModelBusRepositoryLocation obj = (ModelBusRepositoryLocation)stream.readObject();
		// if (!tmp.contains(obj)) {
		// tmp.add(obj);
		// }
		// }
		// }
		// catch (EOFException ex) {
		// // EOF, do nothing
		// }
		// finally {
		// if (stream != null) {
		// try {stream.close();} catch (Exception ex) {}
		// }
		// }
		// this.repositories = tmp.toArray(new IRepositoryLocation[tmp.size()]);
	}

}
