/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.modelbusstorage;

import java.io.Serializable;

import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Depth;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusConnectorException;
import org.modelbus.team.eclipse.core.connector.ModelBusEntry;
import org.modelbus.team.eclipse.core.connector.ModelBusEntry.Fields;
import org.modelbus.team.eclipse.core.connector.ModelBusEntry.Kind;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryInfo;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryRevisionReference;
import org.modelbus.team.eclipse.core.connector.ModelBusProperty;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.operation.ModelBusNullProgressMonitor;
import org.modelbus.team.eclipse.core.resource.IRepositoryContainer;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;

/**
 * ModelBus based representation of IRepositoryContainer
 * 
 * @author Alexander Gurov
 */
public class ModelBusRepositoryContainer extends ModelBusRepositoryResource implements IRepositoryContainer, Serializable {
	private static final long					serialVersionUID	= -6380931196819185798L;

	protected transient IRepositoryResource[]	children;

	// serialization conventional constructor
	protected ModelBusRepositoryContainer() {
		super();
	}

	public ModelBusRepositoryContainer(String url, ModelBusRevision selectedRevision) {
		super(url, selectedRevision);
	}

	public boolean isChildrenCached() {
		return this.children != null;
	}

	public synchronized void refresh() {
		super.refresh();
		this.children = null;
	}

	public IRepositoryResource[] getChildren() throws ModelBusConnectorException {
		IRepositoryResource[] retVal = this.children;

		// synchronize only assignment in order to avoid deadlock with this Sync and UI Sync locked from callback
		// in result we can perform excessive work but it is acceptable in that case
		if (retVal == null) {
			String thisUrl = this.getUrl();
			ModelBusEntry[] children = null;

			IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
			try {
				children = ModelBusUtility.list(proxy, ModelBusUtility.getEntryRevisionReference(this), Depth.IMMEDIATES, Fields.ALL,
						IModelBusConnector.Options.FETCH_LOCKS, new ModelBusNullProgressMonitor());
			} finally {
			}

			synchronized (this) {
				retVal = new IRepositoryResource[children.length];

				for (int i = 0; i < children.length; i++) {
					if (children[i].revision == ModelBusRevision.INVALID_REVISION_DESCRIPTOR) {
						// FIXME -1 for ModelBus Kit 1.2.0 if resource is not exists
						throw new ModelBusConnectorException("-1 for ModelBus Kit 1.2.0 if resource is not exists");
					}
					String childUrl =
					// thisUrl + "/" +
					children[i].path;
					ModelBusRepositoryResource resource = children[i].nodeKind == Kind.DIR ? (ModelBusRepositoryResource) this.asRepositoryContainer(
							childUrl, false) : (ModelBusRepositoryResource) this.asRepositoryFile(childUrl, false);
					resource.setRevision(children[i].revision);
					resource.setInfo(new IRepositoryResource.Information(children[i].lock, children[i].size, children[i].author, children[i].revision.getDate(),
							children[i].hasProperties));
					retVal[i] = resource;
				}

				this.children = retVal;
			}
		}

		return retVal;
	}

	protected void getRevisionImpl(IModelBusConnector proxy) throws ModelBusConnectorException {
		ModelBusEntryRevisionReference reference = ModelBusUtility.getEntryRevisionReference(this);
		ModelBusEntryInfo[] infos = ModelBusUtility.info(proxy, reference, Depth.EMPTY, new ModelBusNullProgressMonitor());
		if (infos != null && infos.length > 0 && infos[0].lastChangedRevision != ModelBusRevision.INVALID_REVISION_DESCRIPTOR) {
			this.lastRevision = infos[0].lastChangedRevision;
			ModelBusProperty[] data = ModelBusUtility.properties(proxy, reference, new ModelBusNullProgressMonitor());
			this.setInfo(new IRepositoryResource.Information(infos[0].lock, 0, infos[0].lastChangedAuthor, infos[0].lastChangedDate, data != null
					&& data.length > 0));
		}
	}

	public boolean equals(Object obj) {
		if (obj == null || !(obj instanceof IRepositoryContainer)) {
			return false;
		}
		return super.equals(obj);
	}

}
