/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.modelbusstorage;

import java.io.Serializable;

import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusConnectorCancelException;
import org.modelbus.team.eclipse.core.connector.ModelBusConnectorException;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.resource.IRepositoryContainer;
import org.modelbus.team.eclipse.core.resource.IRepositoryFile;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryRoot;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;

/**
 * ModelBus based representation of IRepositoryResource
 * 
 * @author Alexander Gurov
 */
public abstract class ModelBusRepositoryResource extends ModelBusRepositoryBase implements IRepositoryResource, Serializable {
	private static final long							serialVersionUID	= 8854704746872311777L;

	private transient ModelBusRevision					selectedRevision;							// should be managed using setters and getters
																									// regarding to "transient" modifier
	private transient ModelBusRevision					pegRevision;								// revision where we found this item
	protected transient ModelBusRevision				lastRevision;
	protected transient IRepositoryRoot					root;
	protected transient IRepositoryResource.Information	info;

	// serialization conventional constructor
	protected ModelBusRepositoryResource() {
		super();
	}

	public ModelBusRepositoryResource(String url, ModelBusRevision selectedRevision) {
		super(url);
		this.selectedRevision = selectedRevision;
	}

	public void setInfo(IRepositoryResource.Information info) {
		this.info = info;
	}

	public Information getInfo() {
		return this.info;
	}

	public ModelBusRevision getPegRevision() {
		return this.pegRevision == null ? ModelBusRevision.HEAD : this.pegRevision;
	}

	public void setPegRevision(ModelBusRevision pegRevision) {
		this.pegRevision = pegRevision;
	}

	public ModelBusRevision getSelectedRevision() {
		if (this.selectedRevision == null) {
			this.selectedRevision = ModelBusRevision.HEAD;
		}
		return this.selectedRevision;
	}

	public void setSelectedRevision(ModelBusRevision revision) {
		this.selectedRevision = revision;
	}

	public boolean isInfoCached() {
		return this.lastRevision != null;
	}

	public synchronized void refresh() {
		this.lastRevision = null;
	}

	public void setRevision(ModelBusRevision revisionString) {
		this.lastRevision = revisionString;
	}

	public synchronized ModelBusRevision getRevision() throws ModelBusConnectorException {
		if (this.lastRevision == null) {
			this.lastRevision = ModelBusRevision.INVALID_REVISION_DESCRIPTOR;
			IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
			try {
				this.getRevisionImpl(proxy);
			} finally {
			}
		}
		return this.lastRevision;
	}

	public boolean exists() throws ModelBusConnectorException {
		try {
			return this.getRevision() != ModelBusRevision.INVALID_REVISION_DESCRIPTOR;
		} catch (ModelBusConnectorException ex) {
			// FIXME uncomment this when the WI is resolved ("Unknown node kind" exception instead of "Path not found" (PLC-1008))
			// if (ex instanceof ClientExceptionEx) {
			// if (((ClientExceptionEx)ex).getErrorMessage().getErrorCode().equals(ModelBusErrorCode.RA_DAV_PATH_NOT_FOUND)) {
			// return false;
			// }
			// }
			// throw ex;
			if (ex instanceof ModelBusConnectorCancelException) {
				throw ex;
			}
			return false;
		}
	}

	public IRepositoryResource getParent() {
		String parentUrl = ModelBusUtility.normalizeURL(this.getUrl());
		int idx = parentUrl.lastIndexOf('/');
		if (idx == -1) {
			throw new IllegalArgumentException(parentUrl);
		}
		return this.asRepositoryContainer(parentUrl.substring(0, idx), true);
	}

	public IRepositoryResource getRoot() {
		if (this.root == null) {
			IRepositoryResource parent = this;
			while (!(parent instanceof IRepositoryRoot)) {
				parent = parent.getParent();
			}
			this.root = (IRepositoryRoot) parent;
		}
		return this.root;
	}

	public boolean equals(Object obj) {
		if (obj == null || !(obj instanceof IRepositoryResource)) {
			return false;
		}
		IRepositoryResource other = (IRepositoryResource) obj;
		return super.equals(obj) && this.getSelectedRevision().equals(other.getSelectedRevision())
				&& this.getPegRevision().equals(other.getPegRevision());
	}

	public IRepositoryContainer asRepositoryContainer(String url, boolean allowsNull) {
		String thisUrl = this.getUrl();
		if ("http://".equals(thisUrl)) {
			thisUrl = thisUrl.substring(0, 6);
		}
		IRepositoryContainer retVal = ModelBusConnector.asRepositoryContainer(url.indexOf('/') != -1 ? url : (/* this.getUrl() */thisUrl + "/" + url),
				allowsNull);
		if (retVal == null) {
			return null;
		}
		retVal.setPegRevision(this.getPegRevision());
		retVal.setSelectedRevision(this.getSelectedRevision());
		return retVal;
	}

	public IRepositoryFile asRepositoryFile(String url, boolean allowsNull) {
		IRepositoryFile retVal = ModelBusConnector.asRepositoryFile(url.indexOf('/') != -1 ? url : (this.getUrl() + "/" + url), allowsNull);
		if (retVal == null) {
			return null;
		}
		retVal.setPegRevision(this.getPegRevision());
		retVal.setSelectedRevision(this.getSelectedRevision());
		return retVal;
	}

	protected abstract void getRevisionImpl(IModelBusConnector proxy) throws ModelBusConnectorException;

}
