/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.file.management;

import java.io.File;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.ModelBusTeamPlugin;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Depth;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.operation.file.AbstractFileOperation;
import org.modelbus.team.eclipse.core.operation.file.IFileProvider;
import org.modelbus.team.eclipse.core.operation.local.management.ShareProjectOperation;
import org.modelbus.team.eclipse.core.resource.IRepositoryContainer;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.core.utility.ProgressMonitorUtility;

/**
 * Share selected folders with the specified repository
 * 
 * @author Alexander Gurov
 */
public class ShareOperation extends AbstractFileOperation {
	public static final int	LAYOUT_DEFAULT	= 0;
	public static final int	LAYOUT_SINGLE	= 1;
	public static final int	LAYOUT_MULTIPLE	= 2;

	public interface IFolderNameMapper {
		public String getRepositoryFolderName(File folder);
	}

	protected IFolderNameMapper	mapper;
	protected String			rootName;
	protected int				shareLayout;
	protected boolean			managementFoldersEnabled;
	protected String			commitComment;

	public ShareOperation(File[] files, IFolderNameMapper mapper, String rootName, int shareLayout, boolean managementFoldersEnabled,
			String commitComment) {
		super("Operation.ShareFile", files);
		this.mapper = mapper;
		this.rootName = rootName;
		this.shareLayout = shareLayout;
		this.managementFoldersEnabled = managementFoldersEnabled;
		this.commitComment = commitComment;
	}

	public ShareOperation(IFileProvider provider, IFolderNameMapper mapper, String rootName, int shareLayout, boolean managementFoldersEnabled,
			String commitComment) {
		super("Operation.ShareFile", provider);
		this.mapper = mapper;
		this.rootName = rootName;
		this.shareLayout = shareLayout;
		this.managementFoldersEnabled = managementFoldersEnabled;
		this.commitComment = commitComment;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		File[] folders = FileUtility.shrinkChildNodes(this.operableData(), true);
		HashMap<File, IRepositoryContainer> local2remote = new HashMap<File, IRepositoryContainer>();
		for (int i = 0; i < folders.length; i++) {
			String url = this.getTargetUrl(this.mapper == null ? folders[i].getName() : this.mapper.getRepositoryFolderName(folders[i]),
					this.managementFoldersEnabled);
			IRepositoryContainer remote = ModelBusConnector.asRepositoryContainer(url, false);
			local2remote.put(folders[i], remote);
		}
		if (this.commitComment == null) {
			this.commitComment = "";
			for (Iterator<?> it = local2remote.entrySet().iterator(); it.hasNext();) {
				Map.Entry entry = (Map.Entry) it.next();
				String commentPart = ShareOperation.getDefaultComment((File) entry.getKey(), (IRepositoryContainer) entry.getValue());
				this.commitComment += this.commitComment.length() == 0 ? commentPart : ("\n" + commentPart);
			}
		}

		final IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		try {
			IRepositoryResource[] resourceSet = null;
			switch (this.shareLayout) {
				case ShareProjectOperation.LAYOUT_DEFAULT: {
					resourceSet = this.doDefaultLayout(local2remote);
					break;
				}
				case ShareProjectOperation.LAYOUT_SINGLE: {
					resourceSet = this.doSingleLayout(local2remote);
					break;
				}
				case ShareProjectOperation.LAYOUT_MULTIPLE: {
					resourceSet = this.doMultipleLayout(local2remote);
					break;
				}
				default: {
					String message = this.getNationalizedString("Error.UnknownProjectLayoutType");
					throw new Exception(MessageFormat.format(message, new Object[] { String.valueOf(this.shareLayout) }));
				}
			}

			this.mkdir(proxy, resourceSet, monitor);

			for (Iterator<?> it = local2remote.entrySet().iterator(); it.hasNext() && !monitor.isCanceled();) {
				final Map.Entry entry = (Map.Entry) it.next();
				this.protectStep(new IUnprotectedOperation() {
					public void run(IProgressMonitor monitor) throws Exception {
						IRepositoryContainer remote = (IRepositoryContainer) entry.getValue();
						File local = (File) entry.getKey();
						proxy.checkout(ModelBusUtility.getEntryRevisionReference(remote), local.getAbsolutePath(), Depth.EMPTY,
								IModelBusConnector.Options.NONE, new ModelBusProgressMonitor(ShareOperation.this, monitor, null));
					}
				}, monitor, local2remote.size());
			}
		} finally {
		}
	}

	public static String getDefaultComment(File folder, IRepositoryResource remote) {
		return ModelBusTeamPlugin.instance().getResource("Operation.ShareFile.DefaultComment", new String[] { folder.getName(), remote.getUrl() });
	}

	public String getTargetUrl(String projectName, boolean managementFoldersEnabled) {
		return ModelBusUtility.normalizeURL(this.getTargetUrlImpl(projectName, managementFoldersEnabled));
	}

	protected String getTargetUrlImpl(String projectName, boolean managementFoldersEnabled) {
		// String trunkName = managementFoldersEnabled ? ("/" + this.getTrunkName()) : "";
		// switch (this.shareLayout) {
		// case ShareProjectOperation.LAYOUT_DEFAULT: {
		// return this.location.getUrl() + trunkName + "/" + projectName;
		// }
		// case ShareProjectOperation.LAYOUT_SINGLE: {
		// return this.location.getUrl() + "/" + projectName + trunkName;
		// }
		// case ShareProjectOperation.LAYOUT_MULTIPLE: {
		// return this.location.getUrl() + "/" + this.rootName + trunkName + "/" + projectName;
		// }
		// default: {
		// String message = this.getNationalizedString("Error.UnknownProjectLayoutType");
		// throw new RuntimeException(MessageFormat.format(message, new Object[] {String.valueOf(this.shareLayout)}));
		// }
		// }
		throw new RuntimeException("TODOMWA32");
	}

	protected void mkdir(IModelBusConnector proxy, IRepositoryResource[] resourceSet, IProgressMonitor monitor) throws Exception {
		ArrayList<String> urlsList = new ArrayList<String>();
		for (int i = 0; i < resourceSet.length && !monitor.isCanceled(); i++) {
			ProgressMonitorUtility.setTaskInfo(monitor, this, resourceSet[i].getUrl());
			if (!resourceSet[i].exists()) {
				urlsList.add(ModelBusUtility.encodeURL(resourceSet[i].getUrl()));
			}
			ProgressMonitorUtility.progress(monitor, IProgressMonitor.UNKNOWN, resourceSet.length);
		}
		String[] urls = urlsList.toArray(new String[urlsList.size()]);
		proxy.mkdir(urls, this.commitComment, IModelBusConnector.Options.INCLUDE_PARENTS, null, new ModelBusProgressMonitor(this, monitor, null));
	}

	protected IRepositoryResource[] doDefaultLayout(Map<File, IRepositoryContainer> local2remote) {
		HashSet<IRepositoryResource> fullSet = new HashSet<IRepositoryResource>();
		for (Iterator<IRepositoryContainer> it = local2remote.values().iterator(); it.hasNext();) {
			IRepositoryContainer remote = it.next();
			IRepositoryResource[] resources = ModelBusUtility.makeResourceSet(ModelBusConnector.getRepositoryRoot(), remote);
			fullSet.addAll(Arrays.asList(resources));
		}
		return this.getOrderedSet(fullSet);
	}

	protected IRepositoryResource[] doSingleLayout(Map<File, IRepositoryContainer> local2remote) {
		if (this.managementFoldersEnabled) {
			HashSet<IRepositoryResource> fullSet = new HashSet<IRepositoryResource>();
			for (Iterator<IRepositoryContainer> it = local2remote.values().iterator(); it.hasNext();) {
				IRepositoryContainer remote = it.next();
				IRepositoryResource[] resources = ModelBusUtility.makeResourceSet(ModelBusConnector.getRepositoryRoot(), remote);
				fullSet.addAll(Arrays.asList(resources));
				IRepositoryContainer parent = (IRepositoryContainer) remote.getParent();
				fullSet.add(this.makeChild(parent, this.getTagsName()));
				fullSet.add(this.makeChild(parent, this.getBranchesName()));
			}
			return this.getOrderedSet(fullSet);
		}
		return this.doDefaultLayout(local2remote);
	}

	protected IRepositoryResource[] doMultipleLayout(Map<File, IRepositoryContainer> local2remote) {
		if (this.managementFoldersEnabled) {
			HashSet<IRepositoryResource> fullSet = new HashSet<IRepositoryResource>();
			for (Iterator<IRepositoryContainer> it = local2remote.values().iterator(); it.hasNext();) {
				IRepositoryContainer remote = it.next();
				IRepositoryResource[] resources = ModelBusUtility.makeResourceSet(ModelBusConnector.getRepositoryRoot(), remote);
				fullSet.addAll(Arrays.asList(resources));

				String targetUrl = this.getTargetUrl("", false);
				IRepositoryContainer parent = ModelBusConnector.asRepositoryContainer(targetUrl, false);
				fullSet.add(this.makeChild(parent, this.getTagsName()));
				fullSet.add(this.makeChild(parent, this.getBranchesName()));
			}
			return this.getOrderedSet(fullSet);
		}
		return this.doDefaultLayout(local2remote);
	}

	protected IRepositoryContainer makeChild(IRepositoryContainer parent, String name) {
		return ModelBusConnector.asRepositoryContainer(parent.getUrl() + "/" + name, false);
	}

	protected IRepositoryResource[] getOrderedSet(Set<IRepositoryResource> fullSet) {
		IRepositoryResource[] resources = fullSet.toArray(new IRepositoryResource[fullSet.size()]);
		Arrays.sort(resources, new Comparator<Object>() {
			public int compare(Object arg0, Object arg1) {
				IRepositoryResource first = (IRepositoryResource) arg0;
				IRepositoryResource second = (IRepositoryResource) arg1;
				return first.getUrl().compareTo(second.getUrl());
			}
		});
		return resources;
	}

	protected String getTrunkName() {
		if (true) {
			System.out.println("TODOMWA22");
			// return this.location.getTrunkLocation();
		}
		return ModelBusConnector.getRepositoryRootUrl();
		// return ModelBusTeamPlugin.instance().getOptionProvider().getDefaultTrunkName();
	}

	protected String getBranchesName() {
		// if (this.location.isStructureEnabled()) {
		// return this.location.getBranchesLocation();
		// }
		System.out.println("TODOMWA23");
		return ModelBusConnector.getRepositoryRootUrl();
		// return ModelBusTeamPlugin.instance().getOptionProvider().getDefaultBranchesName();
	}

	protected String getTagsName() {
		// if (this.location.isStructureEnabled()) {
		// return this.location.getTagsLocation();
		// }
		System.out.println("TODOMWA24");
		return ModelBusConnector.getRepositoryRootUrl();

		// return ModelBusTeamPlugin.instance().getOptionProvider().getDefaultTagsName();
	}

}
