/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.file.refactor;

import java.io.File;
import java.io.FileFilter;
import java.text.MessageFormat;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.jobs.ISchedulingRule;
import org.eclipse.core.runtime.jobs.MultiRule;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.operation.UnreportableException;
import org.modelbus.team.eclipse.core.operation.file.AbstractFileOperation;
import org.modelbus.team.eclipse.core.operation.file.IFileProvider;
import org.modelbus.team.eclipse.core.operation.file.ModelBusFileStorage;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.core.utility.ProgressMonitorUtility;

/**
 * Move resources between folders in WC
 * 
 * @author Alexander Gurov
 */
public class MoveOperation extends AbstractFileOperation {
	protected File		localTo;
	protected boolean	forceNonModelBus;

	public MoveOperation(File[] files, File localTo, boolean forceNonModelBus) {
		super("Operation.MoveFile", files);
		this.localTo = localTo;
		this.forceNonModelBus = forceNonModelBus;
	}

	public MoveOperation(IFileProvider provider, File localTo, boolean forceNonModelBus) {
		super("Operation.MoveFile", provider);
		this.localTo = localTo;
		this.forceNonModelBus = forceNonModelBus;
	}

	public ISchedulingRule getSchedulingRule() {
		ISchedulingRule parentRule = super.getSchedulingRule();
		return MultiRule.combine(new LockingRule(this.localTo), parentRule);
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		File[] files = this.operableData();
		// allows moving of child and parent resources at the same time
		FileUtility.reorder(files, false);

		IRepositoryResource remoteTo = ModelBusFileStorage.instance().asRepositoryResource(this.localTo, true);
		final IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		for (int i = 0; i < files.length && !monitor.isCanceled(); i++) {
			final File current = files[i];
			this.protectStep(new IUnprotectedOperation() {
				public void run(IProgressMonitor monitor) throws Exception {
					IRepositoryResource remote = ModelBusFileStorage.instance().asRepositoryResource(current, true);
					File checked = MoveOperation.this.getRenameTo(current);
					if (remote == null) {
						MoveOperation.this.nonModelBusMove(checked, current, monitor);
					} else if (proxy == null || MoveOperation.this.forceNonModelBus) {
						MoveOperation.this.nonModelBusCopy(current, monitor);
						ProgressMonitorUtility.doTaskExternal(new DeleteOperation(new File[] { current }), monitor);
					} else {
						proxy.move(new String[] { current.getAbsolutePath() }, checked.getAbsolutePath(), IModelBusConnector.Options.FORCE,
								new ModelBusProgressMonitor(MoveOperation.this, monitor, null));
					}
				}
			}, monitor, files.length);
		}

	}

	protected File getRenameTo(File what) {
		File checked = new File(this.localTo.getAbsolutePath() + "/" + what.getName());
		if (checked.exists()) {
			String message = this.getNationalizedString("Error.AlreadyExists");
			throw new UnreportableException(MessageFormat.format(message, new Object[] { checked.getAbsolutePath() }));
		}
		return checked;
	}

	protected void nonModelBusMove(File renameTo, File what, IProgressMonitor monitor) throws Exception {
		if (!what.renameTo(renameTo)) {
			this.nonModelBusCopy(what, monitor);
			FileUtility.deleteRecursive(what, monitor);
		}
	}

	protected void nonModelBusCopy(File what, IProgressMonitor monitor) throws Exception {
		FileUtility.copyAll(this.localTo, what, FileUtility.COPY_NO_OPTIONS, new FileFilter() {
			public boolean accept(File pathname) {
				return !pathname.getName().equals(ModelBusUtility.getModelBusFolderName());
			}
		}, monitor);
	}

}
