/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.local;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Depth;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusNotification.NodeStatus;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRemoteStorage;
import org.modelbus.team.eclipse.core.operation.IActionOperation;
import org.modelbus.team.eclipse.core.operation.IConsoleStream;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.resource.IRemoteStorage;
import org.modelbus.team.eclipse.core.resource.IResourceProvider;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;

/**
 * Update operation implementation
 * 
 * @author Alexander Gurov
 */
public class UpdateOperation extends AbstractConflictDetectionOperation implements IResourceProvider {
	protected ModelBusRevision	selectedRevision;
	protected boolean			doRecursiveUpdate;

	public UpdateOperation(IResource[] resources, boolean doRecursiveUpdate) {
		this(resources, null, doRecursiveUpdate);
	}

	public UpdateOperation(IResourceProvider provider, boolean doRecursiveUpdate) {
		this(provider, null, doRecursiveUpdate);
	}

	public UpdateOperation(IResourceProvider provider, ModelBusRevision selectedRevision, boolean doRecursiveUpdate) {
		super("Operation.Update", provider);
		this.doRecursiveUpdate = doRecursiveUpdate;
		this.selectedRevision = selectedRevision == null ? ModelBusRevision.HEAD : selectedRevision;
	}

	public UpdateOperation(IResource[] resources, ModelBusRevision selectedRevision, boolean doRecursiveUpdate) {
		super("Operation.Update", resources);
		this.selectedRevision = selectedRevision == null ? ModelBusRevision.HEAD : selectedRevision;
		this.doRecursiveUpdate = doRecursiveUpdate;
	}

	public int getOperationWeight() {
		return 19;
	}

	public IResource[] getResources() {
		return this.getProcessed();
	}

	protected void runImpl(final IProgressMonitor monitor) throws Exception {
		IResource[] resources = this.operableData();

		// container with resources that is really updated
		this.defineInitialResourceSet(resources);

		IRemoteStorage storage = ModelBusRemoteStorage.instance();
		Map wc2Resources = ModelBusUtility.splitWorkingCopies(resources);
		for (Iterator it = wc2Resources.entrySet().iterator(); it.hasNext() && !monitor.isCanceled();) {
			Map.Entry entry = (Map.Entry) it.next();
			IResource[] wcResources = (IResource[]) ((List) entry.getValue()).toArray(new IResource[0]);
			if (this.doRecursiveUpdate) {
				wcResources = FileUtility.shrinkChildNodes(wcResources);
			} else {
				FileUtility.reorder(wcResources, true);
			}
			final String[] paths = FileUtility.asPathArray(wcResources);

			this.complexWriteToConsole(new Runnable() {
				public void run() {
					UpdateOperation.this.writeToConsole(IConsoleStream.LEVEL_CMD, "modelbus update");
					for (int i = 0; i < paths.length && !monitor.isCanceled(); i++) {
						UpdateOperation.this.writeToConsole(IConsoleStream.LEVEL_CMD, " \"" + paths[i] + "\"");
					}
					UpdateOperation.this.writeToConsole(IConsoleStream.LEVEL_CMD, " -r " + UpdateOperation.this.selectedRevision
							+ (UpdateOperation.this.doRecursiveUpdate ? "" : " -N") + FileUtility.getUsernameParam(ModelBusConnector.getUserName())
							+ "\n");
				}
			});

			final IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
			this.protectStep(new IUnprotectedOperation() {
				public void run(IProgressMonitor monitor) throws Exception {
					proxy.update(paths, UpdateOperation.this.selectedRevision, Depth.unknownOrFiles(UpdateOperation.this.doRecursiveUpdate),
							IModelBusConnector.Options.NONE, new ConflictDetectionProgressMonitor(UpdateOperation.this, monitor, null));
				}
			}, monitor, wc2Resources.size());
		}
	}

	protected class ConflictDetectionProgressMonitor extends ModelBusProgressMonitor {
		public ConflictDetectionProgressMonitor(IActionOperation parent, IProgressMonitor monitor, IPath root) {
			super(parent, monitor, root);
		}

		public void progress(int current, int total, ItemState state) {
			super.progress(current, total, state);
			if (state.contentState == NodeStatus.CONFLICTED || state.propState == NodeStatus.CONFLICTED) {
				UpdateOperation.this.hasUnresolvedConflict = true;
				for (Iterator<IResource> it = UpdateOperation.this.processed.iterator(); it.hasNext();) {
					IResource res = it.next();
					IPath conflictPath = new Path(state.path);
					IPath resourcePath = FileUtility.getResourcePath(res);
					if (resourcePath.isPrefixOf(conflictPath)) {
						if (resourcePath.equals(conflictPath)) {
							it.remove();
						}
						IResource conflictResource = ResourcesPlugin.getWorkspace().getRoot()
								.findMember(res.getFullPath().append(conflictPath.removeFirstSegments(resourcePath.segmentCount())));
						if (conflictResource != null) {
							UpdateOperation.this.unprocessed.add(conflictResource);
						}
						break;
					}
				}
			}
		}

	}

}
