/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Sergiy Logvin (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.local.property;

import java.util.HashMap;
import java.util.Set;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceVisitor;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.IStateFilter;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryRevisionReference;
import org.modelbus.team.eclipse.core.connector.ModelBusProperty;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRemoteStorage;
import org.modelbus.team.eclipse.core.operation.AbstractActionOperation;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.resource.ILocalResource;
import org.modelbus.team.eclipse.core.resource.IResourceProvider;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.core.utility.ProgressMonitorUtility;

/**
 * Operation to get properties for multiple resources
 * 
 * @author Sergiy Logvin
 */
public class GetMultiPropertiesOperation extends AbstractActionOperation implements IResourceProvider, IPropertyProvider {
	protected IResource[]								resources;
	protected HashMap<IResource, ModelBusProperty[]>	properties;
	protected String									propertyName;
	protected int										depth;
	protected IStateFilter								filter;

	/**
	 * @param resources
	 *            the resources for which the properties are requested
	 * @param propertyName
	 *            the name of a property which value is requested. If null then all the properties will be computed for the resources.
	 */
	public GetMultiPropertiesOperation(IResource[] resources, int depth, IStateFilter filter, String propertyName) {
		super("Operation.GetMultiProperties");
		this.resources = resources;
		this.propertyName = propertyName;
		this.filter = filter != null ? filter : IStateFilter.SF_VERSIONED;
		this.properties = new HashMap<IResource, ModelBusProperty[]>();
		this.depth = depth;
	}

	public IResource[] getResources() {
		Set<IResource> resources = this.properties.keySet();
		return resources.toArray(new IResource[resources.size()]);
	}

	public ModelBusProperty[] getProperties(IResource resource) {
		return this.properties.get(resource);
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		for (int i = 0; i < this.resources.length && !monitor.isCanceled(); i++) {
			final IResource current = this.resources[i];
			final IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
			try {
				this.protectStep(new IUnprotectedOperation() {
					public void run(final IProgressMonitor monitor) throws Exception {
						FileUtility.visitNodes(current, new IResourceVisitor() {
							public boolean visit(IResource resource) throws CoreException {
								if (monitor.isCanceled()) {
									return false;
								}
								ILocalResource local = ModelBusRemoteStorage.instance().asLocalResourceAccessible(resource);
								if (GetMultiPropertiesOperation.this.filter.accept(local)) {
									GetMultiPropertiesOperation.this.processResource(proxy, resource, monitor);
								}
								return GetMultiPropertiesOperation.this.filter.allowsRecursion(local);
							}
						}, GetMultiPropertiesOperation.this.depth);
					}
				}, monitor, this.resources.length);
			} finally {
			}
		}
	}

	protected void processResource(final IModelBusConnector proxy, final IResource current, IProgressMonitor monitor) {
		ProgressMonitorUtility.setTaskInfo(monitor, this, current.getFullPath().toString());
		this.protectStep(new IUnprotectedOperation() {
			public void run(IProgressMonitor monitor) throws Exception {
				String wcPath = FileUtility.getWorkingCopyPath(current);
				if (GetMultiPropertiesOperation.this.propertyName != null) {
					ModelBusProperty data = proxy.getProperty(new ModelBusEntryRevisionReference(wcPath, null, ModelBusRevision.WORKING),
							GetMultiPropertiesOperation.this.propertyName, new ModelBusProgressMonitor(GetMultiPropertiesOperation.this, monitor,
									null));
					if (data != null) {
						GetMultiPropertiesOperation.this.properties.put(current, new ModelBusProperty[] { data });
					}
				} else {
					ModelBusProperty[] data = ModelBusUtility.properties(proxy, new ModelBusEntryRevisionReference(wcPath, null,
							ModelBusRevision.WORKING), new ModelBusProgressMonitor(GetMultiPropertiesOperation.this, monitor, null));
					if (data != null && data.length != 0) {
						GetMultiPropertiesOperation.this.properties.put(current, data);
					}
				}
			}
		}, monitor, 1);
	}

}
