/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.remote;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.InputStream;
import java.lang.reflect.InvocationTargetException;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.internal.resources.mapping.SimpleResourceMapping;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceVisitor;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.core.runtime.jobs.ISchedulingRule;
import org.eclipse.core.runtime.jobs.MultiRule;
import org.eclipse.team.core.RepositoryProvider;
import org.eclipse.team.core.TeamException;
import org.eclipse.team.core.variants.IResourceVariant;
import org.eclipse.team.core.variants.ThreeWaySynchronizer;
import org.modelbus.core.lib.IRepositoryHelper;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.operation.AbstractActionOperation;
import org.modelbus.team.eclipse.core.operation.IConsoleStream;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.operation.ModelBusResourceRuleFactory;
import org.modelbus.team.eclipse.core.operation.UnreportableException;
import org.modelbus.team.eclipse.core.operation.local.RestoreProjectMetaOperation;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.core.utility.ProgressMonitorUtility;
import org.modelbus.team.eclipse.repository.ModelBusRepositoryPlugin;
import org.modelbus.team.eclipse.repository.ModelBusRepositoryProvider;
import org.modelbus.team.eclipse.repository.Policy;
import org.modelbus.team.eclipse.repository.subscriber.ModelBusRepositoryResourceVariant;
import org.modelbus.team.eclipse.repository.subscriber.ModelBusThreeWaySynchronizer;
import org.modelbus.team.eclipse.repository.ui.GetOperation;
import org.modelbus.team.eclipse.repository.ui.ModelBusRepositoryOperation;

/**
 * Checkout content into the project which has the specified name
 * 
 * @author Alexander Gurov
 */
public class CheckoutAsOperation extends AbstractActionOperation {
	public final static byte[]				bytes	= new byte[0];
	protected IProject						project;
	protected IRepositoryResource			resource;
	protected String						projectLocation;
	protected List<IProject>				overlappingProjects;
	protected int							recureDepth;
	protected boolean						ignoreExternals;
	protected RestoreProjectMetaOperation	restoreOp;

	public CheckoutAsOperation(String projectName, IRepositoryResource resource, int recureDepth) {
		this(projectName, resource, Platform.getLocation().toString(), recureDepth, false);
	}

	public CheckoutAsOperation(String projectName, IRepositoryResource resource, int recureDepth, boolean ignoreExternals) {
		this(projectName, resource, Platform.getLocation().toString(), recureDepth, ignoreExternals);
	}

	public CheckoutAsOperation(String projectName, IRepositoryResource resource, boolean respectHierarchy, String location, int recureDepth,
			boolean ignoreExternals) {
		this(projectName, resource, location == null ? Platform.getLocation().toString() : location
				+ (respectHierarchy ? ModelBusUtility.getResourceParent(resource) : ""), recureDepth, ignoreExternals);
	}

	public int getOperationWeight() {
		return 19;
	}

	public CheckoutAsOperation(String projectName, IRepositoryResource resource, String projectLocation, int recureDepth, boolean ignoreExternals) {
		super("Operation.CheckOutAs");
		projectName = FileUtility.formatResourceName(projectName);
		if (FileUtility.isCaseInsensitiveOS()) {
			IProject[] projects = ResourcesPlugin.getWorkspace().getRoot().getProjects();
			IProject projectToCheckOut = null;
			for (int i = 0; i < projects.length; i++) {
				if (projects[i].getName().equalsIgnoreCase(projectName)) {
					projectToCheckOut = projects[i];
				}
			}
			this.project = projectToCheckOut != null ? projectToCheckOut : ResourcesPlugin.getWorkspace().getRoot().getProject(projectName);
		} else {
			this.project = ResourcesPlugin.getWorkspace().getRoot().getProject(projectName);
		}
		this.resource = resource;
		this.projectLocation = projectLocation;
		this.recureDepth = recureDepth;
		this.ignoreExternals = ignoreExternals;
		this.overlappingProjects = new ArrayList<IProject>();
		IProject[] projects = ResourcesPlugin.getWorkspace().getRoot().getProjects();
		for (int i = 0; i < projects.length; i++) {
			if (new Path(this.projectLocation).append(this.project.getName()).isPrefixOf(projects[i].getLocation())) {
				this.overlappingProjects.add(projects[i]);
			}
		}
		this.overlappingProjects.add(this.project);
	}

	public ISchedulingRule getSchedulingRule() {
		IProject[] projects = this.overlappingProjects.toArray(new IProject[this.overlappingProjects.size()]);
		ISchedulingRule[] rules = new ISchedulingRule[this.overlappingProjects.size()];
		for (int i = 0; i < projects.length; i++) {
			rules[i] = ModelBusResourceRuleFactory.INSTANCE.modifyRule(projects[i]);
		}
		return new MultiRule(rules);
	}

	public IProject getProject() {
		return this.project;
	}

	public void setRestoreOperation(RestoreProjectMetaOperation restoreOp) {
		this.restoreOp = restoreOp;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		String projectName = this.project.isAccessible() ? this.project.getLocation().toString()
				.substring(this.project.getLocation().toString().lastIndexOf("/") + 1) : this.project.getName();
		final IPath destination = new Path(this.projectLocation).append(projectName);

		/* MES: Original checkout */
		/*
		 * ProgressMonitorUtility.doSubTask(this, new IUnprotectedOperation() { public void run(IProgressMonitor monitor) throws Exception {
		 * CheckoutAsOperation.this.doCheckout(monitor, destination); } }, monitor, 20, 19);
		 * 
		 * 
		 * ProgressMonitorUtility.doSubTask(this, new IUnprotectedOperation() { public void run(IProgressMonitor monitor) throws Exception {
		 * CheckoutAsOperation.this.doOpen(monitor, destination); } }, monitor, 20, 1);
		 */

		/*
		 * MES: Modified checkout using GetOperation from org.modelbus.team.eclipse plugin. Works. Sluggish due to conflicting scheduling rules.
		 * Upsets the subscriber.
		 */
		/*
		 * ProgressMonitorUtility.doSubTask(this, new IUnprotectedOperation() { public void run(IProgressMonitor monitor) throws Exception {
		 * CheckoutAsOperation.this.deleteStaleProjects(monitor, destination); } }, monitor, 20, 1);
		 * 
		 * ProgressMonitorUtility.doSubTask(this, new IUnprotectedOperation() { public void run(IProgressMonitor monitor) throws Exception {
		 * CheckoutAsOperation.this.doOpen(monitor, destination); } }, monitor, 20, 1);
		 * 
		 * ProgressMonitorUtility.doSubTask(this, new IUnprotectedOperation() { public void run(IProgressMonitor monitor) throws Exception {
		 * CheckoutAsOperation.this.doCheckoutViaGetOperation(monitor, destination); } }, monitor, 20, 17);
		 * 
		 * ProgressMonitorUtility.doSubTask(this, new IUnprotectedOperation() { public void run(IProgressMonitor monitor) throws Exception {
		 * CheckoutAsOperation.this.checkRestore(monitor, destination); } }, monitor, 20, 1);
		 */

		/*
		 * MES: Modified checkout with manual setting of base and remote bytes. Fast. Observed refresh and sync-state issues. Upsets the subscriber.
		 * TODOMES: Reset/flush bytes in case of errors. Log errors correctly. Use ProgressMonitor to output more info. Maybe take a look at SVNKit
		 * implementation to see how this is done.
		 */
		/*
		 * ProgressMonitorUtility.doSubTask(this, new IUnprotectedOperation() { public void run(IProgressMonitor monitor) throws Exception {
		 * CheckoutAsOperation.this.doCheckout(monitor, destination); } }, monitor, 20, 9);
		 * 
		 * 
		 * ProgressMonitorUtility.doSubTask(this, new IUnprotectedOperation() { public void run(IProgressMonitor monitor) throws Exception {
		 * CheckoutAsOperation.this.doOpen(monitor, destination); } }, monitor, 20, 2);
		 * 
		 * 
		 * ProgressMonitorUtility.doSubTask(this, new IUnprotectedOperation() { public void run(IProgressMonitor monitor) throws Exception {
		 * CheckoutAsOperation.this.setBaseAndRemoteBytes(monitor, getSynchronizer(), project, (ModelBusRepositoryProvider) ModelBusRepositoryProvider
		 * .getProvider(project)); } }, monitor, 20, 9);
		 */

		/*
		 * MES: Best approach yet. Still upsets the subscriber, though. Need to map the project early to be able to use the ModelBusRepositoryProvider
		 * to fetch IResourceVariants. TODOMES: Cache for IResourceVariants is still not fixed!
		 */

		ProgressMonitorUtility.doSubTask(this, new IUnprotectedOperation() {
			public void run(IProgressMonitor monitor) throws Exception {
				CheckoutAsOperation.this.deleteStaleProjects(monitor, destination);
			}
		}, monitor, 20, 1);

		ProgressMonitorUtility.doSubTask(this, new IUnprotectedOperation() {
			public void run(IProgressMonitor monitor) throws Exception {
				CheckoutAsOperation.this.doOpen(monitor, destination);
			}
		}, monitor, 20, 1);

		ProgressMonitorUtility.doSubTask(this, new IUnprotectedOperation() {
			public void run(IProgressMonitor monitor) throws Exception {
				CheckoutAsOperation.this.doCheckoutViaWorkspaceAPI(monitor);
			}
		}, monitor, 20, 18);
	}

	/*
	 * MES. Somehow upsets the synchronization state. Projects get checked-out correctly, but are perceived as out-of-sync by the synchronize view.
	 * Might be a problem with the timestamps, since commenting the refreshLocal()-call at the bottom seems to fix the problem.
	 */
	private void setBaseAndRemoteBytes(final IProgressMonitor monitor, final ThreeWaySynchronizer synchronizer, final IProject project,
			final ModelBusRepositoryProvider provider) throws Exception {

		final byte[] bytes = new byte[0];
		try {
			project.accept(new IResourceVisitor() {

				public boolean visit(IResource resource) throws CoreException {
					/* TODOMES: Check if setting bytes on project causes any undesired side-effects */
					if (resource.getType() == IResource.PROJECT) {
						try {
							// Bytes for IProject: [byte[0], byte[0]]
							synchronizer.setBaseBytes(project, bytes);
							return true;
						} catch (TeamException e) {
							throw e;
						}
					}

					if (resource.getType() == IResource.FOLDER) {
						try {
							/* Bytes for IFolder: [byte[0], byte[0]] */
							synchronizer.setBaseBytes(resource, bytes);
							return true;
						} catch (TeamException e) {
							throw e;
						}
					}

					if (resource.getType() == IResource.FILE) {
						try {
							/*
							 * Bytes for freshly created files w/o remote (e.g.: .project): [null, null]. In other words: do nothing.
							 */
							/* Bytes for regularly checked-out files: [remote.asBytes(), remote.asBytes()] */
							IResourceVariant remote = provider.getResourceVariant(resource);
							// Beware! Returns null in case there is no corresponding remote resource!
							if (remote != null) {
								byte[] remoteBytes = remote.asBytes();
								synchronizer.setBaseBytes(resource, remoteBytes);
							}
							return true;
						} catch (Exception e) {
							throw new RuntimeException(e);
						}
					}
					return false;
				}
			});
			/* MES: Absolutely need to do this to be able to open files, without having to trigger manual refresh. */
			project.refreshLocal(IResource.DEPTH_INFINITE, monitor);
		} catch (CoreException e) {
			e.printStackTrace();
			// Cannot go on if sync bytes were not set properly.
			this.deleteProject(project, monitor);
		}
	}

	protected void doOpen(IProgressMonitor monitor, IPath destination) throws Exception {
		if (this.restoreOp != null) {
			this.reportStatus(this.restoreOp.run(monitor).getStatus());
		}

		IProjectDescription description = ResourcesPlugin.getWorkspace().newProjectDescription(this.project.getName());
		if (!Platform.getLocation().equals(new Path(this.projectLocation))) {
			description.setLocation(destination);
		}
		this.project.create(description, monitor);
		this.project.open(new SubProgressMonitor(monitor, IProgressMonitor.UNKNOWN));
		/*
		 * MES: To revert to the original implementation uncomment the following line and comment the try-catch-block instead.
		 */
		// MES: Not strictly necessary any longer. There are no such things as team-private resources no more.
		// ModelBusTeamProjectMapper.map(this.project, this.resource);
		/*
		 * MES: Copied from the org.modelbus.team.eclipse plugin's. ConfigurationWizard (Sharing Wizard). Prerequisite for checking out via the
		 * workspace API. PROJECT MUST BE OPEN!
		 */
		try {
			/*
			 * MES: Map the provider and set the location. Be aware of the fact that creating, opening and mapping the project to a provider triggers
			 * (at least) resource change events, which may get handled on another thread before the provider's target location was set. If a
			 * synchronization change listener is on the prowl, it will try to recalculate the sync-state for the freshly created project folder and
			 * .project file immediately and ask the provider for the yet unset target location (ModelBusRepositoryProvider#getRepositoryURL()), which
			 * yields NULL. Clients of ModelBusRepositoryProvider need to guard against that possibility.
			 */
			RepositoryProvider.map(project, ModelBusRepositoryPlugin.PROVIDER_ID);
			ModelBusRepositoryProvider provider = (ModelBusRepositoryProvider) RepositoryProvider.getProvider(project);
			provider.setTargetLocation(resource.getUrl());
		} catch (TeamException e) {
			this.reportError(e);
		}
	}

	/* MES */
	private void deleteStaleProjects(IProgressMonitor monitor, IPath destination) throws Exception {
		// prepare workspace...
		ProgressMonitorUtility.setTaskInfo(monitor, this, this.getOperationResource("PrepareFS"));

		for (Iterator<IProject> it = this.overlappingProjects.iterator(); it.hasNext() && !monitor.isCanceled();) {
			IProject overlappingProject = it.next();
			this.deleteProject(overlappingProject, monitor);
		}
		this.deleteFolderContent(destination.toString(), monitor);
	}

	/* MES */
	private void doCheckoutViaGetOperation(IProgressMonitor monitor, IPath destination) throws Exception {
		try {
			String path = destination.toString();
			this.writeToConsole(
					IConsoleStream.LEVEL_CMD,
					"modelbus checkout \"" + this.resource.getUrl() + "@" + this.resource.getPegRevision() + "\" -r "
							+ this.resource.getSelectedRevision() + ModelBusUtility.getDepthArg(this.recureDepth) + " \""
							+ FileUtility.normalizePath(path) + "\"" + FileUtility.getUsernameParam(ModelBusConnector.getUserName()) + "\n");

			/*
			 * MES: Direct use of ModelBusOperations#get() does not work. Updates to the project are lost. TODOMES: Avoid passing null for target
			 * part.
			 */
			GetOperation operation = new GetOperation(null, ModelBusRepositoryOperation.createScopeManager(
					Policy.bind("GetAction.working"), new SimpleResourceMapping[] { new SimpleResourceMapping(project) })); //$NON-NLS-1$
			operation.setOverwriteOutgoing(true);
			operation.run();
		} catch (InvocationTargetException e) {
		} catch (InterruptedException e) {
		}
		// Ignore
		finally {
			/* TODOMES: Refresh or triggering of resource change event necessary here? */
		}
	}

	/* MES */
	private void checkRestore(IProgressMonitor monitor, IPath destination) {
		if (this.restoreOp != null) {
			this.reportStatus(this.restoreOp.run(monitor).getStatus());
		}
	}

	/* MES */
	private void doCheckoutViaWorkspaceAPI(IProgressMonitor monitor) throws Exception {

		ThreeWaySynchronizer synchronizer = null;
		ModelBusRepositoryProvider provider = null;

		try {
			// ensure the progress monitor is not null
			ProgressMonitorUtility.setTaskInfo(monitor, this, "Checking out " + project.getName());

			// get the tools needed for doing the checkout and setting the bytes
			synchronizer = getSynchronizer();
			provider = (ModelBusRepositoryProvider) ModelBusRepositoryProvider.getProvider(project);

			// obtain the project remote resource
			ModelBusRepositoryResourceVariant projectVariant = null;
			// the revision
			String resourceRevision = IRepositoryHelper.REVISION_HEAD;
			if (ModelBusRevision.HEAD.equals(this.resource.getSelectedRevision())) { // HEAD selected
				projectVariant = (ModelBusRepositoryResourceVariant) provider.getResourceVariant(project);
				resourceRevision = projectVariant.getRepositoryDirEntry().getRevision();
			} else { // older revision selected (in number format, but doesn't stick to our -1 conventio for HEAD, so previous check required.
				projectVariant = (ModelBusRepositoryResourceVariant) provider.getResourceVariant(project, this.resource.getRevision().getRevision());
				resourceRevision = this.resource.getRevision().getRevision();
			}
			// set bytes for IProject: [null, byte[0]]
			// TODOMES: Check if setting bytes on project causes any undesired side-effects */
			// synchronizer.setBaseBytes(project, bytes); // sets both slots at the same time, triggers notification event.
			((ModelBusThreeWaySynchronizer) synchronizer).setBaseBytes(project, bytes, resourceRevision);
			// treat children (depth-first)
			ModelBusRepositoryResourceVariant[] remoteChildren = projectVariant.members(); // check members!
			for (ModelBusRepositoryResourceVariant child : remoteChildren) {
				if (child != null) {
					if (child.isFile()) {
						internalCheckoutFileViaWorkspaceAPI(monitor, project, child, resourceRevision, synchronizer);
					}
					if (child.isContainer()) {
						internalCheckoutContainerViaWorkspaceAPI(monitor, project, child, resourceRevision, synchronizer);
					}
				}
			}

			// Something went really wrong. Better flush the whole mess :)
		} catch (Exception e) {
			reportError(e);
			e.printStackTrace();
			if (synchronizer != null) {
				synchronizer.flush(project, IResource.DEPTH_INFINITE);
			}
			ModelBusRepositoryProvider.unmap(project);
			deleteProject(project, monitor);
		}
	}

	/* MES */
	// private void internalCheckoutContainerViaWorkspaceAPI(IProgressMonitor monitor, IContainer localParent, ModelBusRepositoryResourceVariant
	// remoteChild, final ThreeWaySynchronizer synchronizer)
	// throws Exception {
	//
	// // obtain IFolder-handle from parent container
	// IFolder folder = (localParent instanceof IProject ? ((IProject) localParent).getFolder(remoteChild.getName()) : ((IFolder)
	// localParent).getFolder(remoteChild.getName()));
	// // update ProgressMonitor
	// ProgressMonitorUtility.setTaskInfo(monitor, this, "Checking out " + folder.getName());
	// // force folder creation (encountering existing folders reeks of errors, since this is a check-out operation)
	// if (!folder.exists()) {
	// folder.create(true, true, monitor);
	// }
	// // set bytes for IFolder: [byte[0], byte[0]]
	// synchronizer.setBaseBytes(folder, bytes);
	// // deal with children (depth-first)
	// ModelBusRepositoryResourceVariant[] remoteGrandChildren = remoteChild.members();
	// for (ModelBusRepositoryResourceVariant remoteGrandChild : remoteGrandChildren) {
	// if (remoteGrandChild != null) {
	// if (remoteGrandChild.isFile()) {
	// internalCheckoutFileViaWorkspaceAPI(monitor, folder, remoteGrandChild, synchronizer);
	// }
	// if (remoteGrandChild.isContainer()) {
	// internalCheckoutContainerViaWorkspaceAPI(monitor, folder, remoteGrandChild, synchronizer);
	// }
	// }
	// }
	//
	// }

	private void internalCheckoutContainerViaWorkspaceAPI(IProgressMonitor monitor, IContainer localParent,
			ModelBusRepositoryResourceVariant remoteChild, String revision, final ThreeWaySynchronizer synchronizer) throws Exception {

		// obtain IFolder-handle from parent container
		IFolder folder = (localParent instanceof IProject ? ((IProject) localParent).getFolder(remoteChild.getName()) : ((IFolder) localParent)
				.getFolder(remoteChild.getName()));
		// update ProgressMonitor
		ProgressMonitorUtility.setTaskInfo(monitor, this, "Checking out " + folder.getName());
		// force folder creation (encountering existing folders reeks of errors, since this is a check-out operation)
		if (!folder.exists()) {
			folder.create(true, true, monitor);
		}
		// set bytes for IFolder: [byte[0], byte[0]]
		((ModelBusThreeWaySynchronizer) synchronizer).setBaseBytes(folder, bytes, revision);
		// deal with children (depth-first)
		ModelBusRepositoryResourceVariant[] remoteGrandChildren = remoteChild.members();
		for (ModelBusRepositoryResourceVariant remoteGrandChild : remoteGrandChildren) {
			if (remoteGrandChild != null) {
				if (remoteGrandChild.isFile()) {
					internalCheckoutFileViaWorkspaceAPI(monitor, folder, remoteGrandChild, revision, synchronizer);
				}
				if (remoteGrandChild.isContainer()) {
					internalCheckoutContainerViaWorkspaceAPI(monitor, folder, remoteGrandChild, revision, synchronizer);
				}
			}
		}

	}

	/* MES */
	// private void internalCheckoutFileViaWorkspaceAPI(IProgressMonitor monitor, IContainer parent, ModelBusRepositoryResourceVariant child, final
	// ThreeWaySynchronizer synchronizer) throws Exception {
	//
	// InputStream remoteFileInputStream = null;
	//
	// // obtain IFile-handle from parent
	// IFile file = (parent instanceof IProject ? ((IProject) parent).getFile(child.getName()) : ((IFolder) parent).getFile(child.getName()));
	// // update ProgressMonitor
	// ProgressMonitorUtility.setTaskInfo(monitor, this, "Checking out " + file.getName());
	// // force file creation (eclipse is very picky here)
	// try {
	// remoteFileInputStream = child.getStorage(monitor).getContents();
	// // empty file (checkout: assuming that remote file exists, since it was there when we selected it :))
	// if (remoteFileInputStream == null) {
	// remoteFileInputStream = new ByteArrayInputStream(bytes);
	// }
	// if (!file.exists()) {
	// file.create(remoteFileInputStream, true, monitor);
	// } else {
	// file.setContents(remoteFileInputStream, true, true, monitor);
	// }
	// // set bytes for IFile [remote.asBytes(), remote.asBytes()]
	// byte baseBytes[] = child.asBytes();
	// if (baseBytes == null) {
	// baseBytes = bytes;
	// }
	// synchronizer.setBaseBytes(file, baseBytes);
	// } finally {
	// // just to make sure everything is fine
	// if (remoteFileInputStream != null)
	// remoteFileInputStream.close();
	// }
	// }

	private void internalCheckoutFileViaWorkspaceAPI(IProgressMonitor monitor, IContainer parent, ModelBusRepositoryResourceVariant child,
			String revision, final ThreeWaySynchronizer synchronizer) throws Exception {

		InputStream remoteFileInputStream = null;

		// obtain IFile-handle from parent
		IFile file = (parent instanceof IProject ? ((IProject) parent).getFile(child.getName()) : ((IFolder) parent).getFile(child.getName()));
		// update ProgressMonitor
		ProgressMonitorUtility.setTaskInfo(monitor, this, "Checking out " + file.getName());
		// force file creation (eclipse is very picky here)
		try {
			remoteFileInputStream = child.getStorage(monitor).getContents();
			// empty file (checkout: assuming that remote file exists, since it was there when we selected it :))
			if (remoteFileInputStream == null) {
				remoteFileInputStream = new ByteArrayInputStream(bytes);
			}
			if (!file.exists()) {
				file.create(remoteFileInputStream, true, monitor);
			} else {
				file.setContents(remoteFileInputStream, true, true, monitor);
			}
			// set bytes for IFile [remote.asBytes(), remote.asBytes()]
			byte baseBytes[] = child.asBytes();
			if (baseBytes == null) {
				baseBytes = bytes;
			}
			((ModelBusThreeWaySynchronizer) synchronizer).setBaseBytes(file, baseBytes, revision);
		} finally {
			// just to make sure everything is fine
			if (remoteFileInputStream != null)
				remoteFileInputStream.close();
		}
	}

	protected void doCheckout(IProgressMonitor monitor, IPath destination) throws Exception {
		// prepare workspace...
		ProgressMonitorUtility.setTaskInfo(monitor, this, this.getOperationResource("PrepareFS"));

		for (Iterator<IProject> it = this.overlappingProjects.iterator(); it.hasNext() && !monitor.isCanceled();) {
			IProject overlappingProject = it.next();
			this.deleteProject(overlappingProject, monitor);
		}
		this.deleteFolderContent(destination.toString(), monitor);

		// check out files from the repository
		// using parent because deleted project does not have any location (null
		// value returned)
		IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		try {
			String path = destination.toString();
			this.writeToConsole(
					IConsoleStream.LEVEL_CMD,
					"modelbus checkout \"" + this.resource.getUrl() + "@" + this.resource.getPegRevision() + "\" -r "
							+ this.resource.getSelectedRevision() + ModelBusUtility.getDepthArg(this.recureDepth) + " \""
							+ FileUtility.normalizePath(path) + "\"" + FileUtility.getUsernameParam(ModelBusConnector.getUserName()) + "\n");
			proxy.checkout(ModelBusUtility.getEntryRevisionReference(this.resource), path, this.recureDepth,
					this.ignoreExternals ? IModelBusConnector.Options.IGNORE_EXTERNALS : IModelBusConnector.Options.NONE,
					new ModelBusProgressMonitor(this, monitor, this.project.getFullPath()));
		} finally {
		}
	}

	protected void deleteProject(IProject project, IProgressMonitor monitor) throws Exception {
		IPath location = project.getLocation();
		project.delete(false, true, null);
		if (location != null) {
			this.deleteFolderContent(location.toString(), monitor);
		}
	}

	protected void deleteFolderContent(String targetPath, IProgressMonitor monitor) throws Exception {
		File target = new File(targetPath);
		if (!FileUtility.deleteRecursive(target, monitor)) {
			File[] children = target.listFiles();
			if (children != null && children.length > 0) {
				String message = this.getNationalizedString("Error.LockedExternally");
				throw new UnreportableException(MessageFormat.format(message, new Object[] { children[0].getAbsolutePath() }));
			}
		}
	}

	protected String getShortErrorMessage(Throwable t) {
		return MessageFormat.format(super.getShortErrorMessage(t), new Object[] { this.resource.getUrl() });
	}

	private ModelBusThreeWaySynchronizer getSynchronizer() {
		return new ModelBusThreeWaySynchronizer(new QualifiedName(ModelBusRepositoryPlugin.ID, "workspace-sync"));
	}
}
