/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.remote;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusLogEntry;
import org.modelbus.team.eclipse.core.connector.ModelBusLogPath;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision.Kind;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.resource.IRepositoryFile;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryResourceProvider;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.core.utility.ProgressMonitorUtility;

/**
 * Locate resource URL that corresponds to revision
 * 
 * @author Alexander Gurov
 */
public class LocateResourceURLInHistoryOperation extends AbstractRepositoryOperation implements IRepositoryResourceProvider {
	protected IRepositoryResource[]	converted;

	public LocateResourceURLInHistoryOperation(IRepositoryResource[] resources) {
		super("Operation.LocateURLInHistory", resources);
	}

	public LocateResourceURLInHistoryOperation(IRepositoryResourceProvider provider) {
		super("Operation.LocateURLInHistory", provider);
	}

	public IRepositoryResource[] getRepositoryResources() {
		return this.converted;
	}

	protected void runImpl(final IProgressMonitor monitor) throws Exception {
		IRepositoryResource[] resources = this.operableData();
		this.converted = new IRepositoryResource[resources.length];
		System.arraycopy(resources, 0, this.converted, 0, resources.length);

		for (int i = 0; i < resources.length && !monitor.isCanceled(); i++) {
			final int idx = i;
			ProgressMonitorUtility.setTaskInfo(monitor, this, resources[i].getUrl());
			if (this.converted[i].getSelectedRevision().getKind() == Kind.STRING) {
				this.protectStep(new IUnprotectedOperation() {
					public void run(IProgressMonitor monitor) throws Exception {
						IRepositoryResource result = LocateResourceURLInHistoryOperation.this.processEntry(
								LocateResourceURLInHistoryOperation.this.converted[idx], monitor);
						LocateResourceURLInHistoryOperation.this.converted[idx] = LocateResourceURLInHistoryOperation.this.converted[idx] == result ? ModelBusUtility
								.copyOf(result) : result;
						// when URL is corrected peg can be set to selected revision number
						LocateResourceURLInHistoryOperation.this.converted[idx]
								.setPegRevision(LocateResourceURLInHistoryOperation.this.converted[idx].getSelectedRevision());
					}
				}, monitor, resources.length);
			}
		}
	}

	protected IRepositoryResource processEntry(IRepositoryResource current, IProgressMonitor monitor) throws Exception {
		IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		ModelBusLogEntry[] msgs = null;
		int index = 0;
		try {
			IRepositoryResource pegNode = ModelBusUtility.copyOf(current);
			pegNode.setSelectedRevision(pegNode.getPegRevision());
			if (pegNode.exists()) {
				msgs = ModelBusUtility.logEntries(proxy, ModelBusUtility.getEntryReference(pegNode), ModelBusRevision.fromString("0"),
						pegNode.getPegRevision(), IModelBusConnector.Options.STOP_ON_COPY | IModelBusConnector.Options.DISCOVER_PATHS,
						IModelBusConnector.EMPTY_LOG_ENTRY_PROPS, 1, new ModelBusProgressMonitor(this, monitor, null));
			} else if (pegNode.getParent() != null) {
				msgs = ModelBusUtility.logEntries(proxy, ModelBusUtility.getEntryReference(pegNode.getParent()), ModelBusRevision.fromString("0"),
						pegNode.getPegRevision(), IModelBusConnector.Options.STOP_ON_COPY | IModelBusConnector.Options.DISCOVER_PATHS,
						IModelBusConnector.EMPTY_LOG_ENTRY_PROPS, 1, new ModelBusProgressMonitor(this, monitor, null));
				if (msgs != null) {
					for (int j = 0; j < msgs.length; j++) {
						ModelBusLogPath[] paths = msgs[j].changedPaths;
						if (paths != null) {
							for (int k = 0; k < paths.length; k++) {
								if (paths[k] != null && current.getUrl().endsWith(paths[k].path)) {
									index = j;
									break;
								}
							}
						}
					}
				}
			}
		} finally {
		}
		if (msgs != null && msgs.length > index && msgs[index] != null) {
			ModelBusLogPath[] paths = msgs[index].changedPaths;
			if (paths == null) {
				return current;
			}
			Path pattern = new Path(current.getUrl().substring(ModelBusConnector.getRepositoryRootUrl().length()));
			int idx = -1;
			int prefLen = 0;
			for (int i = 0; i < paths.length; i++) {
				Path path = new Path(paths[i].path);
				int tLen = path.segmentCount();
				if (path.isPrefixOf(pattern) && paths[i].copiedFromPath != null && tLen > prefLen) {
					idx = i;
				}
			}
			if (idx == -1) {
				return current;
			}
			String copiedFrom = ModelBusConnector.getRepositoryRootUrl() + paths[idx].copiedFromPath
					+ pattern.toString().substring(paths[idx].path.length());

			ModelBusRevision rev = paths[idx].copiedFromRevision;
			ModelBusRevision searchRevision = current.getSelectedRevision();
			
			if (rev.isLowerThan(searchRevision)) {
				if (msgs[index].modelBusRevision.isLowerThanOrEqual(searchRevision)) {
					return current;
				}
				searchRevision = rev;
			}

			IRepositoryResource retVal = current instanceof IRepositoryFile ? (IRepositoryResource) ModelBusConnector.asRepositoryFile(copiedFrom,
					false) : ModelBusConnector.asRepositoryContainer(copiedFrom, false);
			retVal.setPegRevision(rev);
			retVal.setSelectedRevision(searchRevision);
			return rev.isLowerThanOrEqual(searchRevision) ? retVal : this.processEntry(retVal, monitor);
		}
		return current;
	}

}
