/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *    Gabor Liptak - Speedup Pattern's usage
 *******************************************************************************/

package org.modelbus.team.eclipse.core.utility;

import java.io.File;
import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Stack;
import java.util.StringTokenizer;

import org.eclipse.core.internal.preferences.Base64;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.team.core.Team;
import org.modelbus.team.eclipse.core.ModelBusTeamPlugin;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Depth;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Options;
import org.modelbus.team.eclipse.core.connector.IModelBusDiffStatusCallback;
import org.modelbus.team.eclipse.core.connector.IModelBusEntryCallback;
import org.modelbus.team.eclipse.core.connector.IModelBusEntryInfoCallback;
import org.modelbus.team.eclipse.core.connector.IModelBusLogEntryCallback;
import org.modelbus.team.eclipse.core.connector.IModelBusMergeStatusCallback;
import org.modelbus.team.eclipse.core.connector.IModelBusNotificationCallback;
import org.modelbus.team.eclipse.core.connector.IModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.connector.IModelBusPropertyCallback;
import org.modelbus.team.eclipse.core.connector.IModelbusEntryStatusCallback;
import org.modelbus.team.eclipse.core.connector.ModelBusChangeStatus;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusConnectorException;
import org.modelbus.team.eclipse.core.connector.ModelBusDiffStatus;
import org.modelbus.team.eclipse.core.connector.ModelBusEntry;
import org.modelbus.team.eclipse.core.connector.ModelBusEntry.Kind;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryInfo;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryReference;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryRevisionReference;
import org.modelbus.team.eclipse.core.connector.ModelBusLogEntry;
import org.modelbus.team.eclipse.core.connector.ModelBusMergeStatus;
import org.modelbus.team.eclipse.core.connector.ModelBusProperty;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.connector.ModelBusRevisionRange;
import org.modelbus.team.eclipse.core.extension.CoreExtensionsManager;
import org.modelbus.team.eclipse.core.extension.options.IIgnoreRecommendations;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRemoteStorage;
import org.modelbus.team.eclipse.core.operation.ModelBusNullProgressMonitor;
import org.modelbus.team.eclipse.core.operation.UnreportableException;
import org.modelbus.team.eclipse.core.resource.ILocalResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryFile;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryRoot;

/**
 * ModelBus Utility functions
 * 
 * @author Alexander Gurov
 */
public final class ModelBusUtility {
	private static String	modelbusFolderName	= null;

	public static IRepositoryResource asRepositoryResource(String url, boolean isFolder) {
		if (!ModelBusUtility.isValidModelBusURL(url)) {
			return null;
		}
		IRepositoryRoot[] roots = ModelBusUtility.findRoots(url, true);
		IRepositoryResource retVal = null;
		if (roots.length > 0) {
			retVal = isFolder ? roots[0].asRepositoryContainer(url, false) : roots[0].asRepositoryFile(url, false);
		} else {
			retVal = isFolder ? ModelBusConnector.asRepositoryContainer(url, false) : ModelBusConnector.asRepositoryFile(url, false);
		}
		return retVal;
	}

	public static IRepositoryResource getCopiedFrom(IResource resource) {
		ILocalResource local = ModelBusRemoteStorage.instance().asLocalResource(resource);
		if (local.isCopied()) {
			IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
			final ModelBusChangeStatus[] st = new ModelBusChangeStatus[1];
			try {
				final String path = FileUtility.getWorkingCopyPath(resource);
				proxy.status(path, IModelBusConnector.Depth.EMPTY, IModelBusConnector.Options.INCLUDE_UNCHANGED | Options.IGNORE_EXTERNALS, null,
						new IModelbusEntryStatusCallback() {
							public void next(ModelBusChangeStatus status) {
								if (path.equals(status.path)) {
									st[0] = status;
								}
							}
						}, new ModelBusNullProgressMonitor());
			} catch (ModelBusConnectorException ex) {
				return null;
			} finally {
			}

			if (st[0] != null) {
				String url = st[0].urlCopiedFrom;
				if (url == null) {
					IResource parent = resource.getParent();
					if (parent != null && parent.getType() != IResource.ROOT) {
						IRepositoryResource tmp = ModelBusUtility.getCopiedFrom(parent);
						if (tmp != null) {
							url = tmp.getUrl() + "/" + resource.getName();
						}
					}
				} else {
					url = ModelBusUtility.decodeURL(url);
				}
				IRepositoryResource retVal = ModelBusRemoteStorage.instance().asRepositoryResource(url, resource.getType() == IResource.FILE);
				retVal.setSelectedRevision(st[0].revisionCopiedFrom == ModelBusRevision.INVALID_REVISION_DESCRIPTOR ? st[0].revision : st[0].revisionCopiedFrom);
				return retVal;
			}
		}
		return null;
	}

	public static Map<String, ModelBusEntryRevisionReference> parseModelBusExternalsProperty(String property, IRepositoryResource propertyHolder) {
		if (property == null) {
			return Collections.emptyMap();
		}
		Map<String, ModelBusEntryRevisionReference> retVal = new HashMap<String, ModelBusEntryRevisionReference>();
		String[] externals = property.trim().split("[\\n|\\r\\n]+"); // it seems different clients have different behaviours wrt trailing whitespace..
																		// so
																		// trim() to be safe
		for (int i = 0; i < externals.length; i++) {
			String[] parts = externals[i].trim().split("[\\t ]+");
			// 2 - name + URL
			// 3 - name + -rRevision + URL
			// 4 - name + -r + Revision + URL
			// or in ModelBus 1.5 format
			// 2 - URL@peg + name
			// 3 - -rRevision + URL@peg + name
			// 4 - -r + Revision + URL@peg + name
			if (parts.length < 2 || parts.length > 4) {
				throw new UnreportableException("Malformed external, " + parts.length + ", " + externals[i]);
			}

			String name = null;
			String url = null;
			ModelBusRevision revision = null;
			ModelBusRevision pegRevision = null;

			if (ModelBusUtility.isValidModelBusURL(parts[parts.length - 1])) {
				name = parts[0];
				try {
					url = parts[1];
					if (parts.length == 4) {
						revision = ModelBusRevision.fromString(parts[2]);
						url = parts[3];
					} else if (parts.length == 3) {
						revision = ModelBusRevision.fromString(parts[1].substring(2));
						url = parts[2];
					}
				} catch (Exception ex) {
					throw new UnreportableException("Malformed external, " + parts.length + ", " + externals[i]);
				}
			} else {
				name = parts[parts.length - 1];
				try {
					url = parts[0];
					if (parts.length == 4) {
						revision = ModelBusRevision.fromString(parts[1]);
						url = parts[2];
					} else if (parts.length == 3) {
						revision = ModelBusRevision.fromString(parts[0].substring(2));
						url = parts[1];
					}
				} catch (Exception ex) {
					throw new UnreportableException("Malformed external, " + parts.length + ", " + externals[i]);
				}
				if (!ModelBusUtility.isValidModelBusURL(url)) {
					if (url.startsWith("^/")) {
						url = ModelBusConnector.getRepositoryRootUrl() + url.substring(1);
					} else if (url.startsWith("//")) {
						try {
							String protocol = ModelBusUtility.getModelBusUrl(propertyHolder.getUrl()).getProtocol();
							if (propertyHolder.getUrl().indexOf(":///") != -1) {
								url = protocol + ":/" + url;
							} else {
								url = protocol + ":" + url;
							}
						} catch (MalformedURLException e) {
							// cannot be thrown
						}
					} else if (url.startsWith("/")) {
						String prefix = propertyHolder.getUrl();
						int idx = prefix.lastIndexOf("//");
						idx = prefix.indexOf('/', idx + 2);
						url = prefix.substring(0, idx) + url;
					} else if (url.startsWith("../")) {
						IRepositoryResource prefix = propertyHolder;
						while (url.startsWith("../")) {
							url = url.substring(3);
							prefix = prefix.getParent();
							if (prefix == null) {
								throw new UnreportableException("Malformed external, " + parts.length + ", " + externals[i]);
							}
						}
						url = prefix.getUrl() + "/" + url;
					} else {
						throw new UnreportableException("Malformed external, " + parts.length + ", " + externals[i]);
					}
				}
				ModelBusEntryReference ref = ModelBusUtility.asEntryReference(url);
				url = ref.path;
				pegRevision = ref.pegRevision;
			}

			try {
				url = ModelBusUtility.decodeURL(url);
			} catch (IllegalArgumentException ex) {
				// the URL is not encoded
			}
			url = ModelBusUtility.normalizeURL(url);

			retVal.put(name, new ModelBusEntryRevisionReference(url, pegRevision, revision));
		}
		return retVal;
	}

	public static ModelBusEntryReference asEntryReference(String url) {
		if (url == null) {
			return null;
		}
		int idx = url.lastIndexOf('@');
		ModelBusRevision peg = null;
		if (idx != -1) {
			try {
				peg = ModelBusRevision.fromString(url.substring(idx + 1));
				url = url.substring(0, idx);
			} catch (IllegalArgumentException ex) {
				// it is not a revision at the end of the URL
			}
		}
		return new ModelBusEntryReference(url, peg);
	}

	public static boolean useSingleReferenceSignature(ModelBusEntryRevisionReference reference1, ModelBusEntryRevisionReference reference2) {
		int kind1 = reference1.revision.getKind();
		int kind2 = reference2.revision.getKind();
		if ((kind1 == ModelBusRevision.Kind.BASE || kind1 == ModelBusRevision.Kind.WORKING)
				&& (kind2 == ModelBusRevision.Kind.BASE || kind2 == ModelBusRevision.Kind.WORKING)) {
			return false;
		}
		return reference1.path.equals(reference2.path)
				&& (reference1.pegRevision == reference2.pegRevision || (reference1.pegRevision != null && reference1.pegRevision
						.equals(reference2.pegRevision)));
	}

	public static ModelBusEntryRevisionReference getEntryRevisionReference(IRepositoryResource resource) {
		return new ModelBusEntryRevisionReference(ModelBusUtility.encodeURL(resource.getUrl()), resource.getPegRevision(),
				resource.getSelectedRevision());
	}

	public static ModelBusEntryReference getEntryReference(IRepositoryResource resource) {
		return new ModelBusEntryReference(ModelBusUtility.encodeURL(resource.getUrl()), resource.getPegRevision());
	}

	public static ModelBusProperty[] properties(IModelBusConnector proxy, ModelBusEntryRevisionReference reference, IModelBusProgressMonitor monitor)
			throws ModelBusConnectorException {
		final ModelBusProperty[][] retVal = new ModelBusProperty[1][];
		proxy.getProperties(reference, Depth.EMPTY, null, new IModelBusPropertyCallback() {
			public void next(String path, ModelBusProperty[] data) {
				retVal[0] = data;
			}
		}, monitor);
		return retVal[0];
	}

	public static ModelBusChangeStatus[] status(IModelBusConnector proxy, String path, int depth, long options, IModelBusProgressMonitor monitor)
			throws ModelBusConnectorException {
		final ArrayList<ModelBusChangeStatus> statuses = new ArrayList<ModelBusChangeStatus>();
		proxy.status(path, depth, options, null, new IModelbusEntryStatusCallback() {
			public void next(ModelBusChangeStatus status) {
				statuses.add(status);
			}
		}, monitor);
		return statuses.toArray(new ModelBusChangeStatus[statuses.size()]);
	}

	public static void diffStatus(IModelBusConnector proxy, final Collection<ModelBusDiffStatus> statuses, ModelBusEntryRevisionReference reference1,
			ModelBusEntryRevisionReference reference2, int depth, long options, IModelBusProgressMonitor monitor) throws ModelBusConnectorException {
		proxy.diffStatus(reference1, reference2, depth, options, null, new IModelBusDiffStatusCallback() {
			public void next(ModelBusDiffStatus status) {
				statuses.add(status);
			}
		}, monitor);
	}

	public static void diffStatus(IModelBusConnector proxy, final Collection<ModelBusDiffStatus> statuses, ModelBusEntryReference reference,
			ModelBusRevision revision1, ModelBusRevision revision2, int depth, long options, IModelBusProgressMonitor monitor)
			throws ModelBusConnectorException {
		proxy.diffStatus(reference, revision1, revision2, depth, options, null, new IModelBusDiffStatusCallback() {
			public void next(ModelBusDiffStatus status) {
				statuses.add(status);
			}
		}, monitor);
	}

	public static ModelBusMergeStatus[] mergeStatus(IModelBusConnector proxy, ModelBusEntryReference reference, ModelBusRevisionRange[] revisions,
			String path, int depth, long options, IModelBusProgressMonitor monitor) throws ModelBusConnectorException {
		final ArrayList<ModelBusMergeStatus> statuses = new ArrayList<ModelBusMergeStatus>();
		proxy.mergeStatus(reference, revisions, path, depth, options, new IModelBusMergeStatusCallback() {
			public void next(ModelBusMergeStatus status) {
				statuses.add(status);
			}
		}, monitor);
		return statuses.toArray(new ModelBusMergeStatus[statuses.size()]);
	}

	public static ModelBusEntry[] list(IModelBusConnector proxy, ModelBusEntryRevisionReference reference, int depth, int direntFields, long options,
			IModelBusProgressMonitor monitor) throws ModelBusConnectorException {
		final ArrayList<ModelBusEntry> entries = new ArrayList<ModelBusEntry>();
		proxy.list(reference, depth, direntFields, options, new IModelBusEntryCallback() {
			public void next(ModelBusEntry entry) {
				entries.add(entry);
			}
		}, monitor);
		return entries.toArray(new ModelBusEntry[entries.size()]);
	}

	public static ModelBusLogEntry[] logEntries(IModelBusConnector proxy, ModelBusEntryReference reference, ModelBusRevision revisionStart,
			ModelBusRevision revisionEnd, long options, String[] revProps, long limit, IModelBusProgressMonitor monitor)
			throws ModelBusConnectorException {
		final ArrayList<ModelBusLogEntry> entries = new ArrayList<ModelBusLogEntry>();
		proxy.logEntries(reference, revisionStart, revisionEnd, revProps, limit, options, new IModelBusLogEntryCallback() {
			private Stack<ModelBusLogEntry>	mergeTreeBuilder	= new Stack<ModelBusLogEntry>();

			public void next(ModelBusLogEntry log) {
				if (log.revision == ModelBusRevision.INVALID_REVISION_STRING) {
					log = this.mergeTreeBuilder.pop();
					if (this.mergeTreeBuilder.isEmpty()) {
						entries.add(log);
					}
					return;
				}

				if (!this.mergeTreeBuilder.isEmpty()) {
					this.mergeTreeBuilder.peek().add(log);
				} else if (!log.hasChildren()) {
					entries.add(log);
				}
				if (log.hasChildren()) {
					this.mergeTreeBuilder.push(log);
				}
			}
		}, monitor);

		return entries.toArray(new ModelBusLogEntry[entries.size()]);
	}

	public static ModelBusEntryInfo[] info(IModelBusConnector proxy, ModelBusEntryRevisionReference reference, int depth,
			IModelBusProgressMonitor monitor) throws ModelBusConnectorException {
		final ArrayList<ModelBusEntryInfo> infos = new ArrayList<ModelBusEntryInfo>();
		proxy.info(reference, depth, null, new IModelBusEntryInfoCallback() {
			public void next(ModelBusEntryInfo info) {
				infos.add(info);
			}
		}, monitor);
		return infos.toArray(new ModelBusEntryInfo[infos.size()]);
	}

	public static String getStatusText(String status) {
		if (status == null) {
			status = "NotExists";
		}
		return ModelBusTeamPlugin.instance().getResource("Status." + status);
	}

	public static String getOldRoot(String oldUrl, IRepositoryResource[] rootChildren) {
		for (int i = 0; i < rootChildren.length; i++) {
			String childName = rootChildren[i].getName();
			int idx = oldUrl.indexOf(childName);
			if (idx > 0 && oldUrl.charAt(idx - 1) == '/' && (oldUrl.endsWith(childName) || oldUrl.charAt(idx + childName.length()) == '/')) {
				return oldUrl.substring(0, idx - 1);
			}
		}
		return null;
	}

	// public static IRepositoryRoot getTrunkLocation(IRepositoryResource resource) {
	// return ModelBusUtility.getRootLocation(resource, resource.getRepositoryLocation().getTrunkLocation());
	// }

	public static IRepositoryRoot getTrunkLocation(IRepositoryResource resource) {
		return ModelBusUtility.getRootLocation(resource, "trunk");
	}

	public static IRepositoryRoot getBranchesLocation(IRepositoryResource resource) {
		return ModelBusUtility.getRootLocation(resource, "branches");
	}

	public static IRepositoryRoot getTagsLocation(IRepositoryResource resource) {
		return ModelBusUtility.getRootLocation(resource, "tags");
	}

	// public static IRepositoryRoot getBranchesLocation(IRepositoryResource resource) {
	// return ModelBusUtility.getRootLocation(resource, resource.getRepositoryLocation().getBranchesLocation());
	// }
	//
	// public static IRepositoryRoot getTagsLocation(IRepositoryResource resource) {
	// return ModelBusUtility.getRootLocation(resource, resource.getRepositoryLocation().getTagsLocation());
	// }
	//
	// public static IRepositoryContainer getProposedTrunk(IRepositoryLocation location) {
	// return location.asRepositoryContainer(ModelBusUtility.getProposedTrunkLocation(location), false);
	// }
	//
	// public static IRepositoryContainer getProposedBranches(IRepositoryLocation location) {
	// return location.asRepositoryContainer(ModelBusUtility.getProposedBranchesLocation(location), false);
	// }
	//
	// public static IRepositoryContainer getProposedTags(IRepositoryLocation location) {
	// return location.asRepositoryContainer(ModelBusUtility.getProposedTagsLocation(location), false);
	// }
	//
	// public static String getProposedTrunkLocation(IRepositoryLocation location) {
	// String baseUrl = location.getUrl();
	// return location.isStructureEnabled() ? (baseUrl + "/" + location.getTrunkLocation()) : baseUrl;
	// }

	// public static String getProposedBranchesLocation(IRepositoryLocation location) {
	// String baseUrl = location.getUrl();
	// return location.isStructureEnabled() ? (baseUrl + "/" + location.getBranchesLocation()) : baseUrl;
	// }
	//
	// public static String getProposedTagsLocation(IRepositoryLocation location) {
	// String baseUrl = location.getUrl();
	// return location.isStructureEnabled() ? (baseUrl + "/" + location.getTagsLocation()) : baseUrl;
	// }

	public static IRepositoryRoot[] findRoots(String resourceUrl, boolean longestOnly) {
		if (!ModelBusUtility.isValidModelBusURL(resourceUrl)) {
			return new IRepositoryRoot[0];
		}
		IPath url = new Path(resourceUrl);
		// IRepositoryLocation []locations = ModelBusRemoteStorage.instance().getRepositoryLocations();
		ArrayList<IRepositoryRoot> roots = new ArrayList<IRepositoryRoot>();
		// for (int i = 0; i < locations.length; i++) {
		// IPath locationUrl = new Path(locations[i].getUrl());
		// if (url.segmentCount() < locationUrl.segmentCount() && !url.isPrefixOf(locationUrl)) {
		// continue;
		// }
		// if (locationUrl.isPrefixOf(url) || // performance optimization: repository root URL detection [if is not cached] requires interaction with
		// a
		// remote host
		// new Path(locations[i].getRepositoryRootUrl()).isPrefixOf(url)) {
		// ModelBusUtility.addRepositoryRoot(roots, (IRepositoryRoot)locations[i].asRepositoryContainer(resourceUrl, false).getRoot(), longestOnly);
		// }
		// }
		System.out.println("TODOMWA16!!!!");

		IRepositoryRoot[] repositoryRoots = roots.toArray(new IRepositoryRoot[roots.size()]);
		if (!longestOnly) {
			Arrays.sort(repositoryRoots, new Comparator<IRepositoryRoot>() {
				public int compare(IRepositoryRoot first, IRepositoryRoot second) {
					return second.getUrl().compareTo(first.getUrl());
				}
			});
		}
		return repositoryRoots;
	}

	private static void addRepositoryRoot(List<IRepositoryRoot> container, IRepositoryRoot root, boolean longestOnly) {
		if (longestOnly && container.size() > 0) {
			int cnt = new Path(root.getUrl()).segmentCount();
			int cnt2 = new Path(container.get(0).getUrl()).segmentCount();
			if (cnt > cnt2) {
				container.clear();
				container.add(root);
			} else if (cnt == cnt2) {
				container.add(root);
			}
		} else {
			container.add(root);
		}
	}

	public static synchronized String getModelBusFolderName() {
		if (ModelBusUtility.modelbusFolderName == null) {
			String name = FileUtility.getEnvironmentVariables().get("ModelBus_ASP_DOT_NET_HACK") != null ? "_modelbus" : ".modelbus";
			ModelBusUtility.modelbusFolderName = System.getProperty("javamodelbus.admindir", name);
		}
		return ModelBusUtility.modelbusFolderName;
	}

	public static String getResourceParent(IRepositoryResource resource) {
		String parent = "";
		String url = resource.getUrl();
		String rootUrl = resource.getRoot().getUrl();
		if (url.equals(rootUrl)) {
			return "";
		}
		parent = url.substring(rootUrl.length(), url.length() - resource.getName().length() - 1);
		return parent;
	}

	public static IRepositoryResource copyOf(IRepositoryResource resource) {
		String url = resource.getUrl();
		return resource instanceof IRepositoryFile ? (IRepositoryResource) resource.asRepositoryFile(url, false) : resource.asRepositoryContainer(
				url, false);
	}

	public static IRepositoryResource[] makeResourceSet(IRepositoryResource upPoint, String relativeReference, boolean isFile) {
		String url = ModelBusUtility.normalizeURL(upPoint.getUrl() + "/" + relativeReference);
		// IRepositoryLocation location = upPoint.getRepositoryLocation();
		IRepositoryResource downPoint = isFile ? (IRepositoryResource) ModelBusConnector.asRepositoryFile(url, false) : ModelBusConnector
				.asRepositoryContainer(url, false);
		downPoint.setPegRevision(upPoint.getPegRevision());
		downPoint.setSelectedRevision(upPoint.getSelectedRevision());
		return ModelBusUtility.makeResourceSet(upPoint, downPoint);
	}

	public static IRepositoryResource[] makeResourceSet(IRepositoryResource upPoint, IRepositoryResource downPoint) {
		ArrayList<IRepositoryResource> resourceSet = new ArrayList<IRepositoryResource>();
		while (downPoint != null && !downPoint.equals(upPoint)) {
			resourceSet.add(0, downPoint);
			downPoint = downPoint.getParent();
		}
		return resourceSet.toArray(new IRepositoryResource[resourceSet.size()]);
	}

	public static boolean isValidModelBusURL(String url) {
		try {
			URL modelbusUrl = ModelBusUtility.getModelBusUrl(url);
			String host = modelbusUrl.getHost();
			if (!host.matches("[a-zA-Z0-9_\\-]+(?:\\.[a-zA-Z0-9_\\-]+)*") && host.length() > 0 || host.length() == 0
					&& !"file".equals(modelbusUrl.getProtocol())) {
				return false;
			}
			return true;
		} catch (MalformedURLException e) {
			return false;
		}
	}

	public static URL getModelBusUrl(String url) throws MalformedURLException {
		return ModelBusUtility.getModelBusUrlStreamHandler(url).getURL();
	}

	public static ModelBusURLStreamHandler getModelBusUrlStreamHandler(String url) throws MalformedURLException {
		ModelBusURLStreamHandler retVal = new ModelBusURLStreamHandler();
		new URL(null, url, retVal);
		return retVal;
	}

	public static String base64Encode(String data) {
		if (data == null) {
			return null;
		}
		return new String(Base64.encode(data.getBytes()));
	}

	public static String base64Decode(String encoded) {
		if (encoded == null) {
			return null;
		}
		return new String(Base64.decode(encoded.getBytes()));
	}

	public synchronized static void addModelBusNotifyListener(IModelBusConnector proxy, IModelBusNotificationCallback listener) {
		Notify2Composite composite = (Notify2Composite) proxy.getNotificationCallback();
		if (composite == null) {
			proxy.setNotificationCallback(composite = new Notify2Composite());
		}
		composite.add(listener);
	}

	public synchronized static void removeModelBusNotifyListener(IModelBusConnector proxy, IModelBusNotificationCallback listener) {
		Notify2Composite composite = (Notify2Composite) proxy.getNotificationCallback();
		if (composite != null) {
			composite.remove(listener);
		}
	}

	public static void reorder(ModelBusDiffStatus[] statuses, final boolean parent2Child) {
		Arrays.sort(statuses, new Comparator<ModelBusDiffStatus>() {
			public int compare(ModelBusDiffStatus d1, ModelBusDiffStatus d2) {
				return parent2Child ? d1.pathPrev.compareTo(d2.pathPrev) : d2.pathPrev.compareTo(d1.pathPrev);
			}

			public boolean equals(Object obj) {
				return false;
			}
		});
	}

	public static void reorder(ModelBusChangeStatus[] statuses, final boolean parent2Child) {
		Arrays.sort(statuses, new Comparator<ModelBusChangeStatus>() {
			public int compare(ModelBusChangeStatus o1, ModelBusChangeStatus o2) {
				String s1 = o1.path;
				String s2 = o2.path;
				return parent2Child ? s1.compareTo(s2) : s2.compareTo(s1);
			}

			public boolean equals(Object obj) {
				return false;
			}
		});
	}

	public static void reorder(IRepositoryResource[] resources, final boolean parent2Child) {
		Arrays.sort(resources, new Comparator<IRepositoryResource>() {
			public int compare(IRepositoryResource o1, IRepositoryResource o2) {
				String s1 = o1.getUrl();
				String s2 = o2.getUrl();
				return parent2Child ? s1.compareTo(s2) : s2.compareTo(s1);
			}

			public boolean equals(Object obj) {
				return false;
			}
		});
	}

	public static String encodeURL(String url) {
		try {
			url = ModelBusUtility.normalizeURL(url);
			int idx = url.startsWith("file:///") ? "file:///".length() : (url.startsWith("file://") ? (url.indexOf("/", "file://".length()) + 1)
					: (url.indexOf("://") + 3));
			idx = url.indexOf("/", idx);
			if (idx == -1) {
				return url;
			}
			String retVal = url.substring(0, idx);
			StringTokenizer tok = new StringTokenizer(url.substring(idx), "/ ", true);
			// user name should be never encoded
			idx = retVal.indexOf('@');
			if (idx != -1) {
				String protocol = retVal.substring(0, retVal.indexOf("://") + 3);
				String serverPart = retVal.substring(idx);
				retVal = protocol + retVal.substring(protocol.length(), idx) + serverPart;
			}
			while (tok.hasMoreTokens()) {
				String token = tok.nextToken();
				if (token.equals("/")) {
					retVal += token;
				} else if (token.equals(" ")) {
					retVal += "%20";
				} else {
					retVal += URLEncoder.encode(token, "UTF-8");
				}
			}
			return retVal;
		} catch (UnsupportedEncodingException ex) {
			throw new RuntimeException(ex);
		}
	}

	public static String decodeURL(String url) {
		try {
			url = ModelBusUtility.normalizeURL(url);
			int idx = url.startsWith("file:///") ? "file:///".length() : (url.startsWith("file://") ? (url.indexOf("/", "file://".length()) + 1)
					: (url.indexOf("://") + 3));
			idx = url.indexOf("/", idx);
			if (idx == -1) {
				return url;
			}
			String retVal = url.substring(0, idx);
			StringTokenizer tok = new StringTokenizer(url.substring(idx), "/+", true);
			// user name should be never decoded
			idx = retVal.indexOf('@');
			if (idx != -1) {
				String protocol = retVal.substring(0, retVal.indexOf("://") + 3);
				String serverPart = retVal.substring(idx);
				retVal = protocol + retVal.substring(protocol.length(), idx) + serverPart;
			}
			while (tok.hasMoreTokens()) {
				String token = tok.nextToken();
				if (token.equals("/") || token.equals("+")) {
					retVal += token;
				} else {
					retVal += URLDecoder.decode(token, "UTF-8");
				}
			}
			return retVal;
		} catch (UnsupportedEncodingException ex) {
			throw new RuntimeException(ex);
		}
	}

	public static String normalizeURL(String url) {
		StringTokenizer tokenizer = new StringTokenizer(PatternProvider.replaceAll(url, "([\\\\])+", "/"), "/", false);
		String retVal = "";
		while (tokenizer.hasMoreTokens()) {
			String token = tokenizer.nextToken();
			retVal += retVal.length() == 0 ? token : ("/" + token);
		}
		int idx = retVal.indexOf(':') + 1;
		return idx == 0 ? retVal : retVal.substring(0, idx) + (url.startsWith("file:///") ? "//" : "/") + retVal.substring(idx);
	}

	// public static Exception validateRepositoryLocation(IRepositoryLocation location) {
	// IModelBusConnector proxy = location.acquireModelBusProxy();
	// try {
	// proxy.list(new ModelBusEntryRevisionReference(ModelBusUtility.encodeURL(location.getUrl()), null, null), Depth.EMPTY,
	// ModelBusEntry.Fields.NONE,
	// IModelBusConnector.Options.NONE, new IModelBusEntryCallback() {
	// public void next(ModelBusEntry entry) {
	// }
	// }, new ModelBusNullProgressMonitor());
	// }
	// catch (Exception e) {
	// return e;
	// }
	// finally {
	// location.releaseModelBusProxy(proxy);
	// location.dispose();
	// }
	// return null;
	// }

	// public static void configureProxy(IModelBusConnector proxy) {
	// proxy.setUsername(location.getUsername());
	// proxy.setPassword(location.getPassword());
	//
	// SSLSettings sslSettings = location.getSSLSettings();
	// String host = "localhost";
	// String protocol = "file";
	// try {
	// protocol = ModelBusUtility.getModelBusUrl(location.getUrl()).getProtocol();
	// if (!protocol.equals("file")) {
	// host = ModelBusUtility.getModelBusUrl(location.getUrl()).getHost();
	// }
	// }
	// catch (MalformedURLException ex) {
	// //skip
	// }
	// IProxyService proxyService = ModelBusTeamPlugin.instance().getProxyService();
	// String proxyType = protocol.equals("https") ? IProxyData.HTTPS_PROXY_TYPE : IProxyData.HTTP_PROXY_TYPE;
	// ModelBusCachedProxyCredentialsManager proxyCredetialsManager = ModelBusRemoteStorage.instance().getProxyCredentialsManager();
	// IProxyData proxyData = proxyService.getProxyDataForHost(host, proxyType);
	// if (proxyService.isProxiesEnabled() && proxyData != null) {
	// proxy.setProxy(proxyData.getHost(), proxyData.getPort(), proxyCredetialsManager.getUsername(), proxyCredetialsManager.getPassword());
	// }
	// else {
	// proxy.setProxy(null, -1, null, null);
	// }
	// if (sslSettings.isAuthenticationEnabled()) {
	// proxy.setClientSSLCertificate(sslSettings.getCertificatePath(), sslSettings.getPassPhrase());
	// }
	// else {
	// proxy.setClientSSLCertificate(null, null);
	// }
	//
	// SSHSettings sshSettings = location.getSSHSettings();
	// if (!sshSettings.isUseKeyFile()) {
	// proxy.setSSHCredentials(location.getUsername(), location.getPassword(), sshSettings.getPort());
	// }
	// else if (sshSettings.getPrivateKeyPath().length() > 0) {
	// proxy.setSSHCredentials(location.getUsername(), sshSettings.getPrivateKeyPath(), sshSettings.getPassPhrase(), sshSettings.getPort());
	// }
	// else {
	// proxy.setSSHCredentials(null, null, null, -1);
	// }
	// }

	public static ModelBusChangeStatus getModelBusInfoForNotConnected(IResource root) {
		// @SuppressWarnings("unused")
		// IPath location = FileUtility.getResourcePath(root);
		System.out.println("TODOMWA38");
		return null;
		// IPath checkedPath = root.getType() == IResource.FILE ? location.removeLastSegments(1) : location;
		// if (!checkedPath.append(ModelBusUtility.getModelBusFolderName()).toFile().exists()) {
		// return null;
		// }
		// IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		// //CoreExtensionsManager.instance().getModelBusConnectorFactory().newInstance();
		// try {
		// ModelBusChangeStatus []st = ModelBusUtility.status(proxy, location.toString(), Depth.IMMEDIATES,
		// IModelBusConnector.Options.INCLUDE_UNCHANGED,
		// new ModelBusNullProgressMonitor());
		// if (st != null && st.length > 0) {
		// ModelBusUtility.reorder(st, true);
		// return st[0].url == null ? null : st[0];
		// }
		// return null;
		// }
		// catch (Exception ex) {
		// return null;
		// }
		// finally {
		// proxy.dispose();
		// }
		// throw new RuntimeException("NYI");

	}

	public static String getPropertyForNotConnected(IResource root, String propertyName) {
		String location = FileUtility.getWorkingCopyPath(root);
		IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		// CoreExtensionsManager.instance().getModelBusConnectorFactory().newInstance();
		try {
			ModelBusProperty data = proxy.getProperty(new ModelBusEntryRevisionReference(location, null, ModelBusRevision.WORKING), propertyName,
					new ModelBusNullProgressMonitor());
			return data == null ? null : data.value;
		} catch (Exception ex) {
			return null;
		} finally {
			proxy.dispose();
		}
	}

	public static boolean isIgnored(IResource resource) {
		// Ignore WorkspaceRoot, derived and team-private resources and resources from TeamHints
		if (resource instanceof IWorkspaceRoot || resource.isDerived() || FileUtility.isModelBusInternals(resource) || Team.isIgnoredHint(resource)
				|| ModelBusUtility.isMergeParts(resource)) {
			return true;
		}
		try {
			IIgnoreRecommendations[] ignores = CoreExtensionsManager.instance().getIgnoreRecommendations();
			for (int i = 0; i < ignores.length; i++) {
				if (ignores[i].isAcceptableNature(resource) && ignores[i].isIgnoreRecommended(resource)) {
					return true;
				}
			}
		} catch (Exception ex) {
			// cannot be correctly processed in the caller context
		}
		return false;
	}

	public static Map<IProject, List<IResource>> splitWorkingCopies(IResource[] resources) {
		Map<IProject, List<IResource>> wc2Resources = new HashMap<IProject, List<IResource>>();

		for (int i = 0; i < resources.length; i++) {
			IProject wcRoot = resources[i].getProject();

			List<IResource> wcResources = wc2Resources.get(wcRoot);
			if (wcResources == null) {
				wc2Resources.put(wcRoot, wcResources = new ArrayList<IResource>());
			}
			wcResources.add(resources[i]);
		}

		return wc2Resources;
	}

	public static Map splitWorkingCopies(File[] files) {
		Map wc2Resources = new HashMap();

		IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		// CoreExtensionsManager.instance().getModelBusConnectorFactory().newInstance();
		try {
			Map<File, ModelBusEntryInfo> file2info = new HashMap<File, ModelBusEntryInfo>();
			for (int i = 0; i < files.length; i++) {
				file2info.put(files[i], ModelBusUtility.getModelBusInfo(files[i], proxy));
			}

			ArrayList<File> restOfFiles = new ArrayList<File>(Arrays.asList(files));
			while (restOfFiles.size() > 0) {
				File current = restOfFiles.get(0);
				ModelBusEntryInfo info = file2info.get(current);
				Object[] wcRoot = ModelBusUtility.getWCRoot(proxy, current, info);

				List wcResources = (List) wc2Resources.get(wcRoot[0]);
				if (wcResources == null) {
					wc2Resources.put(wcRoot[0], wcResources = new ArrayList());
				}

				Path rootPath = new Path(((File) wcRoot[0]).getAbsolutePath());
				Path rootInfoPath = new Path(((ModelBusEntryInfo) wcRoot[1]).url);
				for (Iterator it = restOfFiles.iterator(); it.hasNext();) {
					File checked = (File) it.next();
					if (rootPath.isPrefixOf(new Path(checked.getAbsolutePath()))) {
						if (rootInfoPath.isPrefixOf(new Path(file2info.get(checked).url))) {
							wcResources.add(checked);
							it.remove();
						}
					}
				}
			}
		} finally {
			proxy.dispose();
		}

		return wc2Resources;
	}

	private static Object[] getWCRoot(IModelBusConnector proxy, File node, ModelBusEntryInfo info) {
		File oldRoot = node;
		ModelBusEntryInfo oldInfo = info;

		node = node.getParentFile();
		while (node != null) {
			ModelBusEntryInfo rootInfo = ModelBusUtility.getModelBusInfo(node, proxy);
			if (rootInfo != null) {
				if (oldInfo == null) {
					oldInfo = rootInfo;
				} else if (!new Path(rootInfo.url).isPrefixOf(new Path(oldInfo.url))) {
					return new Object[] { oldRoot, oldInfo };
				}
				oldRoot = node;
				node = node.getParentFile();
			} else if (oldInfo != null) {
				return new Object[] { oldRoot, oldInfo };
			}
		}

		if (oldInfo == null) {
			String errMessage = ModelBusTeamPlugin.instance().getResource("Error.NonModelBusPath", new String[] { oldRoot.getAbsolutePath() });
			throw new RuntimeException(errMessage);
		}
		return new Object[] { oldRoot, oldInfo };
	}

	public static ModelBusEntryInfo getModelBusInfo(File root) {
		IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		// CoreExtensionsManager.instance().getModelBusConnectorFactory().newInstance();
		try {
			return ModelBusUtility.getModelBusInfo(root, proxy);
		} finally {
			proxy.dispose();
		}
	}

	public static ModelBusEntryInfo getModelBusInfo(File root, IModelBusConnector proxy) {
		if (root.exists()) {
			File modelbusMeta = root.isDirectory() ? root : root.getParentFile();
			modelbusMeta = new File(modelbusMeta.getAbsolutePath() + "/" + ModelBusUtility.getModelBusFolderName());
			if (modelbusMeta.exists()) {
				try {
					ModelBusEntryInfo[] st = ModelBusUtility.info(proxy, new ModelBusEntryRevisionReference(root.getAbsolutePath(), null,
							ModelBusRevision.BASE), Depth.EMPTY, new ModelBusNullProgressMonitor());
					return st != null && st.length != 0 ? st[0] : null;
				} catch (Exception ex) {
					return null;
				}
			}
		}
		return null;
	}

	public static String[] asURLArray(IRepositoryResource[] resources, boolean encode) {
		String[] retVal = new String[resources.length];
		for (int i = 0; i < resources.length; i++) {
			retVal[i] = encode ? ModelBusUtility.encodeURL(resources[i].getUrl()) : resources[i].getUrl();
		}
		return retVal;
	}

	// public static Map splitRepositoryLocations(IRepositoryResource []resources) throws Exception {
	// Map repository2Resources = new HashMap();
	// for (int i = 0; i < resources.length; i++) {
	// IRepositoryLocation location = resources[i].getRepositoryLocation();
	//
	// List tResources = (List)repository2Resources.get(location);
	// if (tResources == null) {
	// repository2Resources.put(location, tResources = new ArrayList());
	// }
	// tResources.add(resources[i]);
	// }
	// return ModelBusUtility.combineLocationsByUUID(repository2Resources);
	// }

	// public static Map splitRepositoryLocations(IResource []resources) throws Exception{
	// Map repository2Resources = new HashMap();
	// ModelBusRemoteStorage storage = ModelBusRemoteStorage.instance();
	// for (int i = 0; i < resources.length; i++) {
	// IRepositoryLocation location = storage.getRepositoryLocation(resources[i]);
	//
	// List tResources = (List)repository2Resources.get(location);
	// if (tResources == null) {
	// repository2Resources.put(location, tResources = new ArrayList());
	// }
	// tResources.add(resources[i]);
	// }
	// return ModelBusUtility.combineLocationsByUUID(repository2Resources);
	// }

	// public static Map splitRepositoryLocations(File []files) throws Exception{
	// Map repository2Resources = new HashMap();
	// for (int i = 0; i < files.length; i++) {
	// IRepositoryResource resource = ModelBusFileStorage.instance().asRepositoryResource(files[i], false);
	// IRepositoryLocation location = resource.getRepositoryLocation();
	//
	// List tResources = (List)repository2Resources.get(location);
	// if (tResources == null) {
	// repository2Resources.put(location, tResources = new ArrayList());
	// }
	// tResources.add(files[i]);
	// }
	// return ModelBusUtility.combineLocationsByUUID(repository2Resources);
	// }

	public static int getNodeKind(String path, int kind, boolean ignoreNone) {
		File f = new File(path);
		if (f.exists()) {
			return f.isDirectory() ? Kind.DIR : Kind.FILE;
		} else if (kind == Kind.DIR) {
			return Kind.DIR;
		} else if (kind == Kind.FILE) {
			return Kind.FILE;
		}
		// ignore files absent in the WC base and WC working. But what is the reason why it is reported ?
		if (ignoreNone) {
			return Kind.NONE;
		}
		String errMessage = ModelBusTeamPlugin.instance().getResource("Error.UnrecognizedNodeKind", new String[] { String.valueOf(kind), path });
		throw new RuntimeException(errMessage);
	}

	public static IRepositoryResource[] shrinkChildNodes(IRepositoryResource[] resources) {
		Set<IRepositoryResource> roots = new HashSet<IRepositoryResource>(Arrays.asList(resources));
		for (int i = 0; i < resources.length; i++) {
			if (ModelBusUtility.hasRoots(roots, resources[i])) {
				roots.remove(resources[i]);
			}
		}
		return roots.toArray(new IRepositoryResource[roots.size()]);
	}

	public static IRepositoryResource[] getCommonParents(IRepositoryResource[] resources) {
		Map<IRepositoryResource, ArrayList> byRepositoryRoots = new HashMap<IRepositoryResource, ArrayList>();
		for (int i = 0; i < resources.length; i++) {
			IRepositoryResource root = resources[i].getRoot();
			ArrayList tmp = byRepositoryRoots.get(root);
			if (tmp == null) {
				byRepositoryRoots.put(root, tmp = new ArrayList());
			}
			tmp.add(resources[i]);
		}
		HashSet<IRepositoryResource> container = new HashSet<IRepositoryResource>();
		for (ArrayList tmp : byRepositoryRoots.values()) {
			IRepositoryResource parent = ModelBusUtility.getCommonParent((IRepositoryResource[]) tmp.toArray(new IRepositoryResource[tmp.size()]));
			if (parent != null) {
				container.add(parent);
			}
		}
		return container.toArray(new IRepositoryResource[container.size()]);
	}

	public static String getAscendant(IRepositoryResource resource) {
		String pathUpToRoot = ModelBusUtility.getPathUpToRoot(resource);
		int idx = pathUpToRoot.indexOf('/');
		return idx == -1 ? pathUpToRoot : pathUpToRoot.substring(0, idx);
	}

	public static String getDescendant(IRepositoryResource resource) {
		String pathUpToRoot = ModelBusUtility.getPathUpToRoot(resource);
		int idx = pathUpToRoot.lastIndexOf('/');
		return idx == -1 ? pathUpToRoot : pathUpToRoot.substring(idx + 1);
	}

	public static String getPathUpToRoot(IRepositoryResource resource) {
		IRepositoryResource root = resource.getRoot();
		return root == resource ? resource.getName() : resource.getUrl().substring(root.getUrl().length() + 1);
	}

	/**
	 * Compares two revisions
	 * 
	 * @return: 0 if equal, -1 if less, 1 if more
	 */
	public static int compareRevisions(ModelBusRevision first, ModelBusRevision second, ModelBusEntryRevisionReference referenceFirst,
			ModelBusEntryRevisionReference referenceSecond, IModelBusConnector proxy) throws ModelBusConnectorException {
		if (first.getKind() == ModelBusRevision.Kind.STRING && second.getKind() == ModelBusRevision.Kind.STRING) {
			ModelBusRevision fromNumber = first;
			ModelBusRevision toNumber = second;
			return first.isGreaterThan(second) ? 1 : (fromNumber.equals(toNumber) ? 0 : -1);
		}
		ModelBusRevision.DateRevision fromDate = null;
		ModelBusRevision.DateRevision toDate = null;
		if (first.getKind() == ModelBusRevision.Kind.DATE) {
			fromDate = (ModelBusRevision.DateRevision) first;
		} else {
			ModelBusEntryInfo[] entryInfo = ModelBusUtility.info(proxy, referenceFirst, Depth.UNKNOWN, new ModelBusNullProgressMonitor());
			fromDate = ModelBusRevision.fromDate(entryInfo[0].lastChangedDate);
		}
		if (second.getKind() == ModelBusRevision.Kind.DATE) {
			toDate = (ModelBusRevision.DateRevision) second;
		} else {
			ModelBusEntryInfo[] entryInfo = ModelBusUtility.info(proxy, referenceSecond, Depth.UNKNOWN, new ModelBusNullProgressMonitor());
			toDate = ModelBusRevision.fromDate(entryInfo[0].lastChangedDate);
		}
		return fromDate.getDate() > toDate.getDate() ? 1 : (fromDate.getDate() == toDate.getDate() ? 0 : -1);
	}

	private static boolean isMergeParts(IResource resource) {
		String ext = resource.getFileExtension();
		return ext != null && ext.matches("r(\\d)+");
	}

	// private static Map combineLocationsByUUID(Map repository2Resources) throws Exception{
	// Map locationUtility2Resources = new HashMap();
	// for (Iterator it = repository2Resources.entrySet().iterator(); it.hasNext(); ) {
	// Map.Entry entry = (Map.Entry)it.next();
	// // IRepositoryLocation location = (IRepositoryLocation)entry.getKey();
	// List tResources = (List)entry.getValue();
	// RepositoryLocationUtility locationUtility = new RepositoryLocationUtility(location);
	// List tResources2 = (List)locationUtility2Resources.get(locationUtility);
	// if (tResources2 == null) {
	// locationUtility2Resources.put(locationUtility, tResources2 = new ArrayList());
	// }
	// tResources2.addAll(tResources);
	// }
	// repository2Resources.clear();
	// for (Iterator it = locationUtility2Resources.entrySet().iterator(); it.hasNext(); ) {
	// Map.Entry entry = (Map.Entry)it.next();
	// RepositoryLocationUtility locationUtility = (RepositoryLocationUtility)entry.getKey();
	// repository2Resources.put(locationUtility.location, entry.getValue());
	// }
	// return repository2Resources;
	// }

	private static boolean hasRoots(Set<IRepositoryResource> roots, IRepositoryResource node) {
		while ((node = node.getParent()) != null) {
			if (roots.contains(node)) {
				return true;
			}
		}
		return false;
	}

	private static IRepositoryResource getCommonParent(IRepositoryResource[] resources) {
		if (resources == null || resources.length == 0) {
			return null;
		}
		IRepositoryResource base = resources[0].getParent();
		while (base != null) { // can be null for resources from different repositories
			int startsCnt = 0;
			Path baseUrl = new Path(base.getUrl());
			for (int i = 0; i < resources.length; i++) {
				if (baseUrl.isPrefixOf(new Path(resources[i].getUrl()))) {
					startsCnt++;
				}
			}
			if (startsCnt == resources.length) {
				break;
			}
			base = base.getParent();
		}
		return base;
	}

	private static IRepositoryRoot getRootLocation(IRepositoryResource resource, String rootName) {
		// IRepositoryLocation location = resource.getRepositoryLocation();
		IRepositoryRoot root = (IRepositoryRoot) resource.getRoot();
		// if (!location.isStructureEnabled()) {
		// return root;
		// }

		int rootKind = root.getKind();
		if (rootKind == IRepositoryRoot.KIND_ROOT) {
			return (IRepositoryRoot) root.asRepositoryContainer(rootName, false);
		} else if (rootKind == IRepositoryRoot.KIND_LOCATION_ROOT) {
			IRepositoryResource parent = root.getParent();
			if (parent == null) {
				return (IRepositoryRoot) root.asRepositoryContainer(rootName, false);
			}
			IRepositoryRoot tmp = (IRepositoryRoot) parent.getRoot();
			rootKind = tmp.getKind();
			if (rootKind == IRepositoryRoot.KIND_ROOT) {
				return (IRepositoryRoot) root.asRepositoryContainer(rootName, false);
			}
			root = tmp;
		}
		IRepositoryResource rootParent = root.getParent();
		return (IRepositoryRoot) rootParent.asRepositoryContainer(rootName, false);
	}

	public static boolean isTagOperated(IResource[] resources) {
		for (int i = 0; i < resources.length; i++) {
			if (((IRepositoryRoot) ModelBusRemoteStorage.instance().asRepositoryResource(resources[i]).getRoot()).getKind() == IRepositoryRoot.KIND_TAGS) {
				return true;
			}
		}
		return false;
	}

	public static String getDepthArg(int depth) {
		String depthArg = " --depth ";
		if (depth == Depth.EMPTY) {
			return depthArg + "empty ";
		}
		if (depth == Depth.INFINITY) {
			return depthArg + "infinity";
		}
		if (depth == Depth.IMMEDIATES) {
			return depthArg + "immediates ";
		}
		return depthArg + "files ";
	}

	private ModelBusUtility() {

	}

	// MES
	@SuppressWarnings("unchecked")
	public static Map<?, ?> fillRepository2ResourcesMap(IRepositoryResource[] resources) {
		Map repository2Resources = new HashMap();
		List innerList = new ArrayList();
		String modelBusRepositoryLocation = ModelBusConnector.getRepositoryRootUrl(); // dummy!

		repository2Resources.put(modelBusRepositoryLocation, innerList);
		for (int i = 0; i < resources.length; i++) {
			innerList.add(resources[i]);
		}
		return repository2Resources;
	}

}
