/*******************************************************************************
 * Copyright (c) 2000, 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.modelbus.team.eclipse.repository;

import java.io.IOException;

import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPluginDescriptor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.swt.widgets.Display;
import org.eclipse.team.core.TeamException;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.modelbus.core.lib.notification.NotificationListenerManager;
import org.modelbus.team.eclipse.repository.ui.LockHelper;
import org.modelbus.team.eclipse.repository.ui.ModelBusRepositoryDecoratorPing;
import org.modelbus.team.eclipse.repository.ui.ModelBusRepositoryStatus;
import org.osgi.framework.BundleContext;

/**
 * This is the plugin class for the file system examples. It provides the following:
 * 
 * <ol>
 * <li>public fields for the plugin and provider IDs as defined in the plugin.xml
 * <li>initialization on startup of Policy class that provides internationalization of strings
 * <li>helper methods for outputing IStatus objects to the log
 * <li>helper methods for converting CoreExceptions and IOExceptions to TeamExceptions
 * </ol>
 */
public class ModelBusRepositoryPlugin extends AbstractUIPlugin /* implements ModelBusConsumer */{

	/**
	 * This is the ID of the plugin as defined in the plugin.xml
	 */
	public static final String				ID				= "org.modelbus.team.eclipse.repository";	//$NON-NLS-1$

	/**
	 * This is the provider ID of the plugin as defined in the plugin.xml
	 */
	public static final String				PROVIDER_ID		= ID + ".ModelBusRepositoryProvider";		//$NON-NLS-1$

	private static String					repLocation;

	// This static field will hold the singleton instance of the plugin class
	private static ModelBusRepositoryPlugin	plugin;

	// private IRepositoryHelper repositoryHelper;
	// private PessimisticFilesystemProviderPlugin pessPlugin;

	private ModelBusRepositoryDecoratorPing	modelBusRepositoryDecoratorPing;

	private final long						pingInterval	= 5000;

	private final ModelBusRepositoryStatus	modelBusRepositoryStatus;

	/**
	 * Override the standard plugin constructor.
	 * 
	 * @param descriptor
	 *            the plugin descriptor
	 */
	public ModelBusRepositoryPlugin(IPluginDescriptor descriptor) {
		super(descriptor);
		// record this instance as the singleton
		plugin = this;
		modelBusRepositoryStatus = new ModelBusRepositoryStatus();
		// Instantiate pessimistic provider
		// pessPlugin = new PessimisticFilesystemProviderPlugin(descriptor);
	}

	/**
	 * Return the singleton instance of the plugin class to allow other classes in the plugin access to plugin instance methods such as those for
	 * logging errors, etc.
	 */
	public static ModelBusRepositoryPlugin getPlugin() {
		return plugin;
	}

	/**
	 * Helper method to convert a CoreException into a TeamException. We do this to maintain the core status and code. This type of mapping may not be
	 * appropriate in more complicated exception handling situations.
	 * 
	 * @param e
	 *            the CoreException
	 */
	public static TeamException wrapException(CoreException e) {
		return new TeamException(e.getStatus());
	}

	/**
	 * Helper method to convert an IOException into a TeamException. This type of mapping may not be appropriate in more complicated exception
	 * handling situations.
	 * 
	 * @param e
	 *            the CoreException
	 */
	public static TeamException wrapException(IOException e) {
		return new TeamException(new Status(IStatus.ERROR, ModelBusRepositoryPlugin.ID, TeamException.IO_FAILED, e.getMessage(), e));
	}

	public ModelBusRepositoryStatus getRepositoryStatus() {
		return modelBusRepositoryStatus;
	}

	/**
	 * Helper method to log an exception status.
	 * 
	 * @param status
	 *            the status to be logged
	 */
	public static void log(IStatus status) {
		plugin.getLog().log(status);
	}

	/**
	 * Returns the standard display to be used. The method first checks, if the thread calling this method has an associated display. If so, this
	 * display is returned. Otherwise the method returns the default display.
	 */
	public static Display getStandardDisplay() {
		Display display = Display.getCurrent();
		if (display == null) {
			display = Display.getDefault();
		}
		return display;
	}

	@Override
	public void start(BundleContext context) throws Exception {
		super.start(context);
		// ModelBusCoreLib.getRepositoryHelper().addConsumer(this);
		IWorkspace workspace = ResourcesPlugin.getWorkspace();
		workspace.addResourceChangeListener(ModelBusRepositoryHelper.getModelBusRepositoryHelper());
		org.modelbus.preferences.dependencies.DependenciesPreferencePlugin.getDefault().getPreferenceStore()
				.addPropertyChangeListener(ModelBusRepositoryHelper.getModelBusRepositoryHelper());
		// org.modelbus.preferences.ModelBusPreferencePlugin.getDefault().getPreferenceStore().addPropertyChangeListener(ModelBusRepositoryHelper.getModelBusRepositoryHelper());
		// Call startup on the Pessimistic Plugin
		// pessPlugin.startup();
		LockHelper lockHelper = LockHelper.getLockHelper();

		// final NotificationListenerManager nManager = NotificationListenerManager.getNotificationListenerManager();
		//
		// if (nManager != null) {
		// nManager.addNotificationListener(lockHelper);
		// }

		// modelBusRepositoryStatus.addObserver(lockHelper);
		modelBusRepositoryDecoratorPing = new ModelBusRepositoryDecoratorPing(pingInterval);
		modelBusRepositoryDecoratorPing.start();

	}

	@Override
	public void stop(BundleContext context) throws Exception {
		try {
			// if (pessPlugin != null)
			// pessPlugin.shutdown();
		} finally {
			IWorkspace workspace = ResourcesPlugin.getWorkspace();
			ModelBusRepositoryHelper modelBusRepositoryHelper = ModelBusRepositoryHelper.getModelBusRepositoryHelper();
			workspace.removeResourceChangeListener(modelBusRepositoryHelper);
			org.modelbus.preferences.dependencies.DependenciesPreferencePlugin.getDefault().getPreferenceStore()
					.removePropertyChangeListener(modelBusRepositoryHelper);
			// org.modelbus.preferences.ModelBusPreferencePlugin.getDefault().getPreferenceStore().removePropertyChangeListener(ModelBusRepositoryHelper.getModelBusRepositoryHelper());
			LockHelper lockHelper = LockHelper.getLockHelper();

			final NotificationListenerManager nManager = NotificationListenerManager.getNotificationListenerManager();

			if (nManager != null) {
				nManager.removeNotificationListener(lockHelper);
			}

			modelBusRepositoryStatus.deleteObservers();
			modelBusRepositoryDecoratorPing.kill();

			super.stop(context);
		}
	}

	// @Override
	// public void registerRepositoryHelper(IRepositoryHelper helper) {
	// this.repositoryHelper = helper;
	// // System.out.println("TeamProvider: core lib is there");
	// }
	//
	// public IRepositoryHelper getRepositoryHelper() {
	// return repositoryHelper;
	// }

	public String getResource(String key) {
		return ResourceUtil.getResource(Platform.getResourceBundle(this.getBundle()), key);
	}

	protected static String getRepositoryLocation() {
		if (repLocation == null) {
			repLocation = RepositoryLocationHelper.getPropertyRepositoryLocation();
		}
		return repLocation;
	}

}
