/*******************************************************************************
 * ongoing check MWA
 *******************************************************************************/
package org.modelbus.team.eclipse.repository;

import java.io.File;
import java.io.IOException;
import java.io.UnsupportedEncodingException;

import org.eclipse.core.resources.IFileModificationValidator;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceRuleFactory;
import org.eclipse.core.resources.team.ResourceRuleFactory;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.emf.common.util.URI;
import org.eclipse.team.core.RepositoryProvider;
import org.eclipse.team.core.TeamException;
import org.eclipse.team.core.history.IFileHistoryProvider;
import org.eclipse.team.core.variants.IResourceVariant;
import org.modelbus.core.lib.IRepositoryHelper;
import org.modelbus.dosgi.repository.descriptor.InvalidRevisionException;
import org.modelbus.dosgi.repository.descriptor.NonExistingResourceException;
import org.modelbus.dosgi.repository.descriptor.RepositoryAuthentificationException;
import org.modelbus.dosgi.repository.descriptor.RepositoryDirEntry;
import org.modelbus.dosgi.repository.descriptor.RepositoryNodeKind;
import org.modelbus.team.eclipse.repository.history.ModelBusRepositoryHistoryProvider;
import org.modelbus.team.eclipse.repository.subscriber.ModelBusRepositoryLocalFileResourceVariant;
import org.modelbus.team.eclipse.repository.subscriber.ModelBusRepositoryResourceVariant;
import org.modelbus.team.eclipse.repository.subscriber.ModelBusRepositorySubscriber;
import org.modelbus.team.eclipse.repository.subscriber.ModelBusThreeWaySynchronizer;

/**
 * This example illustrates how to create a concrete implementation of a <code>RepositoryProvider</code> that uses the file system to act as the
 * repository. See the plugin.xml file for the xml required to register this provider with the Team extension point
 * <code>org.eclipse.team.core.repository</code>. The plugin.xml file also contains examples of how to filter menu items using a repository provider's
 * ID.
 * 
 * <p>
 * This example provider illustrates the following:
 * <ol>
 * <li>simple working implementation of <code>RepositoyProvider</code>
 * <li>storage of a persistent property with the project (which provides the target location for the provider)
 * <li>access to an instance of <code>SimpleAccessOperations</code> for performing simple file operations
 * </ol>
 * 
 * <p>
 * Additional functionality that will be illustrated in the future include:
 * <ol>
 * <li>Validate Save/Validate Edit
 * <li>Move/Delete Hook
 * <li>Project Sets
 * <li>Use of the workspace synchronizer (ISynchronizer)
 * <li>Use of decorators
 * <li>combining streams and progress monitors to get responsive UI
 * </ol>
 * 
 */
public class ModelBusRepositoryProvider extends RepositoryProvider {

	/*
	 * Create a custom rule factory to allow more optimistic concurrency
	 */
	private static final ResourceRuleFactory			RESOURCE_RULE_FACTORY	= new ResourceRuleFactory() {
																					// Just need a subclass to instantiate
																				};

	// The url where the repository is deployed.
	private String										repositoryURL;

	// The location of the folder on file system where the repository is stored.
	private IPath										root;

	// The QualifiedName that is used to persist the location across workspace
	// as a persistent property on a resource
	public static QualifiedName							MODELBUS_REPO_LOC		= new QualifiedName(ModelBusRepositoryPlugin.ID,
																						"modelbus_repository_location");	//$NON-NLS-1$

	private static ModelBusRepositoryHistoryProvider	fileHistoryProvider;

	/**
	 * Create a new ModelBusRepositoryProvider.
	 */
	public ModelBusRepositoryProvider() {
		super();
	}

	/**
	 * This method is invoked when the provider is mapped to a project. Although we have access to the project at this point (using
	 * <code>getProject()</code>, we don't know the root location so there is nothing we can do yet.
	 * 
	 * @see org.eclipse.team.core.RepositoryProvider#configureProject()
	 */
	public void configureProject() throws CoreException {
		ModelBusRepositorySubscriber.getInstance().handleRootChanged(getProject(), true /* added */);
	}

	/**
	 * This method is invoked when the provider is unmapped from its project.
	 * 
	 * @see org.eclipse.core.resources.IProjectNature#deconfigure()
	 */
	public void deconfigure() throws CoreException {
		// Clear the persistant property containing the location
		getProject().setPersistentProperty(MODELBUS_REPO_LOC, null);
		ModelBusRepositorySubscriber.getInstance().handleRootChanged(getProject(), false /* removed */);
	}

	/**
	 * Return the provider ID as specified in the plugin.xml
	 * 
	 * @see RepositoryProvider#getID()
	 */
	public String getID() {
		return ModelBusRepositoryPlugin.PROVIDER_ID;
	}

	/**
	 * Set the file system location for the provider. This mist be invoked after the provider is mapped and configured but before the provider is used
	 * to perform any operations.
	 * 
	 * @param location
	 *            the path representing the location where the project contents will be stored.
	 * @throws TeamException
	 */

	public void setTargetLocation(String location) throws TeamException, RepositoryAuthentificationException, NonExistingResourceException {

		// set the instance variable to the provided path
		repositoryURL = location;

		// String repositoryLocation = org.modelbus.preferences.ModelBusPreferencePlugin.getDefault().getPreferenceStore().getString(
		// org.modelbus.preferences.PreferenceConstants.REPOSITORY_LOCATION_STRING);
		boolean checkDependencies = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin.getDefault().getPreferenceStore()
				.getBoolean(org.modelbus.preferences.dependencies.preferences.PreferenceConstants.CHECK_DEPENDENCIES);
		String[] modelExtensions = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin.getDefault().getExtensionPreference();

		IRepositoryHelper repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();
		try {

			// ensure that the location is a folder (if it exists)
			// dealing with HEAD
			if (repositoryHelper.exists(UserSessionHelper.getSession(), URI.createURI(location), IRepositoryHelper.REVISION_HEAD)) {
				if (!RepositoryNodeKind.DIR.equals(repositoryHelper.checkPath(UserSessionHelper.getSession(), URI.createURI(location), IRepositoryHelper.REVISION_HEAD))) {
					throw new TeamException(Policy.bind("ModelBusRepositoryProvider.mustBeFolder", location)); //$NON-NLS-1$
				}
			} else {
				// TODO: pass logMessage here
				final String logMessage = "TODO: logMessage";
				repositoryHelper.createDir(UserSessionHelper.getSession(), URI.createURI(location), logMessage);
			}

			// record the location as a persistent property so it will be
			// remembered across platform invocations
			getProject().setPersistentProperty(MODELBUS_REPO_LOC, location);
			// mark project as in-sync to forestall detection of a
			// pseudo-conflicting addition on synchronize
			ModelBusRepositorySubscriber.getInstance().getSynchronizer().setBaseBytes(getProject(), new byte[0]);
			String revision = repositoryHelper.getInfo(UserSessionHelper.getSession(), URI.createURI(location), IRepositoryHelper.REVISION_HEAD).getRevision();
			((ModelBusThreeWaySynchronizer) ModelBusRepositorySubscriber.getInstance().getSynchronizer()).setWorkingCopyRevision(getProject(),
					revision);
		} catch (CoreException e) {
			e.printStackTrace();
			throw ModelBusRepositoryPlugin.wrapException(e);
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw ModelBusRepositoryPlugin.wrapException(e);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw ModelBusRepositoryPlugin.wrapException(e);
		} catch (InvalidRevisionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new RuntimeException(e);
		}
	}

	// public void setTargetLocation(String location) throws TeamException {
	//
	// // set the instance variable to the provided path
	// repositoryURL = location;
	//
	// String repositoryLocation =
	// org.modelbus.preferences.ModelBusPreferencePlugin.getDefault().getPreferenceStore().getString(
	// org.modelbus.preferences.PreferenceConstants.REPOSITORY_LOCATION_STRING);
	//
	// RepositoryHelper repositoryHelper = new
	// RepositoryHelper(repositoryLocation);
	// try {
	//
	// // ensure that the location is a folder (if it exists)
	//
	// if
	// (!RepositoryNodeKind.DIR.equals(repositoryHelper.checkPath(UserSessionHelper.getSession(),
	// location))) {
	//				throw new TeamException(Policy.bind("ModelBusRepositoryProvider.mustBeFolder", location)); //$NON-NLS-1$
	// }
	//
	// // record the location as a persistant property so it will be
	// // remembered across platform invokations
	// getProject().setPersistentProperty(MODELBUS_REPO_LOC, location);
	// } catch (AxisFault e) {
	// throw ModelBusRepositoryPlugin.wrapException(e);
	// } catch (RemoteException e) {
	// throw ModelBusRepositoryPlugin.wrapException(e);
	// } catch (CoreException e) {
	// throw ModelBusRepositoryPlugin.wrapException(e);
	// }
	// }

	/**
	 * Returns the folder in the file system to which the provider is connected. Return <code>null</code> if there is no location or there was a
	 * problem determining it.
	 * 
	 * @return IPath The path to the root of the repository. MAY BE NULL!!!!!
	 */
	public String getRepositoryURL() {
		if (repositoryURL == null) {
			try {
				String location = getProject().getPersistentProperty(MODELBUS_REPO_LOC);
				if (location == null) {
					return null;
				}
				repositoryURL = location;
			} catch (CoreException e) {
				// log the problem and carry on
				ModelBusRepositoryPlugin.log(e.getStatus());
				return null;
			}
		}
		return repositoryURL;
	}

	/**
	 * Return an object that provides the operations for transfering data to and from the provider's location.
	 */
	public ModelBusRepositoryOperations getOperations() {
		return new ModelBusRepositoryOperations(this);
	}

	/**
	 * @see org.eclipse.team.core.RepositoryProvider#getFileModificationValidator()
	 */
	public IFileModificationValidator getFileModificationValidator() {
		return new FileModificationValidator(this);
	}

	/**
	 * Return the resource variant for the local resource using the bytes to identify the variant.
	 * 
	 * @param resource
	 *            the resource
	 * @param bytes
	 *            the bytes that identify the resource variant
	 * @return the resource variant handle
	 */
	public IResourceVariant getResourceVariant(IResource resource, byte[] bytes) {
		if (bytes == null) {
			return null;
		}
		String filelocation = getRepositoryPath(resource);
		if (filelocation == null) {
			return null;
		}

		// try {
		// String repositoryLocation = org.modelbus.preferences.ModelBusPreferencePlugin.getDefault().getPreferenceStore().getString(
		// org.modelbus.preferences.PreferenceConstants.REPOSITORY_LOCATION_STRING);
		boolean checkDependencies = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin.getDefault().getPreferenceStore()
				.getBoolean(org.modelbus.preferences.dependencies.preferences.PreferenceConstants.CHECK_DEPENDENCIES);
		String[] modelExtensions = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin.getDefault().getExtensionPreference();
		IRepositoryHelper repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();
		/*
		 * MES: This method gets called when the base tree requests an IResourceVariant (base) during synchronization. To calculate the
		 * ModelBusSyncInfo for the IResource, ModelBusRepositoryResourceVariant#asBytes() and isContainer() are called, which try to contact the
		 * server. If the remote resource has been deleted though those operations run into trouble. Fortunately, server contact is unnecessary if we
		 * simply assume, that the given base bytes are valid and no sync bytes are ever kept for folders. Still, ugly hack :)
		 */
		// TODO CHECKMWA
		return new ModelBusRepositoryResourceVariant(getRepositoryPath(resource), repositoryHelper, bytes);
		// if (repositoryHelper.exists(UserSessionHelper.getSession(),
		// filelocation)) { // needs to be protected since location may have
		// been deleted remotely
		// return new
		// ModelBusRepositoryResourceVariant(repositoryHelper.getInfo(UserSessionHelper.getSession(),
		// filelocation, -1) , bytes);
		// } else {
		// return null;
		// }
		// } catch (AxisFault e) {
		// // TODO Auto-generated catch block
		// e.printStackTrace();
		// throw new RuntimeException(e);
		// } catch (RemoteException e) {
		// // TODO Auto-generated catch block
		// e.printStackTrace();
		// throw new RuntimeException(e);
		// } catch (IOException e) {
		// throw new RuntimeException(e);
		// }
		// return null;
	}

	/**
	 * Return the resource variant for the local resource.
	 * 
	 * @param resource
	 *            the resource
	 * @return the resource variant
	 * @throws InvalidRevisionException
	 * @throws IOException
	 */
	public IResourceVariant getResourceVariant(IResource resource) throws RepositoryAuthentificationException, NonExistingResourceException,
			IOException, InvalidRevisionException {

		String repositoryPath = getRepositoryPath(resource);
		// String repositoryLocation = org.modelbus.preferences.ModelBusPreferencePlugin.getDefault().getPreferenceStore().getString(
		// org.modelbus.preferences.PreferenceConstants.REPOSITORY_LOCATION_STRING);
		boolean checkDependencies = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin.getDefault().getPreferenceStore()
				.getBoolean(org.modelbus.preferences.dependencies.preferences.PreferenceConstants.CHECK_DEPENDENCIES);
		String[] modelExtensions = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin.getDefault().getExtensionPreference();

		IRepositoryHelper repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();

		if (repositoryPath == null || !repositoryHelper.exists(UserSessionHelper.getSession(), URI.createURI(repositoryPath), IRepositoryHelper.REVISION_HEAD)) { // got
			// local
			// resource
			// as
			// only
			// hint
			// to
			// remote.
			// Check
			// for
			// HEAD.

			return null;
		}

		RepositoryDirEntry entry = repositoryHelper.getInfo(UserSessionHelper.getSession(), URI.createURI(repositoryPath), IRepositoryHelper.REVISION_HEAD);
		String revision = entry.getRevision();

		// try {
		return new ModelBusRepositoryResourceVariant(repositoryPath, repositoryHelper, revision);
		// } catch (AxisFault e) {
		// // TODO Auto-generated catch block
		// e.printStackTrace();
		// throw new RuntimeException(e);
		// } catch (RemoteException e) {
		// // TODO Auto-generated catch block
		// e.printStackTrace();
		// throw new RuntimeException(e);
		// }
	}

	/* Called during checkout via CheckoutAsOperation */
	public IResourceVariant getResourceVariant(IResource resource, String revision) throws NonExistingResourceException,
			RepositoryAuthentificationException, IOException, InvalidRevisionException {
		//TODO check!
//		if (revision < 0) {
//			return getResourceVariant(resource);
//		}
		String repositoryPath = getRepositoryPath(resource);
		// String repositoryLocation = org.modelbus.preferences.ModelBusPreferencePlugin.getDefault().getPreferenceStore().getString(
		// org.modelbus.preferences.PreferenceConstants.REPOSITORY_LOCATION_STRING);
		boolean checkDependencies = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin.getDefault().getPreferenceStore()
				.getBoolean(org.modelbus.preferences.dependencies.preferences.PreferenceConstants.CHECK_DEPENDENCIES);
		String[] modelExtensions = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin.getDefault().getExtensionPreference();

		IRepositoryHelper repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();

		if (repositoryPath == null || !repositoryHelper.exists(UserSessionHelper.getSession(), URI.createURI(repositoryPath), revision)) {
			return null;
		}

		return new ModelBusRepositoryResourceVariant(repositoryPath, repositoryHelper, revision);

	}

	public IResourceVariant getLocalResourceVariant(IResource resource) {
		File file = getFile(resource);
		if (file == null || !file.exists())
			return null;
		return new ModelBusRepositoryLocalFileResourceVariant(file);
	}

	public IResourceVariant getLocalResourceVariant(IResource resource, byte[] bytes) {
		if (bytes == null)
			return null;
		File file = getFile(resource);
		if (file == null)
			return null;
		return new ModelBusRepositoryLocalFileResourceVariant(file, bytes);
	}

	public java.io.File getFile(IResource resource) {
		if (resource.getProject().equals(getProject())) {
			IPath rootdir = getRoot();
			return new File(rootdir.append(resource.getProjectRelativePath()).toOSString());
		}
		return null;
	}

	/**
	 * Return the <code>java.io.File</code> that the given resource maps to. Return <code>null</code> if the resource is not a child of this
	 * provider's project.
	 * 
	 * @param resource
	 *            the resource
	 * @return the file that the resource maps to.
	 */
	public String getRepositoryPath(IResource resource) {
		if (resource == null) {
			return null;
		}
		if (resource.getProject() == null) {
			return null;
		}
		if (resource.getProject().equals(getProject())) {
			/*
			 * BE CAREFUL HERE! MAY BE NULL, e.g. if the target location for the provider has not yet been set. See CheckoutAsOperation#doOpen().
			 */
			String repositoryDir = getRepositoryURL();
			// MWATODO
			// TODO check
			if (repositoryDir == null) { // Ignore. Manual synchronize triggered
				// by user will fix this in any
				// case.
				return null;
			}
			return repositoryDir.concat("/" + (resource.getProjectRelativePath()).toString());
		}
		return null;
	}

	public IPath getRoot() {
		if (root == null) {
			try {
				String location = ""; // getProject().getPersistentProperty(FILESYSTEM_REPO_LOC);
				if (location == null) {
					return null;
				}
				root = new Path(location);
			} catch (Exception e) {
				// log the problem and carry on
				// FileSystemPlugin.log(e.getStatus());
				return null;
			}
		}
		return root;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.team.core.RepositoryProvider#getRuleFactory()
	 */
	public IResourceRuleFactory getRuleFactory() {
		return RESOURCE_RULE_FACTORY;
	}

	public IFileHistoryProvider getFileHistoryProvider() {
		if (ModelBusRepositoryProvider.fileHistoryProvider == null) {
			ModelBusRepositoryProvider.fileHistoryProvider = new ModelBusRepositoryHistoryProvider();
		}
		return ModelBusRepositoryProvider.fileHistoryProvider;
	}

}
