/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.modelbus.team.eclipse.repository.subscriber;

import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.team.core.RepositoryProvider;
import org.eclipse.team.core.TeamException;
import org.eclipse.team.core.synchronize.SyncInfo;
import org.eclipse.team.core.variants.IResourceVariant;
import org.eclipse.team.core.variants.ThreeWayRemoteTree;
import org.eclipse.team.core.variants.ThreeWaySubscriber;
import org.eclipse.team.core.variants.ThreeWaySynchronizer;
import org.modelbus.team.eclipse.repository.ModelBusRepositoryPlugin;
import org.modelbus.team.eclipse.repository.ModelBusRepositoryProvider;

/**
 * This is an example file system subscriber that overrides ThreeWaySubscriber. It uses a repository provider (
 * <code>ModelBusRepositoryProvider</code>) to determine and manage the roots and to create resource variants. It also makes use of a file system
 * specific remote tree (<code>ModelBusRepositoryRemoteTree</code>) for provided the remote tree access and refresh.
 * 
 * @see ThreeWaySubscriber
 * @see ThreeWaySynchronizer
 * @see ModelBusRepositoryProvider
 * @see ModelBusRepositoryRemoteTree
 */
public class ModelBusRepositorySubscriber extends ThreeWaySubscriber {

	private static ModelBusRepositorySubscriber	instance;

	/**
	 * Return the file system subscriber singleton.
	 * 
	 * @return the file system subscriber singleton.
	 */
	public static synchronized ModelBusRepositorySubscriber getInstance() {
		if (instance == null) {
			instance = new ModelBusRepositorySubscriber();
		}
		return instance;
	}

	/**
	 * Create the file system subscriber.
	 */
	private ModelBusRepositorySubscriber() {
		super(new ModelBusThreeWaySynchronizer(new QualifiedName(ModelBusRepositoryPlugin.ID, "workspace-sync"))); //$NON-NLS-1$
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.team.core.variants.ThreeWaySubscriber#getResourceVariant(org.eclipse.core.resources.IResource, byte[])
	 */
	public IResourceVariant getResourceVariant(IResource resource, byte[] bytes) throws TeamException {
		RepositoryProvider provider = RepositoryProvider.getProvider(resource.getProject(), ModelBusRepositoryPlugin.PROVIDER_ID);
		if (provider != null) {
			return ((ModelBusRepositoryProvider) provider).getResourceVariant(resource, bytes);
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.team.core.variants.ThreeWaySubscriber#createRemoteTree()
	 */
	protected ThreeWayRemoteTree createRemoteTree() {
		return new ModelBusRepositoryRemoteTree(this);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.team.core.subscribers.Subscriber#getName()
	 */
	public String getName() {
		return "ModelBus Repository Subscriber"; //$NON-NLS-1$
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.team.core.subscribers.Subscriber#roots()
	 */
	public IResource[] roots() {
		List result = new ArrayList();
		IProject[] projects = ResourcesPlugin.getWorkspace().getRoot().getProjects();
		for (int i = 0; i < projects.length; i++) {
			IProject project = projects[i];
			if (project.isAccessible()) {
				RepositoryProvider provider = RepositoryProvider.getProvider(project, ModelBusRepositoryPlugin.PROVIDER_ID);
				if (provider != null) {
					result.add(project);
				}
			}
		}
		return (IProject[]) result.toArray(new IProject[result.size()]);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.team.core.variants.ThreeWaySubscriber#handleRootChanged(org.eclipse.core.resources.IResource, boolean)
	 */
	public void handleRootChanged(IResource resource, boolean added) {
		// Override to allow ModelBusRepositoryProvider to signal the addition and removal of roots
		super.handleRootChanged(resource, added);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.team.core.variants.ResourceVariantTreeSubscriber#getSyncInfo(org.eclipse.core.resources.IResource,
	 * org.eclipse.team.core.variants.IResourceVariant, org.eclipse.team.core.variants.IResourceVariant)
	 */
	protected SyncInfo getSyncInfo(IResource local, IResourceVariant base, IResourceVariant remote) throws TeamException {
		// Override to use a custom sync info
		ModelBusRepositorySyncInfo info = new ModelBusRepositorySyncInfo(local, base, remote, this.getResourceComparator());
		info.init();
		return info;
	}

	/**
	 * Make the resource in-sync.
	 * 
	 * @param resource
	 *            the resource
	 * @throws TeamException
	 * @throws UnsupportedEncodingException
	 */
	public void makeInSync(IResource resource) throws TeamException, UnsupportedEncodingException {
		ThreeWaySynchronizer synchronizer = getSynchronizer();
		// byte[] remoteBytes = synchronizer.getRemoteBytes(resource);
		/*
		 * MES: Can't ask the synchronizer here. Will return byte[0] for a deleted remote folder instead of null. Do another server lookup.
		 * ModelBusRepositoryResourceVariant reflects the situation correctly.
		 */
		IResourceVariant remote = getRemoteTree().getResourceVariant(resource);

		// copied from ModelBusRepositoryOperation#internalPut(IFile,...)
		List<IContainer> parentContainers = new ArrayList<IContainer>();
		IContainer parent = resource.getParent();
		while (parent != null) {
			parentContainers.add(parent);
			parent = parent.getParent();
		}

		if (remote == null && !resource.exists()) {
			synchronizer.flush(resource, IResource.DEPTH_ZERO);
		} else {

			byte[] remoteBytes = synchronizer.getRemoteBytes(resource);
			if (remoteBytes == null) {
				if (!resource.exists())
					synchronizer.flush(resource, IResource.DEPTH_ZERO);
			} else {
				synchronizer.setBaseBytes(resource, remoteBytes);
				((ModelBusThreeWaySynchronizer) synchronizer).setWorkingCopyRevision(resource,
						((ModelBusRepositoryResourceVariant) remote).getRevision());
			}
		}

		// when updating a single file/folder out of complex file system tree, all parents of the resource must be marked as in-sync to avoid
		// conflicts
		for (Object ctr : parentContainers) {

			byte[] remoteBytes = synchronizer.getRemoteBytes((IResource) ctr);

			if (synchronizer.isLocallyModified((IResource) ctr) && remoteBytes != null) {
				synchronizer.setBaseBytes((IResource) ctr, remoteBytes);
				remote = getRemoteTree().getResourceVariant((IResource) ctr);
				((ModelBusThreeWaySynchronizer) synchronizer).setWorkingCopyRevision((IResource) ctr,
						((ModelBusRepositoryResourceVariant) remote).getRevision());
			}

		}

	}

	/**
	 * Make the change an outgoing change
	 * 
	 * @param resource
	 * @throws TeamException
	 * @throws UnsupportedEncodingException
	 */
	public void markAsMerged(IResource resource, IProgressMonitor monitor) throws TeamException, UnsupportedEncodingException {
		makeInSync(resource);
		try {
			/*
			 * MES: Guard to prevent exception when attempting to ignore (pseudo-conflicting) folder deletion. Modification timestamp won't be read
			 * when synchronizing folders.
			 */
			if (resource.exists() && resource.getType() == IResource.FILE) {
				resource.touch(monitor);
			}
		} catch (CoreException e) {
			throw TeamException.asTeamException(e);
		}
	}

}
