/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.modelbus.team.eclipse.repository.subscriber;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.Assert;
import org.eclipse.team.core.TeamException;
import org.eclipse.team.core.synchronize.SyncInfo;
import org.eclipse.team.core.variants.IResourceVariant;
import org.eclipse.team.core.variants.IResourceVariantComparator;

/**
 * Provide a custom sync info that will report files that exist both locally and remotely as in-sync and will return a null base if there is an
 * incoming change.
 */
public class ModelBusRepositorySyncInfo extends SyncInfo {

	public ModelBusRepositorySyncInfo(IResource local, IResourceVariant base, IResourceVariant remote, IResourceVariantComparator comparator) {
		super(local, base, remote, comparator);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.team.core.subscribers.SyncInfo#calculateKind(org.eclipse.core.runtime.IProgressMonitor)
	 */
	protected int calculateKind() throws TeamException {
		/*
		 * Special treatment for folders. They either exist or they do not. No modifications are calculated. MES: Maybe this does unnecessarily
		 * duplicate code, but relying on the local_calculateKind() caused problems, so this seemed the easiest approach to sort things out for now.
		 */
		if (getLocal().getType() != IResource.FILE) {
			if (getBase() != null) {
				if (getLocal().exists()) {
					if (getRemote() != null) {
						return IN_SYNC;
					} else {
						return INCOMING | DELETION;
					}
				} else {
					if (getRemote() != null) {
						return OUTGOING | DELETION;
					} else {
						return CONFLICTING | DELETION | PSEUDO_CONFLICT;
					}
				}
			} else {
				if (getLocal().exists()) {
					if (getRemote() != null) {
						return CONFLICTING | ADDITION | PSEUDO_CONFLICT;
					} else {
						return OUTGOING | ADDITION;
					}
				} else {
					if (getRemote() != null) {
						return INCOMING | ADDITION;
					} else {
						return IN_SYNC;
					}
				}
			}
		}

		// TODO che
		// and new calculator
		/* Analyzing sync state for a non-container resource. Use default algorithm. */
		return local_calculateKind();
	}

	private int local_calculateKind() {
		int description = IN_SYNC;

		boolean localExists = this.getLocal().exists();

		if (this.getComparator().isThreeWay()) {
			if (getBase() == null) {
				if (getRemote() == null) {
					if (!localExists) {
						description = IN_SYNC;
					} else {
						description = OUTGOING | ADDITION;
					}
				} else {
					if (!localExists) {
						description = INCOMING | ADDITION;
					} else {
						description = CONFLICTING | ADDITION;
						if (getComparator().compare(getLocal(), getRemote())) {
							description |= PSEUDO_CONFLICT;
						}
					}
				}
			} else {
				if (!localExists) {
					if (getRemote() == null) {
						description = CONFLICTING | DELETION | PSEUDO_CONFLICT;
					} else {
						// MES:
						try {
							// System.out.println("SyncInfo. Contents of base ResourceVariant:\n " +
							// Base64.encode(getBase().asBytes()));
							// System.out.println("SyncInfo. Contents of remote ResourceVariant:\n " + Base64.encode());
						} catch (Throwable t) {
							System.out.println("hello");
						}
						if (getComparator().compare(getBase(), getRemote()))
							description = OUTGOING | DELETION;
						else
							description = CONFLICTING | CHANGE;
					}
				} else {
					if (getRemote() == null) {
						if (getComparator().compare(getLocal(), getBase()))
							description = INCOMING | DELETION;
						else
							description = CONFLICTING | CHANGE;
					} else {
						boolean ay = getComparator().compare(getLocal(), getBase());
						boolean am = getComparator().compare(getBase(), getRemote());
						if (ay && am) {
							// in-sync
						} else if (ay && !am) {
							description = INCOMING | CHANGE;
						} else if (!ay && am) {
							description = OUTGOING | CHANGE;
						} else {
							if (!getComparator().compare(getLocal(), getRemote())) {
								description = CONFLICTING | CHANGE;
							}
						}
					}
				}
			}
		} else { // two compare without access to base contents
			if (getRemote() == null) {
				if (!localExists) {
					Assert.isTrue(false);
					// shouldn't happen
				} else {
					description = DELETION;
				}
			} else {
				if (!localExists) {
					description = ADDITION;
				} else {
					if (!getComparator().compare(getLocal(), getRemote()))
						description = CHANGE;
				}
			}
		}
		return description;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.team.core.synchronize.SyncInfo#getBase()
	 */
	public IResourceVariant getBase() {
		// If the kind has been set and there is an incoming change
		// return null as the base since the server does not keep the
		// base contents
		// if ((getKind() & INCOMING) > 0) {
		// return null;
		// }
		return super.getBase();
	}
}
