/*******************************************************************************
 * Copyright (c) 2000, 2003 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.modelbus.team.eclipse.repository.ui;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.common.util.URI;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.modelbus.core.lib.IRepositoryHelper;
import org.modelbus.core.lib.util.ModelBusCoreUtil.URIUtil;
import org.modelbus.dosgi.repository.descriptor.RepositoryDirEntry;
import org.modelbus.dosgi.repository.descriptor.RepositoryNodeKind;
import org.modelbus.team.eclipse.repository.ModelBusRepositoryHelper;
import org.modelbus.team.eclipse.repository.Policy;
import org.modelbus.team.eclipse.repository.RepositoryLocationHelper;
import org.modelbus.team.eclipse.repository.UserSessionHelper;

/**
 * This class provides the main page of the file system repository configuration wizard. It allows the user to select a location on disk. Once the
 * page is finished, the location can be accessed using the <code>getLocation()</code> method.
 */
public class ModelBusRepositoryMainPage extends WizardPage {

	private String				projectName;
	private Text				locationText;
	private Tree				tree;
	private static final int	COMBO_HISTORY_LENGTH		= 5;
	private static final String	INCOMPLETE_ROOT_URL_STRING	= "http:/";
	private String				repositoryLocation;

	// org.modelbus.preferences.ModelBusPreferencePlugin.getDefault().getPreferenceStore().getString(
	// org.modelbus.preferences.PreferenceConstants.REPOSITORY_LOCATION_STRING);

	// private String location;

	private IRepositoryHelper	repositoryHelper;

	/*
	 * WizardPage constructor comment. @param pageName the name of the page
	 * 
	 * @param title the title of the page @param description the description of the page @param titleImage the image for the page
	 */

	public ModelBusRepositoryMainPage(String pageName, String title, String description, ImageDescriptor titleImage) {
		super(pageName, title, titleImage);
		setDescription(description);
		setTitle(title);
	}

	// gdu
	public ModelBusRepositoryMainPage(String projectName, String pageName, String title, String description, ImageDescriptor titleImage) {
		this(pageName, title, description, titleImage);
		this.projectName = projectName;
	}

	/*
	 * Creates a new checkbox instance and sets the default layout data.
	 * 
	 * @param group the composite in which to create the checkbox @param label the string to set into the checkbox @return the new checkbox
	 */
	protected Button createCheckBox(Composite group, String label) {
		Button button = new Button(group, SWT.CHECK | SWT.LEFT);
		button.setText(label);
		GridData data = new GridData();
		data.horizontalSpan = 2;
		button.setLayoutData(data);
		return button;
	}

	/*
	 * Utility method that creates a combo box
	 * 
	 * @param parent the parent for the new label @return the new widget
	 */
	protected Combo createCombo(Composite parent) {
		Combo combo = new Combo(parent, SWT.READ_ONLY);
		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = IDialogConstants.ENTRY_FIELD_WIDTH;
		combo.setLayoutData(data);
		return combo;
	}

	/*
	 * Creates composite control and sets the default layout data.
	 * 
	 * @param parent the parent of the new composite @param numColumns the number of columns for the new composite @return the newly-created coposite
	 */
	protected Composite createComposite(Composite parent, int numColumns) {
		Composite composite = new Composite(parent, SWT.NULL);

		// GridLayout
		GridLayout layout = new GridLayout();
		layout.numColumns = numColumns;
		composite.setLayout(layout);

		// GridData
		GridData data = new GridData();
		data.verticalAlignment = GridData.FILL;
		data.horizontalAlignment = GridData.FILL;
		composite.setLayoutData(data);
		return composite;
	}

	/*
	 * Utility method that creates a label instance and sets the default layout data.
	 * 
	 * @param parent the parent for the new label @param text the text for the new label @return the new label
	 */
	protected Label createLabel(Composite parent, String text) {
		Label label = new Label(parent, SWT.LEFT);
		label.setText(text);
		GridData data = new GridData();
		data.horizontalSpan = 1;
		data.horizontalAlignment = GridData.FILL;
		label.setLayoutData(data);
		return label;
	}

	/*
	 * Create a text field specific for this application
	 * 
	 * @param parent the parent of the new text field @return the new text field
	 */
	protected Text createTextField(Composite parent) {
		Text text = new Text(parent, SWT.SINGLE | SWT.BORDER);
		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		data.verticalAlignment = GridData.CENTER;
		data.grabExcessVerticalSpace = false;
		data.widthHint = IDialogConstants.ENTRY_FIELD_WIDTH;
		text.setLayoutData(data);
		return text;
	}

	/*
	 * Adds an entry to a history, while taking care of duplicate history items and excessively long histories. The assumption is made that all
	 * histories should be of length <code>ConfigurationWizardMainPage.COMBO_HISTORY_LENGTH</code>.
	 * 
	 * @param history the current history @param newEntry the entry to add to the history @return the history with the new entry appended
	 */
	protected String[] addToHistory(String[] history, String newEntry) {
		ArrayList l = new ArrayList(Arrays.asList(history));
		addToHistory(l, newEntry);
		String[] r = new String[l.size()];
		l.toArray(r);
		return r;
	}

	/*
	 * Adds an entry to a history, while taking care of duplicate history items and excessively long histories. The assumption is made that all
	 * histories should be of length <code>ConfigurationWizardMainPage.COMBO_HISTORY_LENGTH</code>.
	 * 
	 * @param history the current history @param newEntry the entry to add to the history
	 */
	protected void addToHistory(List history, String newEntry) {
		history.remove(newEntry);
		history.add(0, newEntry);

		// since only one new item was added, we can be over the limit
		// by at most one item
		if (history.size() > COMBO_HISTORY_LENGTH)
			history.remove(COMBO_HISTORY_LENGTH);
	}

	// Dialog store id constants
	private static final String	STORE_LOCATION	= "ModelBusRepositoryWizardMainPage.STORE_LOCATION";	//$NON-NLS-1$

	public void createControl(Composite parent) {
		Composite composite = new Composite(parent, SWT.NULL);
		composite.setLayoutData(new GridData(GridData.FILL_BOTH));
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		composite.setLayout(layout);
		setControl(composite);

		Label label = new Label(composite, SWT.NULL);
		label.setText(Policy.bind("ModelBusRepositoryMainPage.location")); //$NON-NLS-1$
		label.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));

		locationText = new Text(composite, SWT.BORDER);
		locationText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		new Label(composite, SWT.NONE);

		final TreeViewer treeViewer = new TreeViewer(composite, SWT.BORDER);
		tree = treeViewer.getTree();
		tree.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		tree.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				// moduleTree.getSelection();
				handleSelection();
			}
		});

		initializeValues();
		updatePackageTree();
		validateFields();
	}

	public String getLocation() {
		return locationText.getText();
	}

	private void handleSelection() {
		TreeItem[] sels = tree.getSelection();
		// modules= new ModuleDef[sels.length];
		// for (int i=0;i<sels.length;i++){
		// modules[i]=(ModuleDef)sels[i].getData();
		// }
		if (sels.length > 0) {
			TreeItem sel = sels[0];
			RepositoryDirEntry entry = (RepositoryDirEntry) sel.getData();
			String url = entry.getUri().toString();
			/*
			 * Makes sure that "http://" is shown for the repository root entry in the editable text field. Displaying "http:/" is a constant source
			 * of incorrect user entries for fresh project locations.
			 */
			if (INCOMPLETE_ROOT_URL_STRING.equals(url)) {
				url += "/";
			}

			if (!url.endsWith("/")) {
				url += "/";
			}
			url += projectName;
			locationText.setText(url);
			/*
			 * current_package = (_CCMPackage) sel.getData();
			 * 
			 * try { ModelNameText.setText(current_package.medini_get_name()); } catch (MofError e) { // TODO Auto-generated catch block
			 * e.printStackTrace(); }
			 */
		}

		dialogChanged();

	}

	private void dialogChanged() {
		String location = locationText.getText();

		if (location.length() == 0) {
			updateStatus("Registry Location must be specified");
			return;
		}

		/*
		 * if (current_package == null){ updateStatus("Choose a Repository-Package."); return; }
		 */// this is not longer needed in this page
		updateStatus(null);
		// updateModuleTree();
	}

	/**
	 * Updates the ModelTree.
	 */
	private void updatePackageTree() {
		// Properties modelBusProperties = new Properties();
		// modelBusProperties.put(AdapterStub.PROP_REGISTRY_LOCATION,registrytext.getText()
		// );

		tree.removeAll();
		tree.setEnabled(false);
		TreeItem root_package = new TreeItem(tree, SWT.NULL);
		// repositoryLocation =
		// org.modelbus.preferences.ModelBusPreferencePlugin.getDefault().getPreferenceStore().getString(org.modelbus.preferences.PreferenceConstants.REPOSITORY_LOCATION_STRING);
		// boolean checkDependencies = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
		// .getDefault()
		// .getPreferenceStore()
		// .getBoolean(org.modelbus.preferences.dependencies.preferences.PreferenceConstants.CHECK_DEPENDENCIES);
		// String[] modelExtensions = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
		// .getDefault().getExtensionPreference();
		root_package.setText(this.getRepositoryLocation());
		try {
			// adapterStub = new AdapterStubImpl(modelBusProperties);

			// repositoryHelper = new RepositoryHelper(repositoryLocation,checkDependencies,modelExtensions);
			repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();
			RepositoryDirEntry root = repositoryHelper.getRoot(UserSessionHelper.getSession());
			root_package.setData(root);
			try {
				addEntries(root_package,
						repositoryHelper.getDirEntries(UserSessionHelper.getSession(), URI.createURI(root.getUri()), root.getRevision()));
			} catch (Exception e) {
				e.printStackTrace();
			}

			tree.setEnabled(true);
			updateStatus("Please select a model or a file");
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private void addEntries(TreeItem treeitem, RepositoryDirEntry[] entries) throws RemoteException {
		if (entries == null) {
			return;
		}
		for (int i = 0; i < entries.length; i++) {
			RepositoryDirEntry entry = entries[i];
			if (RepositoryNodeKind.DIR.equals(entry.getKind())) {
				TreeItem item = new TreeItem(treeitem, SWT.ARROW);
				item.setText(entry.getName());
				item.setData(entry);
				try {
					RepositoryDirEntry[] childs = repositoryHelper.getDirEntries(UserSessionHelper.getSession(), URI.createURI(entry.getUri()),
							entry.getRevision());
					addEntries(item, childs);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}

		}
	}

	private void updateStatus(String message) {
		setErrorMessage(message);
		setPageComplete(message == null);

		// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		setPageComplete(true);
		// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	}

	public boolean finish(IProgressMonitor monitor) {
		saveWidgetValues();
		return true;
	}

	/**
	 * Initializes states of the controls.
	 */
	private void initializeValues() {
		IDialogSettings settings = getDialogSettings();
		if (settings != null) {
			String[] locations = settings.getArray(STORE_LOCATION);
			if (locations != null) {
				for (int i = 0; i < locations.length; i++) {
					// repositoryURL_Text.setText(locations[i]);
				}
				// locationCombo.select(0);
			}

		}
	}

	/**
	 * Saves the widget values
	 */
	private void saveWidgetValues() {
		// Update history
		IDialogSettings settings = getDialogSettings();
		if (settings != null) {
			String[] locations = settings.getArray(STORE_LOCATION);
			if (locations == null)
				locations = new String[0];
			settings.put(STORE_LOCATION, locations);
		}
	}

	/*
	 * Validates the contents of the editable fields and set page completion and error messages appropriately.
	 */
	private void validateFields() {

		String location = locationText.getText();

		if (location.length() == 0) {
			setErrorMessage(null);
			setPageComplete(false);
			return;
		} else {
			if (!URIUtil.isHTTPURI(location)) { // Does not get called at the moment
				setErrorMessage("Malformed URL");
				setPageComplete(false);
				return;
			}
			// File file = new File(location);
			// if(!file.exists() || !file.isDirectory()) {
			// setErrorMessage(Policy.bind("ModelBusRepositoryMainPage.notValidLocation"));
			// //$NON-NLS-1$
			// setPageComplete(false);
			// return;
			// }
			// TODO check if Repository exists, well-formedness of the uri.
		}
		setErrorMessage(null);
		setPageComplete(true);
	}

	protected String getRepositoryLocation() {
		if (repositoryLocation == null) {
			repositoryLocation = RepositoryLocationHelper.getPropertyRepositoryLocation();
			;
		}

		return repositoryLocation;
	}
}
