/*******************************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.modelbus.team.eclipse.repository.ui;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.mapping.ResourceMapping;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.team.core.subscribers.SubscriberScopeManager;
import org.eclipse.team.internal.ui.Utils;
import org.eclipse.team.ui.synchronize.ISynchronizePageConfiguration;
import org.eclipse.team.ui.synchronize.ModelParticipantAction;
import org.modelbus.model.user.User;
import org.modelbus.model.user.UserModel;
import org.modelbus.model.user.UserPackage;
import org.modelbus.team.eclipse.repository.ModelBusRepositoryPlugin;
import org.modelbus.team.eclipse.repository.UserSessionHelper;

/**
 * A put action for use in the file system synchronize participant.
 */
public class ModelPutAction extends ModelParticipantAction {

	public ModelPutAction(String text, ISynchronizePageConfiguration configuration) {
		super(text, configuration);
	}

	protected boolean isEnabledForSelection(IStructuredSelection selection) {
		// Only enable the put in outgoing or both modes
		int mode = getConfiguration().getMode();
		if (mode == ISynchronizePageConfiguration.OUTGOING_MODE || mode == ISynchronizePageConfiguration.BOTH_MODE) {
			return getResourceMappings(selection).length > 0;
		}
		return false;
	}

	private ResourceMapping[] getResourceMappings(IStructuredSelection selection) {
		List mappings = new ArrayList();
		for (Iterator iter = selection.iterator(); iter.hasNext();) {
			Object element = iter.next();
			ResourceMapping mapping = Utils.getResourceMapping(element);
			if (mapping != null)
				mappings.add(mapping);
		}
		return (ResourceMapping[]) mappings.toArray(new ResourceMapping[mappings.size()]);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.Action#run()
	 */
	public void run() {
		ResourceMapping[] resourceMappings = getResourceMappings(getStructuredSelection());
		SubscriberScopeManager manager = ModelBusRepositoryOperation.createScopeManager("Put", resourceMappings);
		try {
			Shell shell = ModelBusRepositoryPlugin.getPlugin().getWorkbench().getActiveWorkbenchWindow().getShell();

			IStructuredSelection selection = getStructuredSelection();
			Object firstElement = selection.getFirstElement();
			if (firstElement instanceof IResource) {

				IResource[] allChildren = getAllChildren((IResource) firstElement, new ArrayList<IResource>());

				for (int i = 0; i < allChildren.length; i++) {
					IResource child = allChildren[i];
					String childName = child.getName();
					if (childName.equals("ModelBus.user")) {

						ResourceSet resourceSet = new ResourceSetImpl();
						resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put("user", new XMIResourceFactoryImpl());
						resourceSet.getPackageRegistry().put(UserPackage.eNS_URI, UserPackage.eINSTANCE);
						Resource res = resourceSet.getResource(URI.createFileURI(child.getLocation().toString()), true);
						UserModel userModel = (UserModel) res.getContents().get(0);
						EList<User> ownedUser = userModel.getOwnedUser();
						for (User user : ownedUser) {
							if (UserSessionHelper.getPropertyUserName().equals(user.getName())
									&& !UserSessionHelper.getPropertyUserPassword().equals(user.getPassword())) {

								MessageDialog messageDialog = new MessageDialog(
										shell,
										"Changing current password",
										null,
										"You are about to change your password!\nPlease change your password in the ModelBus Preferences Page after committing the file.",
										MessageDialog.INFORMATION, new String[] { "OK", "CANCEL" }, 0);
								if (messageDialog.open() == 1) {
									return;
								}
								break;
							}
						}
						break;
					}
				}

			}

			MultiLineInputDialog inputDialog = new MultiLineInputDialog(shell, "Commit Message Dialog");

			String commitMessage = "";
			if (inputDialog.open() == Window.OK) {
				commitMessage = inputDialog.getValue();
				new PutOperation(getConfiguration().getSite().getPart(), manager, commitMessage).run();
			}
			// new PutOperation(getConfiguration().getSite().getPart(), manager,commitMessage).run();
		} catch (InvocationTargetException e) {
			IStatus status = getStatus(e);
			ErrorDialog.openError(getConfiguration().getSite().getShell(), null, null, status);
		} catch (InterruptedException e) {
			// Ignore
		} catch (CoreException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private IResource[] getAllChildren(IResource resource, ArrayList<IResource> allChildren) throws CoreException {

		if (!allChildren.contains(resource)) {
			allChildren.add(resource);
		}
		if (resource instanceof IContainer) {
			IResource[] members = ((IContainer) resource).members();
			if (!(members.length == 0)) {
				// allChildren.addAll(Arrays.asList(members));
				for (int i = 0; i < members.length; i++) {
					IResource member = members[i];
					getAllChildren(member, allChildren);

				}
			}
		}

		return allChildren.toArray(new IResource[allChildren.size()]);
	}

	private IStatus getStatus(Throwable throwable) {
		if (throwable instanceof InvocationTargetException) {
			return getStatus(((InvocationTargetException) throwable).getCause());
		}
		return new Status(IStatus.ERROR, ModelBusRepositoryPlugin.ID, 0, "An error occurred during the put.", throwable);
	}
}
