/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *    Alessandro Nistico - [patch] Change Set's implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui;

import java.net.MalformedURLException;
import java.net.URL;
import java.text.MessageFormat;

import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Preferences;
import org.eclipse.jface.action.StatusLineManager;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.team.internal.core.subscribers.SubscriberChangeSetManager;
import org.eclipse.ui.IPerspectiveDescriptor;
import org.eclipse.ui.IPerspectiveListener;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PerspectiveAdapter;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.internal.WorkbenchWindow;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.modelbus.team.eclipse.core.ModelBusTeamPlugin;
import org.modelbus.team.eclipse.core.operation.IConsoleStream;
import org.modelbus.team.eclipse.core.operation.LoggedOperation;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.ui.console.ModelBusConsole;
import org.modelbus.team.eclipse.ui.preferences.ModelBusTeamPreferences;
import org.modelbus.team.eclipse.ui.status.ModelBusRepositoryPing;
import org.modelbus.team.eclipse.ui.status.ModelBusStatusLineContributionItem;
import org.modelbus.team.eclipse.ui.status.ModelBusStatusProvider;
import org.modelbus.team.eclipse.ui.synchronize.update.UpdateSubscriber;
import org.osgi.framework.BundleContext;

/**
 * Plugin entry point. Implements "system facade" pattern
 * 
 * @author Alexander Gurov
 */
public class ModelBusTeamUIPlugin extends AbstractUIPlugin {
	private volatile static ModelBusTeamUIPlugin	instance		= null;

	private ProjectCloseListener					pcListener;
	private URL										baseUrl;
	// private ProblemListener problemListener;

	private ModelBusConsole							console;
	private SubscriberChangeSetManager				changeSetManager;

	private ModelBusRepositoryPing					pingThread;
	private ModelBusStatusProvider					status;

	private final long								pingInterval	= 5000;

	public ModelBusTeamUIPlugin() {
		super();

		this.pcListener = new ProjectCloseListener();
		// this.problemListener = new ProblemListener();

		this.status = new ModelBusStatusProvider();

		ModelBusTeamUIPlugin.instance = this;
	}

	public static ModelBusTeamUIPlugin instance() {
		return ModelBusTeamUIPlugin.instance;
	}

	public IConsoleStream getConsoleStream() {
		return this.console == null ? null : this.console.getConsoleStream();
	}

	public String getResource(String key) {
		return FileUtility.getResource(Platform.getResourceBundle(this.getBundle()), key);
	}

	public String getResource(String key, Object[] args) {
		String message = this.getResource(key);
		return MessageFormat.format(message, args);
	}

	public ImageDescriptor getImageDescriptor(String path) {
		try {
			return ImageDescriptor.createFromURL(new URL(this.baseUrl, path));
		} catch (MalformedURLException e) {
			LoggedOperation.reportError(this.getResource("Error.GetImageDescriptor"), e);
			return null;
		}
	}

	public String getVersionString() {
		return (String) this.getBundle().getHeaders().get(org.osgi.framework.Constants.BUNDLE_VERSION);
	}

	public void start(BundleContext context) throws Exception {
		super.start(context);

		// Platform.addLogListener(this.problemListener);

		ModelBusTeamPreferences.setDefaultValues(this.getPreferenceStore());

		Preferences corePreferences = ModelBusTeamPlugin.instance().getPluginPreferences();

		// Earlier Subversive releases save connector id in ModelBusTeamPlugin store
		// To be compatible with earlier releases copy saved preferences to
		// ModelBusTeamUIPlugin store and clear ModelBusTeamPlugin store
		String connector = corePreferences.getString(ModelBusTeamPlugin.CORE_ModelBusCLIENT_NAME).trim();
		if (connector.length() != 0) {
			ModelBusTeamPreferences.setCoreString(this.getPreferenceStore(), ModelBusTeamPreferences.CORE_ModelBusCONNECTOR_NAME, connector);
			corePreferences.setValue(ModelBusTeamPlugin.CORE_ModelBusCLIENT_NAME, "");
			ModelBusTeamPlugin.instance().savePluginPreferences();
		}

		this.baseUrl = context.getBundle().getEntry("/");

		IWorkspace workspace = ResourcesPlugin.getWorkspace();
		workspace.addResourceChangeListener(ModelBusTeamUIPlugin.this.pcListener, IResourceChangeEvent.PRE_CLOSE | IResourceChangeEvent.PRE_DELETE);

		this.console = new ModelBusConsole();

		final IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();

		final IPerspectiveListener perspectiveListener = new PerspectiveAdapter() {
			private boolean	itemAdded	= false;

			@SuppressWarnings("restriction")
			@Override
			public void perspectiveActivated(final IWorkbenchPage page, final IPerspectiveDescriptor perspective) {
				if (window instanceof WorkbenchWindow && !this.itemAdded) {
					final WorkbenchWindow workbenchWindow = (WorkbenchWindow) window;
					final StatusLineManager manager = workbenchWindow.getStatusLineManager();

					final ModelBusStatusLineContributionItem item = new ModelBusStatusLineContributionItem("modelbus.status");
					ModelBusTeamUIPlugin.this.status.addObserver(item);

					manager.add(item);
					item.update();
					manager.update(true);
					this.itemAdded = true;
				}
			}
		};
		
		if (window != null) {
			window.addPerspectiveListener(perspectiveListener);
		}

		this.pingThread = new ModelBusRepositoryPing(this.pingInterval);
		this.pingThread.start();
	}

	public void stop(BundleContext context) throws Exception {
		this.console.shutdown();

		IWorkspace workspace = ResourcesPlugin.getWorkspace();

		workspace.removeResourceChangeListener(this.pcListener);

		if (this.changeSetManager != null) {
			this.changeSetManager.dispose();
		}

		this.pingThread.kill();

		// Platform.removeLogListener(this.problemListener);
		super.stop(context);
	}

	public synchronized SubscriberChangeSetManager getChangeSetManager() {
		if (this.changeSetManager == null) {
			this.changeSetManager = new SubscriberChangeSetManager(UpdateSubscriber.instance());
		}
		
		return this.changeSetManager;
	}

	public ModelBusStatusProvider getStatus() {
		return this.status;
	}

}
