/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Sergiy Logvin - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.operation;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.text.MessageFormat;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.preference.IPreferenceStore;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusConnectorException;
import org.modelbus.team.eclipse.core.operation.AbstractActionOperation;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.remote.GetFileContentOperation;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryResourceProvider;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ProgressMonitorUtility;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.preferences.ModelBusTeamPreferences;

/**
 * Obtain project names from .project files operation
 * 
 * @author Sergiy Logvin
 */
public class ObtainProjectNameOperation extends AbstractActionOperation {

	protected IRepositoryResourceProvider			resourceProvider;
	protected IRepositoryResource[]					resources;
	protected HashMap<String, IRepositoryResource>	names2Resources;

	public ObtainProjectNameOperation(IRepositoryResource[] resources) {
		super("Operation.ObtainProjectName");
		this.resources = resources;
		this.names2Resources = new HashMap<String, IRepositoryResource>();
	}

	public ObtainProjectNameOperation(IRepositoryResourceProvider resourceProvider) {
		this((IRepositoryResource[]) null);
		this.resourceProvider = resourceProvider;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		IPreferenceStore store = ModelBusTeamUIPlugin.instance().getPreferenceStore();
		final boolean doObtainFromDotProject = ModelBusTeamPreferences.getCheckoutBoolean(store,
				ModelBusTeamPreferences.CHECKOUT_USE_DOT_PROJECT_NAME);
		final Set<String> lowerCaseNames = new HashSet<String>();
		final boolean caseInsensitiveOS = FileUtility.isCaseInsensitiveOS();
		if (this.resourceProvider != null) {
			this.resources = this.resourceProvider.getRepositoryResources();
		}
		for (int i = 0; i < this.resources.length && !monitor.isCanceled(); i++) {
			ProgressMonitorUtility.setTaskInfo(monitor, this,
					MessageFormat.format(this.getOperationResource("Scanning"), new Object[] { this.resources[i].getName() }));

			final int j = i;
			this.protectStep(new IUnprotectedOperation() {
				public void run(IProgressMonitor monitor) throws Exception {
					String name = null;
					String newName = null;
					try {
						name = doObtainFromDotProject ? ObtainProjectNameOperation.this.obtainProjectName(
								ObtainProjectNameOperation.this.resources[j], monitor) : null;
					} catch (ModelBusConnectorException ex) {
						// do nothing
					}
					if (name == null) {
						IRepositoryResource resource = ObtainProjectNameOperation.this.resources[j];
						// IRepositoryLocation location = resource.getRepositoryLocation();
						// if (location.isStructureEnabled() && location.getTrunkLocation().equals(resource.getName())) {
						// if (resource.getParent() != null) {
						// name = resource.getParent().getName();
						// }
						// else {
						// name = new Path(location.getUrl()).lastSegment();
						// }
						// name = FileUtility.formatResourceName(name);
						// }
						// else {
						// System.out.println("TODOMWA4");
						name = resource.getName();
						// }
					}
					if (!ObtainProjectNameOperation.this.names2Resources.containsKey(name)
							&& (caseInsensitiveOS ? !lowerCaseNames.contains(name.toLowerCase()) : true)) {
						ObtainProjectNameOperation.this.names2Resources.put(name, ObtainProjectNameOperation.this.resources[j]);
						if (caseInsensitiveOS) {
							lowerCaseNames.add(name.toLowerCase());
						}
					} else {
						for (int k = 1;; k++) {
							newName = name + " (" + k + ")";
							if (!ObtainProjectNameOperation.this.names2Resources.containsKey(newName)
									&& (caseInsensitiveOS ? !lowerCaseNames.contains(newName.toLowerCase()) : true)) {
								ObtainProjectNameOperation.this.names2Resources.put(newName, ObtainProjectNameOperation.this.resources[j]);
								if (caseInsensitiveOS) {
									lowerCaseNames.add(newName.toLowerCase());
								}
								break;
							}
						}
					}

				}
			}, monitor, this.resources.length);

		}
	}

	protected String obtainProjectName(IRepositoryResource resource, IProgressMonitor monitor) throws Exception {
		String projFileUrl = resource.getUrl() + "/.project";
		IRepositoryResource projFile = ModelBusConnector.asRepositoryFile(projFileUrl, false);
		if (projFile.exists()) {
			GetFileContentOperation op = new GetFileContentOperation(projFile);
			ProgressMonitorUtility.doTaskExternal(op, monitor);
			InputStream is = op.getContent();
			BufferedReader reader = null;
			try {
				reader = new BufferedReader(new InputStreamReader(is, "UTF-8"));
				String currentString;
				int first;
				int last;
				while ((currentString = reader.readLine()) != null) {
					// FIXME incorrect search: could be on different lines
					if ((first = currentString.indexOf("<name>")) >= 0 && (last = currentString.indexOf("</name>")) >= 0) {
						String name = currentString.substring(first + "<name>".length(), last);
						return name.length() > 0 ? name : null;
					}
				}
			} finally {
				if (reader != null) {
					try {
						reader.close();
					} catch (Exception ex) {
					}
				}
				try {
					is.close();
				} catch (Exception ex) {
				}
			}
		}

		return null;
	}

	public HashMap<String, IRepositoryResource> getNames2Resources() {
		return this.names2Resources;
	}

}
