/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.operation;

import java.util.HashSet;

import org.eclipse.compare.CompareConfiguration;
import org.eclipse.compare.CompareUI;
import org.eclipse.compare.internal.CompareEditor;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.jobs.ISchedulingRule;
import org.eclipse.core.runtime.jobs.MultiRule;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Depth;
import org.modelbus.team.eclipse.core.connector.ModelBusChangeStatus;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRemoteStorage;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusNullProgressMonitor;
import org.modelbus.team.eclipse.core.operation.ModelBusResourceRuleFactory;
import org.modelbus.team.eclipse.core.operation.local.AbstractWorkingCopyOperation;
import org.modelbus.team.eclipse.core.resource.IRemoteStorage;
import org.modelbus.team.eclipse.core.resource.IResourceProvider;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.ui.compare.ComparePanel;
import org.modelbus.team.eclipse.ui.compare.ConflictingFileEditorInput;
import org.modelbus.team.eclipse.ui.dialog.DefaultDialog;
import org.modelbus.team.eclipse.ui.utility.UIMonitorUtility;

/**
 * Show conflict editor operation implementation
 * 
 * @author Alexander Gurov
 */
public class ShowConflictEditorOperation extends AbstractWorkingCopyOperation {

	protected boolean	showInDialog;

	public ShowConflictEditorOperation(IResource[] resources, boolean showInDialog) {
		super("Operation.ShowConflictEditor", resources);
		this.showInDialog = showInDialog;
	}

	public ShowConflictEditorOperation(IResourceProvider provider, boolean showInDialog) {
		super("Operation.ShowConflictEditor", provider);
		this.showInDialog = showInDialog;
	}

	public ISchedulingRule getSchedulingRule() {
		ISchedulingRule rule = super.getSchedulingRule();
		if (rule instanceof IWorkspaceRoot) {
			return rule;
		}
		IResource[] resources = this.operableData();
		HashSet<ISchedulingRule> ruleSet = new HashSet<ISchedulingRule>();
		for (int i = 0; i < resources.length; i++) {
			ruleSet.add(ModelBusResourceRuleFactory.INSTANCE.refreshRule(resources[i].getParent()));
		}
		return new MultiRule(ruleSet.toArray(new IResource[ruleSet.size()]));
	}

	public int getOperationWeight() {
		return 0;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		IResource[] conflictingResources = this.operableData();

		for (int i = 0; i < conflictingResources.length && !monitor.isCanceled(); i++) {
			final IResource current = conflictingResources[i];
			if (current.getType() == IResource.FILE) {
				this.protectStep(new IUnprotectedOperation() {
					public void run(IProgressMonitor monitor) throws Exception {
						ShowConflictEditorOperation.this.showEditorFor((IFile) current, monitor);
					}
				}, monitor, conflictingResources.length);
			}
		}
	}

	protected void showEditorFor(IFile resource, IProgressMonitor monitor) throws Exception {
		IRemoteStorage storage = ModelBusRemoteStorage.instance();

		IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();

		try {
			ModelBusChangeStatus[] status = ModelBusUtility.status(proxy, FileUtility.getWorkingCopyPath(resource), Depth.EMPTY,
					IModelBusConnector.Options.NONE, new ModelBusNullProgressMonitor());
			if (status.length == 1) {
				IContainer parent = resource.getParent();
				parent.refreshLocal(IResource.DEPTH_ONE, monitor);
				IFile local = parent.getFile(new Path(status[0].conflictWorking));
				IFile remote = parent.getFile(new Path(status[0].conflictNew));
				IFile ancestor = parent.getFile(new Path(status[0].conflictOld));
				this.openEditor(resource, local, remote, ancestor, monitor);
			}
		} finally {
		}
	}

	protected void openEditor(final IFile target, IFile left, IFile right, IFile ancestor, IProgressMonitor monitor) throws Exception {
		CompareConfiguration cc = new CompareConfiguration();
		cc.setProperty(CompareEditor.CONFIRM_SAVE_PROPERTY, Boolean.TRUE);
		final ConflictingFileEditorInput compare = new ConflictingFileEditorInput(cc, target, left, right, ancestor);
		compare.run(monitor);
		UIMonitorUtility.getDisplay().syncExec(new Runnable() {
			public void run() {
				if (ShowConflictEditorOperation.this.showInDialog) {
					ComparePanel panel = new ComparePanel(compare, target);
					DefaultDialog dlg = new DefaultDialog(UIMonitorUtility.getShell(), panel);
					dlg.open();
					// CompareUI.openCompareDialog(compare);
				} else {
					CompareUI.openCompareEditor(compare);
				}
			}
		});
	}

}
