/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Sergiy Logvin - Initial API and implementation
 *    Rene Link - [patch] NPE in Interactive Merge UI
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.panel.common;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.resource.IRepositoryBase;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryRoot;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.composite.RepositoryTreeComposite;
import org.modelbus.team.eclipse.ui.panel.AbstractDialogPanel;
import org.modelbus.team.eclipse.ui.repository.model.IParentTreeNode;
import org.modelbus.team.eclipse.ui.repository.model.IResourceTreeNode;
import org.modelbus.team.eclipse.ui.repository.model.RepositoryFictiveNode;
import org.modelbus.team.eclipse.ui.repository.model.RepositoryFile;
import org.modelbus.team.eclipse.ui.repository.model.RepositoryFolder;
import org.modelbus.team.eclipse.ui.repository.model.RepositoryLocationFilter;
import org.modelbus.team.eclipse.ui.repository.model.RepositoryResource;
import org.modelbus.team.eclipse.ui.repository.model.RepositoryRevisions;

/**
 * Repository tree panel
 * 
 * @author Sergiy Logvin
 */
public class RepositoryTreePanel extends AbstractDialogPanel {
	protected RepositoryTreeComposite	repositoryTree;
	protected IRepositoryBase			root;
	protected IRepositoryResource		selectedResource;
	protected IRepositoryResource[]		selectedResources;
	protected boolean					allowSourcesInTree;
	protected boolean					allowFiles;
	protected boolean					autoExpandFirstLevel;

	public RepositoryTreePanel(String title, IRepositoryResource[] resources, boolean allowSourcesInTree) {
		this(title, ModelBusTeamUIPlugin.instance().getResource("RepositoryTreePanel.Description"), AbstractDialogPanel
				.makeToBeOperatedMessage(resources), resources, allowSourcesInTree);
	}

	public RepositoryTreePanel(String title, String description, String message, IRepositoryResource[] resources, boolean allowSourcesInTree) {
		this(title, description, message, resources, allowSourcesInTree, null);
	}

	public void setAutoExpandFirstLevel(boolean autoExpandFirstLevel) {
		this.autoExpandFirstLevel = autoExpandFirstLevel;
	}

	public RepositoryTreePanel(String title, String description, String message, IRepositoryResource[] resources, boolean allowSourcesInTree,
			IRepositoryBase root) {
		super();
		this.dialogTitle = title;
		this.dialogDescription = description;
		this.defaultMessage = message;
		this.selectedResources = resources;
		this.allowSourcesInTree = allowSourcesInTree;
		this.allowFiles = false;
		this.root = root;
	}

	public boolean isAllowFiles() {
		return this.allowFiles;
	}

	public void setAllowFiles(boolean allowFiles) {
		this.allowFiles = allowFiles;
	}

	public IRepositoryResource getSelectedResource() {
		return this.selectedResource;
	}

	public void createControlsImpl(Composite parent) {
		if (this.root != null) {
			this.repositoryTree = new RepositoryTreeComposite(parent, SWT.BORDER, false, this.root);
		} else if (this.selectedResources.length > 0) {
			this.repositoryTree = new RepositoryTreeComposite(parent, SWT.BORDER, false, new ProjectRoot(this.selectedResources[0]));
		} else {
			this.repositoryTree = new RepositoryTreeComposite(parent, SWT.BORDER);
		}
		this.repositoryTree.setAutoExpandFirstLevel(this.autoExpandFirstLevel);
		if (this.repositoryTree.getRepositoryTreeViewer().getInput() instanceof ProjectRoot) {
			ProjectRoot root = (ProjectRoot) this.repositoryTree.getRepositoryTreeViewer().getInput();
			this.repositoryTree.getRepositoryTreeViewer().setExpandedElements(new Object[] { root.getChildren(null)[0] });
		}
		if (this.root == null && this.selectedResources.length > 0) {
			String url = ModelBusConnector.getRepositoryRootUrl();
			// this.selectedResources[0].getRepositoryLocation().getRepositoryRootUrl();
			this.repositoryTree.setFilter(new RepositoryLocationFilter(url) {
				public boolean accept(Object obj) {
					if (obj instanceof RepositoryFile && !RepositoryTreePanel.this.allowFiles || obj instanceof RepositoryRevisions
							|| obj instanceof RepositoryFolder && RepositoryTreePanel.this.isSource(((RepositoryFolder) obj).getRepositoryResource())) {
						return false;
					}
					return super.accept(obj);
				}
			});
		}
		GridData data = new GridData(GridData.FILL_BOTH);
		data.heightHint = 185;
		this.repositoryTree.setLayoutData(data);
		this.repositoryTree.getRepositoryTreeViewer().addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {
				if (RepositoryTreePanel.this.manager != null) {
					IStructuredSelection selection = (IStructuredSelection) RepositoryTreePanel.this.repositoryTree.getRepositoryTreeViewer()
							.getSelection();
					RepositoryTreePanel.this.manager.setButtonEnabled(0, !selection.isEmpty()
							&& selection.getFirstElement() instanceof IResourceTreeNode);
				}
			}
		});
	}

	public String getHelpId() {
		return "org.modelbus.team.eclipse.help.copyMoveToDialogContext";
	}

	protected void saveChangesImpl() {
		IStructuredSelection selection = (IStructuredSelection) this.repositoryTree.getRepositoryTreeViewer().getSelection();
		if (!selection.isEmpty() && selection.getFirstElement() instanceof RepositoryResource) {
			this.selectedResource = ((IResourceTreeNode) selection.getFirstElement()).getRepositoryResource();
		}
	}

	protected void cancelChangesImpl() {
	}

	protected boolean isSource(IRepositoryResource resource) {
		if (!this.allowSourcesInTree) {
			for (int i = 0; i < this.selectedResources.length; i++) {
				if (resource.equals(this.selectedResources[i])) {
					return true;
				}
			}
		}
		return false;
	}

	public static class ProjectRoot extends RepositoryFictiveNode implements IParentTreeNode {
		protected RepositoryResource[]	children;

		public ProjectRoot(IRepositoryResource resource) {
			IRepositoryResource projectRoot = ModelBusUtility.getTrunkLocation(resource);
			if (((IRepositoryRoot) projectRoot).getKind() == IRepositoryRoot.KIND_TRUNK) {
				projectRoot = projectRoot.getParent();
			}
			System.out.println("CHECKMWA34");
			this.children = new RepositoryResource[] { RepositoryFolder.wrapChild(null, /* ModelBusConnector.getRepositoryRoot() */projectRoot),
					RepositoryFolder.wrapChild(null, ModelBusConnector.getRepositoryRoot()) };
		}

		public boolean hasChildren() {
			return true;
		}

		public Object[] getChildren(Object o) {
			return this.children;
		}

		public ImageDescriptor getImageDescriptor(Object object) {
			return null;
		}

		public String getLabel(Object o) {
			return null;
		}

	}

}
