/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   Alexander Gurov - Initial API and implementation
 *******************************************************************************/
package org.modelbus.team.eclipse.ui.properties;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.PlatformUI;
import org.modelbus.team.eclipse.core.IStateFilter;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRemoteStorage;
import org.modelbus.team.eclipse.core.operation.CompositeOperation;
import org.modelbus.team.eclipse.core.operation.IActionOperation;
import org.modelbus.team.eclipse.core.operation.IResourcePropertyProvider;
import org.modelbus.team.eclipse.core.operation.local.property.GetPropertiesOperation;
import org.modelbus.team.eclipse.core.operation.remote.GetRemotePropertiesOperation;
import org.modelbus.team.eclipse.core.resource.ILocalResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.ui.AbstractModelBusView;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.composite.PropertiesComposite;
import org.modelbus.team.eclipse.ui.preferences.ModelBusTeamPreferences;
import org.modelbus.team.eclipse.ui.utility.UIMonitorUtility;

/**
 * Properties view
 * 
 * @author Alexander Gurov
 */
public class PropertiesView extends AbstractModelBusView {

	public static final String			VIEW_ID	= PropertiesView.class.getName();

	protected PropertiesComposite		propertiesComposite;
	protected IResourcePropertyProvider	propertyProvider;
	protected IAdaptable				adaptable;
	protected Action					linkWithEditorAction;
	protected Action					linkWithEditorDropDownAction;
	protected boolean					backgroundExecution;

	public PropertiesView() {
		super(ModelBusTeamUIPlugin.instance().getResource("PropertiesView.Description"));
	}

	public void setFocus() {
	}

	public void setResource(IAdaptable resource, IResourcePropertyProvider propertyProvider, boolean backgroundExecution) {
		if (resource instanceof IRepositoryResource) {
			this.repositoryResource = (IRepositoryResource) resource;
			this.wcResource = null;
		} else if (resource instanceof IResource) {
			this.wcResource = (IResource) resource;
			this.repositoryResource = null;
		}
		this.adaptable = resource;
		this.propertyProvider = propertyProvider;
		this.backgroundExecution = backgroundExecution;

		this.propertiesComposite.setResource(resource, propertyProvider);
		this.refreshView();
	}

	public void refreshView() {
		boolean operationToFollow = this.propertyProvider != null && this.propertyProvider.getExecutionState() != IStatus.OK;
		this.propertiesComposite.setPending(operationToFollow);
		this.getSite().getShell().getDisplay().syncExec(new Runnable() {
			public void run() {
				PropertiesView.this.showResourceLabel();
				PropertiesView.this.propertiesComposite.initializeComposite();
			}
		});
		CompositeOperation composite = new CompositeOperation("Operation.ShowProperties");
		if (this.propertyProvider != null && this.propertyProvider.getExecutionState() != IStatus.OK) {
			composite.add(this.propertyProvider);
			composite.add(this.propertiesComposite.getRefreshViewOperation(), new IActionOperation[] { this.propertyProvider });
		} else {
			composite.add(this.propertiesComposite.getRefreshViewOperation());
		}

		if (this.backgroundExecution) {
			UIMonitorUtility.doTaskScheduledDefault(composite);
		} else {
			UIMonitorUtility.doTaskScheduledDefault(this, composite);
		}
	}

	public void createPartControl(Composite parent) {
		super.createPartControl(parent);
		IPreferenceStore store = ModelBusTeamUIPlugin.instance().getPreferenceStore();
		this.isLinkWithEditorEnabled = ModelBusTeamPreferences.getPropertiesBoolean(store, ModelBusTeamPreferences.PROPERTY_LINK_WITH_EDITOR_NAME);
		this.propertiesComposite = new PropertiesComposite(parent);
		this.propertiesComposite.setLayoutData(new GridData(GridData.FILL_BOTH));
		this.refreshView();

		// drop-down menu
		IActionBars actionBars = this.getViewSite().getActionBars();
		IMenuManager actionBarsMenu = actionBars.getMenuManager();

		this.linkWithEditorDropDownAction = new Action(ModelBusTeamUIPlugin.instance().getResource("ModelBusView.LinkWith.Label"),
				Action.AS_CHECK_BOX) {
			public void run() {
				PropertiesView.this.linkWithEditor();
				PropertiesView.this.linkWithEditorAction.setChecked(PropertiesView.this.isLinkWithEditorEnabled);
			}
		};
		this.linkWithEditorDropDownAction.setChecked(this.isLinkWithEditorEnabled);

		actionBarsMenu.add(this.linkWithEditorDropDownAction);

		IToolBarManager tbm = actionBars.getToolBarManager();
		tbm.removeAll();
		Action action = new Action(ModelBusTeamUIPlugin.instance().getResource("ModelBusView.Refresh.Label")) {
			public void run() {
				PropertiesView.this.refreshAction();
			}
		};
		action.setImageDescriptor(ModelBusTeamUIPlugin.instance().getImageDescriptor("icons/common/refresh.gif"));
		tbm.add(action);
		tbm.add(this.getLinkWithEditorAction());

		tbm.update(true);

		this.getSite().getPage().addSelectionListener(this.selectionListener);

		// Setting context help
		PlatformUI.getWorkbench().getHelpSystem().setHelp(parent, "org.modelbus.team.eclipse.help.propertiesViewContext");
	}

	protected void disconnectView() {
		this.propertiesComposite.disconnectComposite();
		this.wcResource = null;
	}

	protected void refreshAction() {
		if (PropertiesView.this.repositoryResource != null) {
			PropertiesView.this.propertyProvider = new GetRemotePropertiesOperation(PropertiesView.this.repositoryResource);
			PropertiesView.this.propertiesComposite.setResource(PropertiesView.this.adaptable, PropertiesView.this.propertyProvider);
		}
		PropertiesView.this.refreshView();
	}

	protected Action getLinkWithEditorAction() {
		this.linkWithEditorAction = new Action(ModelBusTeamUIPlugin.instance().getResource("ModelBusView.LinkWith.Label"), IAction.AS_CHECK_BOX) {
			public void run() {
				PropertiesView.this.linkWithEditor();
				PropertiesView.this.linkWithEditorDropDownAction.setChecked(PropertiesView.this.isLinkWithEditorEnabled);
			}
		};
		this.linkWithEditorAction.setToolTipText(ModelBusTeamUIPlugin.instance().getResource("ModelBusView.LinkWith.ToolTip"));
		this.linkWithEditorAction.setDisabledImageDescriptor(ModelBusTeamUIPlugin.instance().getImageDescriptor(
				"icons/views/history/link_with_disabled.gif"));
		this.linkWithEditorAction.setHoverImageDescriptor(ModelBusTeamUIPlugin.instance().getImageDescriptor("icons/views/history/link_with.gif"));

		this.linkWithEditorAction.setChecked(this.isLinkWithEditorEnabled);

		return this.linkWithEditorAction;
	}

	protected void linkWithEditor() {
		this.isLinkWithEditorEnabled = !this.isLinkWithEditorEnabled;
		IPreferenceStore store = ModelBusTeamUIPlugin.instance().getPreferenceStore();
		ModelBusTeamPreferences.setPropertiesBoolean(store, ModelBusTeamPreferences.PROPERTY_LINK_WITH_EDITOR_NAME, this.isLinkWithEditorEnabled);
		if (this.isLinkWithEditorEnabled) {
			this.editorActivated(this.getSite().getPage().getActiveEditor());
		}
	}

	protected void updateViewInput(IRepositoryResource resource) {
		if (this.repositoryResource != null && this.repositoryResource.equals(resource)) {
			return;
		}
		if (resource != null) {
			this.setResource(resource, new GetRemotePropertiesOperation(resource), true);
		}
	}

	protected void updateViewInput(IResource resource) {
		ILocalResource local = ModelBusRemoteStorage.instance().asLocalResource(resource);
		if (IStateFilter.SF_EXCLUDE_PREREPLACED_AND_DELETED.accept(local)) {
			if (local.getResource().equals(this.wcResource)) {
				return;
			}
			this.setResource(resource, new GetPropertiesOperation(resource), true);
		}
	}

	protected boolean needsLinkWithEditorAndSelection() {
		return true;
	}

}
