/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alessandro Nistico - [patch] Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.synchronize;

import java.text.MessageFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.resources.IWorkspaceRunnable;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.team.core.synchronize.SyncInfo;
import org.eclipse.team.internal.core.subscribers.ChangeSet;
import org.eclipse.team.internal.core.subscribers.CheckedInChangeSet;
import org.eclipse.team.internal.ui.synchronize.SyncInfoSetChangeSetCollector;
import org.eclipse.team.ui.synchronize.ISynchronizePageConfiguration;
import org.modelbus.team.eclipse.core.ModelBusTeamPlugin;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.resource.ILocalResource;
import org.modelbus.team.eclipse.core.resource.IResourceChange;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.ui.synchronize.variant.ResourceVariant;
import org.modelbus.team.eclipse.ui.utility.DateFormatter;

/**
 * Change Set collector implementation.
 * 
 * @author Alessandro Nistico
 */
public class ModelBusChangeSetCollector extends SyncInfoSetChangeSetCollector {
	public ModelBusChangeSetCollector(ISynchronizePageConfiguration configuration) {
		super(configuration);
	}

	public void dispose() {
		ModelBusChangeSetCapability.isEnabled = false;
		super.dispose();
	}

	protected void add(SyncInfo[] infos) {
		if (infos == null || infos.length == 0) {
			return;
		}
		Map<String, ModelBusCheckedInChangeSet> sets = new HashMap<String, ModelBusCheckedInChangeSet>();
		final Set<ModelBusCheckedInChangeSet> added = new HashSet<ModelBusCheckedInChangeSet>();
		for (ChangeSet set : this.getSets()) {
			ModelBusCheckedInChangeSet modelbusSet = (ModelBusCheckedInChangeSet) set;
			sets.put(modelbusSet.getRevision().getRevision(), modelbusSet);
		}
		// change set name format is: revisionNum (date) [author] ...comment...
		String modelbusAuthor = ModelBusTeamPlugin.instance().getResource("ModelBusInfo.Author");
		String modelbusDate = ModelBusTeamPlugin.instance().getResource("ModelBusInfo.Date");
		String modelbusNoAuthor = ModelBusTeamPlugin.instance().getResource("ModelBusInfo.NoAuthor");
		String modelbusNoDate = ModelBusTeamPlugin.instance().getResource("ModelBusInfo.NoDate");
		for (SyncInfo info : infos) {
			if ((info.getKind() & SyncInfo.INCOMING) == 0) {
				continue;
			}
			ResourceVariant remote = (ResourceVariant) info.getRemote();
			ILocalResource resource = remote.getResource();
			ModelBusRevision revision = resource.getRevision();
			ModelBusCheckedInChangeSet set = sets.get(revision);
			boolean updateName = false;
			if (set == null) {
				set = new ModelBusCheckedInChangeSet();
				set.author = resource.getAuthor();
				set.date = new Date(resource.getLastCommitDate());
				set.revision = revision;
				if (resource instanceof IResourceChange) {
					set.comment = ((IResourceChange) resource).getComment();
				}
				updateName = true;
				sets.put(revision.getRevision(), set);
				added.add(set);
			} else if (set.date.getTime() == 0) {
				updateName = true;
				set.date = new Date(resource.getLastCommitDate());
			} else if (set.author == null) {
				updateName = true;
				set.author = resource.getAuthor();
			}
			if (updateName) {
				// rebuild name
				String name = String.valueOf(revision)
						+ " "
						+ (resource.getLastCommitDate() == 0 ? modelbusNoDate : MessageFormat.format(modelbusDate,
								new Object[] { DateFormatter.formatDate(set.date) }))
						+ " "
						+ (resource.getAuthor() == null ? modelbusNoAuthor : MessageFormat.format(modelbusAuthor,
								new Object[] { resource.getAuthor() }));
				if (set.comment != null) {
					String comment = set.comment;
					if (FileUtility.isWindows()) {
						comment = comment.replaceAll("\r\n|\r|\n", " ");
					}
					name += " " + comment;
				}
				set.setName(name);
			}

			set.add(info);
		}

		// lesser UI blinking and thread safe...
		this.performUpdate(new IWorkspaceRunnable() {
			public void run(IProgressMonitor monitor) throws CoreException {
				for (ModelBusCheckedInChangeSet set : added) {
					ModelBusChangeSetCollector.this.add(set);
				}
			}
		}, true, new NullProgressMonitor());
	}

	protected void initializeSets() {
	}

	public static class ModelBusCheckedInChangeSet extends CheckedInChangeSet {
		private String	author;

		private String	comment;

		private Date	date;

		private ModelBusRevision	revision;

		public ModelBusRevision getRevision() {
			return this.revision;
		}

		public String getAuthor() {
			return this.author;
		}

		public Date getDate() {
			return this.date;
		}

		public String getComment() {
			return this.comment;
		}

		public void setName(String name) {
			super.setName(name);
		}
	}

}
