/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.synchronize.update;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.team.core.synchronize.SyncInfo;
import org.modelbus.team.eclipse.core.IStateFilter;
import org.modelbus.team.eclipse.core.connector.ModelBusChangeStatus;
import org.modelbus.team.eclipse.core.connector.ModelBusEntry;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryStatus;
import org.modelbus.team.eclipse.core.connector.ModelBusLogEntry;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRemoteStorage;
import org.modelbus.team.eclipse.core.operation.CompositeOperation;
import org.modelbus.team.eclipse.core.operation.IActionOperation;
import org.modelbus.team.eclipse.core.operation.local.IRemoteStatusOperation;
import org.modelbus.team.eclipse.core.operation.local.RemoteStatusOperation;
import org.modelbus.team.eclipse.core.operation.remote.GetLogMessagesOperation;
import org.modelbus.team.eclipse.core.resource.IChangeStateProvider;
import org.modelbus.team.eclipse.core.resource.ICommentProvider;
import org.modelbus.team.eclipse.core.resource.IFileChange;
import org.modelbus.team.eclipse.core.resource.ILocalResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.resource.IResourceChange;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.core.utility.ProgressMonitorUtility;
import org.modelbus.team.eclipse.ui.synchronize.AbstractModelbusSubscriber;

/**
 * Synchronize view data provider
 * 
 * @author Alexander Gurov
 */
public class UpdateSubscriber extends AbstractModelbusSubscriber {
	private static UpdateSubscriber			instance	= null;

	protected Map<ModelBusRevision, String>	comments;

	public static synchronized UpdateSubscriber instance() {
		if (UpdateSubscriber.instance == null) {
			UpdateSubscriber.instance = new UpdateSubscriber();
		}
		return UpdateSubscriber.instance;
	}

	public void refresh(IResource[] resources, int depth, IProgressMonitor monitor) {
		this.comments.clear();
		super.refresh(resources, depth, monitor);
	}

	protected IRemoteStatusOperation addStatusOperation(CompositeOperation op, IResource[] resources, int depth) {
		RemoteStatusOperation rStatus = new RemoteStatusOperation(resources);
		op.add(rStatus);
		return rStatus;
	}

	protected SyncInfo getModelBusSyncInfo(ILocalResource localStatus, IResourceChange remoteStatus) {
		return new UpdateSyncInfo(localStatus, remoteStatus, this.getResourceComparator());
	}

	protected IResourceChange handleResourceChange(IRemoteStatusOperation rStatusOp, final ModelBusEntryStatus status) {
		final ModelBusChangeStatus current = (ModelBusChangeStatus) status;
		if (current.textStatus == ModelBusEntryStatus.Kind.EXTERNAL) {
			return null;
		}
		final IResource[] scope = rStatusOp.getScope();
		IChangeStateProvider provider = new IChangeStateProvider() {
			public long getChangeDate() {
				return current.reposLastCmtRevision == ModelBusRevision.INVALID_REVISION_DESCRIPTOR ? current.lastChangedDate : current.reposLastCmtDate;
			}

			public String getChangeAuthor() {
				return current.reposLastCmtRevision == ModelBusRevision.INVALID_REVISION_DESCRIPTOR ? current.lastCommitAuthor
						: current.reposLastCmtAuthor;
			}

			public ModelBusRevision getChangeRevision() {
				ModelBusRevision changeRev = ModelBusRevision.INVALID_REVISION_DESCRIPTOR.equals(current.reposLastCmtRevision) ? current.lastChangedRevision
						: current.reposLastCmtRevision;
				return ModelBusRevision.INVALID_REVISION_STRING.equals(changeRev) ? null : changeRev;
			}

			public int getTextChangeType() {
				return current.repositoryTextStatus;
			}

			public int getPropertiesChangeType() {
				return current.repositoryPropStatus;
			}

			public int getNodeKind() {
				int kind = ModelBusUtility.getNodeKind(current.path, current.nodeKind, true);
				return kind == ModelBusEntry.Kind.NONE ? ModelBusUtility.getNodeKind(current.path, current.reposKind, true) : kind;
			}

			public String getLocalPath() {
				return current.path;
			}

			public String getComment() {
				return null;
			}

			public boolean isCopied() {
				return current.isCopied;
			}

			public boolean isSwitched() {
				return current.isSwitched;
			}

			public IResource getExact(IResource[] set) {
				return FileUtility.selectOneOf(scope, set);
			}
		};
		if (provider.getNodeKind() == ModelBusEntry.Kind.NONE) {
			return null;
		}
		IResourceChange resourceChange = ModelBusRemoteStorage.instance().asResourceChange(provider, true);
		if (resourceChange == null || resourceChange.getRevision() == ModelBusRevision.INVALID_REVISION_DESCRIPTOR) {
			return null;
		}
		IResourceChange checkForReplacement = ModelBusRemoteStorage.instance().resourceChangeFromBytes(
				this.statusCache.getBytes(resourceChange.getResource()));
		if (checkForReplacement != null) {
			if (IStateFilter.SF_ADDED.accept(checkForReplacement)) {
				if (IStateFilter.SF_DELETED.accept(resourceChange)) {
					checkForReplacement.treatAsReplacement();
				}
				return checkForReplacement;
			}
			if (IStateFilter.SF_DELETED.accept(checkForReplacement) && IStateFilter.SF_ADDED.accept(resourceChange)) {
				resourceChange.treatAsReplacement();
			}
		}

		rStatusOp.setPegRevision(resourceChange);
		IRepositoryResource originator = ModelBusRemoteStorage.instance().asRepositoryResource(resourceChange.getResource());
		if (originator != null) {
			// for case sensitive name changes, nulls allowed for externals roots
			String url = ModelBusUtility.decodeURL(current.url);
			IRepositoryResource tOriginator = resourceChange instanceof IFileChange ? (IRepositoryResource) originator.asRepositoryFile(url, true)
					: (IRepositoryResource) originator.asRepositoryContainer(url, true);
			if (tOriginator != null) {
				originator = tOriginator;
			}
			originator.setSelectedRevision(resourceChange.getRevision());
			originator.setPegRevision(resourceChange.getPegRevision());
			resourceChange.setOriginator(originator);
		}

		resourceChange.setCommentProvider(new ICommentProvider() {
			public String getComment(IResource resource, ModelBusRevision rev, ModelBusRevision peg) {
				// Null is also valid value if no comment was specified for revision. So, check for key presence.
				if (!UpdateSubscriber.this.comments.containsKey(rev)) {
					this.cacheComments(resource, rev, peg);
				}
				return UpdateSubscriber.this.comments.get(rev);
			}

			public void cacheComments(IResource resource, ModelBusRevision rev, ModelBusRevision peg) {
				if (rev == ModelBusRevision.INVALID_REVISION_DESCRIPTOR || peg != null && peg == ModelBusRevision.INVALID_REVISION_DESCRIPTOR) {
					return;
				}
				// we optimized comment fetching by speed regarding to fact that only number revision used by this implementation of ICommentProvider
				// and select messages for project root (helpful in case of multiple-project layouts)...
				IRepositoryResource remote = ModelBusRemoteStorage.instance().asRepositoryResource(resource).getRoot();
				remote.setSelectedRevision(rev);
				remote.setPegRevision(peg);
				GetLogMessagesOperation op = new GetLogMessagesOperation(remote);
				op.setLimit(20);
				op.setDiscoverPaths(false);
				ProgressMonitorUtility.doTaskExternalDefault(op, new NullProgressMonitor());
				if (op.getExecutionState() == IActionOperation.OK) {
					for (ModelBusLogEntry entry : op.getMessages()) {
						UpdateSubscriber.this.comments.put(ModelBusRevision.fromString(entry.revision), entry.message);
					}
				}
			}
		});
		return resourceChange;
	}

	protected boolean isIncoming(ModelBusEntryStatus status) {
		ModelBusChangeStatus st = (ModelBusChangeStatus) status;
		return st.repositoryPropStatus == ModelBusEntryStatus.Kind.MODIFIED || st.repositoryTextStatus != ModelBusEntryStatus.Kind.NONE;
	}

	private UpdateSubscriber() {
		super();
		this.comments = new HashMap<ModelBusRevision, String>();
	}

}
