/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.synchronize.update.action;

import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.eclipse.compare.structuremergeviewer.IDiffElement;
import org.eclipse.core.resources.IResource;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.team.core.synchronize.FastSyncInfoFilter;
import org.eclipse.team.core.synchronize.SyncInfo;
import org.eclipse.team.ui.synchronize.ISynchronizePageConfiguration;
import org.modelbus.team.eclipse.core.IStateFilter;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.modelbusstorage.ResourcesParentsProvider;
import org.modelbus.team.eclipse.core.operation.CompositeOperation;
import org.modelbus.team.eclipse.core.operation.IActionOperation;
import org.modelbus.team.eclipse.core.operation.local.RefreshResourcesOperation;
import org.modelbus.team.eclipse.core.operation.local.RestoreProjectMetaOperation;
import org.modelbus.team.eclipse.core.operation.local.SaveProjectMetaOperation;
import org.modelbus.team.eclipse.core.operation.local.UpdateOperation;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.operation.ClearUpdateStatusesOperation;
import org.modelbus.team.eclipse.ui.operation.NotifyUnresolvedConflictOperation;
import org.modelbus.team.eclipse.ui.synchronize.action.AbstractSynchronizeModelAction;
import org.modelbus.team.eclipse.ui.synchronize.update.UpdateSyncInfo;
import org.modelbus.team.eclipse.ui.utility.UnacceptableOperationNotificator;

/**
 * Synchronize view update action implementation
 * 
 * @author Alexander Gurov
 */
public class UpdateAction extends AbstractSynchronizeModelAction {
	protected boolean	advancedMode;

	public UpdateAction(String text, ISynchronizePageConfiguration configuration) {
		super(text, configuration);
		this.advancedMode = false;
	}

	public UpdateAction(String text, ISynchronizePageConfiguration configuration, ISelectionProvider selectionProvider) {
		super(text, configuration, selectionProvider);
		this.advancedMode = true;
	}

	protected FastSyncInfoFilter getSyncInfoFilter() {
		// FastSyncInfoFilter.AndSyncInfoFilter()
		// FastSyncInfoFilter.OrSyncInfoFilter()
		// FastSyncInfoFilter.SyncInfoDirectionFilter()
		// FastSyncInfoFilter.SyncInfoChangeTypeFilter()
		return new FastSyncInfoFilter.SyncInfoDirectionFilter(new int[] { SyncInfo.INCOMING, SyncInfo.CONFLICTING }) {
			public boolean select(SyncInfo info) {
				return super.select(info) && !IStateFilter.SF_OBSTRUCTED.accept(((UpdateSyncInfo) info).getLocalResource());
			}
		};
	}

	protected IActionOperation getOperation(ISynchronizePageConfiguration configuration, IDiffElement[] elements) {
		// IStateFilter.SF_NONVERSIONED not versioned locally
		IResource[] resources = UnacceptableOperationNotificator.shrinkResourcesWithNotOnRespositoryParents(configuration.getSite().getShell(),
				this.syncInfoSelector.getSelectedResources());
		if (resources == null || resources.length == 0) {
			return null;
		}

		resources = FileUtility.addOperableParents(resources, IStateFilter.SF_UNVERSIONED);

		final IResource[] missing = FileUtility.getResourcesRecursive(resources, IStateFilter.SF_MISSING);// , IResource.DEPTH_ZERO
		if (missing.length > 0) {
			if (!org.modelbus.team.eclipse.ui.action.local.UpdateAction.updateMissing(configuration.getSite().getShell(), missing)) {
				return null;
			}
		}
		if (this.advancedMode) {
			String message;
			if (resources.length == 1) {
				message = ModelBusTeamUIPlugin.instance().getResource("UpdateAll.Message.Single");
			} else {
				message = ModelBusTeamUIPlugin.instance().getResource("UpdateAll.Message.Multi", new String[] { String.valueOf(resources.length) });
			}
			MessageDialog dlg = new MessageDialog(configuration.getSite().getShell(), ModelBusTeamUIPlugin.instance().getResource("UpdateAll.Title"),
					null, message, MessageDialog.QUESTION, new String[] { IDialogConstants.YES_LABEL, IDialogConstants.NO_LABEL }, 0);
			if (dlg.open() != 0) {
				return null;
			}
		}

		CompositeOperation op = new CompositeOperation("Operation.Update");

		SaveProjectMetaOperation saveOp = new SaveProjectMetaOperation(resources);
		op.add(saveOp);

		Map<ModelBusRevision, Set<IResource>> splitted = UpdateAction.splitByPegRevision(this, resources);

		for (Map.Entry<ModelBusRevision, Set<IResource>> entry : splitted.entrySet()) {
			UpdateOperation mainOp = new UpdateOperation(entry.getValue().toArray(new IResource[0]), entry.getKey(), true);
			op.add(mainOp);
			op.add(new ClearUpdateStatusesOperation(mainOp));
			op.add(new NotifyUnresolvedConflictOperation(mainOp));
		}

		op.add(new RestoreProjectMetaOperation(saveOp));
		op.add(new RefreshResourcesOperation(new ResourcesParentsProvider(resources)/*
																					 * , IResource.DEPTH_INFINITE,
																					 * RefreshResourcesOperation.REFRESH_ALL
																					 */));

		return op;
	}

	public static Map<ModelBusRevision, Set<IResource>> splitByPegRevision(AbstractSynchronizeModelAction action, IResource[] resources) {
		Map<ModelBusRevision, Set<IResource>> splitted = new HashMap<ModelBusRevision, Set<IResource>>();
		// NOTE See bug 237204: ModelBus replaces files with second last revision after commit: https://bugs.eclipse.org/bugs/show_bug.cgi?id=237204
		// Due to described problem we should roll back code which prepares update to revisions saved in synchronize view, because potential damage
		// from updating of unapproved changes many times smaller than reverting all committed files to its previous states
		splitted.put(ModelBusRevision.HEAD, new HashSet<IResource>(Arrays.asList(resources)));
		// for (IResource resource : resources) {
		// try {
		// AbstractModelBusSyncInfo info = (AbstractModelBusSyncInfo)UpdateSubscriber.instance().getSyncInfo(resource);
		// ILocalResource local = ((RemoteResourceVariant)info.getRemote()).getResource();
		// // can be ILocalResource in context of OverrideAndUpdateAction
		// if (local instanceof IResourceChange) {
		// ModelBusRevision pegRev = ((IResourceChange)local).getPegRevision();
		// Set<IResource> list = splitted.get(pegRev);
		// if (list == null) {
		// splitted.put(pegRev, list = new HashSet<IResource>());
		// }
		// list.add(resource);
		// }
		// }
		// catch (TeamException ex) {
		// UILoggedOperation.reportError(action.getText(), ex);
		// return null;
		// }
		// }
		return splitted;
	}

}
